@@ -10,8 +10,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkBitmap.c,v 1.10.2.1 2006/02/27 11:36:02 dkf Exp $
  */
 
 #include "tkPort.h"
@@ -26,7 +24,7 @@
  * temporarily turn off that warning message.
  */
 
-#if defined(__WIN32__) || defined(_WIN32)
+#if defined(_MSC_VER)
 #pragma warning (disable : 4305)
 #endif
 
@@ -41,7 +39,7 @@
 #include "question.bmp"
 #include "warning.bmp"
 
-#if defined(__WIN32__) || defined(_WIN32)
+#if defined(_MSC_VER)
 #pragma warning (default : 4305)
 #endif
 
