@@ -10,8 +10,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkButton.c,v 1.20.2.2 2003/11/12 00:05:13 hobbs Exp $
  */
 
 #include "tkButton.h"
@@ -27,14 +25,14 @@ static Tcl_ThreadDataKey dataKey;
  * in tkButton.h.
  */
 
-static char *classNames[] = {"Label", "Button", "Checkbutton", "Radiobutton"};
+static CONST char *CONST classNames[] = {"Label", "Button", "Checkbutton", "Radiobutton"};
 
 /*
  * The following table defines the legal values for the -default option.
  * It is used together with the "enum defaultValue" declaration in tkButton.h.
  */
 
-static char *defaultStrings[] = {
+static CONST char *CONST defaultStrings[] = {
     "active", "disabled", "normal", (char *) NULL
 };
 
@@ -43,7 +41,7 @@ static char *defaultStrings[] = {
  * It is used together with the "enum state" declaration in tkButton.h.
  */
 
-static char *stateStrings[] = {
+static CONST char *CONST stateStrings[] = {
     "active", "disabled", "normal", (char *) NULL
 };
 
@@ -52,16 +50,18 @@ static char *stateStrings[] = {
  * It is used with the "enum compound" declaration in tkButton.h
  */
 
-static char *compoundStrings[] = {
+static CONST char *CONST compoundStrings[] = {
     "bottom", "center", "left", "none", "right", "top", (char *) NULL
 };
 
+char tkDefButtonBorderWidth[TCL_INTEGER_SPACE] = DEF_BUTTON_BORDER_WIDTH;
+
 /*
  * Information used for parsing configuration options.  There is a
  * separate table for each of the four widget classes.
  */
 
-static Tk_OptionSpec labelOptionSpecs[] = {
+static CONST Tk_OptionSpec labelOptionSpecs[] = {
     {TK_OPTION_BORDER, "-activebackground", "activeBackground", "Foreground",
 	DEF_BUTTON_ACTIVE_BG_COLOR, -1, Tk_Offset(TkButton, activeBorder),
 	0, (ClientData) DEF_BUTTON_ACTIVE_BG_MONO, 0},
@@ -81,7 +81,7 @@ static Tk_OptionSpec labelOptionSpecs[] = {
 	DEF_BUTTON_BITMAP, -1, Tk_Offset(TkButton, bitmap),
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
-	DEF_BUTTON_BORDER_WIDTH, Tk_Offset(TkButton, borderWidthPtr),
+	tkDefButtonBorderWidth, Tk_Offset(TkButton, borderWidthPtr),
 	Tk_Offset(TkButton, borderWidth), 0, 0, 0},
     {TK_OPTION_STRING_TABLE, "-compound", "compound", "Compound",
 	 DEF_BUTTON_COMPOUND, -1, Tk_Offset(TkButton, compound), 0,
@@ -147,7 +147,7 @@ static Tk_OptionSpec labelOptionSpecs[] = {
 	(char *) NULL, 0, 0, 0, 0}
 };
 
-static Tk_OptionSpec buttonOptionSpecs[] = {
+static CONST Tk_OptionSpec buttonOptionSpecs[] = {
     {TK_OPTION_BORDER, "-activebackground", "activeBackground", "Foreground",
 	DEF_BUTTON_ACTIVE_BG_COLOR, -1, Tk_Offset(TkButton, activeBorder),
 	0, (ClientData) DEF_BUTTON_ACTIVE_BG_MONO, 0},
@@ -167,7 +167,7 @@ static Tk_OptionSpec buttonOptionSpecs[] = {
 	DEF_BUTTON_BITMAP, -1, Tk_Offset(TkButton, bitmap),
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
-	DEF_BUTTON_BORDER_WIDTH, Tk_Offset(TkButton, borderWidthPtr),
+	tkDefButtonBorderWidth, Tk_Offset(TkButton, borderWidthPtr),
 	Tk_Offset(TkButton, borderWidth), 0, 0, 0},
     {TK_OPTION_STRING, "-command", "command", "Command",
 	DEF_BUTTON_COMMAND, Tk_Offset(TkButton, commandPtr), -1,
@@ -249,7 +249,7 @@ static Tk_OptionSpec buttonOptionSpecs[] = {
 	(char *) NULL, 0, -1, 0, 0, 0}
 };
 
-static Tk_OptionSpec checkbuttonOptionSpecs[] = {
+static CONST Tk_OptionSpec checkbuttonOptionSpecs[] = {
     {TK_OPTION_BORDER, "-activebackground", "activeBackground", "Foreground",
 	DEF_BUTTON_ACTIVE_BG_COLOR, -1, Tk_Offset(TkButton, activeBorder),
 	0, (ClientData) DEF_BUTTON_ACTIVE_BG_MONO, 0},
@@ -269,7 +269,7 @@ static Tk_OptionSpec checkbuttonOptionSpecs[] = {
 	DEF_BUTTON_BITMAP, -1, Tk_Offset(TkButton, bitmap),
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
-	DEF_BUTTON_BORDER_WIDTH, Tk_Offset(TkButton, borderWidthPtr),
+	tkDefButtonBorderWidth, Tk_Offset(TkButton, borderWidthPtr),
 	Tk_Offset(TkButton, borderWidth), 0, 0, 0},
     {TK_OPTION_STRING, "-command", "command", "Command",
 	DEF_BUTTON_COMMAND, Tk_Offset(TkButton, commandPtr), -1,
@@ -358,7 +358,7 @@ static Tk_OptionSpec checkbuttonOptionSpecs[] = {
 	(char *) NULL, 0, -1, 0, 0, 0}
 };
 
-static Tk_OptionSpec radiobuttonOptionSpecs[] = {
+static CONST Tk_OptionSpec radiobuttonOptionSpecs[] = {
     {TK_OPTION_BORDER, "-activebackground", "activeBackground", "Foreground",
 	DEF_BUTTON_ACTIVE_BG_COLOR, -1, Tk_Offset(TkButton, activeBorder),
 	0, (ClientData) DEF_BUTTON_ACTIVE_BG_MONO, 0},
@@ -378,7 +378,7 @@ static Tk_OptionSpec radiobuttonOptionSpecs[] = {
 	DEF_BUTTON_BITMAP, -1, Tk_Offset(TkButton, bitmap),
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
-	DEF_BUTTON_BORDER_WIDTH, Tk_Offset(TkButton, borderWidthPtr),
+	tkDefButtonBorderWidth, Tk_Offset(TkButton, borderWidthPtr),
 	Tk_Offset(TkButton, borderWidth), 0, 0, 0},
     {TK_OPTION_STRING, "-command", "command", "Command",
 	DEF_BUTTON_COMMAND, Tk_Offset(TkButton, commandPtr), -1,
@@ -472,7 +472,7 @@ static Tk_OptionSpec radiobuttonOptionSpecs[] = {
  * class of widgets.
  */
 
-static Tk_OptionSpec *optionSpecs[] = {
+static CONST Tk_OptionSpec *CONST optionSpecs[] = {
     labelOptionSpecs,
     buttonOptionSpecs,
     checkbuttonOptionSpecs,
@@ -631,7 +631,7 @@ ButtonCreate(clientData, interp, objc, objv, type)
 	Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     if (!tsdPtr->defaultsInitialized) {
-	TkpButtonSetDefaults(optionSpecs[type]);
+	TkpButtonSetDefaults(NULL);
 	tsdPtr->defaultsInitialized = 1;
     }
 
