@@ -9,8 +9,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkCanvLine.c,v 1.7.2.2 2001/08/24 23:59:11 hobbs Exp $
  */
 
 #include <stdio.h>
@@ -302,6 +300,10 @@ CreateLine(interp, canvas, itemPtr, objc, objv)
     LineItem *linePtr = (LineItem *) itemPtr;
     int i;
 
+    if (objc == 0) {
+	panic("canvas did not pass any coords\n");
+    }
+
     /*
      * Carry out initialization that is needed to set defaults and to
      * allow proper cleanup after errors during the the remainder of
@@ -330,13 +332,13 @@ CreateLine(interp, canvas, itemPtr, objc, objv)
      * start with a digit or a minus sign followed by a digit.
      */
 
-    for (i = 0; i < objc; i++) {
+    for (i = 1; i < objc; i++) {
 	char *arg = Tcl_GetString(objv[i]);
 	if ((arg[0] == '-') && (arg[1] >= 'a') && (arg[1] <= 'z')) {
 	    break;
 	}
     }
-    if (i && (LineCoords(interp, canvas, itemPtr, i, objv) != TCL_OK)) {
+    if (LineCoords(interp, canvas, itemPtr, i, objv) != TCL_OK) {
 	goto error;
     }
     if (ConfigureLine(interp, canvas, itemPtr, objc-i, objv+i, 0) == TCL_OK) {
@@ -411,14 +413,15 @@ LineCoords(interp, canvas, itemPtr, objc, objv)
 	}
     }
     if (objc & 1) {
-	Tcl_AppendResult(interp,
-		"odd number of coordinates specified for line",
-		(char *) NULL);
+	char buf[64 + TCL_INTEGER_SPACE];
+	sprintf(buf, "wrong # coordinates: expected an even number, got %d",
+		objc);
+	Tcl_SetResult(interp, buf, TCL_VOLATILE);
 	return TCL_ERROR;
     } else if (objc < 4) {
-	Tcl_AppendResult(interp,
-		"too few coordinates specified for line",
-		(char *) NULL);
+	char buf[64 + TCL_INTEGER_SPACE];
+	sprintf(buf, "wrong # coordinates: expected at least 4, got %d", objc);
+	Tcl_SetResult(interp, buf, TCL_VOLATILE);
 	return TCL_ERROR;
     } else {
 	numPoints = objc/2;
@@ -496,8 +499,8 @@ ConfigureLine(interp, canvas, itemPtr, objc, objv, flags)
     Tk_State state;
 
     tkwin = Tk_CanvasTkwin(canvas);
-    if (Tk_ConfigureWidget(interp, tkwin, configSpecs, objc, (char **) objv,
-	    (char *) linePtr, flags|TK_CONFIG_OBJS) != TCL_OK) {
+    if (TCL_OK != Tk_ConfigureWidget(interp, tkwin, configSpecs, objc,
+	    (CONST char **) objv, (char *) linePtr, flags|TK_CONFIG_OBJS)) {
 	return TCL_ERROR;
     }
 
@@ -530,7 +533,15 @@ ConfigureLine(interp, canvas, itemPtr, objc, objv, flags)
 	gcValues.join_style = linePtr->joinStyle;
 	mask |= GCJoinStyle;
 	newGC = Tk_GetGC(tkwin, mask, &gcValues);
+#ifdef MAC_OSX_TK
+	/*
+	 * Mac OS X CG drawing needs access to linewidth even for 
+	 * arrow fills (as linewidth controls antialiasing).
+	 */
+	mask |= GCLineWidth;
+#else
 	gcValues.line_width = 0;
+#endif
 	arrowGC = Tk_GetGC(tkwin, mask, &gcValues);
     } else {
 	newGC = arrowGC = None;
@@ -848,13 +859,11 @@ DisplayLine(canvas, itemPtr, display, drawable, x, y, width, height)
 					 * must be redisplayed (not used). */
 {
     LineItem *linePtr = (LineItem *) itemPtr;
-    XPoint staticPoints[MAX_STATIC_POINTS];
+    XPoint staticPoints[MAX_STATIC_POINTS*3];
     XPoint *pointPtr;
-    XPoint *pPtr;
-    double *coordPtr, linewidth;
-    int i, numPoints;
+    double linewidth;
+    int numPoints;
     Tk_State state = itemPtr->state;
-    Pixmap stipple = linePtr->outline.stipple;
 
     if ((!linePtr->numPoints)||(linePtr->outline.gc==None)) {
 	return;
@@ -865,16 +874,10 @@ DisplayLine(canvas, itemPtr, display, drawable, x, y, width, height)
     }
     linewidth = linePtr->outline.width;
     if (((TkCanvas *)canvas)->currentItemPtr == itemPtr) {
-	if (linePtr->outline.activeStipple != None) {
-	    stipple = linePtr->outline.activeStipple;
-	}
 	if (linePtr->outline.activeWidth != linewidth) {
 	    linewidth = linePtr->outline.activeWidth;
 	}
     } else if (state==TK_STATE_DISABLED) {
-	if (linePtr->outline.disabledStipple != None) {
-	    stipple = linePtr->outline.disabledStipple;
-	}
 	if (linePtr->outline.disabledWidth != linewidth) {
 	    linewidth = linePtr->outline.disabledWidth;
 	}
@@ -897,7 +900,7 @@ DisplayLine(canvas, itemPtr, display, drawable, x, y, width, height)
     if (numPoints <= MAX_STATIC_POINTS) {
 	pointPtr = staticPoints;
     } else {
-	pointPtr = (XPoint *) ckalloc((unsigned) (numPoints * sizeof(XPoint)));
+	pointPtr = (XPoint *)ckalloc((unsigned)(numPoints * 3*sizeof(XPoint)));
     }
 
     if ((linePtr->smooth) && (linePtr->numPoints > 2)) {
@@ -905,11 +908,8 @@ DisplayLine(canvas, itemPtr, display, drawable, x, y, width, height)
 		linePtr->numPoints, linePtr->splineSteps, pointPtr,
 		(double *) NULL);
     } else {
-	for (i = 0, coordPtr = linePtr->coordPtr, pPtr = pointPtr;
-		i < linePtr->numPoints;  i += 1, coordPtr += 2, pPtr++) {
-	    Tk_CanvasDrawableCoords(canvas, coordPtr[0], coordPtr[1],
-		    &pPtr->x, &pPtr->y);
-	}
+	numPoints = TkCanvTranslatePath((TkCanvas*)canvas, numPoints,
+		linePtr->coordPtr, 0, pointPtr);
     }
 
     /*
@@ -1024,9 +1024,12 @@ LineInsert(canvas, itemPtr, beforeThis, obj)
     for(i=beforeThis; i<length; i++) {
 	new[i+objc] = linePtr->coordPtr[i];
     }
-    if(linePtr->coordPtr) ckfree((char *)linePtr->coordPtr);
+    if (linePtr->coordPtr) {
+	ckfree((char *) linePtr->coordPtr);
+    }
     linePtr->coordPtr = new;
-    linePtr->numPoints = (length + objc)/2;
+    length += objc;
+    linePtr->numPoints = length / 2;
 
     if ((length>3) && (state != TK_STATE_HIDDEN)) {
 	/*
@@ -1728,11 +1731,11 @@ GetLineIndex(interp, canvas, itemPtr, obj, indexPtr)
     int *indexPtr;		/* Where to store converted index. */
 {
     LineItem *linePtr = (LineItem *) itemPtr;
-    size_t length;
-    char *string = Tcl_GetStringFromObj(obj, (int *) &length);
+    int length;
+    char *string = Tcl_GetStringFromObj(obj, &length);
 
     if (string[0] == 'e') {
-	if (strncmp(string, "end", length) == 0) {
+	if (strncmp(string, "end", (unsigned) length) == 0) {
 	    *indexPtr = 2*linePtr->numPoints;
 	} else {
 	    badIndex:
@@ -1872,7 +1875,7 @@ ParseArrowShape(clientData, interp, tkwin, value, recordPtr, offset)
     LineItem *linePtr = (LineItem *) recordPtr;
     double a, b, c;
     int argc;
-    char **argv = NULL;
+    CONST char **argv = NULL;
 
     if (offset != Tk_Offset(LineItem, arrowShapeA)) {
 	panic("ParseArrowShape received bogus offset");
