@@ -8,8 +8,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkCanvText.c,v 1.8.2.1 2001/08/24 23:59:11 hobbs Exp $
  */
 
 #include <stdio.h>
@@ -235,22 +233,10 @@ CreateText(interp, canvas, itemPtr, objc, objv)
     Tcl_Obj *CONST objv[];	/* Arguments describing rectangle. */
 {
     TextItem *textPtr = (TextItem *) itemPtr;
-    int i = 2;
+    int i;
 
-    if (objc == 1) {
-	i = 1;
-    } else if (objc > 1) {
-	char *arg = Tcl_GetString(objv[1]);
-	if ((arg[0] == '-') && (arg[1] >= 'a') && (arg[1] <= 'z')) {
-	    i = 1;
-	}
-    }
-
-    if (objc < i) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"",
-		Tk_PathName(Tk_CanvasTkwin(canvas)), " create ",
-		itemPtr->typePtr->name, " x y ?options?\"", (char *) NULL);
-	return TCL_ERROR;
+    if (objc == 0) {
+	panic("canvas did not pass any coords\n");
     }
 
     /*
@@ -288,8 +274,18 @@ CreateText(interp, canvas, itemPtr, objc, objv)
 
     /*
      * Process the arguments to fill in the item record.
+     * Only 1 (list) or 2 (x y) coords are allowed.
      */
 
+    if (objc == 1) {
+	i = 1;
+    } else {
+	char *arg = Tcl_GetString(objv[1]);
+	i = 2;
+	if ((arg[0] == '-') && (arg[1] >= 'a') && (arg[1] <= 'z')) {
+	    i = 1;
+	}
+    }
     if ((TextCoords(interp, canvas, itemPtr, i, objv) != TCL_OK)) {
 	goto error;
     }
@@ -407,8 +403,8 @@ ConfigureText(interp, canvas, itemPtr, objc, objv, flags)
     Tk_State state;
 
     tkwin = Tk_CanvasTkwin(canvas);
-    if (Tk_ConfigureWidget(interp, tkwin, configSpecs, objc, (char **) objv,
-	    (char *) textPtr, flags|TK_CONFIG_OBJS) != TCL_OK) {
+    if (TCL_OK != Tk_ConfigureWidget(interp, tkwin, configSpecs, objc,
+	    (CONST char **) objv, (char *) textPtr, flags|TK_CONFIG_OBJS)) {
 	return TCL_ERROR;
     }
 
@@ -468,7 +464,9 @@ ConfigureText(interp, canvas, itemPtr, objc, objv, flags)
 	    gcValues.fill_style = FillStippled;
 	    mask |= GCStipple|GCFillStyle;
 	}
-	gcValues.foreground = textInfoPtr->selFgColorPtr->pixel;
+	if (textInfoPtr->selFgColorPtr != NULL) {
+	    gcValues.foreground = textInfoPtr->selFgColorPtr->pixel;
+	}
 	newSelGC = Tk_GetGC(tkwin, mask|GCForeground, &gcValues);
     }
     if (textPtr->gc != None) {
@@ -635,8 +633,8 @@ ComputeTextBbox(canvas, textPtr)
      * of the bounding box for the text item.
      */
 
-    leftX = (int) (textPtr->x + 0.5);
-    topY = (int) (textPtr->y + 0.5);
+    leftX = (int) floor(textPtr->x + 0.5);
+    topY = (int) floor(textPtr->y + 0.5);
     switch (textPtr->anchor) {
 	case TK_ANCHOR_NW:
 	case TK_ANCHOR_N:
@@ -828,6 +826,8 @@ DisplayCanvText(canvas, itemPtr, display, drawable, x, y, width, height)
 			    - (textInfoPtr->insertWidth / 2)),
 		    (double) (textPtr->header.y1 + y),
 		    &drawableX, &drawableY);
+	    Tk_SetCaretPos(Tk_CanvasTkwin(canvas), drawableX, drawableY,
+		    height);
 	    if (textInfoPtr->cursorOn) {
 		Tk_Fill3DRectangle(Tk_CanvasTkwin(canvas), drawable,
 			textInfoPtr->insertBorder,
@@ -853,21 +853,30 @@ DisplayCanvText(canvas, itemPtr, display, drawable, x, y, width, height)
 
 
     /*
-     * Display the text in two pieces: draw the entire text item, then
-     * draw the selected text on top of it.  The selected text then
-     * will only need to be drawn if it has different attributes (such
-     * as foreground color) than regular text.
+     * If there is no selected text or the selected text foreground
+     * is the same as the regular text foreground, then draw one
+     * text string. If there is selected text and the foregrounds
+     * differ, draw the regular text up to the selection, draw
+     * the selection, then draw the rest of the regular text.
+     * Drawing the regular text and then the selected text over
+     * it would causes problems with anti-aliased text because the
+     * two anti-aliasing colors would blend together.
      */
 
     Tk_CanvasDrawableCoords(canvas, (double) textPtr->leftEdge,
 	    (double) textPtr->header.y1, &drawableX, &drawableY);
-    Tk_DrawTextLayout(display, drawable, textPtr->gc, textPtr->textLayout,
-	    drawableX, drawableY, 0, -1);
 
     if ((selFirstChar >= 0) && (textPtr->selTextGC != textPtr->gc)) {
+	Tk_DrawTextLayout(display, drawable, textPtr->gc, textPtr->textLayout,
+	    drawableX, drawableY, 0, selFirstChar);
 	Tk_DrawTextLayout(display, drawable, textPtr->selTextGC,
 	    textPtr->textLayout, drawableX, drawableY, selFirstChar,
 	    selLastChar + 1);
+	Tk_DrawTextLayout(display, drawable, textPtr->gc, textPtr->textLayout,
+	    drawableX, drawableY, selLastChar + 1, -1);
+    } else {
+	Tk_DrawTextLayout(display, drawable, textPtr->gc, textPtr->textLayout,
+	    drawableX, drawableY, 0, -1);
     }
 
     if (stipple != None) {
@@ -1238,27 +1247,25 @@ GetTextIndex(interp, canvas, itemPtr, obj, indexPtr)
 				 * index. */
 {
     TextItem *textPtr = (TextItem *) itemPtr;
-    size_t length;
-    int c;
+    int c, length;
     TkCanvas *canvasPtr = (TkCanvas *) canvas;
     Tk_CanvasTextInfo *textInfoPtr = textPtr->textInfoPtr;
-    char *string = Tcl_GetStringFromObj(obj, (int *) &length);
+    char *string = Tcl_GetStringFromObj(obj, &length);
 
     c = string[0];
-    length = strlen(string);
 
-    if ((c == 'e') && (strncmp(string, "end", length) == 0)) {
+    if ((c == 'e') && (strncmp(string, "end", (unsigned) length) == 0)) {
 	*indexPtr = textPtr->numChars;
-    } else if ((c == 'i') && (strncmp(string, "insert", length) == 0)) {
+    } else if ((c=='i') && (strncmp(string, "insert", (unsigned) length)==0)) {
 	*indexPtr = textPtr->insertPos;
-    } else if ((c == 's') && (strncmp(string, "sel.first", length) == 0)
+    } else if ((c=='s') && (strncmp(string, "sel.first", (unsigned) length)==0)
 	    && (length >= 5)) {
 	if (textInfoPtr->selItemPtr != itemPtr) {
 	    Tcl_SetResult(interp, "selection isn't in item", TCL_STATIC);
 	    return TCL_ERROR;
 	}
 	*indexPtr = textInfoPtr->selectFirst;
-    } else if ((c == 's') && (strncmp(string, "sel.last", length) == 0)
+    } else if ((c=='s') && (strncmp(string, "sel.last", (unsigned) length)==0)
 	    && (length >= 5)) {
 	if (textInfoPtr->selItemPtr != itemPtr) {
 	    Tcl_SetResult(interp, "selection isn't in item", TCL_STATIC);
@@ -1376,7 +1383,8 @@ GetSelText(canvas, itemPtr, offset, buffer, maxBytes)
 {
     TextItem *textPtr = (TextItem *) itemPtr;
     int byteCount; 
-    char *text, *selStart, *selEnd;
+    char *text;
+    CONST char *selStart, *selEnd;
     Tk_CanvasTextInfo *textInfoPtr = textPtr->textInfoPtr;
 
     if ((textInfoPtr->selectFirst < 0) ||
