@@ -9,13 +9,12 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * SCCS: @(#) tkCanvUtil.c 1.7 96/05/03 10:54:22
  */
 
 #include "tkInt.h"
 #include "tkCanvas.h"
 #include "tkPort.h"
+#include <assert.h>
 
 
 /*
@@ -177,7 +176,7 @@ Tk_CanvasWindowCoords(canvas, x, y, screenXPtr, screenYPtr)
  *	TCL_OK is returned, then everything went well and the
  *	canvas coordinate is stored at *doublePtr;  otherwise
  *	TCL_ERROR is returned and an error message is left in
- *	interp->result.
+ *	the interp's result.
  *
  * Side effects:
  *	None.
@@ -189,7 +188,7 @@ int
 Tk_CanvasGetCoord(interp, canvas, string, doublePtr)
     Tcl_Interp *interp;		/* Interpreter for error reporting. */
     Tk_Canvas canvas;		/* Canvas to which coordinate applies. */
-    char *string;		/* Describes coordinate (any screen
+    CONST char *string;		/* Describes coordinate (any screen
 				 * coordinate form may be used here). */
     double *doublePtr;		/* Place to store converted coordinate. */
 {
@@ -232,7 +231,7 @@ Tk_CanvasGetCoordFromObj(interp, canvas, obj, doublePtr)
     double *doublePtr;		/* Place to store converted coordinate. */
 {
     TkCanvas *canvasPtr = (TkCanvas *) canvas;
-    if (TkGetScreenMMFromObj(canvasPtr->interp, canvasPtr->tkwin, obj,
+    if (Tk_GetMMFromObj(canvasPtr->interp, canvasPtr->tkwin, obj,
 	    doublePtr) != TCL_OK) {
 	return TCL_ERROR;
     }
@@ -277,9 +276,9 @@ Tk_CanvasSetStippleOrigin(canvas, gc)
  *
  * Tk_CanvasSetOffset--
  *
- *	This procedure sets the stipple or tile offset in a graphics
- *	context so that stipples or tiles drawn with the GC will
- *	line up with other stipples or tiles with the same offset.
+ *	This procedure sets the stipple offset in a graphics
+ *	context so that stipples drawn with the GC will
+ *	line up with other stipples with the same offset.
  *
  * Results:
  *	None.
@@ -294,7 +293,7 @@ void
 Tk_CanvasSetOffset(canvas, gc, offset)
     Tk_Canvas canvas;		/* Token for a canvas. */
     GC gc;			/* Graphics context that is about to be
-				 * used to draw a tiled pattern as
+				 * used to draw a stippled pattern as
 				 * part of redisplaying the canvas. */
     Tk_TSOffset *offset;	/* offset (may be NULL pointer)*/
 {
@@ -309,7 +308,7 @@ Tk_CanvasSetOffset(canvas, gc, offset)
 	y += offset->yoffset;
     }
     if ((flags & TK_OFFSET_RELATIVE) && !(flags & TK_OFFSET_INDEX)) {
-	Tk_SetTileOrigin(canvasPtr->tkwin, gc, x - canvasPtr->xOrigin,
+	Tk_SetTSOrigin(canvasPtr->tkwin, gc, x - canvasPtr->xOrigin,
 		y - canvasPtr->yOrigin);
     } else {
 	XSetTSOrigin(canvasPtr->display, gc, x, y);
@@ -369,14 +368,14 @@ Tk_CanvasTagsParseProc(clientData, interp, tkwin, value, widgRec, offset)
     ClientData clientData;		/* Not used.*/
     Tcl_Interp *interp;			/* Used for reporting errors. */
     Tk_Window tkwin;			/* Window containing canvas widget. */
-    char *value;			/* Value of option (list of tag
+    CONST char *value;			/* Value of option (list of tag
 					 * names). */
     char *widgRec;			/* Pointer to record for item. */
     int offset;				/* Offset into item (ignored). */
 {
     register Tk_Item *itemPtr = (Tk_Item *) widgRec;
     int argc, i;
-    char **argv;
+    CONST char **argv;
     Tk_Uid *newPtr;
 
     /*
@@ -454,7 +453,7 @@ Tk_CanvasTagsPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
 	return (char *) itemPtr->tagPtr[0];
     }
     *freeProcPtr = TCL_DYNAMIC;
-    return Tcl_Merge(itemPtr->numTags, (char **) itemPtr->tagPtr);
+    return Tcl_Merge(itemPtr->numTags, (CONST char **) itemPtr->tagPtr);
 }
 
 
@@ -465,7 +464,7 @@ static int	DashConvert _ANSI_ARGS_((char *l, CONST char *p,
 /*
  *--------------------------------------------------------------
  *
- * Tk_CanvasDashParseProc --
+ * TkCanvasDashParseProc --
  *
  *	This procedure is invoked during option processing to handle
  *	"-dash", "-activedash" and "-disableddash" options for canvas
@@ -482,7 +481,7 @@ static int	DashConvert _ANSI_ARGS_((char *l, CONST char *p,
  */
 
 int
-Tk_CanvasDashParseProc(clientData, interp, tkwin, value, widgRec, offset)
+TkCanvasDashParseProc(clientData, interp, tkwin, value, widgRec, offset)
     ClientData clientData;		/* Not used.*/
     Tcl_Interp *interp;			/* Used for reporting errors. */
     Tk_Window tkwin;			/* Window containing canvas widget. */
@@ -496,7 +495,7 @@ Tk_CanvasDashParseProc(clientData, interp, tkwin, value, widgRec, offset)
 /*
  *--------------------------------------------------------------
  *
- * Tk_CanvasDashPrintProc --
+ * TkCanvasDashPrintProc --
  *
  *	This procedure is invoked by the Tk configuration code
  *	to produce a printable string for the "-dash", "-activedash"
@@ -516,7 +515,7 @@ Tk_CanvasDashParseProc(clientData, interp, tkwin, value, widgRec, offset)
  */
 
 char *
-Tk_CanvasDashPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
+TkCanvasDashPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;		/* Ignored. */
     Tk_Window tkwin;			/* Window containing canvas widget. */
     char *widgRec;			/* Pointer to record for item. */
@@ -530,24 +529,25 @@ Tk_CanvasDashPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     char *p;
     int i = dash->number;
 
-    if (i<0) {
+    if (i < 0) {
+	i = -i;
 	*freeProcPtr = TCL_DYNAMIC;
-	buffer = (char *) ckalloc(1-i);
-	p = (-i > sizeof(char *)) ? dash->pattern.pt : dash->pattern.array;
-	memcpy(buffer,p,-i);
-	buffer[-i] = 0;
+	buffer = (char *) ckalloc((unsigned int) (i+1));
+	p = (i > sizeof(char *)) ? dash->pattern.pt : dash->pattern.array;
+	memcpy(buffer, p, (unsigned int) i);
+	buffer[i] = 0;
 	return buffer;
     } else if (!i) {
 	*freeProcPtr = (Tcl_FreeProc *) NULL;
 	return "";
     }
-    buffer = (char *)ckalloc(4*i);
+    buffer = (char *)ckalloc((unsigned int) (4*i));
     *freeProcPtr = TCL_DYNAMIC;
 
     p = (i > sizeof(char *)) ? dash->pattern.pt : dash->pattern.array;
-    sprintf(buffer,"%d",*p++ & 0xff);
+    sprintf(buffer, "%d", *p++ & 0xff);
     while(--i) {
-	sprintf(buffer+strlen(buffer)," %d",*p++ & 0xff);
+	sprintf(buffer+strlen(buffer), " %d", *p++ & 0xff);
     }
     return buffer;
 }
@@ -571,9 +571,11 @@ Tk_CanvasDashPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
  */
 
 Tk_SmoothMethod tkBezierSmoothMethod = {
-    "1",
+    "bezier",
     TkMakeBezierCurve,
-    TkMakeBezierPostscript,
+    (void (*) _ANSI_ARGS_((Tcl_Interp *interp, Tk_Canvas canvas,
+	    double *coordPtr, int numPoints, int numSteps)))
+		TkMakeBezierPostscript,
 };
 
 static void SmoothMethodCleanupProc _ANSI_ARGS_((ClientData clientData,
@@ -590,19 +592,19 @@ Tk_CreateSmoothMethod(interp, smooth)
     Tcl_Interp *interp;
     Tk_SmoothMethod *smooth;
 {
-    SmoothAssocData *method, *typePtr2, *prevPtr, *ptr;
-    method = (SmoothAssocData *) Tcl_GetAssocData(interp, "smoothMethod",
+    SmoothAssocData *methods, *typePtr2, *prevPtr, *ptr;
+    methods = (SmoothAssocData *) Tcl_GetAssocData(interp, "smoothMethod",
 	    (Tcl_InterpDeleteProc **) NULL);
 
     /*
      * If there's already a smooth method with the given name, remove it.
      */
 
-    for (typePtr2 = method, prevPtr = NULL; typePtr2 != NULL;
+    for (typePtr2 = methods, prevPtr = NULL; typePtr2 != NULL;
 	    prevPtr = typePtr2, typePtr2 = typePtr2->nextPtr) {
 	if (!strcmp(typePtr2->smooth.name, smooth->name)) {
 	    if (prevPtr == NULL) {
-		method = typePtr2->nextPtr;
+		methods = typePtr2->nextPtr;
 	    } else {
 		prevPtr->nextPtr = typePtr2->nextPtr;
 	    }
@@ -614,7 +616,7 @@ Tk_CreateSmoothMethod(interp, smooth)
     ptr->smooth.name = smooth->name;
     ptr->smooth.coordProc = smooth->coordProc;
     ptr->smooth.postscriptProc = smooth->postscriptProc;
-    ptr->nextPtr = method;
+    ptr->nextPtr = methods;
     Tcl_SetAssocData(interp, "smoothMethod", SmoothMethodCleanupProc,
 		(ClientData) ptr);
 }
@@ -641,11 +643,11 @@ SmoothMethodCleanupProc(clientData, interp)
 				 * for the interpreter. */
     Tcl_Interp *interp;		/* Interpreter that is being deleted. */
 {
-    SmoothAssocData *ptr, *method = (SmoothAssocData *) clientData;
+    SmoothAssocData *ptr, *methods = (SmoothAssocData *) clientData;
 
-    while (method != NULL) {
-	method = (ptr = method)->nextPtr;
-	ckfree(ptr);
+    while (methods != NULL) {
+	methods = (ptr = methods)->nextPtr;
+	ckfree((char *) ptr);
     }
 }
 /*
@@ -675,34 +677,43 @@ TkSmoothParseProc(clientData, interp, tkwin, value, widgRec, offset)
     char *widgRec;			/* Pointer to record for item. */
     int offset;				/* Offset into item. */
 {
-    register Tk_SmoothMethod **smoothPtr = (Tk_SmoothMethod **) (widgRec + offset);
+    register Tk_SmoothMethod **smoothPtr =
+	(Tk_SmoothMethod **) (widgRec + offset);
     Tk_SmoothMethod *smooth = NULL;
-    int b, length;
-    SmoothAssocData *method;
+    int b;
+    size_t length;
+    SmoothAssocData *methods;
 
-    if(value == NULL || *value == 0) {
+    if (value == NULL || *value == 0) {
 	*smoothPtr = (Tk_SmoothMethod *) NULL;
 	return TCL_OK;
     }
     length = strlen(value);
-    method = (SmoothAssocData *) Tcl_GetAssocData(interp, "smoothMethod",
+    methods = (SmoothAssocData *) Tcl_GetAssocData(interp, "smoothMethod",
 	    (Tcl_InterpDeleteProc **) NULL);
-    while (method != (SmoothAssocData *) NULL) {
-	if (strncmp(value, method->smooth.name, length) == 0) {
+    while (methods != (SmoothAssocData *) NULL) {
+	if (strncmp(value, methods->smooth.name, length) == 0) {
 	    if (smooth != (Tk_SmoothMethod *) NULL) {
 		Tcl_AppendResult(interp, "ambigeous smooth method \"", value,
 			"\"", (char *) NULL);
 		return TCL_ERROR;
 	    }
-	    smooth = &method->smooth;
+	    smooth = &methods->smooth;
 	}
-	method = method->nextPtr;
+	methods = methods->nextPtr;
     }
     if (smooth) {
 	*smoothPtr = smooth;
 	return TCL_OK;
+    } else if (strncmp(value, tkBezierSmoothMethod.name, length) == 0) {
+	/*
+	 * We need to do handle the built-in bezier method.
+	 */
+	*smoothPtr = &tkBezierSmoothMethod;
+	return TCL_OK;
     }
 
+
     if (Tcl_GetBoolean(interp, (char *) value, &b) != TCL_OK) {
 	return TCL_ERROR;
     }
@@ -772,15 +783,16 @@ Tk_GetDash(interp, value, dash)
 				 * store dash information. */
 {
     int argc, i;
-    char **largv, **argv = NULL;
+    CONST char **largv, **argv = NULL;
     char *pt;
 
     if ((value==(char *) NULL) || (*value==0) ) {
 	dash->number = 0;
 	return TCL_OK;
     }
-    if ((*value == '.') || (*value == ',') || (*value == '-') || (*value == '_')) {
-	i = DashConvert((char *) NULL, value, -1, 0);
+    if ((*value == '.') || (*value == ',') ||
+	    (*value == '-') || (*value == '_')) {
+	i = DashConvert((char *) NULL, value, -1, 0.0);
 	if (i>0) {
 	    i = strlen(value);
 	} else {
@@ -791,7 +803,7 @@ Tk_GetDash(interp, value, dash)
 	} else {
 	    pt = dash->pattern.array;
 	}
-	memcpy(pt,value,i);
+	memcpy(pt,value, (unsigned int) i);
 	dash->number = -i;
 	return TCL_OK;
     }
@@ -799,7 +811,8 @@ Tk_GetDash(interp, value, dash)
 	Tcl_ResetResult(interp);
     badDashList:
 	Tcl_AppendResult(interp, "bad dash list \"", value,
-		     "\": must be a list of integers or a format like \"-..\"", (char *) NULL);
+		"\": must be a list of integers or a format like \"-..\"",
+		(char *) NULL);
     syntaxError:
 	if (argv != NULL) {
 	    ckfree((char *) argv);
@@ -813,8 +826,8 @@ Tk_GetDash(interp, value, dash)
     if (ABS(dash->number) > sizeof(char *)) {
 	ckfree((char *) dash->pattern.pt);
     }
-    if ( argc > sizeof(char *)) {
-	dash->pattern.pt = pt = (char *) ckalloc(argc);
+    if (argc > sizeof(char *)) {
+	dash->pattern.pt = pt = (char *) ckalloc((unsigned int) argc);
     } else {
 	pt = dash->pattern.array;
     }
@@ -858,7 +871,7 @@ Tk_GetDash(interp, value, dash)
  */
 
 void Tk_CreateOutline(outline)
-Tk_Outline *outline;
+    Tk_Outline *outline;
 {
     outline->gc = None;
     outline->width = 1.0;
@@ -868,12 +881,9 @@ Tk_Outline *outline;
     outline->dash.number = 0;
     outline->activeDash.number = 0;
     outline->disabledDash.number = 0;
-    outline->tile = NULL;
     outline->tsoffset.flags = 0;
     outline->tsoffset.xoffset = 0;
     outline->tsoffset.yoffset = 0;
-    outline->activeTile = NULL;
-    outline->disabledTile = NULL;
     outline->color = NULL;
     outline->activeColor = NULL;
     outline->disabledColor = NULL;
@@ -900,8 +910,8 @@ Tk_Outline *outline;
  */
 
 void Tk_DeleteOutline(display, outline)
-Display *display;			/* Display containing window */
-Tk_Outline *outline;
+    Display *display;			/* Display containing window */
+    Tk_Outline *outline;
 {
     if (outline->gc != None) {
 	Tk_FreeGC(display, outline->gc);
@@ -915,15 +925,6 @@ Tk_Outline *outline;
     if (ABS(outline->disabledDash.number) > sizeof(char *)) {
 	ckfree((char *) outline->disabledDash.pattern.pt);
     }
-    if (outline->tile != NULL) {
-	Tk_FreeTile(outline->tile);
-    }
-    if (outline->activeTile != NULL) {
-	Tk_FreeTile(outline->activeTile);
-    }
-    if (outline->disabledTile != NULL) {
-	Tk_FreeTile(outline->disabledTile);
-    }
     if (outline->color != NULL) {
 	Tk_FreeColor(outline->color);
     }
@@ -966,19 +967,17 @@ Tk_Outline *outline;
  */
 
 int Tk_ConfigOutlineGC(gcValues, canvas, item, outline)
-XGCValues *gcValues;
-Tk_Canvas canvas;
-Tk_Item *item;
-Tk_Outline *outline;
+    XGCValues *gcValues;
+    Tk_Canvas canvas;
+    Tk_Item *item;
+    Tk_Outline *outline;
 {
     int mask = 0;
     double width;
     Tk_Dash *dash;
-    Tk_Tile tile;
     XColor *color;
     Pixmap stipple;
     Tk_State state = item->state;
-    Pixmap pixmap;
 
     if (outline->width < 0.0) {
 	outline->width = 0.0;
@@ -998,10 +997,9 @@ Tk_Outline *outline;
 	width = 1.0;
     }
     dash = &(outline->dash);
-    tile = outline->tile;
     color = outline->color;
     stipple = outline->stipple;
-    if(state == TK_STATE_NULL) {
+    if (state == TK_STATE_NULL) {
 	state = ((TkCanvas *)canvas)->canvas_state;
     }
     if (((TkCanvas *)canvas)->currentItemPtr == item) {
@@ -1011,9 +1009,6 @@ Tk_Outline *outline;
 	if (outline->activeDash.number != 0) {
 	    dash = &(outline->activeDash);
 	}
-	if (outline->activeTile!=NULL) {
-	    tile = outline->activeTile;
-	}
 	if (outline->activeColor!=NULL) {
 	    color = outline->activeColor;
 	}
@@ -1027,9 +1022,6 @@ Tk_Outline *outline;
 	if (outline->disabledDash.number != 0) {
 	    dash = &(outline->disabledDash);
 	}
-	if (outline->disabledTile!=NULL) {
-	    tile = outline->disabledTile;
-	}
 	if (outline->disabledColor!=NULL) {
 	    color = outline->disabledColor;
 	}
@@ -1038,21 +1030,12 @@ Tk_Outline *outline;
 	}
     }
 
-    Tk_SetTileCanvasItem(outline->tile, canvas, (Tk_Item *) NULL);
-    Tk_SetTileCanvasItem(outline->activeTile, canvas, (Tk_Item *) NULL);
-    Tk_SetTileCanvasItem(outline->disabledTile, canvas, (Tk_Item *) NULL);
-    Tk_SetTileCanvasItem(tile, canvas, item);
-
-    if ((tile==NULL) && (color==NULL)) {
+    if (color==NULL) {
 	return 0;
     }
 
     gcValues->line_width = (int) (width + 0.5);
-    if ((pixmap = Tk_PixmapOfTile(tile)) != None) {
-	gcValues->fill_style = FillTiled;
-	gcValues->tile = pixmap;
-	mask = GCTile|GCFillStyle|GCLineWidth;
-    } else if (color != NULL) {
+    if (color != NULL) {
 	gcValues->foreground = color->pixel;
 	mask = GCForeground|GCLineWidth;
 	if (stipple != None) {
@@ -1069,7 +1052,7 @@ Tk_Outline *outline;
 	} else if (dash->number > 0) {
 	    gcValues->dashes = dash->pattern.array[0];
 	} else {
-	    gcValues->dashes = 4 * width;
+	    gcValues->dashes = (char) (4 * width);
 	}
 	mask |= GCLineStyle|GCDashList|GCDashOffset;
     }
@@ -1099,14 +1082,13 @@ Tk_Outline *outline;
 
 int
 Tk_ChangeOutlineGC(canvas, item, outline)
-Tk_Canvas canvas;
-Tk_Item *item;
-Tk_Outline *outline;
+    Tk_Canvas canvas;
+    Tk_Item *item;
+    Tk_Outline *outline;
 {
     CONST char *p;
     double width;
     Tk_Dash *dash;
-    Tk_Tile tile;
     XColor *color;
     Pixmap stipple;
     Tk_State state = item->state;
@@ -1116,42 +1098,35 @@ Tk_Outline *outline;
 	width = 1.0;
     }
     dash = &(outline->dash);
-    tile = outline->tile;
     color = outline->color;
     stipple = outline->stipple;
-    if(state == TK_STATE_NULL) {
+    if (state == TK_STATE_NULL) {
 	state = ((TkCanvas *)canvas)->canvas_state;
     }
     if (((TkCanvas *)canvas)->currentItemPtr == item) {
-	if (outline->activeWidth>width) {
+	if (outline->activeWidth > width) {
 	    width = outline->activeWidth;
 	}
 	if (outline->activeDash.number != 0) {
 	    dash = &(outline->activeDash);
 	}
-	if (outline->activeTile!=NULL) {
-	    tile = outline->activeTile;
-	}
-	if (outline->activeColor!=NULL) {
+	if (outline->activeColor != NULL) {
 	    color = outline->activeColor;
 	}
-	if (outline->activeStipple!=None) {
+	if (outline->activeStipple != None) {
 	    stipple = outline->activeStipple;
 	}
-    } else if (state==TK_STATE_DISABLED) {
-	if (outline->disabledWidth>width) {
+    } else if (state == TK_STATE_DISABLED) {
+	if (outline->disabledWidth > width) {
 	    width = outline->disabledWidth;
 	}
 	if (outline->disabledDash.number != 0) {
 	    dash = &(outline->disabledDash);
 	}
-	if (outline->disabledTile!=NULL) {
-	    tile = outline->disabledTile;
-	}
-	if (outline->disabledColor!=NULL) {
+	if (outline->disabledColor != NULL) {
 	    color = outline->disabledColor;
 	}
-	if (outline->disabledStipple!=None) {
+	if (outline->disabledStipple != None) {
 	    stipple = outline->disabledStipple;
 	}
     }
@@ -1164,7 +1139,7 @@ Tk_Outline *outline;
 	int i = -dash->number;
 
         p = (i > sizeof(char *)) ? dash->pattern.pt : dash->pattern.array;
-	q = (char *) ckalloc(2*i);
+	q = (char *) ckalloc(2*(unsigned int)i);
 	i = DashConvert(q, p, i, width);
 	XSetDashes(((TkCanvas *)canvas)->display, outline->gc, outline->offset, q, i);
 	ckfree(q);
@@ -1173,16 +1148,12 @@ Tk_Outline *outline;
         p = (char *) (dash->number > sizeof(char *)) ? dash->pattern.pt : dash->pattern.array;
 	XSetDashes(((TkCanvas *)canvas)->display, outline->gc, outline->offset, p, dash->number);
     }
-    if ((tile!= NULL) || (stipple!=None)) {
+    if (stipple!=None) {
 	int w=0; int h=0;
 	Tk_TSOffset *tsoffset = &outline->tsoffset;
 	int flags = tsoffset->flags;
 	if (!(flags & TK_OFFSET_INDEX) && (flags & (TK_OFFSET_CENTER|TK_OFFSET_MIDDLE))) {
-	    if (tile != NULL) {
-		Tk_SizeOfTile(tile, &w, &h);
-	    } else {
-		Tk_SizeOfBitmap(((TkCanvas *)canvas)->display, stipple, &w, &h);
-	    }
+	    Tk_SizeOfBitmap(((TkCanvas *)canvas)->display, stipple, &w, &h);
 	    if (flags & TK_OFFSET_CENTER) {
 		w /= 2;
 	    } else {
@@ -1234,7 +1205,6 @@ Tk_ResetOutlineGC(canvas, item, outline)
     char dashList;
     double width;
     Tk_Dash *dash;
-    Tk_Tile tile;
     XColor *color;
     Pixmap stipple;
     Tk_State state = item->state;
@@ -1244,10 +1214,9 @@ Tk_ResetOutlineGC(canvas, item, outline)
 	width = 1.0;
     }
     dash = &(outline->dash);
-    tile = outline->tile;
     color = outline->color;
     stipple = outline->stipple;
-    if(state == TK_STATE_NULL) {
+    if (state == TK_STATE_NULL) {
 	state = ((TkCanvas *)canvas)->canvas_state;
     }
     if (((TkCanvas *)canvas)->currentItemPtr == item) {
@@ -1257,9 +1226,6 @@ Tk_ResetOutlineGC(canvas, item, outline)
 	if (outline->activeDash.number != 0) {
 	    dash = &(outline->activeDash);
 	}
-	if (outline->activeTile!=NULL) {
-	    tile = outline->activeTile;
-	}
 	if (outline->activeColor!=NULL) {
 	    color = outline->activeColor;
 	}
@@ -1273,9 +1239,6 @@ Tk_ResetOutlineGC(canvas, item, outline)
 	if (outline->disabledDash.number != 0) {
 	    dash = &(outline->disabledDash);
 	}
-	if (outline->disabledTile!=NULL) {
-	    tile = outline->disabledTile;
-	}
 	if (outline->disabledColor!=NULL) {
 	    color = outline->disabledColor;
 	}
@@ -1297,9 +1260,10 @@ Tk_ResetOutlineGC(canvas, item, outline)
 	} else {
 	    dashList = 4;
 	}
-	XSetDashes(((TkCanvas *)canvas)->display, outline->gc, outline->offset, &dashList , 1);
+	XSetDashes(((TkCanvas *)canvas)->display, outline->gc,
+		outline->offset, &dashList , 1);
     }
-    if ((stipple!=None) || (tile!=NULL)) {
+    if (stipple != None) {
 	XSetTSOrigin(((TkCanvas *)canvas)->display, outline->gc, 0, 0);
 	return 1;
     }
@@ -1333,109 +1297,104 @@ Tk_CanvasPsOutline(canvas, item, outline)
     char string[41];
     char pattern[11];
     int i;
-    char *p;
-    char *s = string;
-    char *l = pattern;
+    char *ptr;
+    char *str = string;
+    char *lptr = pattern;
     Tcl_Interp *interp = ((TkCanvas *)canvas)->interp;
     double width;
     Tk_Dash *dash;
-    Tk_Tile tile;
     XColor *color;
     Pixmap stipple;
     Tk_State state = item->state;
 
     width = outline->width;
     dash = &(outline->dash);
-    tile = outline->tile;
     color = outline->color;
     stipple = outline->stipple;
-    if(state == TK_STATE_NULL) {
+    if (state == TK_STATE_NULL) {
 	state = ((TkCanvas *)canvas)->canvas_state;
     }
     if (((TkCanvas *)canvas)->currentItemPtr == item) {
-	if (outline->activeWidth>width) {
+	if (outline->activeWidth > width) {
 	    width = outline->activeWidth;
 	}
-	if (outline->activeDash.number>0) {
+	if (outline->activeDash.number > 0) {
 	    dash = &(outline->activeDash);
 	}
-	if (outline->activeTile!=NULL) {
-	    tile = outline->activeTile;
-	}
-	if (outline->activeColor!=NULL) {
+	if (outline->activeColor != NULL) {
 	    color = outline->activeColor;
 	}
-	if (outline->activeStipple!=None) {
+	if (outline->activeStipple != None) {
 	    stipple = outline->activeStipple;
 	}
-    } else if (state==TK_STATE_DISABLED) {
-	if (outline->disabledWidth>0) {
+    } else if (state == TK_STATE_DISABLED) {
+	if (outline->disabledWidth > 0) {
 	    width = outline->disabledWidth;
 	}
-	if (outline->disabledDash.number>0) {
+	if (outline->disabledDash.number > 0) {
 	    dash = &(outline->disabledDash);
 	}
-	if (outline->disabledTile!=NULL) {
-	    tile = outline->disabledTile;
-	}
-	if (outline->disabledColor!=NULL) {
+	if (outline->disabledColor != NULL) {
 	    color = outline->disabledColor;
 	}
-	if (outline->disabledStipple!=None) {
+	if (outline->disabledStipple != None) {
 	    stipple = outline->disabledStipple;
 	}
     }
     sprintf(string, "%.15g setlinewidth\n", width);
     Tcl_AppendResult(interp, string, (char *) NULL);
 
-    if (dash->number>10) {
-	s = (char *)ckalloc(1 + 4*dash->number);
-    } else if (dash->number<-5) {
-	s = (char *)ckalloc(1 - 8*dash->number);
-	l = (char *)ckalloc(1 - 2*dash->number);
+    if (dash->number > 10) {
+	str = (char *)ckalloc((unsigned int) (1 + 4*dash->number));
+    } else if (dash->number < -5) {
+	str = (char *)ckalloc((unsigned int) (1 - 8*dash->number));
+	lptr = (char *)ckalloc((unsigned int) (1 - 2*dash->number));
     }
-    p = (char *) ((ABS(dash->number) > sizeof(char *)) ) ? dash->pattern.pt : dash->pattern.array;
+    ptr = (char *) ((ABS(dash->number) > sizeof(char *)) ) ?
+	dash->pattern.pt : dash->pattern.array;
     if (dash->number > 0) {
-	sprintf(s,"[%d",*p++ & 0xff);
-	i=dash->number-1;
-	while(i--) {
-	    sprintf(s+strlen(s)," %d",*p++ & 0xff);
+	char *ptr0 = ptr;
+	sprintf(str, "[%d", *ptr++ & 0xff);
+	i = dash->number-1;
+	while (i--) {
+	    sprintf(str+strlen(str), " %d", *ptr++ & 0xff);
 	}
-	Tcl_AppendResult(interp,s,(char *)NULL);
+	Tcl_AppendResult(interp, str, (char *)NULL);
 	if (dash->number&1) {
-	    Tcl_AppendResult(interp," ",s+1,(char *)NULL);
+	    Tcl_AppendResult(interp, " ", str+1, (char *)NULL);
 	}
-	sprintf(s,"] %d setdash\n",outline->offset);
-	Tcl_AppendResult(interp,s,(char *)NULL);
+	sprintf(str, "] %d setdash\n", outline->offset);
+	Tcl_AppendResult(interp, str, (char *)NULL);
+	ptr = ptr0;
     } else if (dash->number < 0) {
-	if ((i = DashConvert(l ,p, -dash->number, width)) != 0) {
-	    sprintf(s,"[%d",*l++ & 0xff);
-	    while(--i) {
-		sprintf(s+strlen(s)," %d",*l++ & 0xff);
+	if ((i = DashConvert(lptr, ptr, -dash->number, width)) != 0) {
+	    char *lptr0 = lptr;
+	    sprintf(str, "[%d", *lptr++ & 0xff);
+	    while (--i) {
+		sprintf(str+strlen(str), " %d", *lptr++ & 0xff);
 	    }
-	    Tcl_AppendResult(interp,s,(char *)NULL);
-	    sprintf(s,"] %d setdash\n",outline->offset);
-	    Tcl_AppendResult(interp,s,(char *)NULL);
+	    Tcl_AppendResult(interp, str, (char *)NULL);
+	    sprintf(str, "] %d setdash\n", outline->offset);
+	    Tcl_AppendResult(interp, str, (char *)NULL);
+	    lptr = lptr0;
 	} else {
-	    Tcl_AppendResult(interp,"[] 0 setdash\n",(char *)NULL);
+	    Tcl_AppendResult(interp, "[] 0 setdash\n", (char *)NULL);
 	}
     } else {
-	Tcl_AppendResult(interp,"[] 0 setdash\n",(char *)NULL);
+	Tcl_AppendResult(interp, "[] 0 setdash\n", (char *)NULL);
     }
-    if (s!=string) {
-	ckfree(s);
+    if (str != string) {
+	ckfree(str);
     }
-    if (l!=pattern) {
-	ckfree(l);
+    if (lptr != pattern) {
+	ckfree(lptr);
     }
-    if (Tk_CanvasPsColor(interp, canvas, color)
-	    != TCL_OK) {
+    if (Tk_CanvasPsColor(interp, canvas, color) != TCL_OK) {
 	return TCL_ERROR;
     }
     if (stipple != None) {
 	Tcl_AppendResult(interp, "StrokeClip ", (char *) NULL);
-	if (Tk_CanvasPsStipple(interp, canvas,
-		stipple) != TCL_OK) {
+	if (Tk_CanvasPsStipple(interp, canvas, stipple) != TCL_OK) {
 	    return TCL_ERROR;
 	}
     } else {
@@ -1519,3 +1478,278 @@ DashConvert (l, p, n, width)
     }
     return result;
 }
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * translateAndAppendCoords --
+ *
+ *	This is a helper routine for TkCanvTranslatePath() below.
+ *
+ *	Given an (x,y) coordinate pair within a canvas, this procedure
+ *	computes the corresponding coordinates at which the point should
+ *	be drawn in the drawable used for display.  Those coordinates are
+ *	then written into outArr[numOut*2] and outArr[numOut*2+1].
+ *
+ * Results:
+ *	There is no return value.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void
+translateAndAppendCoords(canvPtr, x, y, outArr, numOut)
+    TkCanvas *canvPtr;			/* The canvas. */
+    double x, y;			/* Coordinates in canvas space. */
+    XPoint *outArr;                     /* Write results into this array */
+    int numOut;                         /* Num of prior entries in outArr[] */
+{
+    double tmp;
+
+    tmp = x - canvPtr->drawableXOrigin;
+    if (tmp > 0) {
+	tmp += 0.5;
+    } else {
+	tmp -= 0.5;
+    }
+    outArr[numOut].x = (short) tmp;
+
+    tmp = y  - canvPtr->drawableYOrigin;
+    if (tmp > 0) {
+	tmp += 0.5;
+    } else {
+	tmp -= 0.5;
+    }
+    outArr[numOut].y = (short) tmp;
+}
+
+/*
+ *--------------------------------------------------------------
+ *
+ * TkCanvTranslatePath
+ *
+ *	Translate a line or polygon path so that all vertices are
+ *	within a rectangle that is 1000 pixels larger than the total
+ *	size of the canvas window.  This will prevent pixel coordinates
+ *	from overflowing the 16-bit integer size limitation imposed by
+ *	most windowing systems.
+ *
+ *      coordPtr must point to an array of doubles, two doubles per
+ *      vertex.  There are a total of numVertex vertices, or 2*numVertex
+ *      entries in coordPtr.  The result vertices written into outArr
+ *      have their coordinate origin shifted to canvPtr->drawableXOrigin
+ *      by canvPtr->drawableYOrigin.  There might be as many as 3 times
+ *      more output vertices than there are input vertices.  The calling
+ *      function should allocate space accordingly.
+ *
+ *	This routine limits the width and height of a canvas window
+ *	to 31767 pixels.  At the highest resolution display devices
+ *	available today (210 ppi in Jan 2003) that's a window that is
+ *      over 13 feet wide and tall.  Should be enough for the near
+ *	future.
+ *
+ * Results:
+ *      Clipped and translated path vertices are written into outArr[].
+ *      There might be as many as twice the vertices in outArr[] as there
+ *      are in coordPtr[].  The return value is the number of vertices
+ *      actually written into outArr[].
+ *
+ * Side effects:
+ *	None
+ *
+ *--------------------------------------------------------------
+ */
+int
+TkCanvTranslatePath (canvPtr, numVertex, coordArr, closedPath, outArr)
+    TkCanvas *canvPtr;  /* The canvas */
+    int numVertex;      /* Number of vertices specified by coordArr[] */
+    double *coordArr;   /* X and Y coordinates for each vertex */
+    int closedPath;     /* True if this is a closed polygon */
+    XPoint *outArr;     /* Write results here, if not NULL */
+{
+    int numOutput = 0;  /* Number of output coordinates */
+    double lft, rgh;    /* Left and right sides of the bounding box */
+    double top, btm;    /* Top and bottom sizes of the bounding box */
+    double *tempArr;    /* Temporary storage used by the clipper */
+    double *a, *b, *t;  /* Pointers to parts of the temporary storage */
+    int i, j;           /* Loop counters */
+    int maxOutput;      /* Maximum number of outputs that we will allow */
+    double limit[4];          /* Boundries at which clipping occurs */
+    double staticSpace[480];  /* Temp space from the stack */
+
+    /*
+    ** Constrain all vertices of the path to be within a box that is no
+    ** larger than 32000 pixels wide or height.  The top-left corner of
+    ** this clipping box is 1000 pixels above and to the left of the top
+    ** left corner of the window on which the canvas is displayed.
+    **
+    ** This means that a canvas will not display properly on a canvas
+    ** window that is larger than 31000 pixels wide or high.  That is not
+    ** a problem today, but might someday become a factor for ultra-high
+    ** resolutions displays.
+    **
+    ** The X11 protocol allows us (in theory) to expand the size of the
+    ** clipping box to 32767 pixels.  But we have found experimentally that
+    ** XFree86 sometimes fails to draw lines correctly if they are longer
+    ** than about 32500 pixels.  So we have left a little margin in the
+    ** size to mask that bug.
+    */
+    lft = canvPtr->xOrigin - 1000.0;
+    top = canvPtr->yOrigin - 1000.0;
+    rgh = lft + 32000.0;
+    btm = top + 32000.0;
+
+    /* Try the common case first - no clipping.  Loop over the input
+    ** coordinates and translate them into appropriate output coordinates.
+    ** But if a vertex outside of the bounding box is seen, break out of
+    ** the loop.
+    **
+    ** Most of the time, no clipping is needed, so this one loop is 
+    ** sufficient to do the translation.
+    */
+    for(i=0; i<numVertex; i++){
+        double x, y;
+        x = coordArr[i*2];
+        y = coordArr[i*2+1];
+        if( x<lft || x>rgh || y<top || y>btm ) break;
+        translateAndAppendCoords(canvPtr, x, y, outArr, numOutput++);
+    }
+    if( i==numVertex ){
+        assert( numOutput==numVertex );
+        return numOutput;
+    }
+
+    /* If we reach this point, it means that some clipping is required.
+    ** Begin by allocating some working storage - at least 6 times as much space
+    ** as coordArr[] requires.  Divide this space into two separate arrays
+    ** a[] and b[].  Initialize a[] to be equal to coordArr[].
+    */
+    if( numVertex*12 <= sizeof(staticSpace)/sizeof(staticSpace[0]) ){
+        tempArr = staticSpace;
+    } else {
+        tempArr = (double*)ckalloc( numVertex*12*sizeof(tempArr[0]) );
+    }
+    for(i=0; i<numVertex*2; i++){
+        tempArr[i] = coordArr[i];
+    }
+    a = tempArr;
+    b = &tempArr[numVertex*6];
+
+    /* We will make four passes through the input data.  On each pass,
+    ** we copy the contents of a[] over into b[].  As we copy, we clip
+    ** any line segments that extend to the right past xClip then we
+    ** rotate the coordinate system 90 degrees clockwise.  After each
+    ** pass is complete, we interchange a[] and b[] in preparation for
+    ** the next pass.
+    **
+    ** Each pass clips line segments that extend beyond a single side
+    ** of the bounding box, and four passes rotate the coordinate system
+    ** back to its original value.  I'm not an expert on graphics 
+    ** algorithms, but I think this is called Cohen-Sutherland polygon
+    ** clipping.
+    **
+    ** The limit[] array contains the xClip value used for each of the
+    ** four passes.
+    */
+    limit[0] = rgh;
+    limit[1] = -top;
+    limit[2] = -lft;
+    limit[3] = btm;
+
+    /* This is the loop that makes the four passes through the data.
+    */
+    maxOutput = numVertex*3;
+    for(j=0; j<4; j++){
+        double xClip = limit[j];
+        int inside = a[0]<xClip;
+        double priorY = a[1];
+        numOutput = 0;
+
+        /* Clip everything to the right of xClip.  Store the results in
+        ** b[] rotated by 90 degrees clockwise.
+        */
+        for(i=0; i<numVertex; i++){
+            double x = a[i*2];
+            double y = a[i*2+1];
+            if( x>=xClip ){
+                /* The current vertex is to the right of xClip.
+                */
+                if( inside ){
+                    /* If the current vertex is to the right of xClip but
+                    ** the previous vertex was left of xClip, then draw a
+                    ** line segment from the previous vertex to until it
+                    ** intersects the vertical at xClip.
+                    */
+                    double x0, y0, yN;
+                    assert( i>0 );
+                    x0 = a[i*2-2];
+                    y0 = a[i*2-1];
+                    yN = y0 + (y - y0)*(xClip-x0)/(x-x0);
+                    b[numOutput*2] = -yN;
+                    b[numOutput*2+1] = xClip;
+                    numOutput++;
+                    assert( numOutput<=maxOutput );
+                    priorY = yN;
+                    inside = 0;
+                }else if( i==0 ){
+                    /* If the first vertex is to the right of xClip, add
+                    ** a vertex that is the projection of the first vertex
+                    ** onto the vertical xClip line.
+                    */
+                    b[0] = -y;
+                    b[1] = xClip;
+                    numOutput = 1;
+                    priorY = y;
+                }
+            }else{
+                /* The current vertex is to the left of xClip
+                */
+                if( !inside ){
+                    /* If the current vertex is on the left of xClip and
+                    ** one or more prior vertices where to the right, then
+                    ** we have to draw a line segment along xClip that extends
+                    ** from the spot where we first crossed from left to right
+                    ** to the spot where we cross back from right to left.
+                    */
+                    double x0, y0, yN;
+                    assert( i>0 );
+                    x0 = a[i*2-2];
+                    y0 = a[i*2-1];
+                    yN = y0 + (y - y0)*(xClip-x0)/(x-x0);
+                    if( yN!=priorY ){
+                        b[numOutput*2] = -yN;
+                        b[numOutput*2+1] = xClip;
+                        numOutput++;
+                        assert( numOutput<=maxOutput );
+                    }
+                    inside = 1;
+                }
+                b[numOutput*2] = -y;
+                b[numOutput*2+1] = x;
+                numOutput++;
+                assert( numOutput<=maxOutput );
+            }
+        }
+
+        /* Interchange a[] and b[] in preparation for the next pass.
+        */
+        t = a;
+        a = b;
+        b = t;
+        numVertex = numOutput;
+    }
+
+    /* All clipping is now finished.  Convert the coordinates from doubles
+    ** into XPoints and translate the origin for the drawable.
+    */
+    for(i=0; i<numVertex; i++){
+        translateAndAppendCoords(canvPtr, a[i*2], a[i*2+1], outArr, i);
+    }
+    if( tempArr!=staticSpace ){
+        ckfree((char *) tempArr);
+    }
+    return numOutput;
+}
