@@ -8,8 +8,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkCanvWind.c,v 1.4.2.1 2001/08/24 23:59:11 hobbs Exp $
  */
 
 #include <stdio.h>
@@ -184,23 +182,10 @@ CreateWinItem(interp, canvas, itemPtr, objc, objv)
     Tcl_Obj *CONST objv[];		/* Arguments describing window. */
 {
     WindowItem *winItemPtr = (WindowItem *) itemPtr;
-    int i = 2;
+    int i;
 
-    if (objc == 1) {
-	i = 1;
-    } else if (objc > 1) {
-	char *arg = Tcl_GetString(objv[1]);
-	if ((arg[0] == '-') && (arg[1] >= 'a') && (arg[1] <= 'z')) {
-	    i = 1;
-	}
-    }
-
-    if (objc < i) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"",
-		Tk_PathName(Tk_CanvasTkwin(canvas)), " create ",
-		itemPtr->typePtr->name, " x y ?options?\"",
-		(char *) NULL);
-	return TCL_ERROR;
+    if (objc == 0) {
+	panic("canvas did not pass any coords\n");
     }
 
     /*
@@ -215,12 +200,23 @@ CreateWinItem(interp, canvas, itemPtr, objc, objv)
 
     /*
      * Process the arguments to fill in the item record.
+     * Only 1 (list) or 2 (x y) coords are allowed.
      */
 
-    if ((WinItemCoords(interp, canvas, itemPtr, i, objv) != TCL_OK)) {
+    if (objc == 1) {
+	i = 1;
+    } else {
+	char *arg = Tcl_GetString(objv[1]);
+	i = 2;
+	if ((arg[0] == '-') && (arg[1] >= 'a') && (arg[1] <= 'z')) {
+	    i = 1;
+	}
+    }
+    if (WinItemCoords(interp, canvas, itemPtr, i, objv) != TCL_OK) {
 	goto error;
     }
-    if (ConfigureWinItem(interp, canvas, itemPtr, objc-i, objv+i, 0) == TCL_OK) {
+    if (ConfigureWinItem(interp, canvas, itemPtr, objc-i, objv+i, 0)
+	    == TCL_OK) {
 	return TCL_OK;
     }
 
@@ -329,8 +325,8 @@ ConfigureWinItem(interp, canvas, itemPtr, objc, objv, flags)
 
     oldWindow = winItemPtr->tkwin;
     canvasTkwin = Tk_CanvasTkwin(canvas);
-    if (Tk_ConfigureWidget(interp, canvasTkwin, configSpecs, objc, (char **) objv,
-	    (char *) winItemPtr, flags|TK_CONFIG_OBJS) != TCL_OK) {
+    if (TCL_OK != Tk_ConfigureWidget(interp, canvasTkwin, configSpecs, objc,
+	    (CONST char **) objv, (char *) winItemPtr, flags|TK_CONFIG_OBJS)) {
 	return TCL_ERROR;
     }
 
@@ -353,7 +349,7 @@ ConfigureWinItem(interp, canvas, itemPtr, objc, objv, flags)
 	    /*
 	     * Make sure that the canvas is either the parent of the
 	     * window associated with the item or a descendant of that
-	     * parent.  Also, don't allow a top-level window to be
+	     * parent.  Also, don't allow a top-of-hierarchy window to be
 	     * managed inside a canvas.
 	     */
 
@@ -363,7 +359,7 @@ ConfigureWinItem(interp, canvas, itemPtr, objc, objv, flags)
 		if (ancestor == parent) {
 		    break;
 		}
-		if (((Tk_FakeWin *) (ancestor))->flags & TK_TOP_LEVEL) {
+		if (((Tk_FakeWin *) (ancestor))->flags & TK_TOP_HIERARCHY) {
 		    badWindow:
 		    Tcl_AppendResult(interp, "can't use ",
 			    Tk_PathName(winItemPtr->tkwin),
@@ -372,7 +368,7 @@ ConfigureWinItem(interp, canvas, itemPtr, objc, objv, flags)
 		    return TCL_ERROR;
 		}
 	    }
-	    if (((Tk_FakeWin *) (winItemPtr->tkwin))->flags & TK_TOP_LEVEL) {
+	    if (((Tk_FakeWin *) (winItemPtr->tkwin))->flags & TK_TOP_HIERARCHY) {
 		goto badWindow;
 	    }
 	    if (winItemPtr->tkwin == canvasTkwin) {
@@ -384,6 +380,14 @@ ConfigureWinItem(interp, canvas, itemPtr, objc, objv, flags)
 		    (ClientData) winItemPtr);
 	}
     }
+    if ((winItemPtr->tkwin != NULL)
+	    && (itemPtr->state == TK_STATE_HIDDEN)) {
+	if (canvasTkwin == Tk_Parent(winItemPtr->tkwin)) {
+	    Tk_UnmapWindow(winItemPtr->tkwin);
+	} else {
+	    Tk_UnmaintainGeometry(winItemPtr->tkwin, canvasTkwin);
+	}
+    }
 
     ComputeWindowBbox(canvas, winItemPtr);
 
@@ -590,11 +594,20 @@ DisplayWinItem(canvas, itemPtr, display, drawable, regionX, regionY,
     if (winItemPtr->tkwin == NULL) {
 	return;
     }
-    if(state == TK_STATE_NULL) {
+    if (state == TK_STATE_NULL) {
 	state = ((TkCanvas *)canvas)->canvas_state;
     }
-    if (state == TK_STATE_HIDDEN) {
-	Tk_UnmapWindow(winItemPtr->tkwin);
+
+    /*
+     * A drawable of None is used by the canvas UnmapNotify handler
+     * to indicate that we should no longer display ourselves.
+     */
+    if (state == TK_STATE_HIDDEN || drawable == None) {
+	if (canvasTkwin == Tk_Parent(winItemPtr->tkwin)) {
+	    Tk_UnmapWindow(winItemPtr->tkwin);
+	} else {
+	    Tk_UnmaintainGeometry(winItemPtr->tkwin, canvasTkwin);
+	}
 	return;
     }
     Tk_CanvasWindowCoords(canvas, (double) winItemPtr->header.x1,
@@ -845,7 +858,6 @@ CanvasPsWindow(interp, tkwin, canvas, x, y, width, height)
     int width, height;			/* width/height of window. */
 {
     char buffer[256];
-    TkWindow *winPtr;
     XImage *ximage;
     int result;
     Tcl_DString buffer1, buffer2;
@@ -885,12 +897,6 @@ CanvasPsWindow(interp, tkwin, canvas, x, y, width, height)
 		(char *) NULL);
 	Tcl_DStringFree(&buffer1);
 
-	for (winPtr = ((TkWindow *) tkwin)->childList; winPtr != NULL;
-		winPtr = winPtr->nextPtr) {
-	    if (Tk_IsMapped(winPtr)) {
-/*		printf("child window: %s\n", winPtr->pathName);*/
-	    }
-	}
 	return result;
     }
     Tcl_DStringFree(&buffer1);
@@ -1059,8 +1065,14 @@ WinItemRequestProc(clientData, tkwin)
     WindowItem *winItemPtr = (WindowItem *) clientData;
 
     ComputeWindowBbox(winItemPtr->canvas, winItemPtr);
-    DisplayWinItem(winItemPtr->canvas, (Tk_Item *) winItemPtr,
-	    (Display *) NULL, (Drawable) None, 0, 0, 0, 0);
+
+    /*
+     * A drawable argument of None to DisplayWinItem is used by the canvas
+     * UnmapNotify handler to indicate that we should no longer display
+     * ourselves, so need to pass a (bogus) non-zero drawable value here.
+     */
+    DisplayWinItem(winItemPtr->canvas, (Tk_Item *) winItemPtr, NULL,
+	    (Drawable) -1, 0, 0, 0, 0);
 }
 
 /*
