@@ -10,8 +10,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkCmds.c,v 1.31.2.3 2006/03/13 18:18:59 dgp Exp $
  */
 
 #include "tkPort.h"
@@ -19,8 +17,6 @@
 
 #if defined(WIN32)
 #include "tkWinInt.h"
-#elif defined(MAC_TCL)
-#include "tkMacInt.h"
 #elif defined(MAC_OSX_TK) 
 #include "tkMacOSXInt.h"
 #else
@@ -839,8 +835,6 @@ Tk_TkObjCmd(clientData, interp, objc, objv)
 	    }
 #if defined(WIN32)
 	    windowingsystem = "win32";
-#elif defined(MAC_TCL)
-	    windowingsystem = "classic";
 #elif defined(MAC_OSX_TK)
 	    windowingsystem = "aqua";
 #else
