@@ -1,23 +1,22 @@
-/* 
+/*
  * tkColor.c --
  *
- *	This file maintains a database of color values for the Tk
- *	toolkit, in order to avoid round-trips to the server to
- *	map color names to pixel values.
+ *	This file maintains a database of color values for the Tk toolkit, in
+ *	order to avoid round-trips to the server to map color names to pixel
+ *	values.
  *
  * Copyright (c) 1990-1994 The Regents of the University of California.
  * Copyright (c) 1994-1997 Sun Microsystems, Inc.
  *
- * See the file "license.terms" for information on usage and redistribution
- * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkColor.c,v 1.6.2.1 2000/08/05 23:53:11 hobbs Exp $
+ * See the file "license.terms" for information on usage and redistribution of
+ * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  */
 
+#include "tkInt.h"
 #include "tkColor.h"
 
 /*
- * Structures of the following following type are used as keys for 
+ * Structures of the following following type are used as keys for
  * colorValueTable (in TkDisplay).
  */
 
@@ -28,18 +27,17 @@ typedef struct {
     Display *display;		/* Display for colormap. */
 } ValueKey;
 
-
 /*
- * The structure below is used to allocate thread-local data. 
+ * The structure below is used to allocate thread-local data.
  */
 
 typedef struct ThreadSpecificData {
-    char rgbString[20];            /* */
+    char rgbString[20];		/* */
 } ThreadSpecificData;
 static Tcl_ThreadDataKey dataKey;
 
 /*
- * Forward declarations for procedures defined in this file:
+ * Forward declarations for functions defined in this file:
  */
 
 static void		ColorInit _ANSI_ARGS_((TkDisplay *dispPtr));
@@ -50,11 +48,11 @@ static void		InitColorObj _ANSI_ARGS_((Tcl_Obj *objPtr));
 
 /*
  * The following structure defines the implementation of the "color" Tcl
- * object, which maps a string color name to a TkColor object.  The
- * ptr1 field of the Tcl_Obj points to a TkColor object.
+ * object, which maps a string color name to a TkColor object. The ptr1 field
+ * of the Tcl_Obj points to a TkColor object.
  */
 
-static Tcl_ObjType colorObjType = {
+Tcl_ObjType tkColorObjType = {
     "color",			/* name */
     FreeColorObjProc,		/* freeIntRepProc */
     DupColorObjProc,		/* dupIntRepProc */
@@ -67,21 +65,20 @@ static Tcl_ObjType colorObjType = {
  *
  * Tk_AllocColorFromObj --
  *
- *	Given a Tcl_Obj *, map the value to a corresponding
- *	XColor structure based on the tkwin given.
+ *	Given a Tcl_Obj *, map the value to a corresponding XColor structure
+ *	based on the tkwin given.
  *
  * Results:
- *	The return value is a pointer to an XColor structure that
- *	indicates the red, blue, and green intensities for the color
- *	given by the string in objPtr, and also specifies a pixel value 
- *	to use to draw in that color.  If an error occurs, NULL is 
- *	returned and an error message will be left in interp's result
- *	(unless interp is NULL).
+ *	The return value is a pointer to an XColor structure that indicates
+ *	the red, blue, and green intensities for the color given by the string
+ *	in objPtr, and also specifies a pixel value to use to draw in that
+ *	color. If an error occurs, NULL is returned and an error message will
+ *	be left in interp's result (unless interp is NULL).
  *
  * Side effects:
- *	The color is added to an internal database with a reference count.
- *	For each call to this procedure, there should eventually be a call
- *	to Tk_FreeColorFromObj so that the database is cleaned up when colors
+ *	The color is added to an internal database with a reference count. For
+ *	each call to this function, there should eventually be a call to
+ *	Tk_FreeColorFromObj so that the database is cleaned up when colors
  *	aren't in use anymore.
  *
  *----------------------------------------------------------------------
@@ -98,21 +95,21 @@ Tk_AllocColorFromObj(interp, tkwin, objPtr)
 {
     TkColor *tkColPtr;
 
-    if (objPtr->typePtr != &colorObjType) {
+    if (objPtr->typePtr != &tkColorObjType) {
 	InitColorObj(objPtr);
     }
     tkColPtr = (TkColor *) objPtr->internalRep.twoPtrValue.ptr1;
 
     /*
-     * If the object currently points to a TkColor, see if it's the
-     * one we want.  If so, increment its reference count and return.
+     * If the object currently points to a TkColor, see if it's the one we
+     * want. If so, increment its reference count and return.
      */
 
     if (tkColPtr != NULL) {
 	if (tkColPtr->resourceRefCount == 0) {
 	    /*
-	     * This is a stale reference: it refers to a TkColor that's
-	     * no longer in use.  Clear the reference.
+	     * This is a stale reference: it refers to a TkColor that's no
+	     * longer in use. Clear the reference.
 	     */
 
 	    FreeColorObjProc(objPtr);
@@ -125,9 +122,9 @@ Tk_AllocColorFromObj(interp, tkwin, objPtr)
     }
 
     /*
-     * The object didn't point to the TkColor that we wanted.  Search
-     * the list of TkColors with the same name to see if one of the
-     * other TkColors is the right one.
+     * The object didn't point to the TkColor that we wanted. Search the list
+     * of TkColors with the same name to see if one of the other TkColors is
+     * the right one.
      */
 
     if (tkColPtr != NULL) {
@@ -147,7 +144,7 @@ Tk_AllocColorFromObj(interp, tkwin, objPtr)
     }
 
     /*
-     * Still no luck.  Call Tk_GetColor to allocate a new TkColor object.
+     * Still no luck. Call Tk_GetColor to allocate a new TkColor object.
      */
 
     tkColPtr = (TkColor *) Tk_GetColor(interp, tkwin, Tcl_GetString(objPtr));
@@ -167,31 +164,31 @@ Tk_AllocColorFromObj(interp, tkwin, objPtr)
  *	XColor structure.
  *
  * Results:
- *	The return value is a pointer to an XColor structure that
- *	indicates the red, blue, and green intensities for the color
- *	given by "name", and also specifies a pixel value to use to
- *	draw in that color.  If an error occurs, NULL is returned and
- *	an error message will be left in the interp's result.
+ *	The return value is a pointer to an XColor structure that indicates
+ *	the red, blue, and green intensities for the color given by "name",
+ *	and also specifies a pixel value to use to draw in that color. If an
+ *	error occurs, NULL is returned and an error message will be left in
+ *	the interp's result.
  *
  * Side effects:
- *	The color is added to an internal database with a reference count.
- *	For each call to this procedure, there should eventually be a call
- *	to Tk_FreeColor so that the database is cleaned up when colors
- *	aren't in use anymore.
+ *	The color is added to an internal database with a reference count. For
+ *	each call to this function, there should eventually be a call to
+ *	Tk_FreeColor so that the database is cleaned up when colors aren't in
+ *	use anymore.
  *
  *----------------------------------------------------------------------
  */
 
 XColor *
 Tk_GetColor(interp, tkwin, name)
-    Tcl_Interp *interp;		/* Place to leave error message if
-				 * color can't be found. */
+    Tcl_Interp *interp;		/* Place to leave error message if color can't
+				 * be found. */
     Tk_Window tkwin;		/* Window in which color will be used. */
-    char *name;			/* Name of color to be allocated (in form
+    Tk_Uid name;		/* Name of color to be allocated (in form
 				 * suitable for passing to XParseColor). */
 {
     Tcl_HashEntry *nameHashPtr;
-    int new;
+    int isNew;
     TkColor *tkColPtr;
     TkColor *existingColPtr;
     TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
@@ -201,14 +198,13 @@ Tk_GetColor(interp, tkwin, name)
     }
 
     /*
-     * First, check to see if there's already a mapping for this color
-     * name.
+     * First, check to see if there's already a mapping for this color name.
      */
 
-    nameHashPtr = Tcl_CreateHashEntry(&dispPtr->colorNameTable, name, &new);
-    if (!new) {
+    nameHashPtr = Tcl_CreateHashEntry(&dispPtr->colorNameTable, name, &isNew);
+    if (!isNew) {
 	existingColPtr = (TkColor *) Tcl_GetHashValue(nameHashPtr);
-	for (tkColPtr = existingColPtr;  tkColPtr != NULL;
+	for (tkColPtr = existingColPtr; tkColPtr != NULL;
 		tkColPtr = tkColPtr->nextPtr) {
 	    if ((tkColPtr->screen == Tk_Screen(tkwin))
 		    && (Tk_Colormap(tkwin) == tkColPtr->colormap)) {
@@ -221,8 +217,7 @@ Tk_GetColor(interp, tkwin, name)
     }
 
     /*
-     * The name isn't currently known.  Map from the name to a pixel
-     * value.
+     * The name isn't currently known. Map from the name to a pixel value.
      */
 
     tkColPtr = TkpGetColor(tkwin, name);
@@ -230,28 +225,28 @@ Tk_GetColor(interp, tkwin, name)
 	if (interp != NULL) {
 	    if (*name == '#') {
 		Tcl_AppendResult(interp, "invalid color name \"", name,
-			"\"", (char *) NULL);
+			"\"", NULL);
 	    } else {
 		Tcl_AppendResult(interp, "unknown color name \"", name,
-			"\"", (char *) NULL);
+			"\"", NULL);
 	    }
 	}
-	if (new) {
+	if (isNew) {
 	    Tcl_DeleteHashEntry(nameHashPtr);
 	}
 	return (XColor *) NULL;
     }
 
     /*
-     * Now create a new TkColor structure and add it to colorNameTable
-     * (in TkDisplay).
+     * Now create a new TkColor structure and add it to colorNameTable (in
+     * TkDisplay).
      */
 
     tkColPtr->magic = COLOR_MAGIC;
     tkColPtr->gc = None;
     tkColPtr->screen = Tk_Screen(tkwin);
     tkColPtr->colormap = Tk_Colormap(tkwin);
-    tkColPtr->visual  = Tk_Visual(tkwin);
+    tkColPtr->visual = Tk_Visual(tkwin);
     tkColPtr->resourceRefCount = 1;
     tkColPtr->objRefCount = 0;
     tkColPtr->type = TK_COLOR_BY_NAME;
@@ -267,21 +262,20 @@ Tk_GetColor(interp, tkwin, name)
  *
  * Tk_GetColorByValue --
  *
- *	Given a desired set of red-green-blue intensities for a color,
- *	locate a pixel value to use to draw that color in a given
- *	window.
+ *	Given a desired set of red-green-blue intensities for a color, locate
+ *	a pixel value to use to draw that color in a given window.
  *
  * Results:
- *	The return value is a pointer to an XColor structure that
- *	indicates the closest red, blue, and green intensities available
- *	to those specified in colorPtr, and also specifies a pixel
- *	value to use to draw in that color.
+ *	The return value is a pointer to an XColor structure that indicates
+ *	the closest red, blue, and green intensities available to those
+ *	specified in colorPtr, and also specifies a pixel value to use to draw
+ *	in that color.
  *
  * Side effects:
- *	The color is added to an internal database with a reference count.
- *	For each call to this procedure, there should eventually be a call
- *	to Tk_FreeColor, so that the database is cleaned up when colors
- *	aren't in use anymore.
+ *	The color is added to an internal database with a reference count. For
+ *	each call to this function, there should eventually be a call to
+ *	Tk_FreeColor, so that the database is cleaned up when colors aren't in
+ *	use anymore.
  *
  *----------------------------------------------------------------------
  */
@@ -294,7 +288,7 @@ Tk_GetColorByValue(tkwin, colorPtr)
 {
     ValueKey valueKey;
     Tcl_HashEntry *valueHashPtr;
-    int new;
+    int isNew;
     TkColor *tkColPtr;
     Display *display = Tk_Display(tkwin);
     TkDisplay *dispPtr = TkGetDisplay(display);
@@ -304,26 +298,28 @@ Tk_GetColorByValue(tkwin, colorPtr)
     }
 
     /*
-     * First, check to see if there's already a mapping for this color
-     * name.
+     * First, check to see if there's already a mapping for this color name.
+     * Must clear the structure first; it's not tightly packed on 64-bit
+     * systems. [Bug 2911570]
      */
 
+    memset(&valueKey, 0, sizeof(ValueKey));
     valueKey.red = colorPtr->red;
     valueKey.green = colorPtr->green;
     valueKey.blue = colorPtr->blue;
     valueKey.colormap = Tk_Colormap(tkwin);
     valueKey.display = display;
-    valueHashPtr = Tcl_CreateHashEntry(&dispPtr->colorValueTable, 
-            (char *) &valueKey, &new);
-    if (!new) {
+    valueHashPtr = Tcl_CreateHashEntry(&dispPtr->colorValueTable,
+	    (char *) &valueKey, &isNew);
+    if (!isNew) {
 	tkColPtr = (TkColor *) Tcl_GetHashValue(valueHashPtr);
 	tkColPtr->resourceRefCount++;
 	return &tkColPtr->color;
     }
 
     /*
-     * The name isn't currently known.  Find a pixel value for this
-     * color and add a new structure to colorValueTable (in TkDisplay).
+     * The name isn't currently known. Find a pixel value for this color and
+     * add a new structure to colorValueTable (in TkDisplay).
      */
 
     tkColPtr = TkpGetColorByValue(tkwin, colorPtr);
@@ -331,7 +327,7 @@ Tk_GetColorByValue(tkwin, colorPtr)
     tkColPtr->gc = None;
     tkColPtr->screen = Tk_Screen(tkwin);
     tkColPtr->colormap = valueKey.colormap;
-    tkColPtr->visual  = Tk_Visual(tkwin);
+    tkColPtr->visual = Tk_Visual(tkwin);
     tkColPtr->resourceRefCount = 1;
     tkColPtr->objRefCount = 0;
     tkColPtr->type = TK_COLOR_BY_VALUE;
@@ -346,16 +342,14 @@ Tk_GetColorByValue(tkwin, colorPtr)
  *
  * Tk_NameOfColor --
  *
- *	Given a color, return a textual string identifying
- *	the color.
+ *	Given a color, return a textual string identifying the color.
  *
  * Results:
- *	If colorPtr was created by Tk_GetColor, then the return
- *	value is the "string" that was used to create it.
- *	Otherwise the return value is a string that could have
- *	been passed to Tk_GetColor to allocate that color.  The
- *	storage for the returned string is only guaranteed to
- *	persist up until the next call to this procedure.
+ *	If colorPtr was created by Tk_GetColor, then the return value is the
+ *	"string" that was used to create it. Otherwise the return value is a
+ *	string that could have been passed to Tk_GetColor to allocate that
+ *	color. The storage for the returned string is only guaranteed to
+ *	persist up until the next call to this function.
  *
  * Side effects:
  *	None.
@@ -363,19 +357,20 @@ Tk_GetColorByValue(tkwin, colorPtr)
  *--------------------------------------------------------------
  */
 
-char *
+CONST char *
 Tk_NameOfColor(colorPtr)
     XColor *colorPtr;		/* Color whose name is desired. */
 {
     register TkColor *tkColPtr = (TkColor *) colorPtr;
-    
+
     if ((tkColPtr->magic == COLOR_MAGIC) &&
 	    (tkColPtr->type == TK_COLOR_BY_NAME)) {
 	return tkColPtr->hashPtr->key.string;
     } else {
-	ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
-            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
-	sprintf(tsdPtr->rgbString, "#%04x%04x%04x", colorPtr->red, 
+	ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+		Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+
+	sprintf(tsdPtr->rgbString, "#%04x%04x%04x", colorPtr->red,
 		colorPtr->green, colorPtr->blue);
 	return tsdPtr->rgbString;
     }
@@ -386,15 +381,14 @@ Tk_NameOfColor(colorPtr)
  *
  * Tk_GCForColor --
  *
- *	Given a color allocated from this module, this procedure
- *	returns a GC that can be used for simple drawing with that
- *	color.
+ *	Given a color allocated from this module, this function returns a GC
+ *	that can be used for simple drawing with that color.
  *
  * Results:
- *	The return value is a GC with color set as its foreground
- *	color and all other fields defaulted.  This GC is only valid
- *	as long as the color exists;  it is freed automatically when
- *	the last reference to the color is freed.
+ *	The return value is a GC with color set as its foreground color and
+ *	all other fields defaulted. This GC is only valid as long as the color
+ *	exists; it is freed automatically when the last reference to the color
+ *	is freed.
  *
  * Side effects:
  *	None.
@@ -404,29 +398,28 @@ Tk_NameOfColor(colorPtr)
 
 GC
 Tk_GCForColor(colorPtr, drawable)
-    XColor *colorPtr;		/* Color for which a GC is desired. Must
-				 * have been allocated by Tk_GetColor. */
-    Drawable drawable;		/* Drawable in which the color will be
-				 * used (must have same screen and depth
-				 * as the one for which the color was
-				 * allocated). */
+    XColor *colorPtr;		/* Color for which a GC is desired. Must have
+				 * been allocated by Tk_GetColor. */
+    Drawable drawable;		/* Drawable in which the color will be used
+				 * (must have same screen and depth as the one
+				 * for which the color was allocated). */
 {
     TkColor *tkColPtr = (TkColor *) colorPtr;
     XGCValues gcValues;
 
     /*
-     * Do a quick sanity check to make sure this color was really
-     * allocated by Tk_GetColor.
+     * Do a quick sanity check to make sure this color was really allocated by
+     * Tk_GetColor.
      */
 
     if (tkColPtr->magic != COLOR_MAGIC) {
-	panic("Tk_GCForColor called with bogus color");
+	Tcl_Panic("Tk_GCForColor called with bogus color");
     }
 
     if (tkColPtr->gc == None) {
 	gcValues.foreground = tkColPtr->color.pixel;
-	tkColPtr->gc = XCreateGC(DisplayOfScreen(tkColPtr->screen),
-		drawable, GCForeground, &gcValues);
+	tkColPtr->gc = XCreateGC(DisplayOfScreen(tkColPtr->screen), drawable,
+		GCForeground, &gcValues);
     }
     return tkColPtr->gc;
 }
@@ -436,16 +429,14 @@ Tk_GCForColor(colorPtr, drawable)
  *
  * Tk_FreeColor --
  *
- *	This procedure is called to release a color allocated by
- *	Tk_GetColor.
+ *	This function is called to release a color allocated by Tk_GetColor.
  *
  * Results:
  *	None.
  *
  * Side effects:
- *	The reference count associated with colorPtr is deleted, and
- *	the color is released to X if there are no remaining uses
- *	for it.
+ *	The reference count associated with colorPtr is deleted, and the color
+ *	is released to X if there are no remaining uses for it.
  *
  *----------------------------------------------------------------------
  */
@@ -461,12 +452,12 @@ Tk_FreeColor(colorPtr)
     TkColor *prevPtr;
 
     /*
-     * Do a quick sanity check to make sure this color was really
-     * allocated by Tk_GetColor.
+     * Do a quick sanity check to make sure this color was really allocated by
+     * Tk_GetColor.
      */
 
     if (tkColPtr->magic != COLOR_MAGIC) {
-	panic("Tk_FreeColor called with bogus color");
+	Tcl_Panic("Tk_FreeColor called with bogus color");
     }
 
     tkColPtr->resourceRefCount--;
@@ -476,8 +467,8 @@ Tk_FreeColor(colorPtr)
 
     /*
      * This color is no longer being actively used, so free the color
-     * resources associated with it and remove it from the hash table.
-     * no longer any objects referencing it.
+     * resources associated with it and remove it from the hash table. No
+     * longer any objects referencing it.
      */
 
     if (tkColPtr->gc != None) {
@@ -490,7 +481,7 @@ Tk_FreeColor(colorPtr)
     if (prevPtr == tkColPtr) {
 	if (tkColPtr->nextPtr == NULL) {
 	    Tcl_DeleteHashEntry(tkColPtr->hashPtr);
-	} else  {
+	} else {
 	    Tcl_SetHashValue(tkColPtr->hashPtr, tkColPtr->nextPtr);
 	}
     } else {
@@ -501,10 +492,9 @@ Tk_FreeColor(colorPtr)
     }
 
     /*
-     * Free the TkColor structure if there are no objects referencing
-     * it.  However, if there are objects referencing it then keep the
-     * structure around; it will get freed when the last reference is
-     * cleared
+     * Free the TkColor structure if there are no objects referencing it.
+     * However, if there are objects referencing it then keep the structure
+     * around; it will get freed when the last reference is cleared
      */
 
     if (tkColPtr->objRefCount == 0) {
@@ -517,18 +507,18 @@ Tk_FreeColor(colorPtr)
  *
  * Tk_FreeColorFromObj --
  *
- *	This procedure is called to release a color allocated by
- *	Tk_AllocColorFromObj. It does not throw away the Tcl_Obj *;
- *	it only gets rid of the hash table entry for this color
- *	and clears the cached value that is normally stored in the object.
+ *	This function is called to release a color allocated by
+ *	Tk_AllocColorFromObj. It does not throw away the Tcl_Obj *; it only
+ *	gets rid of the hash table entry for this color and clears the cached
+ *	value that is normally stored in the object.
  *
  * Results:
  *	None.
  *
  * Side effects:
- *	The reference count associated with the color represented by
- *	objPtr is decremented, and the color is released to X if there are 
- *	no remaining uses for it.
+ *	The reference count associated with the color represented by objPtr is
+ *	decremented, and the color is released to X if there are no remaining
+ *	uses for it.
  *
  *----------------------------------------------------------------------
  */
@@ -546,19 +536,18 @@ Tk_FreeColorFromObj(tkwin, objPtr)
 /*
  *---------------------------------------------------------------------------
  *
- * FreeColorObjProc -- 
+ * FreeColorObjProc --
  *
- *	This proc is called to release an object reference to a color.
- *	Called when the object's internal rep is released or when
- *	the cached tkColPtr needs to be changed.
+ *	This proc is called to release an object reference to a color. Called
+ *	when the object's internal rep is released or when the cached tkColPtr
+ *	needs to be changed.
  *
  * Results:
  *	None.
  *
  * Side effects:
- *	The object reference count is decremented. When both it
- *	and the hash ref count go to zero, the color's resources
- *	are released.
+ *	The object reference count is decremented. When both it and the hash
+ *	ref count go to zero, the color's resources are released.
  *
  *---------------------------------------------------------------------------
  */
@@ -571,7 +560,7 @@ FreeColorObjProc(objPtr)
 
     if (tkColPtr != NULL) {
 	tkColPtr->objRefCount--;
-	if ((tkColPtr->objRefCount == 0) 
+	if ((tkColPtr->objRefCount == 0)
 		&& (tkColPtr->resourceRefCount == 0)) {
 	    ckfree((char *) tkColPtr);
 	}
@@ -582,17 +571,17 @@ FreeColorObjProc(objPtr)
 /*
  *---------------------------------------------------------------------------
  *
- * DupColorObjProc -- 
+ * DupColorObjProc --
  *
- *	When a cached color object is duplicated, this is called to
- *	update the internal reps.
+ *	When a cached color object is duplicated, this is called to update the
+ *	internal reps.
  *
  * Results:
  *	None.
  *
  * Side effects:
- *	The color's objRefCount is incremented and the internal rep
- *	of the copy is set to point to it.
+ *	The color's objRefCount is incremented and the internal rep of the
+ *	copy is set to point to it.
  *
  *---------------------------------------------------------------------------
  */
@@ -603,7 +592,7 @@ DupColorObjProc(srcObjPtr, dupObjPtr)
     Tcl_Obj *dupObjPtr;		/* The object we are copying to. */
 {
     TkColor *tkColPtr = (TkColor *) srcObjPtr->internalRep.twoPtrValue.ptr1;
-    
+
     dupObjPtr->typePtr = srcObjPtr->typePtr;
     dupObjPtr->internalRep.twoPtrValue.ptr1 = (VOID *) tkColPtr;
 
@@ -617,17 +606,16 @@ DupColorObjProc(srcObjPtr, dupObjPtr)
  *
  * Tk_GetColorFromObj --
  *
- *	Returns the color referred to by a Tcl object.  The color must
- *	already have been allocated via a call to Tk_AllocColorFromObj
- *	or Tk_GetColor.
+ *	Returns the color referred to by a Tcl object. The color must already
+ *	have been allocated via a call to Tk_AllocColorFromObj or Tk_GetColor.
  *
  * Results:
- *	Returns the XColor * that matches the tkwin and the string rep
- *	of objPtr.
+ *	Returns the XColor * that matches the tkwin and the string rep of
+ *	objPtr.
  *
  * Side effects:
- *	If the object is not already a color, the conversion will free
- *	any old internal representation. 
+ *	If the object is not already a color, the conversion will free any old
+ *	internal representation.
  *
  *----------------------------------------------------------------------
  */
@@ -643,37 +631,38 @@ Tk_GetColorFromObj(tkwin, objPtr)
     Tcl_HashEntry *hashPtr;
     TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
 
-    if (objPtr->typePtr != &colorObjType) {
+    if (objPtr->typePtr != &tkColorObjType) {
 	InitColorObj(objPtr);
     }
-  
+
     /*
-     * First check to see if the internal representation of the object
-     * is defined and is a color that is valid for the current screen
-     * and color map.  If it is, we are done.
+     * First check to see if the internal representation of the object is
+     * defined and is a color that is valid for the current screen and color
+     * map. If it is, we are done.
      */
+
     tkColPtr = (TkColor *) objPtr->internalRep.twoPtrValue.ptr1;
     if ((tkColPtr != NULL)
 	    && (tkColPtr->resourceRefCount > 0)
 	    && (Tk_Screen(tkwin) == tkColPtr->screen)
 	    && (Tk_Colormap(tkwin) == tkColPtr->colormap)) {
 	/*
-	 * The object already points to the right TkColor structure.
-	 * Just return it.
+	 * The object already points to the right TkColor structure. Just
+	 * return it.
 	 */
 
 	return (XColor *) tkColPtr;
     }
 
     /*
-     * If we reach this point, it means that the TkColor structure
-     * that we have cached in the internal representation is not valid
-     * for the current screen and colormap.  But there is a list of
-     * other TkColor structures attached to the TkDisplay.  Walk this
-     * list looking for the right TkColor structure.
+     * If we reach this point, it means that the TkColor structure that we
+     * have cached in the internal representation is not valid for the current
+     * screen and colormap. But there is a list of other TkColor structures
+     * attached to the TkDisplay. Walk this list looking for the right TkColor
+     * structure.
      */
 
-    hashPtr = Tcl_FindHashEntry(&dispPtr->colorNameTable, 
+    hashPtr = Tcl_FindHashEntry(&dispPtr->colorNameTable,
 	    Tcl_GetString(objPtr));
     if (hashPtr == NULL) {
 	goto error;
@@ -689,8 +678,8 @@ Tk_GetColorFromObj(tkwin, objPtr)
 	}
     }
 
-    error:
-    panic(" Tk_GetColorFromObj called with non-existent color!");
+  error:
+    Tcl_Panic("Tk_GetColorFromObj called with non-existent color!");
     /*
      * The following code isn't reached; it's just there to please compilers.
      */
@@ -702,16 +691,15 @@ Tk_GetColorFromObj(tkwin, objPtr)
  *
  * InitColorObj --
  *
- *	Bookeeping procedure to change an objPtr to a color type.
+ *	Bookeeping function to change an objPtr to a color type.
  *
  * Results:
  *	None.
  *
  * Side effects:
- *	The old internal rep of the object is freed. The object's
- *	type is set to color with a NULL TkColor pointer (the pointer
- *	will be set later by either Tk_AllocColorFromObj or
- *	Tk_GetColorFromObj).
+ *	The old internal rep of the object is freed. The object's type is set
+ *	to color with a NULL TkColor pointer (the pointer will be set later by
+ *	either Tk_AllocColorFromObj or Tk_GetColorFromObj).
  *
  *----------------------------------------------------------------------
  */
@@ -723,7 +711,7 @@ InitColorObj(objPtr)
     Tcl_ObjType *typePtr;
 
     /*
-     * Free the old internalRep before setting the new one. 
+     * Free the old internalRep before setting the new one.
      */
 
     Tcl_GetString(objPtr);
@@ -731,7 +719,7 @@ InitColorObj(objPtr)
     if ((typePtr != NULL) && (typePtr->freeIntRepProc != NULL)) {
 	(*typePtr->freeIntRepProc)(objPtr);
     }
-    objPtr->typePtr = &colorObjType;
+    objPtr->typePtr = &tkColorObjType;
     objPtr->internalRep.twoPtrValue.ptr1 = (VOID *) NULL;
 }
 
@@ -756,10 +744,10 @@ ColorInit(dispPtr)
     TkDisplay *dispPtr;
 {
     if (!dispPtr->colorInit) {
-        dispPtr->colorInit = 1;
+	dispPtr->colorInit = 1;
 	Tcl_InitHashTable(&dispPtr->colorNameTable, TCL_STRING_KEYS);
-	Tcl_InitHashTable(&dispPtr->colorValueTable, 
-                sizeof(ValueKey)/sizeof(int));
+	Tcl_InitHashTable(&dispPtr->colorValueTable,
+		sizeof(ValueKey)/sizeof(int));
     }
 }
 
@@ -768,13 +756,13 @@ ColorInit(dispPtr)
  *
  * TkDebugColor --
  *
- *	This procedure returns debugging information about a color.
+ *	This function returns debugging information about a color.
  *
  * Results:
  *	The return value is a list with one sublist for each TkColor
- *	corresponding to "name".  Each sublist has two elements that
- *	contain the resourceRefCount and objRefCount fields from the
- *	TkColor structure.
+ *	corresponding to "name". Each sublist has two elements that contain
+ *	the resourceRefCount and objRefCount fields from the TkColor
+ *	structure.
  *
  * Side effects:
  *	None.
@@ -784,30 +772,142 @@ ColorInit(dispPtr)
 
 Tcl_Obj *
 TkDebugColor(tkwin, name)
-    Tk_Window tkwin;		/* The window in which the color will be
-				 * used (not currently used). */
+    Tk_Window tkwin;		/* The window in which the color will be used
+				 * (not currently used). */
     char *name;			/* Name of the desired color. */
 {
-    TkColor *tkColPtr;
     Tcl_HashEntry *hashPtr;
-    Tcl_Obj *resultPtr, *objPtr;
+    Tcl_Obj *resultPtr;
     TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
 
     resultPtr = Tcl_NewObj();
     hashPtr = Tcl_FindHashEntry(&dispPtr->colorNameTable, name);
     if (hashPtr != NULL) {
-	tkColPtr = (TkColor *) Tcl_GetHashValue(hashPtr);
+	TkColor *tkColPtr = (TkColor *) Tcl_GetHashValue(hashPtr);
+
 	if (tkColPtr == NULL) {
-	    panic("TkDebugColor found empty hash table entry");
+	    Tcl_Panic("TkDebugColor found empty hash table entry");
 	}
 	for ( ; (tkColPtr != NULL); tkColPtr = tkColPtr->nextPtr) {
-	    objPtr = Tcl_NewObj();
+	    Tcl_Obj *objPtr = Tcl_NewObj();
+
 	    Tcl_ListObjAppendElement(NULL, objPtr,
 		    Tcl_NewIntObj(tkColPtr->resourceRefCount));
 	    Tcl_ListObjAppendElement(NULL, objPtr,
-		    Tcl_NewIntObj(tkColPtr->objRefCount)); 
+		    Tcl_NewIntObj(tkColPtr->objRefCount));
 	    Tcl_ListObjAppendElement(NULL, resultPtr, objPtr);
 	}
     }
     return resultPtr;
 }
+
+#ifndef __WIN32__
+
+/* This function is not necessary for Win32,
+ * since XParseColor already does the right thing */
+
+#undef XParseColor
+
+CONST char *CONST tkWebColors[20] = {
+    /* 'a' */ "qua\0#0000ffffffff",
+    /* 'b' */ NULL,
+    /* 'c' */ "rimson\0#dcdc14143c3c",
+    /* 'd' */ NULL,
+    /* 'e' */ NULL,
+    /* 'f' */ "uchsia\0#ffff0000ffff",
+    /* 'g' */ NULL,
+    /* 'h' */ NULL,
+    /* 'i' */ "ndigo\0#4b4b00008282",
+    /* 'j' */ NULL,
+    /* 'k' */ NULL,
+    /* 'l' */ "ime\0#0000ffff0000",
+    /* 'm' */ NULL,
+    /* 'n' */ NULL,
+    /* 'o' */ "live\0#808080800000",
+    /* 'p' */ NULL,
+    /* 'q' */ NULL,
+    /* 'r' */ NULL,
+    /* 's' */ "ilver\0#c0c0c0c0c0c0",
+    /* 't' */ "eal\0#000080808080"
+};
+
+Status
+TkParseColor(display, map, name, color)
+    Display * display;		/* The display */
+    Colormap map;			/* Color map */
+    CONST char* name;     /* String to be parsed */
+    XColor * color;
+{
+    char buf[14];
+    if (*name == '#') {
+	buf[0] = '#'; buf[13] = '\0';
+	if (!*(++name) || !*(++name) || !*(++name)) {
+	    /* Not at least 3 hex digits, so invalid */
+	return 0;
+	} else if (!*(++name)) {
+	    /* Exactly 3 hex digits */
+	    buf[9] = buf[10] = buf[11] = buf[12] = *(--name);
+	    buf[5] = buf[6] = buf[7] = buf[8] = *(--name);
+	    buf[1] = buf[2] = buf[3] = buf[4] = *(--name);
+	    name = buf;
+	} else if (!*(++name)	|| !*(++name)) {
+	    /* Not at least 6 hex digits, so invalid */
+	    return 0;
+	} else if (!*(++name)) {
+	    /* Exactly 6 hex digits */
+	    buf[10] = buf[12] = *(--name);
+	    buf[9] = buf[11] = *(--name);
+	    buf[6] = buf[8] = *(--name);
+	    buf[5] = buf[7] = *(--name);
+	    buf[2] = buf[4] = *(--name);
+	    buf[1] = buf[3] = *(--name);
+	    name = buf;
+	} else if (!*(++name) || !*(++name)) {
+	    /* Not at least 9 hex digits, so invalid */
+	    return 0;
+	} else if (!*(++name)) {
+	    /* Exactly 9 hex digits */
+	    buf[11] = *(--name);
+	    buf[10] = *(--name);
+	    buf[9] = buf[12] = *(--name);
+	    buf[7] = *(--name);
+	    buf[6] = *(--name);
+	    buf[5] = buf[8] = *(--name);
+	    buf[3] = *(--name);
+	    buf[2] = *(--name);
+	    buf[1] = buf[4] = *(--name);
+	    name = buf;
+	} else if (!*(++name) || !*(++name) || *(++name)) {
+	    /* Not exactly 12 hex digits, so invalid */
+	    return 0;
+	} else {
+	    name -= 13;
+	}
+	goto done;
+    } else if (((*name - 'A') & 0xdf) < sizeof(tkWebColors)/sizeof(tkWebColors[0])) {
+	const char *p = tkWebColors[((*name - 'A') & 0x1f)];
+	if (p) {
+	    const char *q = name;
+	    while (!((*p - *(++q)) & 0xdf)) {
+		if (!*p++) {
+		    name = p;
+		    goto done;
+		}
+	    }
+	}
+    }
+    if (strlen(name) > 99) {
+	/* Don't bother to parse this. [Bug 2809525]*/
+	return 0;
+    }
+done:
+    return XParseColor(display, map, name, color);
+}
+#endif /* __WIN32__ */
+/*
+ * Local Variables:
+ * mode: c
+ * c-basic-offset: 4
+ * fill-column: 78
+ * End:
+ */
