@@ -10,8 +10,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkCursor.c,v 1.4 1999/09/02 17:02:28 hobbs Exp $
  */
 
 #include "tkPort.h"
@@ -26,8 +24,8 @@
  */
 
 typedef struct {
-    char *source;		/* Cursor bits. */
-    char *mask;			/* Mask bits. */
+    CONST char *source;		/* Cursor bits. */
+    CONST char *mask;		/* Mask bits. */
     int width, height;		/* Dimensions of cursor (and data
 				 * and mask). */
     int xHot, yHot;		/* Location of cursor hot-spot. */
@@ -44,8 +42,8 @@ static void		DupCursorObjProc _ANSI_ARGS_((Tcl_Obj *srcObjPtr,
 			    Tcl_Obj *dupObjPtr));
 static void		FreeCursor _ANSI_ARGS_((TkCursor *cursorPtr));
 static void		FreeCursorObjProc _ANSI_ARGS_((Tcl_Obj *objPtr));
-static TkCursor *	GetCursor _ANSI_ARGS_((Tcl_Interp *interp,
-			    Tk_Window tkwin, char *name));
+static TkCursor *	TkcGetCursor _ANSI_ARGS_((Tcl_Interp *interp,
+			    Tk_Window tkwin, CONST char *name));
 static TkCursor *	GetCursorFromObj _ANSI_ARGS_((Tk_Window tkwin,
 			    Tcl_Obj *objPtr));
 static void		InitCursorObj _ANSI_ARGS_((Tcl_Obj *objPtr));
@@ -58,7 +56,7 @@ static void		InitCursorObj _ANSI_ARGS_((Tcl_Obj *objPtr));
  * option is set.
  */
 
-static Tcl_ObjType cursorObjType = {
+Tcl_ObjType tkCursorObjType = {
     "cursor",			/* name */
     FreeCursorObjProc,		/* freeIntRepProc */
     DupCursorObjProc,		/* dupIntRepProc */
@@ -101,7 +99,7 @@ Tk_AllocCursorFromObj(interp, tkwin, objPtr)
 {
     TkCursor *cursorPtr;
 
-    if (objPtr->typePtr != &cursorObjType) {
+    if (objPtr->typePtr != &tkCursorObjType) {
 	InitCursorObj(objPtr);
     }
     cursorPtr = (TkCursor *) objPtr->internalRep.twoPtrValue.ptr1;
@@ -147,10 +145,10 @@ Tk_AllocCursorFromObj(interp, tkwin, objPtr)
     }
 
     /*
-     * Still no luck.  Call GetCursor to allocate a new TkCursor object.
+     * Still no luck.  Call TkcGetCursor to allocate a new TkCursor object.
      */
 
-    cursorPtr = GetCursor(interp, tkwin, Tcl_GetString(objPtr));
+    cursorPtr = TkcGetCursor(interp, tkwin, Tcl_GetString(objPtr));
     objPtr->internalRep.twoPtrValue.ptr1 = (VOID *) cursorPtr;
     if (cursorPtr == NULL) {
 	return None;
@@ -189,10 +187,10 @@ Tk_Cursor
 Tk_GetCursor(interp, tkwin, string)
     Tcl_Interp *interp;		/* Interpreter to use for error reporting. */
     Tk_Window tkwin;		/* Window in which cursor will be used. */
-    char *string;		/* Description of cursor.  See manual entry
+    Tk_Uid string;		/* Description of cursor.  See manual entry
 				 * for details on legal syntax. */
 {
-    TkCursor *cursorPtr = GetCursor(interp, tkwin, string);
+    TkCursor *cursorPtr = TkcGetCursor(interp, tkwin, string);
     if (cursorPtr == NULL) {
 	return None;
     }
@@ -202,7 +200,7 @@ Tk_GetCursor(interp, tkwin, string)
 /*
  *----------------------------------------------------------------------
  *
- * GetCursor --
+ * TkcGetCursor --
  *
  *	Given a string describing a cursor, locate (or create if necessary)
  *	a cursor that fits the description. This routine returns the
@@ -227,10 +225,10 @@ Tk_GetCursor(interp, tkwin, string)
  */
 
 static TkCursor *
-GetCursor(interp, tkwin, string)
+TkcGetCursor(interp, tkwin, string)
     Tcl_Interp *interp;		/* Interpreter to use for error reporting. */
     Tk_Window tkwin;		/* Window in which cursor will be used. */
-    char *string;		/* Description of cursor.  See manual entry
+    CONST char *string;		/* Description of cursor.  See manual entry
 				 * for details on legal syntax. */
 {
     Tcl_HashEntry *nameHashPtr;
@@ -276,12 +274,12 @@ GetCursor(interp, tkwin, string)
     cursorPtr->objRefCount = 0;
     cursorPtr->otherTable = &dispPtr->cursorNameTable;
     cursorPtr->hashPtr = nameHashPtr;
+    cursorPtr->nextPtr = existingCursorPtr;
     cursorPtr->idHashPtr = Tcl_CreateHashEntry(&dispPtr->cursorIdTable, 
             (char *) cursorPtr->cursor, &new);
     if (!new) {
 	panic("cursor already registered in Tk_GetCursor");
     }
-    cursorPtr->nextPtr = existingCursorPtr;
     Tcl_SetHashValue(nameHashPtr, cursorPtr);
     Tcl_SetHashValue(cursorPtr->idHashPtr, cursorPtr);
 
@@ -318,8 +316,8 @@ Tk_GetCursorFromData(interp, tkwin, source, mask, width, height,
 	xHot, yHot, fg, bg)
     Tcl_Interp *interp;		/* Interpreter to use for error reporting. */
     Tk_Window tkwin;		/* Window in which cursor will be used. */
-    char *source;		/* Bitmap data for cursor shape. */
-    char *mask;			/* Bitmap data for cursor mask. */
+    CONST char *source;		/* Bitmap data for cursor shape. */
+    CONST char *mask;		/* Bitmap data for cursor mask. */
     int width, height;		/* Dimensions of cursor. */
     int xHot, yHot;		/* Location of hot-spot in cursor. */
     Tk_Uid fg;			/* Foreground color for cursor. */
@@ -359,12 +357,12 @@ Tk_GetCursorFromData(interp, tkwin, source, mask, width, height,
      * available and add it to the database.
      */
 
-    if (XParseColor(dataKey.display, Tk_Colormap(tkwin), fg, &fgColor) == 0) {
+    if (TkParseColor(dataKey.display, Tk_Colormap(tkwin), fg, &fgColor) == 0) {
 	Tcl_AppendResult(interp, "invalid color name \"", fg, "\"",
 		(char *) NULL);
 	goto error;
     }
-    if (XParseColor(dataKey.display, Tk_Colormap(tkwin), bg, &bgColor) == 0) {
+    if (TkParseColor(dataKey.display, Tk_Colormap(tkwin), bg, &bgColor) == 0) {
 	Tcl_AppendResult(interp, "invalid color name \"", bg, "\"",
 		(char *) NULL);
 	goto error;
@@ -383,6 +381,7 @@ Tk_GetCursorFromData(interp, tkwin, source, mask, width, height,
     cursorPtr->objRefCount = 0;
     cursorPtr->idHashPtr = Tcl_CreateHashEntry(&dispPtr->cursorIdTable, 
             (char *) cursorPtr->cursor, &new);
+    cursorPtr->nextPtr = NULL;
 
     if (!new) {
 	panic("cursor already registered in Tk_GetCursorFromData");
@@ -417,7 +416,7 @@ Tk_GetCursorFromData(interp, tkwin, source, mask, width, height,
  *--------------------------------------------------------------
  */
 
-char *
+CONST char *
 Tk_NameOfCursor(display, cursor)
     Display *display;		/* Display for which cursor was allocated. */
     Tk_Cursor cursor;		/* Identifier for cursor whose name is
@@ -561,6 +560,7 @@ Tk_FreeCursorFromObj(tkwin, objPtr)
     Tcl_Obj *objPtr;		/* The Tcl_Obj * to be freed. */
 {
     FreeCursor(GetCursorFromObj(tkwin, objPtr));
+    FreeCursorObjProc(objPtr);
 }
 
 /*
@@ -692,32 +692,35 @@ GetCursorFromObj(tkwin, objPtr)
     Tcl_HashEntry *hashPtr;
     TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
 
-    if (objPtr->typePtr != &cursorObjType) {
+    if (objPtr->typePtr != &tkCursorObjType) {
 	InitCursorObj(objPtr);
     }
 
+    /*
+     * The internal representation is a cache of the last cursor used
+     * with the given name.  But there can be lots different cursors
+     * for each cursor name; one cursor for each display.  Check to
+     * see if the cursor we have cached is the one that is needed.
+     */
     cursorPtr = (TkCursor *) objPtr->internalRep.twoPtrValue.ptr1;
-    if (cursorPtr != NULL) {
-	if (Tk_Display(tkwin) == cursorPtr->display) {
-	    return cursorPtr;
-	}
-	hashPtr = cursorPtr->hashPtr;
-    } else {
-	hashPtr = Tcl_FindHashEntry(&dispPtr->cursorNameTable, 
-                Tcl_GetString(objPtr));
-	if (hashPtr == NULL) {
-	    goto error;
-	}
+    if ((cursorPtr != NULL) && (Tk_Display(tkwin) == cursorPtr->display)) {
+	return cursorPtr;
     }
 
     /*
-     * At this point we've got a hash table entry, off of which hang
-     * one or more TkCursor structures.  See if any of them will work.
+     * If we get to here, it means the cursor we need is not in the cache.
+     * Try to look up the cursor in the TkDisplay structure of the window.
      */
 
+    hashPtr = Tcl_FindHashEntry(&dispPtr->cursorNameTable,
+	    Tcl_GetString(objPtr));
+    if (hashPtr == NULL) {
+	goto error;
+    }
     for (cursorPtr = (TkCursor *) Tcl_GetHashValue(hashPtr);
 	    cursorPtr != NULL; cursorPtr = cursorPtr->nextPtr) {
 	if (Tk_Display(tkwin) == cursorPtr->display) {
+	    FreeCursorObjProc(objPtr);
 	    objPtr->internalRep.twoPtrValue.ptr1 = (VOID *) cursorPtr;
 	    cursorPtr->objRefCount++;
 	    return cursorPtr;
@@ -765,7 +768,7 @@ InitCursorObj(objPtr)
     if ((typePtr != NULL) && (typePtr->freeIntRepProc != NULL)) {
 	(*typePtr->freeIntRepProc)(objPtr);
     }
-    objPtr->typePtr = &cursorObjType;
+    objPtr->typePtr = &tkCursorObjType;
     objPtr->internalRep.twoPtrValue.ptr1 = (VOID *) NULL;
 }
 
@@ -843,6 +846,9 @@ TkDebugCursor(tkwin, name)
     Tcl_Obj *resultPtr, *objPtr;
     TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
 
+    if (!dispPtr->cursorInit) {
+	CursorInit(dispPtr);
+    }
     resultPtr = Tcl_NewObj();
     hashPtr = Tcl_FindHashEntry(&dispPtr->cursorNameTable, name);
     if (hashPtr != NULL) {
