@@ -11,8 +11,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkFrame.c,v 1.15.2.3 2005/08/11 12:17:09 dkf Exp $
  */
 
 #include "default.h"
@@ -172,7 +170,7 @@ enum labelanchor {
     LABELANCHOR_W, LABELANCHOR_WN, LABELANCHOR_WS
 };
 
-static char *labelAnchorStrings[] = {
+static CONST char *CONST labelAnchorStrings[] = {
     "e", "en", "es", "n", "ne", "nw", "s", "se", "sw", "w", "wn", "ws",
     (char *) NULL
 };
@@ -182,7 +180,7 @@ static char *labelAnchorStrings[] = {
  * one common table used by all and one table for each widget class.
  */
 
-static Tk_OptionSpec commonOptSpec[] = {
+static CONST Tk_OptionSpec commonOptSpec[] = {
     {TK_OPTION_BORDER, "-background", "background", "Background",
 	DEF_FRAME_BG_COLOR, -1, Tk_Offset(Frame, border),
 	TK_OPTION_NULL_OK, (ClientData) DEF_FRAME_BG_MONO, 0},
@@ -228,7 +226,7 @@ static Tk_OptionSpec commonOptSpec[] = {
 	(char *) NULL, 0, 0, 0, 0, 0}
 };
 
-static Tk_OptionSpec frameOptSpec[] = {
+static CONST Tk_OptionSpec frameOptSpec[] = {
     {TK_OPTION_SYNONYM, "-bd", (char *) NULL, (char *) NULL,
 	(char *) NULL, 0, -1, 0, (ClientData) "-borderwidth", 0},
     {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
@@ -244,7 +242,7 @@ static Tk_OptionSpec frameOptSpec[] = {
 	(char *) NULL, 0, 0, 0, (ClientData) commonOptSpec, 0}
 };
 
-static Tk_OptionSpec toplevelOptSpec[] = {
+static CONST Tk_OptionSpec toplevelOptSpec[] = {
     {TK_OPTION_SYNONYM, "-bd", (char *) NULL, (char *) NULL,
 	(char *) NULL, 0, -1, 0, (ClientData) "-borderwidth", 0},
     {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
@@ -269,7 +267,7 @@ static Tk_OptionSpec toplevelOptSpec[] = {
 	(char *) NULL, 0, 0, 0, (ClientData) commonOptSpec, 0}
 };
 
-static Tk_OptionSpec labelframeOptSpec[] = {
+static CONST Tk_OptionSpec labelframeOptSpec[] = {
     {TK_OPTION_SYNONYM, "-bd", (char *) NULL, (char *) NULL,
 	(char *) NULL, 0, -1, 0, (ClientData) "-borderwidth", 0},
     {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
@@ -304,17 +302,17 @@ static Tk_OptionSpec labelframeOptSpec[] = {
  * Class names for widgets, indexed by FrameType.
  */
 
-static char *classNames[] = {"Frame", "Toplevel", "Labelframe"};
+static CONST char *classNames[] = {"Frame", "Toplevel", "Labelframe"};
 
 /*
  * The following table maps from FrameType to the option template for
  * that class of widgets.
  */
 
-static Tk_OptionSpec *optionSpecs[] = {
+static CONST Tk_OptionSpec *CONST optionSpecs[] = {
     frameOptSpec,
     toplevelOptSpec,
-    labelframeOptSpec,
+    labelframeOptSpec
 };
 
 /*
@@ -1446,6 +1444,7 @@ DisplayFrame(clientData)
 	    goto noLabel;
 	}
 
+#ifndef TK_NO_DOUBLE_BUFFERING
 	/*
 	 * In order to avoid screen flashes, this procedure redraws the
 	 * frame into off-screen memory, then copies it back on-screen 
@@ -1455,6 +1454,9 @@ DisplayFrame(clientData)
 
 	pixmap = Tk_GetPixmap(framePtr->display, Tk_WindowId(tkwin),
 		Tk_Width(tkwin), Tk_Height(tkwin), Tk_Depth(tkwin));
+#else
+	pixmap = Tk_WindowId(tkwin);
+#endif /* TK_NO_DOUBLE_BUFFERING */
 
 	/* 
 	 * Clear the pixmap.
@@ -1573,6 +1575,7 @@ DisplayFrame(clientData)
 	}
 
 
+#ifndef TK_NO_DOUBLE_BUFFERING
 	/*
 	 * Everything's been redisplayed;  now copy the pixmap onto the screen
 	 * and free up the pixmap.
@@ -1584,6 +1587,7 @@ DisplayFrame(clientData)
 		(unsigned) (Tk_Height(tkwin) - 2 * hlWidth),
 		hlWidth, hlWidth);
 	Tk_FreePixmap(framePtr->display, pixmap);
+#endif /* TK_NO_DOUBLE_BUFFERING */
     }
 
 }
