@@ -10,8 +10,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkImage.c,v 1.5 1999/12/14 06:52:28 hobbs Exp $
  */
 
 #include "tkInt.h"
@@ -69,6 +67,9 @@ typedef struct ImageMaster {
 				 * entry). */
     Image *instancePtr;		/* Pointer to first in list of instances
 				 * derived from this name. */
+    int deleted;		/* Flag set when image is being deleted. */
+    TkWindow *winPtr;		/* Main window of interpreter (used to
+				 * detect when the world is falling apart.) */
 } ImageMaster;
 
 typedef struct ThreadSpecificData {
@@ -83,7 +84,9 @@ static Tcl_ThreadDataKey dataKey;
  * Prototypes for local procedures:
  */
 
-static void		DeleteImage _ANSI_ARGS_((ImageMaster *masterPtr));
+static void	DeleteImage _ANSI_ARGS_((ImageMaster *masterPtr));
+static void	EventuallyDeleteImage _ANSI_ARGS_((ImageMaster *masterPtr,
+						   int forgetHashEntryNow));
 
 /*
  *----------------------------------------------------------------------
@@ -156,12 +159,12 @@ Tk_ImageObjCmd(clientData, interp, objc, objv)
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument strings. */
 {
-    static char *imageOptions[] = {
-	"create", "delete", "height", "names", "type", "types", "width",
-	(char *) NULL
+    static CONST char *imageOptions[] = {
+	"create", "delete", "height", "inuse", "names", "type", "types",
+	    "width", (char *) NULL
     };
     enum options {
-	IMAGE_CREATE, IMAGE_DELETE, IMAGE_HEIGHT, IMAGE_NAMES,
+	IMAGE_CREATE, IMAGE_DELETE, IMAGE_HEIGHT, IMAGE_INUSE, IMAGE_NAMES,
 	IMAGE_TYPE, IMAGE_TYPES, IMAGE_WIDTH
     };
     TkWindow *winPtr = (TkWindow *) clientData;
@@ -228,13 +231,31 @@ Tk_ImageObjCmd(clientData, interp, objc, objv)
 	     */
 
 	    if ((objc == 3) || (*(arg = Tcl_GetString(objv[3])) == '-')) {
-		dispPtr->imageId++;
-		sprintf(idString, "image%d", dispPtr->imageId);
-		name = idString;
+                Tcl_CmdInfo dummy;
+                do {
+		    dispPtr->imageId++;
+		    sprintf(idString, "image%d", dispPtr->imageId);
+		    name = idString;
+                } while (Tcl_GetCommandInfo(interp, name, &dummy) != 0);
 		firstOption = 3;
 	    } else {
+		TkWindow *topWin;
+
 		name = arg;
 		firstOption = 4;
+		/*
+		 * Need to check if the _command_ that we are about to
+		 * create is the name of the current master widget
+		 * command (normally "." but could have been renamed)
+		 * and fail in that case before a really nasty and
+		 * hard to stop crash happens.
+		 */
+		topWin = (TkWindow *) TkToplevelWindowForCommand(interp, name);
+		if (topWin != NULL && winPtr->mainPtr->winPtr == topWin) {
+		    Tcl_AppendResult(interp, "images may not be named the ",
+			    "same as the main window", (char *) NULL);
+		    return TCL_ERROR;
+		}
 	    }
 
 	    /*
@@ -251,6 +272,9 @@ Tk_ImageObjCmd(clientData, interp, objc, objv)
 		masterPtr->tablePtr = &winPtr->mainPtr->imageTable;
 		masterPtr->hPtr = hPtr;
 		masterPtr->instancePtr = NULL;
+		masterPtr->deleted = 0;
+		masterPtr->winPtr = winPtr->mainPtr->winPtr;
+		Tcl_Preserve((ClientData) masterPtr->winPtr);
 		Tcl_SetHashValue(hPtr, masterPtr);
 	    } else {
 		/*
@@ -290,15 +314,18 @@ Tk_ImageObjCmd(clientData, interp, objc, objv)
 		}
 		args[objc] = NULL;
 	    }
+	    Tcl_Preserve((ClientData) masterPtr);
 	    if ((*typePtr->createProc)(interp, name, objc,
 		    args, typePtr, (Tk_ImageMaster) masterPtr,
 		    &masterPtr->masterData) != TCL_OK) {
-		DeleteImage(masterPtr);
+		EventuallyDeleteImage(masterPtr, 0);
+		Tcl_Release((ClientData) masterPtr);
 		if (oldimage) {
 		    ckfree((char *) args);
 		}
 		return TCL_ERROR;
 	    }
+	    Tcl_Release((ClientData) masterPtr);
 	    if (oldimage) {
 		ckfree((char *) args);
 	    }
@@ -322,8 +349,7 @@ Tk_ImageObjCmd(clientData, interp, objc, objv)
 			    "\" doesn't exist", (char *) NULL);
 		    return TCL_ERROR;
 		}
-		masterPtr = (ImageMaster *) Tcl_GetHashValue(hPtr);
-		DeleteImage(masterPtr);
+		DeleteImage((ImageMaster *) Tcl_GetHashValue(hPtr));
 	    }
 	    break;
 	}
@@ -344,18 +370,42 @@ Tk_ImageObjCmd(clientData, interp, objc, objv)
 	    Tcl_SetIntObj(Tcl_GetObjResult(interp), masterPtr->height);
 	    break;
 	}
+
+	case IMAGE_INUSE: {
+	    int count = 0;
+	    char *arg;
+	    if (objc != 3) {
+		Tcl_WrongNumArgs(interp, 2, objv, "name");
+		return TCL_ERROR;
+	    }
+	    arg = Tcl_GetString(objv[2]);
+	    hPtr = Tcl_FindHashEntry(&winPtr->mainPtr->imageTable, arg);
+	    if (hPtr == NULL) {
+		Tcl_AppendResult(interp, "image \"", arg,
+			"\" doesn't exist", (char *) NULL);
+		return TCL_ERROR;
+	    }
+	    masterPtr = (ImageMaster *) Tcl_GetHashValue(hPtr);
+	    if (masterPtr->typePtr != NULL && masterPtr->instancePtr != NULL) {
+		count = 1;
+	    }
+	    Tcl_SetBooleanObj(Tcl_GetObjResult(interp), count);
+	    break;
+	}
+
 	case IMAGE_NAMES: {
 	    if (objc != 2) {
 		Tcl_WrongNumArgs(interp, 2, objv, NULL);
 		return TCL_ERROR;
 	    }
-	    for (hPtr = Tcl_FirstHashEntry(&winPtr->mainPtr->imageTable, &search);
-		 hPtr != NULL; hPtr = Tcl_NextHashEntry(&search)) {
+	    hPtr = Tcl_FirstHashEntry(&winPtr->mainPtr->imageTable, &search);
+	    for ( ; hPtr != NULL; hPtr = Tcl_NextHashEntry(&search)) {
 		Tcl_AppendElement(interp, Tcl_GetHashKey(
 		    &winPtr->mainPtr->imageTable, hPtr));
 	    }
 	    break;
 	}
+	
 	case IMAGE_TYPE: {
 	    char *arg;
 	    if (objc != 3) {
@@ -473,12 +523,15 @@ Tk_ImageChanged(imageMaster, x, y, width, height, imageWidth,
  *----------------------------------------------------------------------
  */
 
-char *
+CONST char *
 Tk_NameOfImage(imageMaster)
     Tk_ImageMaster imageMaster;		/* Token for image. */
 {
     ImageMaster *masterPtr = (ImageMaster *) imageMaster;
 
+    if (masterPtr->hPtr == NULL) {
+	return NULL;
+    }
     return Tcl_GetHashKey(masterPtr->tablePtr, masterPtr->hPtr);
 }
 
@@ -511,7 +564,7 @@ Tk_GetImage(interp, tkwin, name, changeProc, clientData)
 				 * can't be found. */
     Tk_Window tkwin;		/* Token for window in which image will
 				 * be used. */
-    char *name;			/* Name of desired image. */
+    CONST char *name;		/* Name of desired image. */
     Tk_ImageChangedProc *changeProc;
 				/* Procedure to invoke when redisplay is
 				 * needed because image's pixels or size
@@ -600,7 +653,10 @@ Tk_FreeImage(image)
      */
 
     if ((masterPtr->typePtr == NULL) && (masterPtr->instancePtr == NULL)) {
-	Tcl_DeleteHashEntry(masterPtr->hPtr);
+	if (masterPtr->hPtr != NULL) {
+	    Tcl_DeleteHashEntry(masterPtr->hPtr);
+	}
+	Tcl_Release(masterPtr->winPtr);
 	ckfree((char *) masterPtr);
     }
 }
@@ -634,11 +690,31 @@ Tk_PostscriptImage(image, interp, tkwin, psinfo, x, y, width, height, prepass)
     int width, height;		/* Dimensions of region to redraw. */
     int prepass;
 {
+    Image *imagePtr = (Image *) image;
     int result;
     XImage *ximage;
     Pixmap pmap;
     GC newGC;
     XGCValues gcValues;
+
+    if (imagePtr->masterPtr->typePtr == NULL) {
+	/*
+	 * No master for image, so nothing to display on postscript.
+	 */
+	return TCL_OK;
+    }
+
+    /*
+     * Check if an image specific postscript-generation function
+     * exists; otherwise go on with generic code.
+     */
+
+    if (imagePtr->masterPtr->typePtr->postscriptProc != NULL) {
+	return (*imagePtr->masterPtr->typePtr->postscriptProc)(
+	    imagePtr->masterPtr->masterData, interp, tkwin, psinfo,
+	    x, y, width, height, prepass);
+    }
+
     if (prepass) {
 	return TCL_OK;
     }
@@ -800,7 +876,7 @@ void
 Tk_DeleteImage(interp, name)
     Tcl_Interp *interp;		/* Interpreter in which the image was
 				 * created. */
-    char *name;			/* Name of image. */
+    CONST char *name;		/* Name of image. */
 {
     Tcl_HashEntry *hPtr;
     TkWindow *winPtr;
@@ -813,7 +889,7 @@ Tk_DeleteImage(interp, name)
     if (hPtr == NULL) {
 	return;
     }
-    DeleteImage((ImageMaster *) Tcl_GetHashValue(hPtr));
+    DeleteImage((ImageMaster *)Tcl_GetHashValue(hPtr));
 }
 
 /*
@@ -855,11 +931,47 @@ DeleteImage(masterPtr)
 	(*typePtr->deleteProc)(masterPtr->masterData);
     }
     if (masterPtr->instancePtr == NULL) {
-	Tcl_DeleteHashEntry(masterPtr->hPtr);
+	if (masterPtr->hPtr != NULL) {
+	    Tcl_DeleteHashEntry(masterPtr->hPtr);
+	}
+	Tcl_Release((ClientData) masterPtr->winPtr);
 	ckfree((char *) masterPtr);
     }
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * EventuallyDeleteImage --
+ *
+ *	Arrange for an image to be deleted when it is safe to do so.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Image will get freed, though not until it is no longer
+ *	Tcl_Preserve()d by anything.  May be called multiple times on
+ *	the same image without ill effects.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void
+EventuallyDeleteImage(masterPtr, forgetHashEntryNow)
+    ImageMaster *masterPtr;	/* Pointer to main data structure for image. */
+    int forgetHashEntryNow;
+{
+    if (forgetHashEntryNow) {
+	masterPtr->hPtr = NULL;
+    }
+    if (!masterPtr->deleted) {
+	masterPtr->deleted = 1;
+	Tcl_EventuallyFree((ClientData) masterPtr,
+		(Tcl_FreeProc *)DeleteImage);
+    }
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -886,12 +998,10 @@ TkDeleteAllImages(mainPtr)
 {
     Tcl_HashSearch search;
     Tcl_HashEntry *hPtr;
-    ImageMaster *masterPtr;
 
     for (hPtr = Tcl_FirstHashEntry(&mainPtr->imageTable, &search);
 	    hPtr != NULL; hPtr = Tcl_NextHashEntry(&search)) {
-	masterPtr = (ImageMaster *) Tcl_GetHashValue(hPtr);
-	DeleteImage(masterPtr);
+	EventuallyDeleteImage((ImageMaster *) Tcl_GetHashValue(hPtr), 1);
     }
     Tcl_DeleteHashTable(&mainPtr->imageTable);
 }
@@ -921,7 +1031,7 @@ ClientData
 Tk_GetImageMasterData(interp, name, typePtrPtr)
     Tcl_Interp *interp;		/* Interpreter in which the image was
 				 * created. */
-    char *name;			/* Name of image. */
+    CONST char *name;		/* Name of image. */
     Tk_ImageType **typePtrPtr;	/* Points to location to fill in with
 				 * pointer to type information for image. */
 {
@@ -967,7 +1077,7 @@ Tk_SetTSOrigin(tkwin, gc, x, y)
     GC gc;
     int x, y;
 {
-    while (!Tk_IsTopLevel(tkwin)) {
+    while (!Tk_TopWinHierarchy(tkwin)) {
 	x -= Tk_X(tkwin) + Tk_Changes(tkwin)->border_width;
 	y -= Tk_Y(tkwin) + Tk_Changes(tkwin)->border_width;
 	tkwin = Tk_Parent(tkwin);
