@@ -9,8 +9,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkImgBmap.c,v 1.15.2.2 2006/09/22 14:53:06 dkf Exp $
  */
 
 #include "tkInt.h"
@@ -1161,7 +1159,7 @@ ImgBmapPsImagemask(interp, width, height, data)
      * on the left.)  The following array is used to reverse the order of bits
      * within a byte so that the bits will be in the order postscript expects.
      */
-    static unsigned char bit_reverse[] = {
+    static CONST unsigned char bit_reverse[] = {
        0, 128, 64, 192, 32, 160,  96, 224, 16, 144, 80, 208, 48, 176, 112, 240,
        8, 136, 72, 200, 40, 168, 104, 232, 24, 152, 88, 216, 56, 184, 120, 248,
        4, 132, 68, 196, 36, 164, 100, 228, 20, 148, 84, 212, 52, 180, 116, 244,
@@ -1268,7 +1266,7 @@ ImgBmapPostscript(clientData, interp, tkwin, psinfo, x, y, width, height,
      */
     if ((masterPtr->bgUid != NULL) && (masterPtr->bgUid[0] != '\000')) {
 	XColor color;
-	XParseColor(Tk_Display(tkwin), Tk_Colormap(tkwin), masterPtr->bgUid,
+	TkParseColor(Tk_Display(tkwin), Tk_Colormap(tkwin), masterPtr->bgUid,
 		&color);
 	if (Tk_PostscriptColor(interp, psinfo, &color) != TCL_OK) {
 	    return TCL_ERROR;
@@ -1288,7 +1286,7 @@ ImgBmapPostscript(clientData, interp, tkwin, psinfo, x, y, width, height,
      */
     if ( (masterPtr->fgUid != NULL) && (masterPtr->data != NULL) ) {
 	XColor color;
-	XParseColor(Tk_Display(tkwin), Tk_Colormap(tkwin), masterPtr->fgUid,
+	TkParseColor(Tk_Display(tkwin), Tk_Colormap(tkwin), masterPtr->fgUid,
 		&color);
 	if (Tk_PostscriptColor(interp, psinfo, &color) != TCL_OK) {
 	    return TCL_ERROR;
