@@ -28,8 +28,6 @@
  * |   notice appear in supporting documentation.  This software is	|
  * |   provided "as is" without express or implied warranty.		|
  * +-------------------------------------------------------------------+
- *
- * RCS: @(#) $Id: tkImgGIF.c,v 1.24.2.5 2007/09/11 18:01:45 rmax Exp $
  */
 
 /*
@@ -809,8 +807,8 @@ ReadImage(interp, imagePtr, chan, len, rows, cmap,
     int v;
     int xpos = 0, ypos = 0, pass = 0, i;
     register char *pixelPtr;
-    CONST static int interlaceStep[] = { 8, 8, 4, 2 };
-    CONST static int interlaceStart[] = { 0, 4, 2, 1 };
+    static CONST int interlaceStep[] = { 8, 8, 4, 2 };
+    static CONST int interlaceStart[] = { 0, 4, 2, 1 };
     unsigned short prefix[(1 << MAX_LWZ_BITS)];
     unsigned char  append[(1 << MAX_LWZ_BITS)];
     unsigned char  stack[(1 << MAX_LWZ_BITS)*2];
@@ -826,6 +824,12 @@ ReadImage(interp, imagePtr, chan, len, rows, cmap,
 		Tcl_PosixError(interp), (char *) NULL);
 	return TCL_ERROR;
     }
+
+    if (initialCodeSize > MAX_LWZ_BITS) {
+	Tcl_SetResult(interp, "malformed image", TCL_STATIC);
+	return TCL_ERROR;
+    }
+
     if (transparent != -1) {
 	cmap[transparent][CM_RED] = 0;
 	cmap[transparent][CM_GREEN] = 0;
@@ -2092,7 +2096,7 @@ compress(init_bits, handle, readValue)
     out_clear_init = (init_bits <= 3) ? 9 : (out_bump_init-1);
 #ifdef MIGIF_DEBUGGING_ENVARS
     {
-	const char *ocienv;
+	CONST char *ocienv;
 	ocienv = getenv("MIGIF_OUT_CLEAR_INIT");
 	if (ocienv) {
 	    out_clear_init = atoi(ocienv);
