@@ -8,10 +8,10 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * SCCS: @(#) tkInitScript.h 1.3 97/08/11 19:12:28
  */
 
+
+
 /*
  * In order to find tk.tcl during initialization, the following script
  * is invoked by Tk_Init().  It looks in several different directories:
@@ -48,20 +48,7 @@
 static char initScript[] = "if {[info proc tkInit]==\"\"} {\n\
   proc tkInit {} {\n\
     global tk_library tk_version tk_patchLevel\n\
-    rename tkInit {}\n\
-    if {![catch {uplevel #0 source -rsrc tk}]} {\n\
-	set tk_library {}\n\
-	uplevel #0 {\n\
-	    source -rsrc button\n\
-	    source -rsrc entry\n\
-	    source -rsrc listbox\n\
-	    source -rsrc menu\n\
-	    source -rsrc scale\n\
-	    source -rsrc scrlbar\n\
-	    source -rsrc text\n\
-	}\n\
- 	return\n\
-    }\n\
+      rename tkInit {}\n\
     tcl_findLibrary tk $tk_version $tk_patchLevel tk.tcl TK_LIBRARY tk_library\n\
   }\n\
 }\n\
