@@ -1,4 +1,4 @@
-/* 
+/*
  * tkMain.c --
  *
  *	This file contains a generic main program for Tk-based applications.
@@ -12,8 +12,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkMain.c,v 1.15.2.5 2006/09/25 17:28:20 andreas_kupries Exp $
  */
 
 #include <ctype.h>
@@ -30,11 +28,13 @@
 #endif
 #ifdef __WIN32__
 #include "tkWinInt.h"
+#include "../win/tclWinPort.h"
 #endif
 #ifdef MAC_OSX_TK
 #include "tkMacOSXInt.h"
 #endif
 
+extern int TkCygwinMainEx(int, char **, Tcl_AppInitProc *, Tcl_Interp *);
 
 typedef struct ThreadSpecificData {
     Tcl_Interp *interp;         /* Interpreter for this thread. */
@@ -42,7 +42,7 @@ typedef struct ThreadSpecificData {
 				 * into Tcl commands. */
     Tcl_DString line;           /* Used to read the next line from the
 				 * terminal input. */
-    int tty;                    /* Non-zero means standard input is a 
+    int tty;                    /* Non-zero means standard input is a
 				 * terminal-like device.  Zero means it's
 				 * a file. */
 } ThreadSpecificData;
@@ -57,12 +57,46 @@ static Tcl_ThreadDataKey dataKey;
  * some systems.
  */
 
-#if !defined(__WIN32__) && !defined(_WIN32)
-#if !defined(MAC_TCL)
-extern int		isatty _ANSI_ARGS_((int fd));
-#else
-#include <unistd.h>
+#if defined(__WIN32__) || defined(_WIN32)
+#define isatty WinIsTty
+static int WinIsTty(int fd) {
+    HANDLE handle;
+
+    /*
+     * For now, under Windows, we assume we are not running as a console mode
+     * app, so we need to use the GUI console.  In order to enable this, we
+     * always claim to be running on a tty.  This probably isn't the right
+     * way to do it.
+     */
+
+#if !defined(STATIC_BUILD)
+	if (tclStubsPtr->reserved9 && TclpIsAtty) {
+	    /* We are running on Cygwin */
+	    return TclpIsAtty(fd);
+	}
 #endif
+    handle = GetStdHandle(STD_INPUT_HANDLE + fd);
+
+    if ((handle == INVALID_HANDLE_VALUE) || (handle == 0)
+	     || (GetFileType(handle) == FILE_TYPE_UNKNOWN)) {
+	/*
+	 * If it's a bad or closed handle, then it's been connected
+	 * to a wish console window.
+	 */
+
+	return 1;
+    } else if (GetFileType(handle) == FILE_TYPE_CHAR) {
+	/*
+	 * A character file handle is a tty by definition.
+	 */
+
+	return 1;
+    } else {
+	return 0;
+    }
+}
+#else
+extern int		isatty _ANSI_ARGS_((int fd));
 extern char *		strrchr _ANSI_ARGS_((CONST char *string, int c));
 #endif
 
@@ -103,32 +137,62 @@ Tk_MainEx(argc, argv, appInitProc, interp)
     Tcl_Interp *interp;
 {
     Tcl_Obj *argvPtr;
-    int code;
+    int code, nullStdin = 0;
     size_t length;
     Tcl_Channel inChannel, outChannel;
     Tcl_DString appName;
     ThreadSpecificData *tsdPtr;
-#ifdef __WIN32__
-    HANDLE handle;
-#endif
 
     /*
-     * Ensure that we are getting the matching version of Tcl.  This is
-     * really only an issue when Tk is loaded dynamically.
+     * Ensure that we are getting the matching version of Tcl.
      */
 
     if (Tcl_InitStubs(interp, TCL_VERSION, 1) == NULL) {
-	abort();
+	if (Tcl_InitStubs(interp, "8.1", 0) == NULL) {
+	    abort();
+	} else {
+	    Tcl_Panic("%s", Tcl_GetStringResult(interp));
+	}
     }
 
-    tsdPtr = (ThreadSpecificData *) 
+#if defined(__WIN32__) && !defined(__WIN64__) && !defined(STATIC_BUILD)
+
+    if (tclStubsPtr->reserved9) {
+	/* We are running win32 Tk under Cygwin, so let's check
+	 * whether the env("DISPLAY") variable or the -display
+	 * argument is set. If so, we really want to run the
+	 * Tk_MainEx function of libtk8.?.dll, not this one. */
+	if (Tcl_GetVar2(interp, "env", "DISPLAY", TCL_GLOBAL_ONLY)) {
+	loadCygwinTk:
+	    if (TkCygwinMainEx(argc, argv, appInitProc, interp)) {
+		/* Should never reach here. */
+		return;
+	    }
+	} else {
+	    int i;
+
+	    for (i = 1; i < argc; ++i) {
+		if (!strcmp(argv[i], "-display")) {
+		    goto loadCygwinTk;
+		}
+	    }
+	}
+    }
+#endif
+
+    tsdPtr = (ThreadSpecificData *)
 	Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
-    
+
     Tcl_FindExecutable(argv[0]);
     tsdPtr->interp = interp;
     Tcl_Preserve((ClientData) interp);
 
-#if (defined(__WIN32__) || defined(MAC_TCL))
+#if defined(__WIN32__) && !defined(STATIC_BUILD)
+    if (!tclStubsPtr->reserved9) {
+	/* Only initialize console when not running under cygwin */
+	Tk_InitConsoleChannels(interp);
+    }
+#elif  defined(__WIN32__)
     Tk_InitConsoleChannels(interp);
 #endif
 
@@ -137,7 +201,7 @@ Tk_MainEx(argc, argv, appInitProc, interp)
         TkMacOSXDefaultStartupScript();
     }
 #endif
-    
+
 #ifdef TCL_MEM_DEBUG
     Tcl_InitMemory(interp);
 #endif
@@ -163,7 +227,7 @@ Tk_MainEx(argc, argv, appInitProc, interp)
 	    argv++;
 	}
     }
-    
+
     /*
      * Make command-line arguments available in the Tcl variables "argc"
      * and "argv".
@@ -196,40 +260,24 @@ Tk_MainEx(argc, argv, appInitProc, interp)
      * Set the "tcl_interactive" variable.
      */
 
+    tsdPtr->tty = isatty(0);
+
+#if defined(MAC_OSX_TK)
     /*
-     * For now, under Windows, we assume we are not running as a console mode
-     * app, so we need to use the GUI console.  In order to enable this, we
-     * always claim to be running on a tty.  This probably isn't the right
-     * way to do it.
+     * On TkAqua, if we don't have a TTY and stdin is a special character file
+     * of length 0, (e.g. /dev/null, which is what Finder sets when double
+     * clicking Wish) then use the GUI console.
      */
 
-#ifdef __WIN32__
-    handle = GetStdHandle(STD_INPUT_HANDLE);
+    if (!tsdPtr->tty) {
+	struct stat st;
 
-    if ((handle == INVALID_HANDLE_VALUE) || (handle == 0) 
-	     || (GetFileType(handle) == FILE_TYPE_UNKNOWN)) {
-	/*
-	 * If it's a bad or closed handle, then it's been connected
-	 * to a wish console window.
-	 */
-
-	tsdPtr->tty = 1;
-    } else if (GetFileType(handle) == FILE_TYPE_CHAR) {
-	/*
-	 * A character file handle is a tty by definition.
-	 */
-
-	tsdPtr->tty = 1;
-    } else {
-	tsdPtr->tty = 0;
+	nullStdin = fstat(0, &st) || (S_ISCHR(st.st_mode) && !st.st_blocks);
     }
-
-#else
-    tsdPtr->tty = isatty(0);
 #endif
     Tcl_SetVar(interp, "tcl_interactive",
-	    ((TclGetStartupScriptFileName() == NULL) 
-		    && tsdPtr->tty) ? "1" : "0", TCL_GLOBAL_ONLY);
+	    ((TclGetStartupScriptFileName() == NULL) && (tsdPtr->tty
+	    || nullStdin)) ? "1" : "0", TCL_GLOBAL_ONLY);
 
     /*
      * Invoke application-specific initialization.
@@ -331,7 +379,7 @@ StdinProc(clientData, mask)
     char *cmd;
     int code, count;
     Tcl_Channel chan = (Tcl_Channel) clientData;
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
             Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     Tcl_Interp *interp = tsdPtr->interp;
 
@@ -345,7 +393,7 @@ StdinProc(clientData, mask)
 		Tcl_DeleteChannelHandler(chan, StdinProc, (ClientData) chan);
 	    }
 	    return;
-	} 
+	}
     }
 
     (void) Tcl_DStringAppend(&tsdPtr->command, Tcl_DStringValue(
@@ -368,7 +416,7 @@ StdinProc(clientData, mask)
 
     Tcl_CreateChannelHandler(chan, 0, StdinProc, (ClientData) chan);
     code = Tcl_RecordAndEval(interp, cmd, TCL_EVAL_GLOBAL);
-    
+
     chan = Tcl_GetStdChannel(TCL_STDIN);
     if (chan) {
 	Tcl_CreateChannelHandler(chan, TCL_READABLE, StdinProc,
@@ -452,7 +500,7 @@ defaultPrompt:
              * is possible that someone has transferred stderr out of
              * this interpreter with "interp transfer".
              */
-            
+
 	    errChannel = Tcl_GetChannel(interp, "stderr", NULL);
             if (errChannel != (Tcl_Channel) NULL) {
                 Tcl_WriteObj(errChannel, Tcl_GetObjResult(interp));
