@@ -9,8 +9,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkMenubutton.c,v 1.12.2.2 2007/12/13 23:13:52 hobbs Exp $
  */
 
 #include "tkMenubutton.h"
@@ -23,7 +21,7 @@
  * in tkMenubutton.h.
  */
 
-static char *directionStrings[] = {
+static CONST char *CONST directionStrings[] = {
     "above", "below", "flush", "left", "right", (char *) NULL
 };
 
@@ -32,7 +30,7 @@ static char *directionStrings[] = {
  * It is used together with the "enum state" declaration in tkMenubutton.h.
  */
 
-static char *stateStrings[] = {
+static CONST char *CONST stateStrings[] = {
     "active", "disabled", "normal", (char *) NULL
 };
 
@@ -41,7 +39,7 @@ static char *stateStrings[] = {
  * It is used with the "enum compound" declaration in tkMenuButton.h
  */
 
-static char *compoundStrings[] = {
+static CONST char *CONST compoundStrings[] = {
     "bottom", "center", "left", "none", "right", "top", (char *) NULL
 };
 
@@ -49,7 +47,7 @@ static char *compoundStrings[] = {
  * Information used for parsing configuration specs:
  */
 
-static Tk_OptionSpec optionSpecs[] = {
+static CONST Tk_OptionSpec optionSpecs[] = {
     {TK_OPTION_BORDER, "-activebackground", "activeBackground", "Foreground",
         DEF_MENUBUTTON_ACTIVE_BG_COLOR, -1, 
         Tk_Offset(TkMenuButton, activeBorder), 0, 
