@@ -9,8 +9,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkPlace.c,v 1.13.2.2 2007/10/25 16:38:23 dgp Exp $
  */
 
 #include "tkPort.h"
@@ -28,7 +26,7 @@
  *			master's actual window size.
  */
 
-static char *borderModeStrings[] = {
+static CONST char *borderModeStrings[] = {
     "inside", "outside", "ignore", (char *) NULL
 };
 
@@ -81,7 +79,7 @@ typedef struct Slave {
  */
 #define IN_MASK		1
 
-static Tk_OptionSpec optionSpecs[] = {
+static CONST Tk_OptionSpec optionSpecs[] = {
     {TK_OPTION_ANCHOR, "-anchor", NULL, NULL, "nw", -1,
 	 Tk_Offset(Slave, anchor), 0, 0, 0},
     {TK_OPTION_STRING_TABLE, "-bordermode", NULL, NULL, "inside", -1,
