@@ -8,8 +8,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkSelect.h,v 1.2 1998/09/14 18:23:17 stanton Exp $
  */
 
 #ifndef _TKSELECT
@@ -95,6 +93,10 @@ typedef struct TkSelRetrievalInfo {
     int idleTime;		/* Number of seconds that have gone by
 				 * without hearing anything from the
 				 * selection owner. */
+    Tcl_EncodingState encState;	/* Holds intermediate state during translations
+				 * of data that cross buffer boundaries. */
+    int encFlags;		/* Encoding translation state flags. */
+    Tcl_DString buf;		/* Buffer to hold translation data. */
     struct TkSelRetrievalInfo *nextPtr;
 				/* Next in list of all pending
 				 * selection retrievals.  NULL means
@@ -145,14 +147,6 @@ typedef struct TkSelInProgress {
 				/* Next higher nested search. */
 } TkSelInProgress;
 
-/*
- * Declarations for variables shared among the selection-related files:
- */
-
-extern TkSelInProgress *pendingPtr;
-				/* Topmost search in progress, or
-				 * NULL if none. */
-
 /*
  * Chunk size for retrieving selection.  It's defined both in
  * words and in bytes;  the word size is used to allocate
@@ -168,6 +162,11 @@ extern TkSelInProgress *pendingPtr;
  * but shouldn't be used anywhere else in Tk (or by Tk clients):
  */
 
+extern TkSelInProgress * 
+                        TkSelGetInProgress _ANSI_ARGS_((void));
+extern void             TkSelSetInProgress _ANSI_ARGS_((
+                            TkSelInProgress *pendingPtr));
+
 extern void		TkSelClearSelection _ANSI_ARGS_((Tk_Window tkwin,
 			    XEvent *eventPtr));
 extern int		TkSelDefaultSelection _ANSI_ARGS_((
