@@ -12,8 +12,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkTest.c,v 1.21.2.2 2005/11/27 02:44:25 das Exp $
  */
 
 #include "tkInt.h"
@@ -24,7 +22,7 @@
 #include "tkWinInt.h"
 #endif
 
-#if defined(MAC_TCL) || defined(MAC_OSX_TK)
+#if defined(MAC_OSX_TK)
 #include "tkScrollbar.h"
 #endif
 
@@ -179,11 +177,11 @@ static int		TestfontObjCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Obj *CONST objv[]));
 static int		TestmakeexistCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int argc, CONST char **argv));
-#if !(defined(__WIN32__) || defined(MAC_TCL) || defined(MAC_OSX_TK))
+#if !(defined(__WIN32__) || defined(MAC_OSX_TK))
 static int		TestmenubarCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int argc, CONST char **argv));
 #endif
-#if defined(__WIN32__) || defined(MAC_TCL) || defined(MAC_OSX_TK)
+#if defined(__WIN32__) || defined(MAC_OSX_TK)
 static int		TestmetricsCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int argc, CONST char **argv));
 #endif
@@ -203,13 +201,13 @@ static void	CustomOptionFree _ANSI_ARGS_((ClientData clientData,
 			Tk_Window tkwin, char *internalPtr));
 static int		TestpropCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int argc, CONST char **argv));
-#if !(defined(__WIN32__) || defined(MAC_TCL) || defined(MAC_OSX_TK))
+#if !(defined(__WIN32__) || defined(MAC_OSX_TK))
 static int		TestsendCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int argc, CONST char **argv));
 #endif
 static int		TesttextCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int argc, CONST char **argv));
-#if !(defined(__WIN32__) || defined(MAC_TCL) || defined(MAC_OSX_TK))
+#if !(defined(__WIN32__) || defined(MAC_OSX_TK))
 static int		TestwrapperCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int argc, CONST char **argv));
 #endif
@@ -227,7 +225,7 @@ static void		TrivialEventProc _ANSI_ARGS_((ClientData clientData,
 
 extern int		TkplatformtestInit _ANSI_ARGS_((Tcl_Interp *interp));
 
-#if !(defined(__WIN32__) || defined(MAC_TCL) || defined(MAC_OSX_TK))
+#if !(defined(__WIN32__) || defined(MAC_OSX_TK))
 #define TkplatformtestInit(x) TCL_OK
 #endif
 
@@ -285,23 +283,23 @@ Tktest_Init(interp)
 	    (ClientData) Tk_MainWindow(interp), (Tcl_CmdDeleteProc *) NULL);
     Tcl_CreateCommand(interp, "testmakeexist", TestmakeexistCmd,
 	    (ClientData) Tk_MainWindow(interp), (Tcl_CmdDeleteProc *) NULL);
-#if !(defined(__WIN32__) || defined(MAC_TCL) || defined(MAC_OSX_TK))
+#if !(defined(__WIN32__) || defined(MAC_OSX_TK))
     Tcl_CreateCommand(interp, "testmenubar", TestmenubarCmd,
 	    (ClientData) Tk_MainWindow(interp), (Tcl_CmdDeleteProc *) NULL);
 #endif
-#if defined(__WIN32__) || defined(MAC_TCL) || defined(MAC_OSX_TK)
+#if defined(__WIN32__) || defined(MAC_OSX_TK)
     Tcl_CreateCommand(interp, "testmetrics", TestmetricsCmd,
 	    (ClientData) Tk_MainWindow(interp), (Tcl_CmdDeleteProc *) NULL);
 #endif
     Tcl_CreateCommand(interp, "testprop", TestpropCmd,
 	    (ClientData) Tk_MainWindow(interp), (Tcl_CmdDeleteProc *) NULL);
-#if !(defined(__WIN32__) || defined(MAC_TCL) || defined(MAC_OSX_TK))
+#if !(defined(__WIN32__) || defined(MAC_OSX_TK))
     Tcl_CreateCommand(interp, "testsend", TestsendCmd,
 	    (ClientData) Tk_MainWindow(interp), (Tcl_CmdDeleteProc *) NULL);
 #endif
     Tcl_CreateCommand(interp, "testtext", TesttextCmd,
 	    (ClientData) Tk_MainWindow(interp), (Tcl_CmdDeleteProc *) NULL);
-#if !(defined(__WIN32__) || defined(MAC_TCL) || defined(MAC_OSX_TK))
+#if !(defined(__WIN32__) || defined(MAC_OSX_TK))
     Tcl_CreateCommand(interp, "testwrapper", TestwrapperCmd,
 	    (ClientData) Tk_MainWindow(interp), (Tcl_CmdDeleteProc *) NULL);
 #endif
@@ -412,7 +410,7 @@ CBindingEvalProc(clientData, interp, eventPtr, tkwin, keySym)
 
     cbindPtr = (CBinding *) clientData;
     
-    return Tcl_GlobalEval(interp, cbindPtr->command);
+    return Tcl_EvalEx(interp, cbindPtr->command, -1, TCL_EVAL_GLOBAL);
 }
 
 static void
@@ -422,7 +420,7 @@ CBindingFreeProc(clientData)
     CBinding *cbindPtr = (CBinding *) clientData;
     
     if (cbindPtr->delete != NULL) {
-	Tcl_GlobalEval(cbindPtr->interp, cbindPtr->delete);
+	Tcl_EvalEx(cbindPtr->interp, cbindPtr->delete, -1, TCL_EVAL_GLOBAL);
 	ckfree((char *) cbindPtr->delete);
     }
     ckfree((char *) cbindPtr->command);
@@ -651,7 +649,7 @@ TestobjconfigObjCmd(clientData, interp, objc, objv)
 					 * created by commands below; indexed
 					 * with same values as "options"
 					 * array. */
-    static Tk_ObjCustomOption CustomOption = {
+    static CONST Tk_ObjCustomOption CustomOption = {
 	"custom option",
 	    CustomOptionSet,
 	    CustomOptionGet,
@@ -676,7 +674,7 @@ TestobjconfigObjCmd(clientData, interp, objc, objv)
 	Tcl_Obj *extension4ObjPtr;
 	Tcl_Obj *extension5ObjPtr;
     } ExtensionWidgetRecord;
-    static Tk_OptionSpec baseSpecs[] = {
+    static CONST Tk_OptionSpec baseSpecs[] = {
 	{TK_OPTION_STRING,
 		"-one", "one", "One", "one",
 		Tk_Offset(ExtensionWidgetRecord, base1ObjPtr), -1},
@@ -719,9 +717,9 @@ TestobjconfigObjCmd(clientData, interp, objc, objv)
 		Tcl_Obj *customPtr;
 	    } TypesRecord;
 	    TypesRecord *recordPtr;
-	    static char *stringTable[] = {"one", "two", "three", "four", 
+	    static CONST char *CONST stringTable[] = {"one", "two", "three", "four",
 		    (char *) NULL};
-	    static Tk_OptionSpec typesSpecs[] = {
+	    static CONST Tk_OptionSpec typesSpecs[] = {
 		{TK_OPTION_BOOLEAN,
 			"-boolean", "boolean", "Boolean",
 			"1", Tk_Offset(TypesRecord, booleanPtr), -1, 0, 0, 0x1},
@@ -895,7 +893,7 @@ TestobjconfigObjCmd(clientData, interp, objc, objv)
 
 	case CHAIN2: {
 	    ExtensionWidgetRecord *recordPtr;
-	    static Tk_OptionSpec extensionSpecs[] = {
+	    static CONST Tk_OptionSpec extensionSpecs[] = {
 		{TK_OPTION_STRING,
 			"-three", "three", "Three", "three",
 			Tk_Offset(ExtensionWidgetRecord, extension3ObjPtr),
@@ -963,7 +961,7 @@ TestobjconfigObjCmd(clientData, interp, objc, objv)
 		Tcl_Obj *intPtr;
 	    } ErrorWidgetRecord;
 	    ErrorWidgetRecord widgetRecord;
-	    static Tk_OptionSpec errorSpecs[] = {
+	    static CONST Tk_OptionSpec errorSpecs[] = {
 		{TK_OPTION_INT, 
 			"-int", "integer", "Integer",
 			"bogus", Tk_Offset(ErrorWidgetRecord, intPtr)},
@@ -1034,10 +1032,10 @@ TestobjconfigObjCmd(clientData, interp, objc, objv)
 		char *custom;
 	    } InternalRecord;
 	    InternalRecord *recordPtr;
-	    static char *internalStringTable[] = {
+	    static CONST char *internalStringTable[] = {
 		    "one", "two", "three", "four", (char *) NULL
 	    };
-	    static Tk_OptionSpec internalSpecs[] = {
+	    static CONST Tk_OptionSpec internalSpecs[] = {
 		{TK_OPTION_BOOLEAN,
 			"-boolean", "boolean", "Boolean",
 			"1", -1, Tk_Offset(InternalRecord, boolean), 0, 0, 0x1},
@@ -1175,7 +1173,7 @@ TestobjconfigObjCmd(clientData, interp, objc, objv)
 		Tcl_Obj *five;
 	    } FiveRecord;
 	    FiveRecord *recordPtr;
-	    static Tk_OptionSpec smallSpecs[] = {
+	    static CONST Tk_OptionSpec smallSpecs[] = {
 		{TK_OPTION_INT,
 			"-one", "one", "One",
 			"1",
@@ -1241,7 +1239,7 @@ TestobjconfigObjCmd(clientData, interp, objc, objv)
 		Tcl_Obj *fooObjPtr;
 	    } NotEnoughRecord;
 	    NotEnoughRecord record;
-	    static Tk_OptionSpec errorSpecs[] = {
+	    static CONST Tk_OptionSpec errorSpecs[] = {
 		{TK_OPTION_INT, 
 			"-foo", "foo", "Foo",
 			"0", Tk_Offset(NotEnoughRecord, fooObjPtr)},
@@ -1276,7 +1274,7 @@ TestobjconfigObjCmd(clientData, interp, objc, objv)
 		Tcl_Obj *windowPtr;
 	    } SlaveRecord;
 	    SlaveRecord *recordPtr;
-	    static Tk_OptionSpec slaveSpecs[] = {
+	    static CONST Tk_OptionSpec slaveSpecs[] = {
 		{TK_OPTION_WINDOW,
 			"-window", "window", "Window",
 			".bar", Tk_Offset(SlaveRecord, windowPtr), -1,
@@ -1963,7 +1961,7 @@ TestmakeexistCmd(clientData, interp, argc, argv)
  */
 
 	/* ARGSUSED */
-#if !(defined(__WIN32__) || defined(MAC_TCL) || defined(MAC_OSX_TK))
+#if !(defined(__WIN32__) || defined(MAC_OSX_TK))
 static int
 TestmenubarCmd(clientData, interp, argc, argv)
     ClientData clientData;		/* Main window for application. */
@@ -2032,7 +2030,7 @@ TestmenubarCmd(clientData, interp, argc, argv)
  *----------------------------------------------------------------------
  */
 
-#if defined(__WIN32__) || defined(MAC_TCL) || defined(MAC_OSX_TK)
+#if defined(__WIN32__) || defined(MAC_OSX_TK)
 static int
 TestmetricsCmd(clientData, interp, argc, argv)
     ClientData clientData;		/* Main window for application. */
@@ -2184,7 +2182,7 @@ TestpropCmd(clientData, interp, argc, argv)
  */
 
 	/* ARGSUSED */
-#if !(defined(__WIN32__) || defined(MAC_TCL) || defined(MAC_OSX_TK))
+#if !(defined(__WIN32__) || defined(MAC_OSX_TK))
 static int
 TestsendCmd(clientData, interp, argc, argv)
     ClientData clientData;		/* Main window for application. */
@@ -2359,7 +2357,7 @@ TesttextCmd(clientData, interp, argc, argv)
     return TCL_OK;
 }
 
-#if !(defined(__WIN32__) || defined(MAC_TCL) || defined(MAC_OSX_TK))
+#if !(defined(__WIN32__) || defined(MAC_OSX_TK))
 /*
  *----------------------------------------------------------------------
  *
