@@ -7,8 +7,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkUndo.c,v 1.1.4.1 2004/07/14 19:07:48 hobbs Exp $
  */
 
 #include "tkUndo.h"
@@ -190,12 +188,15 @@ void TkUndoSetDepth ( stack, maxdepth )
     stack->maxdepth = maxdepth;
 
     if ((stack->maxdepth > 0) && (stack->depth > stack->maxdepth)) {
-        /* Maximum stack depth exceeded. We have to remove the last compound
-           elements on the stack */
+        /*
+	 * Maximum stack depth exceeded. We have to remove the last compound
+	 * elements on the stack.
+	 */
+
         elem = stack->undoStack;
         prevelem = NULL;
-        while ( sepNumber <= stack->maxdepth ) {
-            if (elem != NULL && (elem->type == TK_UNDO_SEPARATOR) ) {
+        while (elem && (sepNumber <= stack->maxdepth)) {
+            if (elem->type == TK_UNDO_SEPARATOR) {
                 sepNumber++;
             }
             prevelem = elem;
@@ -268,32 +269,9 @@ void TkUndoFreeStack ( stack )
 void TkUndoInsertUndoSeparator ( stack )
     TkUndoRedoStack * stack;
 {
-/*    TkUndoAtom * elem;
-    TkUndoAtom * prevelem;
-    int sepNumber = 0;
-*/
-    
     if ( TkUndoInsertSeparator(&(stack->undoStack)) ) {
         ++(stack->depth);
         TkUndoSetDepth(stack,stack->maxdepth);
-/*        if ((stack->maxdepth > 0) && (stack->depth > stack->maxdepth)) {
-            elem = stack->undoStack;
-            prevelem = NULL;
-            while ( sepNumber < stack->depth ) {
-                if (elem != NULL && (elem->type == TK_UNDO_SEPARATOR) ) {
-                    sepNumber++;
-                }
-                prevelem = elem;
-                elem = elem->next;
-            }
-            prevelem->next = NULL;
-            while ( elem ) {
-               prevelem = elem;
-               elem = elem->next;
-               ckfree((char *) elem);
-            }
-            stack->depth;
-        } */
     }
 }
 
