@@ -1,4 +1,4 @@
-/* 
+/*
  * tkWindow.c --
  *
  *	This file provides basic window-manipulation procedures,
@@ -11,20 +11,20 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkWindow.c,v 1.56.2.15 2008/04/07 23:12:10 hobbs Exp $
  */
 
 #include "tkPort.h"
 #include "tkInt.h"
 
-#if !( defined(__WIN32__) || defined(MAC_TCL) || defined(MAC_OSX_TK))
+#ifdef __WIN32__
+#include "tkWinInt.h"
+#elif !(defined(MAC_OSX_TK))
 #include "tkUnixInt.h"
 #endif
 
 #include "tclInt.h" /* for Tcl_CreateNamespace() */
 
-/* 
+/*
  * Type used to keep track of Window objects that were
  * only partically deallocated by Tk_DestroyWindow.
  */
@@ -52,15 +52,15 @@ typedef struct ThreadSpecificData {
                            /* First in list of partially deallocated
 			    * windows. */
     TkDisplay *displayList;
-                           /* List of all displays currently in use by 
+                           /* List of all displays currently in use by
 			    * the current thread. */
-    int initialized;       /* 0 means the structures above need 
+    int initialized;       /* 0 means the structures above need
 			    * initializing. */
 } ThreadSpecificData;
 static Tcl_ThreadDataKey dataKey;
 
-/* 
- * The Mutex below is used to lock access to the Tk_Uid structs above. 
+/*
+ * The Mutex below is used to lock access to the Tk_Uid structs above.
  */
 
 TCL_DECLARE_MUTEX(windowMutex)
@@ -71,14 +71,14 @@ TCL_DECLARE_MUTEX(windowMutex)
  * events on internal windows:  these events are generated internally.
  */
 
-static XWindowChanges defChanges = {
+static CONST XWindowChanges defChanges = {
     0, 0, 1, 1, 0, 0, Above
 };
 #define ALL_EVENTS_MASK \
     KeyPressMask|KeyReleaseMask|ButtonPressMask|ButtonReleaseMask| \
     EnterWindowMask|LeaveWindowMask|PointerMotionMask|ExposureMask| \
     VisibilityChangeMask|PropertyChangeMask|ColormapChangeMask
-static XSetWindowAttributes defAtts= {
+static CONST XSetWindowAttributes defAtts= {
     None,			/* background_pixmap */
     0,				/* background_pixel */
     CopyFromParent,		/* border_pixmap */
@@ -101,87 +101,85 @@ static XSetWindowAttributes defAtts= {
  * Tk, and the C procedures that execute them.
  */
 
+#define ISSAFE 1
+#define PASSMAINWINDOW 2
+#define NOOBJPROC 4
+#define WINMACONLY 8
+
 typedef struct {
-    char *name;			/* Name of command. */
-    Tcl_CmdProc *cmdProc;	/* Command's string-based procedure. */
-    Tcl_ObjCmdProc *objProc;	/* Command's object-based procedure. */
-    int isSafe;			/* If !0, this command will be exposed in
-                                 * a safe interpreter. Otherwise it will be
-                                 * hidden in a safe interpreter. */
-    int passMainWindow;		/* 0 means provide NULL clientData to
-				 * command procedure; 1 means pass main
-				 * window as clientData to command
-				 * procedure. */
+    CONST char *name;			/* Name of command. */
+    Tcl_ObjCmdProc *objProc;	/* Command's object- (or string-) based procedure. */
+    int flags;
 } TkCmd;
 
-static TkCmd commands[] = {
+static CONST TkCmd commands[] = {
     /*
      * Commands that are part of the intrinsics:
      */
 
-    {"bell",		NULL,			Tk_BellObjCmd,		0, 1},
-    {"bind",		NULL,			Tk_BindObjCmd,		1, 1},
-    {"bindtags",	NULL,			Tk_BindtagsObjCmd,	1, 1},
-    {"clipboard",	NULL,			Tk_ClipboardObjCmd,	0, 1},
-    {"destroy",		NULL,			Tk_DestroyObjCmd,	1, 1},
-    {"event",		NULL,			Tk_EventObjCmd,		1, 1},
-    {"focus",		NULL,			Tk_FocusObjCmd,		1, 1},
-    {"font",		NULL,			Tk_FontObjCmd,		1, 1},
-    {"grab",		NULL,			Tk_GrabObjCmd,		0, 1},
-    {"grid",		NULL,			Tk_GridObjCmd,		1, 1},
-    {"image",		NULL,			Tk_ImageObjCmd,		1, 1},
-    {"lower",		NULL,			Tk_LowerObjCmd,		1, 1},
-    {"option",		NULL,			Tk_OptionObjCmd,	1, 1},
-    {"pack",		NULL,			Tk_PackObjCmd,		1, 1},
-    {"place",		NULL,			Tk_PlaceObjCmd,		1, 0},
-    {"raise",		NULL,			Tk_RaiseObjCmd,		1, 1},
-    {"selection",	NULL,			Tk_SelectionObjCmd,	0, 1},
-    {"tk",		NULL,			Tk_TkObjCmd,		1, 1},
-    {"tkwait",		NULL,			Tk_TkwaitObjCmd,	1, 1},
-#if defined(__WIN32__) || defined(MAC_TCL) || defined(MAC_OSX_TK)
-    {"tk_chooseColor",  NULL,			Tk_ChooseColorObjCmd,	0, 1},
-    {"tk_chooseDirectory", NULL,		Tk_ChooseDirectoryObjCmd, 0, 1},
-    {"tk_getOpenFile",  NULL,			Tk_GetOpenFileObjCmd,	0, 1},
-    {"tk_getSaveFile",  NULL,			Tk_GetSaveFileObjCmd,	0, 1},
+    {"bell",		Tk_BellObjCmd,		PASSMAINWINDOW},
+    {"bind",		Tk_BindObjCmd,		PASSMAINWINDOW|ISSAFE},
+    {"bindtags",	Tk_BindtagsObjCmd,	PASSMAINWINDOW|ISSAFE},
+    {"clipboard",	Tk_ClipboardObjCmd,	PASSMAINWINDOW},
+    {"destroy",		Tk_DestroyObjCmd,	PASSMAINWINDOW|ISSAFE},
+    {"event",		Tk_EventObjCmd,		PASSMAINWINDOW|ISSAFE},
+    {"focus",		Tk_FocusObjCmd,		PASSMAINWINDOW|ISSAFE},
+    {"font",		Tk_FontObjCmd,		PASSMAINWINDOW|ISSAFE},
+    {"grab",		Tk_GrabObjCmd,		PASSMAINWINDOW},
+    {"grid",		Tk_GridObjCmd,		PASSMAINWINDOW|ISSAFE},
+    {"image",		Tk_ImageObjCmd,		PASSMAINWINDOW|ISSAFE},
+    {"lower",		Tk_LowerObjCmd,		PASSMAINWINDOW|ISSAFE},
+    {"option",		Tk_OptionObjCmd,	PASSMAINWINDOW|ISSAFE},
+    {"pack",		Tk_PackObjCmd,		PASSMAINWINDOW|ISSAFE},
+    {"place",		Tk_PlaceObjCmd,		ISSAFE},
+    {"raise",		Tk_RaiseObjCmd,		PASSMAINWINDOW|ISSAFE},
+    {"selection",	Tk_SelectionObjCmd,	PASSMAINWINDOW},
+    {"tk",		Tk_TkObjCmd,		PASSMAINWINDOW|ISSAFE},
+    {"tkwait",		Tk_TkwaitObjCmd,	PASSMAINWINDOW|ISSAFE},
+#if defined(__WIN32__) || defined(MAC_OSX_TK)
+    {"tk_chooseColor",  Tk_ChooseColorObjCmd,	PASSMAINWINDOW},
+    {"tk_chooseDirectory", Tk_ChooseDirectoryObjCmd, WINMACONLY|PASSMAINWINDOW},
+    {"tk_getOpenFile",  Tk_GetOpenFileObjCmd,	WINMACONLY|PASSMAINWINDOW},
+    {"tk_getSaveFile",  Tk_GetSaveFileObjCmd,	WINMACONLY|PASSMAINWINDOW},
 #endif
 #if defined(__WIN32__) || defined(MAC_OSX_TK)
-    {"tk_messageBox",   NULL,			Tk_MessageBoxObjCmd,	0, 1},
+    {"tk_messageBox",   Tk_MessageBoxObjCmd,	PASSMAINWINDOW},
 #endif
-    {"update",		NULL,			Tk_UpdateObjCmd,	1, 1},
-    {"winfo",		NULL,			Tk_WinfoObjCmd,		1, 1},
-    {"wm",		NULL,			Tk_WmObjCmd,		0, 1},
+    {"update",		Tk_UpdateObjCmd,	PASSMAINWINDOW|ISSAFE},
+    {"winfo",		Tk_WinfoObjCmd,		PASSMAINWINDOW|ISSAFE},
+    {"wm",		Tk_WmObjCmd,		PASSMAINWINDOW},
 
     /*
      * Widget class commands.
      */
 
-    {"button",		NULL,			Tk_ButtonObjCmd,	1, 0},
-    {"canvas",		NULL,			Tk_CanvasObjCmd,	1, 1},
-    {"checkbutton",	NULL,			Tk_CheckbuttonObjCmd,	1, 0},
-    {"entry",		NULL,                   Tk_EntryObjCmd,		1, 0},
-    {"frame",		NULL,			Tk_FrameObjCmd,		1, 0},
-    {"label",		NULL,			Tk_LabelObjCmd,		1, 0},
-    {"labelframe",	NULL,			Tk_LabelframeObjCmd,	1, 0},
-    {"listbox",		NULL,			Tk_ListboxObjCmd,	1, 0},
-    {"menubutton",	NULL,                   Tk_MenubuttonObjCmd,	1, 0},
-    {"message",		NULL,			Tk_MessageObjCmd,	1, 0},
-    {"panedwindow",	NULL,			Tk_PanedWindowObjCmd,	1, 0},
-    {"radiobutton",	NULL,			Tk_RadiobuttonObjCmd,	1, 0},
-    {"scale",		NULL,	                Tk_ScaleObjCmd,		1, 0},
-    {"scrollbar",	Tk_ScrollbarCmd,	NULL,			1, 1},
-    {"spinbox",		NULL,                   Tk_SpinboxObjCmd,	1, 0},
-    {"text",		Tk_TextCmd,		NULL,			1, 1},
-    {"toplevel",	NULL,			Tk_ToplevelObjCmd,	0, 0},
+    {"button",		Tk_ButtonObjCmd,	ISSAFE},
+    {"canvas",		Tk_CanvasObjCmd,	PASSMAINWINDOW|ISSAFE},
+    {"checkbutton",	Tk_CheckbuttonObjCmd,	ISSAFE},
+    {"entry",		Tk_EntryObjCmd,		ISSAFE},
+    {"frame",		Tk_FrameObjCmd,		ISSAFE},
+    {"label",		Tk_LabelObjCmd,		ISSAFE},
+    {"labelframe",	Tk_LabelframeObjCmd,	ISSAFE},
+    {"listbox",		Tk_ListboxObjCmd,	ISSAFE},
+    {"menubutton",	Tk_MenubuttonObjCmd,	ISSAFE},
+    {"message",		Tk_MessageObjCmd,	ISSAFE},
+    {"panedwindow",	Tk_PanedWindowObjCmd,	ISSAFE},
+    {"radiobutton",	Tk_RadiobuttonObjCmd,	ISSAFE},
+    {"scale",		Tk_ScaleObjCmd,		ISSAFE},
+    {"scrollbar",	(Tcl_ObjCmdProc *) Tk_ScrollbarCmd,	NOOBJPROC|PASSMAINWINDOW|ISSAFE},
+    {"spinbox",		Tk_SpinboxObjCmd,	ISSAFE},
+    {"text",		(Tcl_ObjCmdProc *) Tk_TextCmd,		NOOBJPROC|PASSMAINWINDOW|ISSAFE},
+    {"toplevel",	Tk_ToplevelObjCmd,	0},
 
     /*
      * Misc.
      */
 
-#if defined(MAC_TCL) || defined(MAC_OSX_TK)
+#if defined(MAC_OSX_TK)
     {"::tk::unsupported::MacWindowStyle",
-	    		NULL,	TkUnsupported1ObjCmd,			1, 1},
+	    		TkUnsupported1ObjCmd,			PASSMAINWINDOW|ISSAFE},
 #endif
-    {(char *) NULL,	(int (*) _ANSI_ARGS_((ClientData, Tcl_Interp *, int, CONST char **))) NULL, NULL, 0}
+    {NULL, NULL, 0}
 };
 
 /*
@@ -224,7 +222,7 @@ static Tk_ArgvInfo argTable[] = {
  */
 
 static Tk_Window	CreateTopLevelWindow _ANSI_ARGS_((Tcl_Interp *interp,
-			    Tk_Window parent, CONST char *name, 
+			    Tk_Window parent, CONST char *name,
 			    CONST char *screenName, unsigned int flags));
 static void		DeleteWindowsExitProc _ANSI_ARGS_((
 			    ClientData clientData));
@@ -337,7 +335,7 @@ CreateTopLevelWindow(interp, parent, name, screenName, flags)
     register TkWindow *winPtr;
     register TkDisplay *dispPtr;
     int screenId;
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
             Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     if (!tsdPtr->initialized) {
@@ -384,9 +382,9 @@ CreateTopLevelWindow(interp, parent, name, screenName, flags)
      * Set the flags specified in the call.
      */
     winPtr->flags |= flags;
-    
+
     /*
-     * Force the window to use a border pixel instead of border pixmap. 
+     * Force the window to use a border pixel instead of border pixmap.
      * This is needed for the case where the window doesn't use the
      * default visual.  In this case, the default border is a pixmap
      * inherited from the root window, which won't work because it will
@@ -449,7 +447,7 @@ GetScreen(interp, screenName, screenPtr)
     CONST char *p;
     int screenId;
     size_t length;
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
             Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     /*
@@ -525,7 +523,7 @@ GetScreen(interp, screenName, screenPtr)
     }
     if (screenId >= ScreenCount(dispPtr->display)) {
 	char buf[32 + TCL_INTEGER_SPACE];
-	
+
 	sprintf(buf, "bad screen number \"%d\"", screenId);
 	Tcl_SetResult(interp, buf, TCL_VOLATILE);
 	return (TkDisplay *) NULL;
@@ -539,7 +537,7 @@ GetScreen(interp, screenName, screenPtr)
  *
  * TkGetDisplay --
  *
- *	Given an X display, TkGetDisplay returns the TkDisplay 
+ *	Given an X display, TkGetDisplay returns the TkDisplay
  *      structure for the display.
  *
  * Results:
@@ -557,7 +555,7 @@ TkGetDisplay(display)
      Display *display;          /* X's display pointer */
 {
     TkDisplay *dispPtr;
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
             Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     for (dispPtr = tsdPtr->displayList; dispPtr != NULL;
@@ -589,9 +587,9 @@ TkGetDisplay(display)
 TkDisplay *
 TkGetDisplayList()
 {
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
             Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
-    
+
     return tsdPtr->displayList;
 }
 
@@ -616,9 +614,9 @@ TkGetDisplayList()
 TkMainInfo *
 TkGetMainInfoList()
 {
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
             Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
-    
+
     return tsdPtr->mainWindowList;
 }
 /*
@@ -859,11 +857,11 @@ TkCreateMainWindow(interp, screenName, baseName)
     Tcl_HashEntry *hPtr;
     register TkMainInfo *mainPtr;
     register TkWindow *winPtr;
-    register TkCmd *cmdPtr;
+    register CONST TkCmd *cmdPtr;
     ClientData clientData;
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
             Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
-    
+
     /*
      * Panic if someone updated the TkWindow structure without
      * also updating the Tk_FakeWin structure (or vice versa).
@@ -882,7 +880,7 @@ TkCreateMainWindow(interp, screenName, baseName)
     if (tkwin == NULL) {
 	return NULL;
     }
-    
+
     /*
      * Create the TkMainInfo structure for this application, and set
      * up name-related information for the new window.
@@ -943,26 +941,32 @@ TkCreateMainWindow(interp, screenName, baseName)
 
     isSafe = Tcl_IsSafe(interp);
     for (cmdPtr = commands; cmdPtr->name != NULL; cmdPtr++) {
-	if ((cmdPtr->cmdProc == NULL) && (cmdPtr->objProc == NULL)) {
-	    panic("TkCreateMainWindow: builtin command with NULL string and object procs");
+	if (cmdPtr->objProc == NULL) {
+	    Tcl_Panic("TkCreateMainWindow: builtin command with NULL string and object procs");
 	}
-	if (cmdPtr->passMainWindow) {
+#if defined(__WIN32__) && !defined(STATIC_BUILD)
+	if ((cmdPtr->flags & WINMACONLY) && tclStubsPtr->reserved9) {
+	    /* We are running on Cygwin, so don't use the win32 dialogs */
+	    continue;
+	}
+#endif
+	if (cmdPtr->flags & PASSMAINWINDOW) {
 	    clientData = (ClientData) tkwin;
 	} else {
 	    clientData = (ClientData) NULL;
 	}
-	if (cmdPtr->cmdProc != NULL) {
-	    Tcl_CreateCommand(interp, cmdPtr->name, cmdPtr->cmdProc,
-		    clientData, (void (*) _ANSI_ARGS_((ClientData))) NULL);
+	if (cmdPtr->flags & NOOBJPROC) {
+	    Tcl_CreateCommand(interp, cmdPtr->name,
+		    (Tcl_CmdProc *) cmdPtr->objProc, clientData, NULL);
 	} else {
 	    Tcl_CreateObjCommand(interp, cmdPtr->name, cmdPtr->objProc,
 		    clientData, NULL);
 	}
-        if (isSafe) {
-            if (!(cmdPtr->isSafe)) {
-                Tcl_HideCommand(interp, cmdPtr->name, cmdPtr->name);
-            }
-        }
+	if (isSafe) {
+	    if (!(cmdPtr->flags & ISSAFE)) {
+		Tcl_HideCommand(interp, cmdPtr->name, cmdPtr->name);
+	    }
+	}
     }
 
     TkCreateMenuCmd(interp);
@@ -1204,11 +1208,11 @@ Tk_CreateWindowFromPath(interp, tkwin, pathName, screenName)
 	return NULL;
     }
     if (((TkWindow *) parent)->flags & TK_ALREADY_DEAD) {
-	Tcl_AppendResult(interp, 
+	Tcl_AppendResult(interp,
 	    "can't create window: parent has been destroyed", (char *) NULL);
 	return NULL;
     } else if (((TkWindow *) parent)->flags & TK_CONTAINER) {
-	Tcl_AppendResult(interp, 
+	Tcl_AppendResult(interp,
 	    "can't create window: its parent has -container = yes",
 		(char *) NULL);
 	return NULL;
@@ -1265,7 +1269,7 @@ Tk_DestroyWindow(tkwin)
     TkDisplay *dispPtr = winPtr->dispPtr;
     XEvent event;
     TkHalfdeadWindow *halfdeadPtr, *prev_halfdeadPtr;
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
             Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     if (winPtr->flags & TK_ALREADY_DEAD) {
@@ -1457,7 +1461,7 @@ Tk_DestroyWindow(tkwin)
 	TkWmRemoveFromColormapWindows(winPtr);
     }
     if (winPtr->window != None) {
-#if defined(MAC_TCL) || defined(MAC_OSX_TK) || defined(__WIN32__)
+#if defined(MAC_OSX_TK) || defined(__WIN32__)
 	XDestroyWindow(winPtr->display, winPtr->window);
 #else
 	if ((winPtr->flags & TK_TOP_HIERARCHY)
@@ -1516,7 +1520,7 @@ Tk_DestroyWindow(tkwin)
 	}
 	winPtr->mainPtr->refCount--;
 	if (winPtr->mainPtr->refCount == 0) {
-	    register TkCmd *cmdPtr;
+	    register CONST TkCmd *cmdPtr;
 
 	    /*
 	     * We just deleted the last window in the application.  Delete
@@ -1537,12 +1541,12 @@ Tk_DestroyWindow(tkwin)
                             (void (*) _ANSI_ARGS_((ClientData))) NULL);
                 }
                 Tcl_CreateCommand(winPtr->mainPtr->interp, "send",
-                        TkDeadAppCmd, (ClientData) NULL, 
+                        TkDeadAppCmd, (ClientData) NULL,
                         (void (*) _ANSI_ARGS_((ClientData))) NULL);
                 Tcl_UnlinkVar(winPtr->mainPtr->interp, "tk_strictMotif");
                 Tcl_UnlinkVar(winPtr->mainPtr->interp, "::tk::AlwaysShowSelection");
             }
-                
+
 	    Tcl_DeleteHashTable(&winPtr->mainPtr->nameTable);
 	    TkBindFree(winPtr->mainPtr);
 	    TkDeleteAllImages(winPtr->mainPtr);
@@ -1551,14 +1555,14 @@ Tk_DestroyWindow(tkwin)
 	    TkStylePkgFree(winPtr->mainPtr);
 
             /*
-             * When embedding Tk into other applications, make sure 
+             * When embedding Tk into other applications, make sure
              * that all destroy events reach the server. Otherwise
              * the embedding application may also attempt to destroy
              * the windows, resulting in an X error
              */
 
             if (winPtr->flags & TK_EMBEDDED) {
-                XSync(winPtr->display, False); 
+                XSync(winPtr->display, False);
             }
 	    ckfree((char *) winPtr->mainPtr);
 
@@ -1567,7 +1571,7 @@ Tk_DestroyWindow(tkwin)
              * display now and relinquish its data structures.
              */
 
-#if !defined(WIN32) && !defined(MAC_TCL) && defined(NOT_YET)
+#if !defined(WIN32) && defined(NOT_YET)
             if (dispPtr->refCount <= 0) {
                 /*
                  * I have disabled this code because on Windows there are
@@ -1582,13 +1586,13 @@ Tk_DestroyWindow(tkwin)
 		 * addressed before this can be enabled.  The current cleanup
 		 * works except for send event issues. -- hobbs 04/2002
 		 */
-                
+
                 TkDisplay *theDispPtr, *backDispPtr;
-                
+
                 /*
                  * Splice this display out of the list of displays.
                  */
-                
+
                 for (theDispPtr = tsdPtr->displayList, backDispPtr = NULL;
                          (theDispPtr != winPtr->dispPtr) &&
                              (theDispPtr != NULL);
@@ -1653,6 +1657,13 @@ Tk_MapWindow(tkwin)
     if (winPtr->window == None) {
 	Tk_MakeWindowExist(tkwin);
     }
+    /*
+     * [Bug 2645457]: the previous call permits events to be processed and can
+     * lead to the destruction of the window under some conditions.
+     */
+    if (winPtr->flags & TK_ALREADY_DEAD) {
+	return;
+    }
     if (winPtr->flags & TK_WIN_MANAGED) {
 	/*
 	 * Lots of special processing has to be done for top-level
@@ -2102,12 +2113,12 @@ Tk_DefineCursor(tkwin, cursor)
 {
     register TkWindow *winPtr = (TkWindow *) tkwin;
 
-#if defined(MAC_TCL) || defined(MAC_OSX_TK)
+#if defined(MAC_OSX_TK)
     winPtr->atts.cursor = (XCursor) cursor;
 #else
     winPtr->atts.cursor = (Cursor) cursor;
 #endif
-    
+
     if (winPtr->window != None) {
 	XDefineCursor(winPtr->display, winPtr->window, winPtr->atts.cursor);
     } else {
@@ -2335,7 +2346,7 @@ Tk_NameToWindow(interp, pathName, tkwin)
 	Tcl_AppendResult(interp, "NULL main window", (char *)NULL);
 	return NULL;
     }
-    
+
     hPtr = Tcl_FindHashEntry(&((TkWindow *) tkwin)->mainPtr->nameTable,
 	    pathName);
     if (hPtr == NULL) {
@@ -2624,7 +2635,7 @@ Tk_MainWindow(interp)
 	return NULL;
     }
 #endif
-    tsdPtr = (ThreadSpecificData *) 
+    tsdPtr = (ThreadSpecificData *)
 	Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     for (mainPtr = tsdPtr->mainWindowList; mainPtr != NULL;
@@ -2693,7 +2704,7 @@ Tk_GetNumMainWindows()
     }
 #endif
 
-    tsdPtr = (ThreadSpecificData *) 
+    tsdPtr = (ThreadSpecificData *)
 	Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     return tsdPtr->numMainWindows;
@@ -2778,7 +2789,7 @@ DeleteWindowsExitProc(clientData)
      * Destroy any remaining main windows.
      */
 
-    while (tsdPtr->mainWindowList != NULL) {        
+    while (tsdPtr->mainWindowList != NULL) {
         interp = tsdPtr->mainWindowList->interp;
         Tcl_Preserve((ClientData) interp);
         Tk_DestroyWindow((Tk_Window) tsdPtr->mainWindowList->winPtr);
@@ -2804,7 +2815,7 @@ DeleteWindowsExitProc(clientData)
          * if it needs to dispatch a message.
          */
 
-        for (tsdPtr->displayList = NULL; dispPtr != NULL; 
+        for (tsdPtr->displayList = NULL; dispPtr != NULL;
                 dispPtr = nextPtr) {
             nextPtr = dispPtr->nextPtr;
             TkCloseDisplay(dispPtr);
@@ -2816,6 +2827,51 @@ DeleteWindowsExitProc(clientData)
     tsdPtr->initialized = 0;
 }
 
+#if defined(__WIN32__) && !defined(__WIN64__)
+
+static HMODULE tkcygwindll = NULL;
+
+/*
+ * Run Tk_MainEx from libtk8.?.dll
+ *
+ * This function is only ever called from wish8.4.exe, the cygwin
+ * port of Tcl. This means that the system encoding is utf-8,
+ * so we don't have to do any encoding conversions.
+ */
+int
+TkCygwinMainEx(argc, argv, appInitProc, interp)
+    int argc;				/* Number of arguments. */
+    char **argv;			/* Array of argument strings. */
+    Tcl_AppInitProc *appInitProc;	/* Application-specific initialization
+					 * procedure to call after most
+					 * initialization but before starting
+					 * to execute commands. */
+    Tcl_Interp *interp;
+{
+    char name[MAX_PATH];
+    int len;
+    void (*sym)(int, char **, Tcl_AppInitProc *, Tcl_Interp *);
+
+    /* construct "<path>/libtk8.?.dll", from "<path>/tk8?.dll" */
+	len = GetModuleFileName(Tk_GetHINSTANCE(), name, MAX_PATH);
+	name[len-2] = '.';
+	name[len-1] = name[len-5];
+	strcpy(name+len, ".dll");
+	memcpy(name+len-8, "libtk8", 6);
+
+	tkcygwindll = LoadLibrary(name);
+	if (!tkcygwindll) {
+	    /* dll is not present */
+	    return 0;
+	}
+	sym = (void (*)(int, char **, Tcl_AppInitProc *, Tcl_Interp *)) GetProcAddress(tkcygwindll, "Tk_MainEx");
+	if (!sym) {
+		return 0;
+	}
+	sym(argc, argv, appInitProc, interp);
+    return 1;
+}
+#endif
 /*
  *----------------------------------------------------------------------
  *
@@ -2844,6 +2900,16 @@ int
 Tk_Init(interp)
     Tcl_Interp *interp;		/* Interpreter to initialize. */
 {
+#if defined(__WIN32__) && !defined(__WIN64__)
+    if (tkcygwindll) {
+	int (*sym)(Tcl_Interp *);
+
+	sym = (int (*)(Tcl_Interp *)) GetProcAddress(tkcygwindll, "Tk_Init");
+	if (sym) {
+	    return sym(interp);
+	}
+    }
+#endif
     return Initialize(interp);
 }
 
@@ -2906,6 +2972,16 @@ Tk_SafeInit(interp)
      *  is checked at several places to differentiate the two initialisations.
      */
 
+#if defined(__WIN32__) && !defined(__WIN64__)
+    if (tkcygwindll) {
+	int (*sym)(Tcl_Interp *);
+
+	sym = (int (*)(Tcl_Interp *)) GetProcAddress(tkcygwindll, "Tk_SafeInit");
+	if (sym) {
+	    return sym(interp);
+	}
+    }
+#endif
     return Initialize(interp);
 }
 
@@ -2934,15 +3010,14 @@ Initialize(interp)
 {
     char *p;
     int argc, code;
-    CONST char **argv; 
+    CONST char **argv;
     char *args[20];
     CONST char *argString = NULL;
     Tcl_DString class;
     ThreadSpecificData *tsdPtr;
-    
+
     /*
-     * Ensure that we are getting the matching version of Tcl.  This is
-     * really only an issue when Tk is loaded dynamically.
+     * Ensure that we are getting the matching version of Tcl.
      */
 
     if (Tcl_InitStubs(interp, TCL_VERSION, 1) == NULL) {
@@ -2954,7 +3029,7 @@ Initialize(interp)
      */
     TkRegisterObjTypes();
 
-    tsdPtr = (ThreadSpecificData *) 
+    tsdPtr = (ThreadSpecificData *)
 	Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     /*
@@ -3032,13 +3107,13 @@ Initialize(interp)
 	     * We don't. (no API to do it and maybe security reasons).
 	     */
 	    Tcl_DStringFree(&ds);
-	    Tcl_AppendResult(interp, 
+	    Tcl_AppendResult(interp,
 		    "not allowed to start Tk by master's safe::TkInit",
 		    (char *) NULL);
 	    goto done;
 	}
 	Tcl_DStringFree(&ds);
-	/* 
+	/*
 	 * Use the master's result as argv.
 	 * Note: We don't use the Obj interfaces to avoid dealing with
 	 * cross interp refcounting and changing the code below.
@@ -3166,11 +3241,6 @@ Initialize(interp)
         geometry = NULL;
     }
 
-    if (Tcl_PkgRequire(interp, "Tcl", TCL_VERSION, 1) == NULL) {
-	code = TCL_ERROR;
-	goto done;
-    }
-
     /*
      * Provide Tk and its stub table.
      */
@@ -3190,9 +3260,7 @@ Initialize(interp)
 	Tcl_SetMainLoop(Tk_MainLoop);
     }
 
-#ifdef Tk_InitStubs
 #undef Tk_InitStubs
-#endif
 
     Tk_InitStubs(interp, TK_VERSION, 1);
 
