@@ -7,10 +7,7 @@
 #	Donal K. Fellows.
 #
 # Copyright (c) 1998-2000 by Ajuba Solutions.
-# All rights reserved.
-# 
-# RCS: @(#) $Id: bgerror.tcl,v 1.23.2.8 2007/05/30 06:37:03 das Exp $
-# $Id: bgerror.tcl,v 1.23.2.8 2007/05/30 06:37:03 das Exp $
+# Copyright (c) 2007 Daniel A. Steffen <das@users.sourceforge.net>
 
 namespace eval ::tk::dialog::error {
     namespace import -force ::tk::msgcat::*
@@ -21,6 +18,7 @@ namespace eval ::tk::dialog::error {
     if {[tk windowingsystem] eq "aqua"} {
 	option add *ErrorDialog*background systemAlertBackgroundActive \
 		widgetDefault
+	option add *ErrorDialog*info.text.background white widgetDefault
 	option add *ErrorDialog*Button.highlightBackground \
 		systemAlertBackgroundActive widgetDefault
     }
@@ -96,8 +94,7 @@ proc ::tk::dialog::error::bgerror err {
     # we use the default dialog then :
     set windowingsystem [tk windowingsystem]
 
-    if {($tcl_platform(platform) eq "macintosh")
-             || ($windowingsystem eq "aqua")} {
+    if {$windowingsystem eq "aqua"} {
 	set ok		[mc Ok]
 	set messageFont	system
 	set textRelief	flat
@@ -145,9 +142,10 @@ proc ::tk::dialog::error::bgerror err {
     wm iconname .bgerrorDialog ErrorDialog
     wm protocol .bgerrorDialog WM_DELETE_WINDOW { }
 
-    if {($tcl_platform(platform) eq "macintosh")
-            || ($windowingsystem eq "aqua")} {
+    if {$windowingsystem eq "aqua"} {
 	::tk::unsupported::MacWindowStyle style .bgerrorDialog moveableAlert {}
+    } elseif {$windowingsystem eq "x11"} {
+	wm attributes .bgerrorDialog -type dialog
     }
 
     frame .bgerrorDialog.bot
@@ -190,17 +188,10 @@ proc ::tk::dialog::error::bgerror err {
     set wrapwidth [expr {$wrapwidth-60-[winfo pixels .bgerrorDialog 9m]}]
     label .bgerrorDialog.msg -justify left -text $text -font $messageFont \
 	    -wraplength $wrapwidth
-    if {($tcl_platform(platform) eq "macintosh")
-            || ($windowingsystem eq "aqua")} {
-	# On the Macintosh, use the stop bitmap
-	label .bgerrorDialog.bitmap -bitmap stop
-    } else {
-	# On other platforms, make the error icon
 	canvas .bgerrorDialog.bitmap -width 32 -height 32 -highlightthickness 0
 	.bgerrorDialog.bitmap create oval 0 0 31 31 -fill red -outline black
 	.bgerrorDialog.bitmap create line 9 9 23 23 -fill white -width 4
 	.bgerrorDialog.bitmap create line 9 23 23 9 -fill white -width 4
-    }
     grid .bgerrorDialog.bitmap .bgerrorDialog.msg \
 	    -in .bgerrorDialog.top	\
 	    -row 0			\
@@ -226,8 +217,7 @@ proc ::tk::dialog::error::bgerror err {
 		-padx 10
 	grid columnconfigure .bgerrorDialog.bot $i -weight 1
 	# We boost the size of some Mac buttons for l&f
-	if {($tcl_platform(platform) eq "macintosh")
-	    || ($windowingsystem eq "aqua")} {
+	if {$windowingsystem eq "aqua"} {
 	    if {($name eq "ok") || ($name eq "dismiss")} {
 		grid columnconfigure .bgerrorDialog.bot $i -minsize 90
 	    }
@@ -242,15 +232,16 @@ proc ::tk::dialog::error::bgerror err {
     bind .bgerrorDialog <Destroy>	[namespace code [list Destroy %W]]
     .bgerrorDialog.function configure -command [namespace code Details]
 
-    # 6. Update all the geometry information so we know how big it wants
-    # to be, then center the window in the display and deiconify it.
+    # 6. Withdraw the window, then update all the geometry information
+    # so we know how big it wants to be, then center the window in the
+    # display (Motif style) and de-iconify it.
 
     ::tk::PlaceWindow .bgerrorDialog
 
     # 7. Ensure that we are topmost.
 
     raise .bgerrorDialog
-    if {$tcl_platform(platform) eq "windows"} {
+    if {[tk windowingsystem] eq "win32"} {
 	# Place it topmost if we aren't at the top of the stacking
 	# order to ensure that it's seen
 	if {[lindex [wm stackorder .] end] ne ".bgerrorDialog"} {
