@@ -4,17 +4,15 @@
 # can be used by non-unix systems that do not have built-in support
 # for shells.
 #
-# RCS: @(#) $Id: console.tcl,v 1.8.2.6 2002/06/22 08:52:24 hobbs Exp $
-#
-# Copyright (c) 1998-1999 Scriptics Corp.
 # Copyright (c) 1995-1997 Sun Microsystems, Inc.
+# Copyright (c) 1998-2000 Ajuba Solutions.
+# Copyright (c) 2007 Daniel A. Steffen <das@users.sourceforge.net>
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 #
 
 # TODO: history - remember partially written command
-package require msgcat
 
 namespace eval ::tk::console {
     variable blinkTime   500 ; # msecs to blink braced range for
@@ -26,6 +24,7 @@ namespace eval ::tk::console {
     variable inPlugin [info exists embed_args]
     variable defaultPrompt  ; # default prompt if tcl_prompt1 isn't used
 
+
     if {$inPlugin} {
 	set defaultPrompt {subst {[history nextid] % }}
     } else {
@@ -36,61 +35,63 @@ namespace eval ::tk::console {
 # simple compat function for tkcon code added for this console
 interp alias {} EvalAttached {} consoleinterp eval
 
-# tkConsoleInit --
+# ::tk::ConsoleInit --
 # This procedure constructs and configures the console windows.
 #
 # Arguments:
 # 	None.
 
-proc tkConsoleInit {} {
+proc ::tk::ConsoleInit {} {
     global tcl_platform
 
     if {![consoleinterp eval {set tcl_interactive}]} {
 	wm withdraw .
     }
 
-    if {[string compare $tcl_platform(platform) "macintosh"]} {
-	set mod "Ctrl"
-    } else {
+    if {[tk windowingsystem] eq "aqua"} {
 	set mod "Cmd"
+    } else {
+	set mod "Ctrl"
     }
 
-    menu .menubar
+    if {[catch {menu .menubar} err]} { bgerror "INIT: $err" }
     .menubar add cascade -label File -menu .menubar.file -underline 0
     .menubar add cascade -label Edit -menu .menubar.edit -underline 0
 
     menu .menubar.file -tearoff 0
-    .menubar.file add command -label "Source..." -underline 0 \
-	    -command tkConsoleSource
-    .menubar.file add command -label "Hide Console" -underline 0 \
-	    -command {wm withdraw .}
-    .menubar.file add command -label [::msgcat::mc "Clear Console"] \
+    .menubar.file add command -label [mc "Source..."] \
+	    -underline 0 -command tk::ConsoleSource
+    .menubar.file add command -label [mc "Hide Console"] \
+	    -underline 0 -command {wm withdraw .}
+    .menubar.file add command -label [mc "Clear Console"] \
 	    -underline 0 -command {.console delete 1.0 "promptEnd linestart"}
-    if {[string compare $tcl_platform(platform) "macintosh"]} {
-	.menubar.file add command -label "Exit" -underline 1 -command exit
+   if {[tk windowingsystem] eq "aqua"} {
+	.menubar.file add command -label [mc "Quit"] \
+		-command exit -accel Cmd-Q
     } else {
-	.menubar.file add command -label "Quit" -command exit -accel Cmd-Q
+	.menubar.file add command -label [mc "Exit"] \
+		-underline 1 -command exit
     }
 
     menu .menubar.edit -tearoff 0
-    .menubar.edit add command -label "Cut" -underline 2 \
+    .menubar.edit add command -label [mc "Cut"] -underline 2 \
 	    -command { event generate .console <<Cut>> } -accel "$mod+X"
-    .menubar.edit add command -label "Copy" -underline 0 \
+    .menubar.edit add command -label [mc "Copy"] -underline 0 \
 	    -command { event generate .console <<Copy>> } -accel "$mod+C"
-    .menubar.edit add command -label "Paste" -underline 1 \
+    .menubar.edit add command -label [mc "Paste"] -underline 1 \
 	    -command { event generate .console <<Paste>> } -accel "$mod+V"
 
-    if {[string compare $tcl_platform(platform) "windows"]} {
-	.menubar.edit add command -label "Clear" -underline 2 \
+    if {$tcl_platform(platform) ne "windows"} {
+	.menubar.edit add command -label [mc "Clear"] -underline 2 \
 		-command { event generate .console <<Clear>> }
     } else {
-	.menubar.edit add command -label "Delete" -underline 0 \
+	.menubar.edit add command -label [mc "Delete"] -underline 0 \
 		-command { event generate .console <<Clear>> } -accel "Del"
-
+	
 	.menubar add cascade -label Help -menu .menubar.help -underline 0
 	menu .menubar.help -tearoff 0
-	.menubar.help add command -label "About..." -underline 0 \
-		-command tkConsoleAbout
+	.menubar.help add command -label [mc "About..."] \
+		-underline 0 -command tk::ConsoleAbout
     }
 
     . configure -menu .menubar
@@ -100,15 +101,17 @@ proc tkConsoleInit {} {
     pack .sb -side right -fill both
     pack $con -fill both -expand 1 -side left
     switch -exact $tcl_platform(platform) {
-	"macintosh" {
-	    $con configure -font {Monaco 9 normal} -highlightthickness 0
-	}
 	"windows" {
 	    $con configure -font systemfixed
 	}
+	"unix" {
+	    if {[tk windowingsystem] eq "aqua"} {
+		$con configure -font {Monaco 10 normal} -highlightthickness 0
+	    }
+	}
     }
 
-    tkConsoleBind $con
+    ConsoleBind $con
 
     $con tag configure stderr	-foreground red
     $con tag configure stdin	-foreground blue
@@ -120,36 +123,58 @@ proc tkConsoleInit {} {
     $con tag configure find	-background \#FFFF00
 
     focus $con
-    
+
     wm protocol . WM_DELETE_WINDOW { wm withdraw . }
-    wm title . "Console"
+    wm title . [mc "Console"]
     flush stdout
     $con mark set output [$con index "end - 1 char"]
-    tkTextSetCursor $con end
+    tk::TextSetCursor $con end
     $con mark set promptEnd insert
     $con mark gravity promptEnd left
+
+    # A variant of ConsolePrompt to avoid a 'puts' call
+    set w $con
+    set temp [$w index "end - 1 char"]
+    $w mark set output end
+    if {![consoleinterp eval "info exists tcl_prompt1"]} {
+	set string [EvalAttached $::tk::console::defaultPrompt]
+	$w insert output $string stdout
+    }
+    $w mark set output $temp
+    ::tk::TextSetCursor $w end
+    $w mark set promptEnd insert
+    $w mark gravity promptEnd left
+
+    if {$tcl_platform(platform) eq "windows"} {
+	# Subtle work-around to erase the '% ' that tclMain.c prints out
+	after idle [subst -nocommand {
+	    if {[$con get 1.0 output] eq "% "} { $con delete 1.0 output }
+	}]
+    }
 }
 
-# tkConsoleSource --
+# ::tk::ConsoleSource --
 #
 # Prompts the user for a file to source in the main interpreter.
 #
 # Arguments:
 # None.
 
-proc tkConsoleSource {} {
+proc ::tk::ConsoleSource {} {
     set filename [tk_getOpenFile -defaultextension .tcl -parent . \
-	    -title "Select a file to source" \
-	    -filetypes {{"Tcl Scripts" .tcl} {"All Files" *}}]
-    if {[string compare $filename ""]} {
+	    -title [mc "Select a file to source"] \
+	    -filetypes [list \
+	    [list [mc "Tcl Scripts"] .tcl] \
+	    [list [mc "All Files"] *]]]
+    if {$filename ne ""} {
     	set cmd [list source $filename]
 	if {[catch {consoleinterp eval $cmd} result]} {
-	    tkConsoleOutput stderr "$result\n"
+	    ConsoleOutput stderr "$result\n"
 	}
     }
 }
 
-# tkConsoleInvoke --
+# ::tk::ConsoleInvoke --
 # Processes the command line input.  If the command is complete it
 # is evaled in the main interpreter.  Otherwise, the continuation
 # prompt is added and more input may be added.
@@ -157,86 +182,86 @@ proc tkConsoleSource {} {
 # Arguments:
 # None.
 
-proc tkConsoleInvoke {args} {
+proc ::tk::ConsoleInvoke {args} {
     set ranges [.console tag ranges input]
     set cmd ""
     if {[llength $ranges]} {
 	set pos 0
-	while {[string compare [lindex $ranges $pos] ""]} {
+	while {[lindex $ranges $pos] ne ""} {
 	    set start [lindex $ranges $pos]
 	    set end [lindex $ranges [incr pos]]
 	    append cmd [.console get $start $end]
 	    incr pos
 	}
     }
-    if {[string equal $cmd ""]} {
-	tkConsolePrompt
+    if {$cmd eq ""} {
+	ConsolePrompt
     } elseif {[info complete $cmd]} {
 	.console mark set output end
 	.console tag delete input
 	set result [consoleinterp record $cmd]
-	if {[string compare $result ""]} {
+	if {$result ne ""} {
 	    puts $result
 	}
-	tkConsoleHistory reset
-	tkConsolePrompt
+	ConsoleHistory reset
+	ConsolePrompt
     } else {
-	tkConsolePrompt partial
+	ConsolePrompt partial
     }
     .console yview -pickplace insert
 }
 
-# tkConsoleHistory --
+# ::tk::ConsoleHistory --
 # This procedure implements command line history for the
 # console.  In general is evals the history command in the
-# main interpreter to obtain the history.  The global variable
-# histNum is used to store the current location in the history.
+# main interpreter to obtain the history.  The variable
+# ::tk::HistNum is used to store the current location in the history.
 #
 # Arguments:
 # cmd -	Which action to take: prev, next, reset.
 
-set histNum 1
-proc tkConsoleHistory {cmd} {
-    global histNum
-    
+set ::tk::HistNum 1
+proc ::tk::ConsoleHistory {cmd} {
+    variable HistNum
+
     switch $cmd {
     	prev {
-	    incr histNum -1
-	    if {$histNum == 0} {
+	    incr HistNum -1
+	    if {$HistNum == 0} {
 		set cmd {history event [expr {[history nextid] -1}]}
 	    } else {
-		set cmd "history event $histNum"
+		set cmd "history event $HistNum"
 	    }
     	    if {[catch {consoleinterp eval $cmd} cmd]} {
-    	    	incr histNum
+    	    	incr HistNum
     	    	return
     	    }
 	    .console delete promptEnd end
     	    .console insert promptEnd $cmd {input stdin}
     	}
     	next {
-	    incr histNum
-	    if {$histNum == 0} {
+	    incr HistNum
+	    if {$HistNum == 0} {
 		set cmd {history event [expr {[history nextid] -1}]}
-	    } elseif {$histNum > 0} {
+	    } elseif {$HistNum > 0} {
 		set cmd ""
-		set histNum 1
+		set HistNum 1
 	    } else {
-		set cmd "history event $histNum"
+		set cmd "history event $HistNum"
 	    }
-	    if {[string compare $cmd ""]} {
+	    if {$cmd ne ""} {
 		catch {consoleinterp eval $cmd} cmd
 	    }
 	    .console delete promptEnd end
 	    .console insert promptEnd $cmd {input stdin}
     	}
     	reset {
-    	    set histNum 1
+    	    set HistNum 1
     	}
     }
 }
 
-# tkConsolePrompt --
+# ::tk::ConsolePrompt --
 # This procedure draws the prompt.  If tcl_prompt1 or tcl_prompt2
 # exists in the main interpreter it will be called to generate the 
 # prompt.  Otherwise, a hard coded default prompt is printed.
@@ -244,9 +269,9 @@ proc tkConsoleHistory {cmd} {
 # Arguments:
 # partial -	Flag to specify which prompt to print.
 
-proc tkConsolePrompt {{partial normal}} {
+proc ::tk::ConsolePrompt {{partial normal}} {
     set w .console
-    if {[string equal $partial "normal"]} {
+    if {$partial eq "normal"} {
 	set temp [$w index "end - 1 char"]
 	$w mark set output end
     	if {[consoleinterp eval "info exists tcl_prompt1"]} {
@@ -265,14 +290,14 @@ proc tkConsolePrompt {{partial normal}} {
     }
     flush stdout
     $w mark set output $temp
-    tkTextSetCursor $w end
+    ::tk::TextSetCursor $w end
     $w mark set promptEnd insert
     $w mark gravity promptEnd left
     ::tk::console::ConstrainBuffer $w $::tk::console::maxLines
     $w see end
 }
 
-# tkConsoleBind --
+# ::tk::ConsoleBind --
 # This procedure first ensures that the default bindings for the Text
 # class have been defined.  Then certain bindings are overridden for
 # the class.
@@ -280,7 +305,7 @@ proc tkConsolePrompt {{partial normal}} {
 # Arguments:
 # None.
 
-proc tkConsoleBind {w} {
+proc ::tk::ConsoleBind {w} {
     bindtags $w [list $w Console PostConsole [winfo toplevel $w] all]
 
     ## Get all Text bindings into Console
@@ -343,13 +368,12 @@ proc tkConsoleBind {w} {
     }
     bind Console <<Console_Eval>> {
 	%W mark set insert {end - 1c}
-	tkConsoleInsert %W "\n"
-	tkConsoleInvoke
+	tk::ConsoleInsert %W "\n"
+	tk::ConsoleInvoke
 	break
     }
     bind Console <Delete> {
-	if {[string compare {} [%W tag nextrange sel 1.0 end]] \
-		&& [%W compare sel.first >= promptEnd]} {
+	if {[%W tag nextrange sel 1.0 end] ne "" && [%W compare sel.first >= promptEnd]} {
 	    %W delete sel.first sel.last
 	} elseif {[%W compare insert >= promptEnd]} {
 	    %W delete insert
@@ -357,8 +381,7 @@ proc tkConsoleBind {w} {
 	}
     }
     bind Console <BackSpace> {
-	if {[string compare {} [%W tag nextrange sel 1.0 end]] \
-		&& [%W compare sel.first >= promptEnd]} {
+	if {[%W tag nextrange sel 1.0 end] ne "" && [%W compare sel.first >= promptEnd]} {
 	    %W delete sel.first sel.last
 	} elseif {[%W compare insert != 1.0] && \
 		[%W compare insert > promptEnd]} {
@@ -370,14 +393,14 @@ proc tkConsoleBind {w} {
 
     bind Console <Home> {
 	if {[%W compare insert < promptEnd]} {
-	    tkTextSetCursor %W {insert linestart}
+	    tk::TextSetCursor %W {insert linestart}
 	} else {
-	    tkTextSetCursor %W promptEnd
+	    tk::TextSetCursor %W promptEnd
 	}
     }
     bind Console <Control-a> [bind Console <Home>]
     bind Console <End> {
-	tkTextSetCursor %W {insert lineend}
+	tk::TextSetCursor %W {insert lineend}
     }
     bind Console <Control-e> [bind Console <End>]
     bind Console <Control-d> {
@@ -426,24 +449,25 @@ proc tkConsoleBind {w} {
 	}
     }
     bind Console <<Console_Prev>> {
-	tkConsoleHistory prev
+	tk::ConsoleHistory prev
     }
     bind Console <<Console_Next>> {
-	tkConsoleHistory next
+	tk::ConsoleHistory next
     }
     bind Console <Insert> {
-	catch {tkConsoleInsert %W [::tk::GetSelection %W PRIMARY]}
+	catch {tk::ConsoleInsert %W [::tk::GetSelection %W PRIMARY]}
     }
     bind Console <KeyPress> {
-	tkConsoleInsert %W %A
+	tk::ConsoleInsert %W %A
     }
     bind Console <F9> {
 	eval destroy [winfo child .]
-	if {[string equal $tcl_platform(platform) "macintosh"]} {
-	    if {[catch {source $tk_library:console.tcl}]} {source -rsrc console}
-	} else {
-	    source [file join $tk_library console.tcl]
-	}
+	source [file join $tk_library console.tcl]
+    }
+    if {[tk windowingsystem] eq "aqua"} {
+	    bind Console <Command-q> {
+		exit
+	    }
     }
     bind Console <<Cut>> {
         # Same as the copy event
@@ -462,12 +486,12 @@ proc tkConsoleBind {w} {
 	catch {
 	    set clip [::tk::GetSelection %W CLIPBOARD]
 	    set list [split $clip \n\r]
-	    tkConsoleInsert %W [lindex $list 0]
+	    tk::ConsoleInsert %W [lindex $list 0]
 	    foreach x [lrange $list 1 end] {
 		%W mark set insert {end - 1c}
-		tkConsoleInsert %W "\n"
-		tkConsoleInvoke
-		tkConsoleInsert %W $x
+		tk::ConsoleInsert %W "\n"
+		tk::ConsoleInvoke
+		tk::ConsoleInsert %W $x
 	    }
 	}
     }
@@ -476,35 +500,35 @@ proc tkConsoleBind {w} {
     ## Bindings for doing special things based on certain keys
     ##
     bind PostConsole <Key-parenright> {
-	if {[string compare \\ [%W get insert-2c]]} {
+	if {"\\" ne [%W get insert-2c]} {
 	    ::tk::console::MatchPair %W \( \) promptEnd
 	}
     }
     bind PostConsole <Key-bracketright> {
-	if {[string compare \\ [%W get insert-2c]]} {
+	if {"\\" ne [%W get insert-2c]} {
 	    ::tk::console::MatchPair %W \[ \] promptEnd
 	}
     }
     bind PostConsole <Key-braceright> {
-	if {[string compare \\ [%W get insert-2c]]} {
+	if {"\\" ne [%W get insert-2c]} {
 	    ::tk::console::MatchPair %W \{ \} promptEnd
 	}
     }
     bind PostConsole <Key-quotedbl> {
-	if {[string compare \\ [%W get insert-2c]]} {
+	if {"\\" ne [%W get insert-2c]} {
 	    ::tk::console::MatchQuote %W promptEnd
 	}
     }
 
     bind PostConsole <KeyPress> {
-	if {"%A" != ""} {
+	if {"%A" ne ""} {
 	    ::tk::console::TagProc %W
 	}
 	break
     }
 }
 
-# tkConsoleInsert --
+# ::tk::ConsoleInsert --
 # Insert a string into a text at the point of the insertion cursor.
 # If there is a selection in the text, and it covers the point of the
 # insertion cursor, then delete the selection before inserting.  Insertion
@@ -514,8 +538,8 @@ proc tkConsoleBind {w} {
 # w -		The text window in which to insert the string
 # s -		The string to insert (usually just a single character)
 
-proc tkConsoleInsert {w s} {
-    if {[string equal $s ""]} {
+proc ::tk::ConsoleInsert {w s} {
+    if {$s eq ""} {
 	return
     }
     catch {
@@ -526,13 +550,13 @@ proc tkConsoleInsert {w s} {
 	}
     }
     if {[$w compare insert < promptEnd]} {
-	$w mark set insert end	
+	$w mark set insert end
     }
     $w insert insert $s {input stdin}
     $w see insert
 }
 
-# tkConsoleOutput --
+# ::tk::ConsoleOutput --
 #
 # This routine is called directly by ConsolePutsCmd to cause a string
 # to be displayed in the console.
@@ -541,14 +565,14 @@ proc tkConsoleInsert {w s} {
 # dest -	The output tag to be used: either "stderr" or "stdout".
 # string -	The string to be displayed.
 
-proc tkConsoleOutput {dest string} {
+proc ::tk::ConsoleOutput {dest string} {
     set w .console
     $w insert output $string $dest
     ::tk::console::ConstrainBuffer $w $::tk::console::maxLines
     $w see insert
 }
 
-# tkConsoleExit --
+# ::tk::ConsoleExit --
 #
 # This routine is called by ConsoleEventProc when the main window of
 # the application is destroyed.  Don't call exit - that probably already
@@ -557,23 +581,22 @@ proc tkConsoleOutput {dest string} {
 # Arguments:
 # None.
 
-proc tkConsoleExit {} {
+proc ::tk::ConsoleExit {} {
     destroy .
 }
 
-# tkConsoleAbout --
+# ::tk::ConsoleAbout --
 #
 # This routine displays an About box to show Tcl/Tk version info.
 #
 # Arguments:
 # None.
 
-proc tkConsoleAbout {} {
-    global tk_patchLevel
-    tk_messageBox -type ok -message "Tcl for Windows
+proc ::tk::ConsoleAbout {} {
+    tk_messageBox -type ok -message "[mc {Tcl for Windows}]
 
-Tcl [info patchlevel]
-Tk $tk_patchLevel"
+Tcl $::tcl_patchLevel
+Tk $::tk_patchLevel"
 }
 
 # ::tk::console::TagProc --
@@ -589,7 +612,7 @@ proc ::tk::console::TagProc w {
     if {!$::tk::console::magicKeys} { return }
     set exp "\[^\\\\\]\[\[ \t\n\r\;{}\"\$\]"
     set i [$w search -backwards -regexp $exp insert-1c promptEnd-1c]
-    if {$i == ""} {set i promptEnd} else {append i +2c}
+    if {$i eq ""} {set i promptEnd} else {append i +2c}
     regsub -all "\[\[\\\\\\?\\*\]" [$w get $i "insert-1c wordend"] {\\\0} c
     if {[llength [EvalAttached [list info commands $c]]]} {
 	$w tag add proc $i "insert-1c wordend"
@@ -622,24 +645,23 @@ proc ::tk::console::TagProc w {
  
 proc ::tk::console::MatchPair {w c1 c2 {lim 1.0}} {
     if {!$::tk::console::magicKeys} { return }
-    if {[string compare {} [set ix [$w search -back $c1 insert $lim]]]} {
+    if {[set ix [$w search -back $c1 insert $lim]] ne ""} {
 	while {
 	    [string match {\\} [$w get $ix-1c]] &&
-	    [string compare {} [set ix [$w search -back $c1 $ix-1c $lim]]]
+	    [set ix [$w search -back $c1 $ix-1c $lim]] ne ""
 	} {}
 	set i1 insert-1c
-	while {[string compare {} $ix]} {
+	while {$ix ne ""} {
 	    set i0 $ix
 	    set j 0
-	    while {[string compare {} [set i0 [$w search $c2 $i0 $i1]]]} {
+	    while {[set i0 [$w search $c2 $i0 $i1]] ne ""} {
 		append i0 +1c
 		if {[string match {\\} [$w get $i0-2c]]} continue
 		incr j
 	    }
 	    if {!$j} break
 	    set i1 $ix
-	    while {$j && [string compare {} \
-		    [set ix [$w search -back $c1 $ix $lim]]]} {
+	    while {$j && [set ix [$w search -back $c1 $ix $lim]] ne ""} {
 		if {[string match {\\} [$w get $ix-1c]]} continue
 		incr j -1
 	    }
@@ -668,7 +690,7 @@ proc ::tk::console::MatchQuote {w {lim 1.0}} {
     if {!$::tk::console::magicKeys} { return }
     set i insert-1c
     set j 0
-    while {[string compare [set i [$w search -back \" $i $lim]] {}]} {
+    while {[set i [$w search -back \" $i $lim]] ne ""} {
 	if {[string match {\\} [$w get $i-1c]]} continue
 	if {!$j} {set i0 $i}
 	incr j
@@ -738,7 +760,7 @@ proc ::tk::console::ConstrainBuffer {w size} {
 proc ::tk::console::Expand {w {type ""}} {
     set exp "\[^\\\\\]\[\[ \t\n\r\\\{\"\\\\\$\]"
     set tmp [$w search -backwards -regexp $exp insert-1c promptEnd-1c]
-    if {$tmp == ""} {set tmp promptEnd} else {append tmp +2c}
+    if {$tmp eq ""} {set tmp promptEnd} else {append tmp +2c}
     if {[$w compare $tmp >= insert]} { return }
     set str [$w get $tmp insert]
     switch -glob $type {
@@ -748,7 +770,7 @@ proc ::tk::console::Expand {w {type ""}} {
 	default {
 	    set res {}
 	    foreach t {Pathname Procname Variable} {
-		if {![catch {Expand$t $str} res] && ($res != "")} { break }
+		if {![catch {Expand$t $str} res] && ($res ne "")} { break }
 	    }
 	}
     }
@@ -757,15 +779,14 @@ proc ::tk::console::Expand {w {type ""}} {
 	set repl [lindex $res 0]
 	$w delete $tmp insert
 	$w insert $tmp $repl {input stdin}
-	if {($len > 1) && $::tk::console::showMatches \
-		&& [string equal $repl $str]} {
+	if {($len > 1) && $::tk::console::showMatches && $repl eq $str} {
 	    puts stdout [lsort [lreplace $res 0 0]]
 	}
     } else { bell }
     return [incr len -1]
 }
 
-## ::tk::console::ExpandPathname --
+# ::tk::console::ExpandPathname --
 #
 # Expand a file pathname based on $str
 # This is based on UNIX file name conventions
@@ -880,7 +901,9 @@ proc ::tk::console::ExpandVariable str {
 	    set vars $ary\([ExpandBestMatch $match $str]
 	    foreach var $match {lappend vars $ary\($var\)}
 	    return $vars
-	} else {set match $ary\($match\)}
+	} elseif {[llength $match] == 1} {
+	    set match $ary\($match\)
+	}
 	## Space transformation avoided for array names.
     } else {
 	set match [EvalAttached [list info vars $str*]]
@@ -921,5 +944,4 @@ proc ::tk::console::ExpandBestMatch {l {e {}}} {
 }
 
 # now initialize the console
-
-tkConsoleInit
+::tk::ConsoleInit 
