@@ -2,8 +2,10 @@
 #
 # This demonstration script creates a toplevel window containing
 # several checkbuttons.
-#
-# SCCS: @(#) check.tcl 1.3 96/02/16 10:49:37
+
+if {![info exists widgetDemo]} {
+    error "This script should be run from the \"widget\" demo."
+}
 
 set w .check
 catch {destroy $w}
