@@ -2,8 +2,6 @@
 #
 # This demonstration script creates a simple canvas that can be
 # scrolled in two dimensions.
-#
-# SCCS: @(#) cscroll.tcl 1.6 97/03/02 16:20:45
 
 if {![info exists widgetDemo]} {
     error "This script should be run from the \"widget\" demo."
@@ -48,9 +46,9 @@ set bg [lindex [$c config -bg] 4]
 for {set i 0} {$i < 20} {incr i} {
     set x [expr {-10 + 3*$i}]
     for {set j 0; set y -10} {$j < 10} {incr j; incr y 3} {
-	$c create rect ${x}c ${y}c [expr $x+2]c [expr $y+2]c \
+	$c create rect ${x}c ${y}c [expr {$x+2}]c [expr {$y+2}]c \
 		-outline black -fill $bg -tags rect
-	$c create text [expr $x+1]c [expr $y+1]c -text "$i,$j" \
+	$c create text [expr {$x+1}]c [expr {$y+1}]c -text "$i,$j" \
 	    -anchor center -tags text
     }
 }
@@ -60,19 +58,33 @@ $c bind all <Any-Leave> "scrollLeave $c"
 $c bind all <1> "scrollButton $c"
 bind $c <2> "$c scan mark %x %y"
 bind $c <B2-Motion> "$c scan dragto %x %y"
+if {[tk windowingsystem] eq "aqua"} {
+    bind $c <MouseWheel> {
+        %W yview scroll [expr {- (%D)}] units
+    }
+    bind $c <Option-MouseWheel> {
+        %W yview scroll [expr {-10 * (%D)}] units
+    }
+    bind $c <Shift-MouseWheel> {
+        %W xview scroll [expr {- (%D)}] units
+    }
+    bind $c <Shift-Option-MouseWheel> {
+        %W xview scroll [expr {-10 * (%D)}] units
+    }
+}
 
 proc scrollEnter canvas {
     global oldFill
     set id [$canvas find withtag current]
     if {[lsearch [$canvas gettags current] text] >= 0} {
-	set id [expr $id-1]
+	set id [expr {$id-1}]
     }
     set oldFill [lindex [$canvas itemconfig $id -fill] 4]
     if {[winfo depth $canvas] > 1} {
 	$canvas itemconfigure $id -fill SeaGreen1
     } else {
 	$canvas itemconfigure $id -fill black
-	$canvas itemconfigure [expr $id+1] -fill white
+	$canvas itemconfigure [expr {$id+1}] -fill white
     }
 }
 
@@ -80,17 +92,17 @@ proc scrollLeave canvas {
     global oldFill
     set id [$canvas find withtag current]
     if {[lsearch [$canvas gettags current] text] >= 0} {
-	set id [expr $id-1]
+	set id [expr {$id-1}]
     }
     $canvas itemconfigure $id -fill $oldFill
-    $canvas itemconfigure [expr $id+1] -fill black
+    $canvas itemconfigure [expr {$id+1}] -fill black
 }
 
 proc scrollButton canvas {
     global oldFill
     set id [$canvas find withtag current]
     if {[lsearch [$canvas gettags current] text] < 0} {
-	set id [expr $id+1]
+	set id [expr {$id+1}]
     }
     puts stdout "You buttoned at [lindex [$canvas itemconf $id -text] 4]"
 }
