@@ -2,8 +2,6 @@
 #
 # This demonstration script creates a canvas widet that displays the
 # floorplan for DEC's Western Research Laboratory.
-#
-# RCS: @(#) $Id: floor.tcl,v 1.2 1998/09/14 18:23:28 stanton Exp $
 
 if {![info exists widgetDemo]} {
     error "This script should be run from the \"widget\" demo."
@@ -94,7 +92,7 @@ proc newRoom w {
 proc roomChanged {w args} {
     global currentRoom floorItems colors
     $w delete highlight
-    if [catch {set item $floorItems($currentRoom)}] {
+    if {[catch {set item $floorItems($currentRoom)}]} {
 	return
     }
     set new [eval \
