@@ -2,8 +2,10 @@
 #
 # This demonstration script creates a canvas widget showing a 2-D
 # plot with data points that can be dragged with the mouse.
-#
-# SCCS: @(#) plot.tcl 1.3 96/02/16 10:49:46
+
+if {![info exists widgetDemo]} {
+    error "This script should be run from the \"widget\" demo."
+}
 
 set w .plot
 catch {destroy $w}
@@ -25,7 +27,7 @@ pack $w.buttons.dismiss $w.buttons.code -side left -expand 1
 canvas $c -relief raised -width 450 -height 300
 pack $w.c -side top -fill x
 
-set plotFont -*-Helvetica-Medium-R-Normal--*-180-*-*-*-*-*-*
+set plotFont {Helvetica 18}
 
 $c create line 100 250 400 250 -width 2
 $c create line 100 250 100 50 -width 2
@@ -34,20 +36,21 @@ $c create text 225 20 -text "A Simple Plot" -font $plotFont -fill brown
 for {set i 0} {$i <= 10} {incr i} {
     set x [expr {100 + ($i*30)}]
     $c create line $x 250 $x 245 -width 2
-    $c create text $x 254 -text [expr 10*$i] -anchor n -font $plotFont
+    $c create text $x 254 -text [expr {10*$i}] -anchor n -font $plotFont
 }
 for {set i 0} {$i <= 5} {incr i} {
     set y [expr {250 - ($i*40)}]
     $c create line 100 $y 105 $y -width 2
-    $c create text 96 $y -text [expr $i*50].0 -anchor e -font $plotFont
+    $c create text 96 $y -text [expr {$i*50}].0 -anchor e -font $plotFont
 }
 
-foreach point {{12 56} {20 94} {33 98} {32 120} {61 180}
-	{75 160} {98 223}} {
+foreach point {
+    {12 56} {20 94} {33 98} {32 120} {61 180} {75 160} {98 223}
+} {
     set x [expr {100 + (3*[lindex $point 0])}]
     set y [expr {250 - (4*[lindex $point 1])/5}]
-    set item [$c create oval [expr $x-6] [expr $y-6] \
-	    [expr $x+6] [expr $y+6] -width 1 -outline black \
+    set item [$c create oval [expr {$x-6}] [expr {$y-6}] \
+	    [expr {$x+6}] [expr {$y+6}] -width 1 -outline black \
 	    -fill SkyBlue2]
     $c addtag point withtag $item
 }
@@ -88,7 +91,7 @@ proc plotDown {w x y} {
 
 proc plotMove {w x y} {
     global plot
-    $w move selected [expr $x-$plot(lastX)] [expr $y-$plot(lastY)]
+    $w move selected [expr {$x-$plot(lastX)}] [expr {$y-$plot(lastY)}]
     set plot(lastX) $x
     set plot(lastY) $y
 }
