@@ -2,8 +2,10 @@
 #
 # This demonstration script creates a text widget that illustrates the
 # various display styles that may be set for tags.
-#
-# SCCS: @(#) style.tcl 1.5 96/02/16 10:49:24
+
+if {![info exists widgetDemo]} {
+    error "This script should be run from the \"widget\" demo."
+}
 
 set w .style
 catch {destroy $w}
@@ -26,10 +28,9 @@ pack $w.text -expand yes -fill both
 
 # Set up display styles
 
-$w.text tag configure bold -font -*-Courier-Bold-O-Normal--*-120-*-*-*-*-*-*
-$w.text tag configure big -font -*-Courier-Bold-R-Normal--*-140-*-*-*-*-*-*
-$w.text tag configure verybig -font \
-	-*-Helvetica-Bold-R-Normal--*-240-*-*-*-*-*-*
+$w.text tag configure bold -font {Courier 12 bold italic}
+$w.text tag configure big -font {Courier 14 bold}
+$w.text tag configure verybig -font {Helvetica 24 bold}
 if {[winfo depth $w] > 1} {
     $w.text tag configure color1 -background #a0b7ce
     $w.text tag configure color2 -foreground red
@@ -50,10 +51,8 @@ $w.text tag configure underline -underline on
 $w.text tag configure overstrike -overstrike on
 $w.text tag configure right -justify right
 $w.text tag configure center -justify center
-$w.text tag configure super -offset 4p \
-	-font -Adobe-Courier-Medium-R-Normal--*-100-*-*-*-*-*-*
-$w.text tag configure sub -offset -2p \
-	-font -Adobe-Courier-Medium-R-Normal--*-100-*-*-*-*-*-*
+$w.text tag configure super -offset 4p -font {Courier 10}
+$w.text tag configure sub -offset -2p -font {Courier 10}
 $w.text tag configure margins -lmargin1 12m -lmargin2 6m -rmargin 10m
 $w.text tag configure spacing -spacing1 10p -spacing2 2p \
 	-lmargin1 12m -lmargin2 6m -rmargin 10m
