@@ -1,8 +1,6 @@
 # vscale.tcl --
 #
 # This demonstration script shows an example with a vertical scale.
-#
-# RCS: @(#) $Id: vscale.tcl,v 1.2 1998/09/14 18:23:30 stanton Exp $
 
 if {![info exists widgetDemo]} {
     error "This script should be run from the \"widget\" demo."
@@ -39,7 +37,7 @@ $w.frame.scale set 75
 
 proc setHeight {w height} {
     incr height 21
-    set y2 [expr $height - 30]
+    set y2 [expr {$height - 30}]
     if {$y2 < 21} {
 	set y2 21
     }
