@@ -3,8 +3,6 @@
 # This file defines the default bindings for Tk entry widgets and provides
 # procedures that help in implementing those bindings.
 #
-# RCS: @(#) $Id: entry.tcl,v 1.11 2000/01/06 02:22:24 hobbs Exp $
-#
 # Copyright (c) 1992-1994 The Regents of the University of California.
 # Copyright (c) 1994-1997 Sun Microsystems, Inc.
 #
@@ -13,7 +11,7 @@
 #
 
 #-------------------------------------------------------------------------
-# Elements of tkPriv that are used in this file:
+# Elements of tk::Priv that are used in this file:
 #
 # afterId -		If non-null, it means that auto-scanning is underway
 #			and it gives the "after" id for the next auto-scan
@@ -33,128 +31,130 @@
 # The code below creates the default class bindings for entries.
 #-------------------------------------------------------------------------
 bind Entry <<Cut>> {
-    if {![catch {tkEntryGetSelection %W} tkPriv(data)]} {
+    if {![catch {tk::EntryGetSelection %W} tk::Priv(data)]} {
 	clipboard clear -displayof %W
-	clipboard append -displayof %W $tkPriv(data)
+	clipboard append -displayof %W $tk::Priv(data)
 	%W delete sel.first sel.last
-	unset tkPriv(data)
+	unset tk::Priv(data)
     }
 }
 bind Entry <<Copy>> {
-    if {![catch {tkEntryGetSelection %W} tkPriv(data)]} {
+    if {![catch {tk::EntryGetSelection %W} tk::Priv(data)]} {
 	clipboard clear -displayof %W
-	clipboard append -displayof %W $tkPriv(data)
-	unset tkPriv(data)
+	clipboard append -displayof %W $tk::Priv(data)
+	unset tk::Priv(data)
     }
 }
 bind Entry <<Paste>> {
     global tcl_platform
     catch {
-	if {[string compare $tcl_platform(platform) "unix"]} {
+	if {[tk windowingsystem] ne "x11"} {
 	    catch {
 		%W delete sel.first sel.last
 	    }
 	}
-	%W insert insert [selection get -displayof %W -selection CLIPBOARD]
-	tkEntrySeeInsert %W
+	%W insert insert [::tk::GetSelection %W CLIPBOARD]
+	tk::EntrySeeInsert %W
     }
 }
 bind Entry <<Clear>> {
-    %W delete sel.first sel.last
+    # ignore if there is no selection
+    catch { %W delete sel.first sel.last }
 }
 bind Entry <<PasteSelection>> {
-    if {!$tkPriv(mouseMoved) || $tk_strictMotif} {
-	tkEntryPaste %W %x
+    if {$tk_strictMotif || ![info exists tk::Priv(mouseMoved)]
+	|| !$tk::Priv(mouseMoved)} {
+	tk::EntryPaste %W %x
     }
 }
 
 # Standard Motif bindings:
 
 bind Entry <1> {
-    tkEntryButton1 %W %x
+    tk::EntryButton1 %W %x
     %W selection clear
 }
 bind Entry <B1-Motion> {
-    set tkPriv(x) %x
-    tkEntryMouseSelect %W %x
+    set tk::Priv(x) %x
+    tk::EntryMouseSelect %W %x
 }
 bind Entry <Double-1> {
-    set tkPriv(selectMode) word
-    tkEntryMouseSelect %W %x
-    catch {%W icursor sel.first}
+    set tk::Priv(selectMode) word
+    tk::EntryMouseSelect %W %x
+    catch {%W icursor sel.last}
 }
 bind Entry <Triple-1> {
-    set tkPriv(selectMode) line
-    tkEntryMouseSelect %W %x
-    %W icursor 0
+    set tk::Priv(selectMode) line
+    tk::EntryMouseSelect %W %x
+    catch {%W icursor sel.last}
 }
 bind Entry <Shift-1> {
-    set tkPriv(selectMode) char
+    set tk::Priv(selectMode) char
     %W selection adjust @%x
 }
 bind Entry <Double-Shift-1>	{
-    set tkPriv(selectMode) word
-    tkEntryMouseSelect %W %x
+    set tk::Priv(selectMode) word
+    tk::EntryMouseSelect %W %x
 }
 bind Entry <Triple-Shift-1>	{
-    set tkPriv(selectMode) line
-    tkEntryMouseSelect %W %x
+    set tk::Priv(selectMode) line
+    tk::EntryMouseSelect %W %x
 }
 bind Entry <B1-Leave> {
-    set tkPriv(x) %x
-    tkEntryAutoScan %W
+    set tk::Priv(x) %x
+    tk::EntryAutoScan %W
 }
 bind Entry <B1-Enter> {
-    tkCancelRepeat
+    tk::CancelRepeat
 }
 bind Entry <ButtonRelease-1> {
-    tkCancelRepeat
+    tk::CancelRepeat
 }
 bind Entry <Control-1> {
     %W icursor @%x
 }
 
 bind Entry <Left> {
-    tkEntrySetCursor %W [expr {[%W index insert] - 1}]
+    tk::EntrySetCursor %W [expr {[%W index insert] - 1}]
 }
 bind Entry <Right> {
-    tkEntrySetCursor %W [expr {[%W index insert] + 1}]
+    tk::EntrySetCursor %W [expr {[%W index insert] + 1}]
 }
 bind Entry <Shift-Left> {
-    tkEntryKeySelect %W [expr {[%W index insert] - 1}]
-    tkEntrySeeInsert %W
+    tk::EntryKeySelect %W [expr {[%W index insert] - 1}]
+    tk::EntrySeeInsert %W
 }
 bind Entry <Shift-Right> {
-    tkEntryKeySelect %W [expr {[%W index insert] + 1}]
-    tkEntrySeeInsert %W
+    tk::EntryKeySelect %W [expr {[%W index insert] + 1}]
+    tk::EntrySeeInsert %W
 }
 bind Entry <Control-Left> {
-    tkEntrySetCursor %W [tkEntryPreviousWord %W insert]
+    tk::EntrySetCursor %W [tk::EntryPreviousWord %W insert]
 }
 bind Entry <Control-Right> {
-    tkEntrySetCursor %W [tkEntryNextWord %W insert]
+    tk::EntrySetCursor %W [tk::EntryNextWord %W insert]
 }
 bind Entry <Shift-Control-Left> {
-    tkEntryKeySelect %W [tkEntryPreviousWord %W insert]
-    tkEntrySeeInsert %W
+    tk::EntryKeySelect %W [tk::EntryPreviousWord %W insert]
+    tk::EntrySeeInsert %W
 }
 bind Entry <Shift-Control-Right> {
-    tkEntryKeySelect %W [tkEntryNextWord %W insert]
-    tkEntrySeeInsert %W
+    tk::EntryKeySelect %W [tk::EntryNextWord %W insert]
+    tk::EntrySeeInsert %W
 }
 bind Entry <Home> {
-    tkEntrySetCursor %W 0
+    tk::EntrySetCursor %W 0
 }
 bind Entry <Shift-Home> {
-    tkEntryKeySelect %W 0
-    tkEntrySeeInsert %W
+    tk::EntryKeySelect %W 0
+    tk::EntrySeeInsert %W
 }
 bind Entry <End> {
-    tkEntrySetCursor %W end
+    tk::EntrySetCursor %W end
 }
 bind Entry <Shift-End> {
-    tkEntryKeySelect %W end
-    tkEntrySeeInsert %W
+    tk::EntryKeySelect %W end
+    tk::EntrySeeInsert %W
 }
 
 bind Entry <Delete> {
@@ -165,7 +165,7 @@ bind Entry <Delete> {
     }
 }
 bind Entry <BackSpace> {
-    tkEntryBackspace %W
+    tk::EntryBackspace %W
 }
 
 bind Entry <Control-space> {
@@ -187,7 +187,8 @@ bind Entry <Control-backslash> {
     %W selection clear
 }
 bind Entry <KeyPress> {
-    tkEntryInsert %W %A
+    tk::CancelRepeat
+    tk::EntryInsert %W %A
 }
 
 # Ignore all Alt, Meta, and Control keypresses unless explicitly bound.
@@ -202,15 +203,15 @@ bind Entry <Escape> {# nothing}
 bind Entry <Return> {# nothing}
 bind Entry <KP_Enter> {# nothing}
 bind Entry <Tab> {# nothing}
-if {[string equal $tcl_platform(platform) "macintosh"]} {
+if {[tk windowingsystem] eq "aqua"} {
 	bind Entry <Command-KeyPress> {# nothing}
 }
 
 # On Windows, paste is done using Shift-Insert.  Shift-Insert already
 # generates the <<Paste>> event, so we don't need to do anything here.
-if {[string compare $tcl_platform(platform) "windows"]} {
+if {[tk windowingsystem] ne "win32"} {
     bind Entry <Insert> {
-	catch {tkEntryInsert %W [selection get -displayof %W]}
+	catch {tk::EntryInsert %W [::tk::GetSelection %W PRIMARY]}
     }
 }
 
@@ -218,12 +219,12 @@ if {[string compare $tcl_platform(platform) "windows"]} {
 
 bind Entry <Control-a> {
     if {!$tk_strictMotif} {
-	tkEntrySetCursor %W 0
+	tk::EntrySetCursor %W 0
     }
 }
 bind Entry <Control-b> {
     if {!$tk_strictMotif} {
-	tkEntrySetCursor %W [expr {[%W index insert] - 1}]
+	tk::EntrySetCursor %W [expr {[%W index insert] - 1}]
     }
 }
 bind Entry <Control-d> {
@@ -233,17 +234,17 @@ bind Entry <Control-d> {
 }
 bind Entry <Control-e> {
     if {!$tk_strictMotif} {
-	tkEntrySetCursor %W end
+	tk::EntrySetCursor %W end
     }
 }
 bind Entry <Control-f> {
     if {!$tk_strictMotif} {
-	tkEntrySetCursor %W [expr {[%W index insert] + 1}]
+	tk::EntrySetCursor %W [expr {[%W index insert] + 1}]
     }
 }
 bind Entry <Control-h> {
     if {!$tk_strictMotif} {
-	tkEntryBackspace %W
+	tk::EntryBackspace %W
     }
 }
 bind Entry <Control-k> {
@@ -253,32 +254,32 @@ bind Entry <Control-k> {
 }
 bind Entry <Control-t> {
     if {!$tk_strictMotif} {
-	tkEntryTranspose %W
+	tk::EntryTranspose %W
     }
 }
 bind Entry <Meta-b> {
     if {!$tk_strictMotif} {
-	tkEntrySetCursor %W [tkEntryPreviousWord %W insert]
+	tk::EntrySetCursor %W [tk::EntryPreviousWord %W insert]
     }
 }
 bind Entry <Meta-d> {
     if {!$tk_strictMotif} {
-	%W delete insert [tkEntryNextWord %W insert]
+	%W delete insert [tk::EntryNextWord %W insert]
     }
 }
 bind Entry <Meta-f> {
     if {!$tk_strictMotif} {
-	tkEntrySetCursor %W [tkEntryNextWord %W insert]
+	tk::EntrySetCursor %W [tk::EntryNextWord %W insert]
     }
 }
 bind Entry <Meta-BackSpace> {
     if {!$tk_strictMotif} {
-	%W delete [tkEntryPreviousWord %W insert] insert
+	%W delete [tk::EntryPreviousWord %W insert] insert
     }
 }
 bind Entry <Meta-Delete> {
     if {!$tk_strictMotif} {
-	%W delete [tkEntryPreviousWord %W insert] insert
+	%W delete [tk::EntryPreviousWord %W insert] insert
     }
 }
 
@@ -286,22 +287,16 @@ bind Entry <Meta-Delete> {
 
 bind Entry <2> {
     if {!$tk_strictMotif} {
-	%W scan mark %x
-	set tkPriv(x) %x
-	set tkPriv(y) %y
-	set tkPriv(mouseMoved) 0
+	::tk::EntryScanMark %W %x
     }
 }
 bind Entry <B2-Motion> {
     if {!$tk_strictMotif} {
-	if {abs(%x-$tkPriv(x)) > 2} {
-	    set tkPriv(mouseMoved) 1
-	}
-	%W scan dragto %x
+	::tk::EntryScanDrag %W %x
     }
 }
 
-# tkEntryClosestGap --
+# ::tk::EntryClosestGap --
 # Given x and y coordinates, this procedure finds the closest boundary
 # between characters to the given coordinates and returns the index
 # of the character just after the boundary.
@@ -310,7 +305,7 @@ bind Entry <B2-Motion> {
 # w -		The entry window.
 # x -		X-coordinate within the window.
 
-proc tkEntryClosestGap {w x} {
+proc ::tk::EntryClosestGap {w x} {
     set pos [$w index @$x]
     set bbox [$w bbox $pos]
     if {($x - [lindex $bbox 0]) < ([lindex $bbox 2]/2)} {
@@ -319,7 +314,7 @@ proc tkEntryClosestGap {w x} {
     incr pos
 }
 
-# tkEntryButton1 --
+# ::tk::EntryButton1 --
 # This procedure is invoked to handle button-1 presses in entry
 # widgets.  It moves the insertion cursor, sets the selection anchor,
 # and claims the input focus.
@@ -328,18 +323,18 @@ proc tkEntryClosestGap {w x} {
 # w -		The entry window in which the button was pressed.
 # x -		The x-coordinate of the button press.
 
-proc tkEntryButton1 {w x} {
-    global tkPriv
+proc ::tk::EntryButton1 {w x} {
+    variable ::tk::Priv
 
-    set tkPriv(selectMode) char
-    set tkPriv(mouseMoved) 0
-    set tkPriv(pressX) $x
-    $w icursor [tkEntryClosestGap $w $x]
+    set Priv(selectMode) char
+    set Priv(mouseMoved) 0
+    set Priv(pressX) $x
+    $w icursor [EntryClosestGap $w $x]
     $w selection from insert
-    if {[string equal [$w cget -state] "normal"]} {focus $w}
+    if {"disabled" ne [$w cget -state]} {focus $w}
 }
 
-# tkEntryMouseSelect --
+# ::tk::EntryMouseSelect --
 # This procedure is invoked when dragging out a selection with
 # the mouse.  Depending on the selection mode (character, word,
 # line) it selects in different-sized units.  This procedure
@@ -350,17 +345,17 @@ proc tkEntryButton1 {w x} {
 # w -		The entry window in which the button was pressed.
 # x -		The x-coordinate of the mouse.
 
-proc tkEntryMouseSelect {w x} {
-    global tkPriv
+proc ::tk::EntryMouseSelect {w x} {
+    variable ::tk::Priv
 
-    set cur [tkEntryClosestGap $w $x]
+    set cur [EntryClosestGap $w $x]
     set anchor [$w index anchor]
-    if {($cur != $anchor) || (abs($tkPriv(pressX) - $x) >= 3)} {
-	set tkPriv(mouseMoved) 1
+    if {($cur != $anchor) || (abs($Priv(pressX) - $x) >= 3)} {
+	set Priv(mouseMoved) 1
     }
-    switch $tkPriv(selectMode) {
+    switch $Priv(selectMode) {
 	char {
-	    if {$tkPriv(mouseMoved)} {
+	    if {$Priv(mouseMoved)} {
 		if {$cur < $anchor} {
 		    $w selection range $cur $anchor
 		} elseif {$cur > $anchor} {
@@ -390,10 +385,13 @@ proc tkEntryMouseSelect {w x} {
 	    $w selection range 0 end
 	}
     }
+    if {$Priv(mouseMoved)} {
+        $w icursor $cur
+    }
     update idletasks
 }
 
-# tkEntryPaste --
+# ::tk::EntryPaste --
 # This procedure sets the insertion cursor to the current mouse position,
 # pastes the selection there, and sets the focus to the window.
 #
@@ -401,15 +399,13 @@ proc tkEntryMouseSelect {w x} {
 # w -		The entry window.
 # x -		X position of the mouse.
 
-proc tkEntryPaste {w x} {
-    global tkPriv
-
-    $w icursor [tkEntryClosestGap $w $x]
-    catch {$w insert insert [selection get -displayof $w]}
-    if {[string equal [$w cget -state] "normal"]} {focus $w}
+proc ::tk::EntryPaste {w x} {
+    $w icursor [EntryClosestGap $w $x]
+    catch {$w insert insert [::tk::GetSelection $w PRIMARY]}
+    if {"disabled" ne [$w cget -state]} {focus $w}
 }
 
-# tkEntryAutoScan --
+# ::tk::EntryAutoScan --
 # This procedure is invoked when the mouse leaves an entry window
 # with button 1 down.  It scrolls the window left or right,
 # depending on where the mouse is, and reschedules itself as an
@@ -419,21 +415,21 @@ proc tkEntryPaste {w x} {
 # Arguments:
 # w -		The entry window.
 
-proc tkEntryAutoScan {w} {
-    global tkPriv
-    set x $tkPriv(x)
+proc ::tk::EntryAutoScan {w} {
+    variable ::tk::Priv
+    set x $Priv(x)
     if {![winfo exists $w]} return
     if {$x >= [winfo width $w]} {
 	$w xview scroll 2 units
-	tkEntryMouseSelect $w $x
+	EntryMouseSelect $w $x
     } elseif {$x < 0} {
 	$w xview scroll -2 units
-	tkEntryMouseSelect $w $x
+	EntryMouseSelect $w $x
     }
-    set tkPriv(afterId) [after 50 [list tkEntryAutoScan $w]]
+    set Priv(afterId) [after 50 [list tk::EntryAutoScan $w]]
 }
 
-# tkEntryKeySelect --
+# ::tk::EntryKeySelect --
 # This procedure is invoked when stroking out selections using the
 # keyboard.  It moves the cursor to a new position, then extends
 # the selection to that position.
@@ -443,7 +439,7 @@ proc tkEntryAutoScan {w} {
 # new -		A new position for the insertion cursor (the cursor hasn't
 #		actually been moved to this position yet).
 
-proc tkEntryKeySelect {w new} {
+proc ::tk::EntryKeySelect {w new} {
     if {![$w selection present]} {
 	$w selection from insert
 	$w selection to $new
@@ -453,7 +449,7 @@ proc tkEntryKeySelect {w new} {
     $w icursor $new
 }
 
-# tkEntryInsert --
+# ::tk::EntryInsert --
 # Insert a string into an entry at the point of the insertion cursor.
 # If there is a selection in the entry, and it covers the point of the
 # insertion cursor, then delete the selection before inserting.
@@ -462,8 +458,8 @@ proc tkEntryKeySelect {w new} {
 # w -		The entry window in which to insert the string
 # s -		The string to insert (usually just a single character)
 
-proc tkEntryInsert {w s} {
-    if {[string equal $s ""]} {
+proc ::tk::EntryInsert {w s} {
+    if {$s eq ""} {
 	return
     }
     catch {
@@ -474,10 +470,10 @@ proc tkEntryInsert {w s} {
 	}
     }
     $w insert insert $s
-    tkEntrySeeInsert $w
+    EntrySeeInsert $w
 }
 
-# tkEntryBackspace --
+# ::tk::EntryBackspace --
 # Backspace over the character just before the insertion cursor.
 # If backspacing would move the cursor off the left edge of the
 # window, reposition the cursor at about the middle of the window.
@@ -485,7 +481,7 @@ proc tkEntryInsert {w s} {
 # Arguments:
 # w -		The entry window in which to backspace.
 
-proc tkEntryBackspace w {
+proc ::tk::EntryBackspace w {
     if {[$w selection present]} {
 	$w delete sel.first sel.last
     } else {
@@ -500,21 +496,21 @@ proc tkEntryBackspace w {
     }
 }
 
-# tkEntrySeeInsert --
+# ::tk::EntrySeeInsert --
 # Make sure that the insertion cursor is visible in the entry window.
 # If not, adjust the view so that it is.
 #
 # Arguments:
 # w -		The entry window.
 
-proc tkEntrySeeInsert w {
+proc ::tk::EntrySeeInsert w {
     set c [$w index insert]
     if {($c < [$w index @0]) || ($c > [$w index @[winfo width $w]])} {
 	$w xview $c
     }
 }
 
-# tkEntrySetCursor -
+# ::tk::EntrySetCursor -
 # Move the insertion cursor to a given position in an entry.  Also
 # clears the selection, if there is one in the entry, and makes sure
 # that the insertion cursor is visible.
@@ -523,13 +519,13 @@ proc tkEntrySeeInsert w {
 # w -		The entry window.
 # pos -		The desired new position for the cursor in the window.
 
-proc tkEntrySetCursor {w pos} {
+proc ::tk::EntrySetCursor {w pos} {
     $w icursor $pos
     $w selection clear
-    tkEntrySeeInsert $w
+    EntrySeeInsert $w
 }
 
-# tkEntryTranspose -
+# ::tk::EntryTranspose -
 # This procedure implements the "transpose" function for entry widgets.
 # It tranposes the characters on either side of the insertion cursor,
 # unless the cursor is at the end of the line.  In this case it
@@ -539,7 +535,7 @@ proc tkEntrySetCursor {w pos} {
 # Arguments:
 # w -		The entry window.
 
-proc tkEntryTranspose w {
+proc ::tk::EntryTranspose w {
     set i [$w index insert]
     if {$i < [$w index end]} {
 	incr i
@@ -548,13 +544,14 @@ proc tkEntryTranspose w {
     if {$first < 0} {
 	return
     }
-    set new [string index [$w get] [expr {$i-1}]][string index [$w get] $first]
+    set data [$w get]
+    set new [string index $data [expr {$i-1}]][string index $data $first]
     $w delete $first $i
     $w insert insert $new
-    tkEntrySeeInsert $w
+    EntrySeeInsert $w
 }
 
-# tkEntryNextWord --
+# ::tk::EntryNextWord --
 # Returns the index of the next word position after a given position in the
 # entry.  The next word is platform dependent and may be either the next
 # end-of-word position or the next start-of-word position after the next
@@ -564,8 +561,8 @@ proc tkEntryTranspose w {
 # w -		The entry window in which the cursor is to move.
 # start -	Position at which to start search.
 
-if {[string equal $tcl_platform(platform) "windows"]}  {
-    proc tkEntryNextWord {w start} {
+if {[tk windowingsystem] eq "win32"}  {
+    proc ::tk::EntryNextWord {w start} {
 	set pos [tcl_endOfWord [$w get] [$w index $start]]
 	if {$pos >= 0} {
 	    set pos [tcl_startOfNextWord [$w get] $pos]
@@ -576,7 +573,7 @@ if {[string equal $tcl_platform(platform) "windows"]}  {
 	return $pos
     }
 } else {
-    proc tkEntryNextWord {w start} {
+    proc ::tk::EntryNextWord {w start} {
 	set pos [tcl_endOfWord [$w get] [$w index $start]]
 	if {$pos < 0} {
 	    return end
@@ -585,7 +582,7 @@ if {[string equal $tcl_platform(platform) "windows"]}  {
     }
 }
 
-# tkEntryPreviousWord --
+# ::tk::EntryPreviousWord --
 #
 # Returns the index of the previous word position before a given
 # position in the entry.
@@ -594,25 +591,61 @@ if {[string equal $tcl_platform(platform) "windows"]}  {
 # w -		The entry window in which the cursor is to move.
 # start -	Position at which to start search.
 
-proc tkEntryPreviousWord {w start} {
+proc ::tk::EntryPreviousWord {w start} {
     set pos [tcl_startOfPreviousWord [$w get] [$w index $start]]
     if {$pos < 0} {
 	return 0
     }
     return $pos
 }
-# tkEntryGetSelection --
+
+# ::tk::EntryScanMark --
+#
+# Marks the start of a possible scan drag operation
+#
+# Arguments:
+# w -	The entry window from which the text to get
+# x -	x location on screen
+
+proc ::tk::EntryScanMark {w x} {
+    $w scan mark $x
+    set ::tk::Priv(x) $x
+    set ::tk::Priv(y) 0 ; # not used
+    set ::tk::Priv(mouseMoved) 0
+}
+
+# ::tk::EntryScanDrag --
+#
+# Marks the start of a possible scan drag operation
+#
+# Arguments:
+# w -	The entry window from which the text to get
+# x -	x location on screen
+
+proc ::tk::EntryScanDrag {w x} {
+    # Make sure these exist, as some weird situations can trigger the
+    # motion binding without the initial press.  [Bug #220269]
+    if {![info exists ::tk::Priv(x)]} { set ::tk::Priv(x) $x }
+    # allow for a delta
+    if {abs($x-$::tk::Priv(x)) > 2} {
+	set ::tk::Priv(mouseMoved) 1
+    }
+    $w scan dragto $x
+}
+
+# ::tk::EntryGetSelection --
 #
 # Returns the selected text of the entry with respect to the -show option.
 #
 # Arguments:
 # w -         The entry window from which the text to get
 
-proc tkEntryGetSelection {w} {
+proc ::tk::EntryGetSelection {w} {
     set entryString [string range [$w get] [$w index sel.first] \
 	    [expr {[$w index sel.last] - 1}]]
-    if {[string compare [$w cget -show] ""]} {
-	regsub -all . $entryString [string index [$w cget -show] 0] entryString
+    if {[$w cget -show] ne ""} {
+	return [string repeat [string index [$w cget -show] 0] \
+		[string length $entryString]]
     }
     return $entryString
 }
