@@ -3,8 +3,6 @@
 # This file defines the procedure tk_optionMenu, which creates
 # an option button and its associated menu.
 #
-# RCS: @(#) $Id: optMenu.tcl,v 1.3 1998/09/14 18:23:24 stanton Exp $
-#
 # Copyright (c) 1994 The Regents of the University of California.
 # Copyright (c) 1994 Sun Microsystems, Inc.
 #
@@ -12,7 +10,7 @@
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 #
 
-# tk_optionMenu --
+# ::tk_optionMenu --
 # This procedure creates an option button named $w and an associated
 # menu.  Together they provide the functionality of Motif option menus:
 # they can be used to select one of many values, and the current value
@@ -27,7 +25,7 @@
 # firstValue -		First of legal values for option (must be >= 1).
 # args -		Any number of additional values.
 
-proc tk_optionMenu {w varName firstValue args} {
+proc ::tk_optionMenu {w varName firstValue args} {
     upvar #0 $varName var
 
     if {![info exists var]} {
