@@ -3,15 +3,13 @@
 # This file contains procedures that change the color palette used
 # by Tk.
 #
-# SCCS: @(#) palette.tcl 1.11 97/06/23 20:35:44
-#
 # Copyright (c) 1995-1997 Sun Microsystems, Inc.
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 #
 
-# tk_setPalette --
+# ::tk_setPalette --
 # Changes the default color scheme for a Tk application by setting
 # default colors in the option database and by modifying all of the
 # color options for existing widgets that have the default value.
@@ -23,8 +21,11 @@
 # option names and values.  The name for an option is the one used
 # for the option database, such as activeForeground, not -activeforeground.
 
-proc tk_setPalette {args} {
-    global tkPalette
+proc ::tk_setPalette {args} {
+    if {[winfo depth .] == 1} {
+	# Just return on monochrome displays, otherwise errors will occur
+	return
+    }
 
     # Create an array that has the complete new palette.  If some colors
     # aren't specified, compute them from other colors that are specified.
@@ -34,41 +35,53 @@ proc tk_setPalette {args} {
     } else {
 	array set new $args
     }
-    if ![info exists new(background)] {
+    if {![info exists new(background)]} {
 	error "must specify a background color"
     }
-    if ![info exists new(foreground)] {
-	set new(foreground) black
-    }
     set bg [winfo rgb . $new(background)]
-    set fg [winfo rgb . $new(foreground)]
-    set darkerBg [format #%02x%02x%02x [expr (9*[lindex $bg 0])/2560] \
-	    [expr (9*[lindex $bg 1])/2560] [expr (9*[lindex $bg 2])/2560]]
+    if {![info exists new(foreground)]} {
+	# Note that the range of each value in the triple returned by
+	# [winfo rgb] is 0-65535, and your eyes are more sensitive to
+	# green than to red, and more to red than to blue.
+	foreach {r g b} $bg {break}
+	if {$r+1.5*$g+0.5*$b > 100000} {
+	    set new(foreground) black
+	} else {
+	    set new(foreground) white
+	}
+    }
+
+    # To avoir too many lindex...
+    foreach {fg_r fg_g fg_b} [winfo rgb . $new(foreground)] {break}
+    foreach {bg_r bg_g bg_b} $bg {break}
+
+    set darkerBg [format #%02x%02x%02x [expr {(9*$bg_r)/2560}] \
+	    [expr {(9*$bg_g)/2560}] [expr {(9*$bg_b)/2560}]]
     foreach i {activeForeground insertBackground selectForeground \
 	    highlightColor} {
-	if ![info exists new($i)] {
+	if {![info exists new($i)]} {
 	    set new($i) $new(foreground)
 	}
     }
-    if ![info exists new(disabledForeground)] {
+    if {![info exists new(disabledForeground)]} {
 	set new(disabledForeground) [format #%02x%02x%02x \
-		[expr (3*[lindex $bg 0] + [lindex $fg 0])/1024] \
-		[expr (3*[lindex $bg 1] + [lindex $fg 1])/1024] \
-		[expr (3*[lindex $bg 2] + [lindex $fg 2])/1024]]
+		[expr {(3*$bg_r + $fg_r)/1024}] \
+		[expr {(3*$bg_g + $fg_g)/1024}] \
+		[expr {(3*$bg_b + $fg_b)/1024}]]
     }
-    if ![info exists new(highlightBackground)] {
+    if {![info exists new(highlightBackground)]} {
 	set new(highlightBackground) $new(background)
     }
-    if ![info exists new(activeBackground)] {
+    if {![info exists new(activeBackground)]} {
 	# Pick a default active background that islighter than the
 	# normal background.  To do this, round each color component
 	# up by 15% or 1/3 of the way to full white, whichever is
 	# greater.
 
-	foreach i {0 1 2} {
-	    set light($i) [expr [lindex $bg $i]/256]
-	    set inc1 [expr ($light($i)*15)/100]
-	    set inc2 [expr (255-$light($i))/3]
+	foreach i {0 1 2} color "$bg_r $bg_g $bg_b" {
+	    set light($i) [expr {$color/256}]
+	    set inc1 [expr {($light($i)*15)/100}]
+	    set inc2 [expr {(255-$light($i))/3}]
 	    if {$inc1 > $inc2} {
 		incr light($i) $inc1
 	    } else {
@@ -81,13 +94,13 @@ proc tk_setPalette {args} {
 	set new(activeBackground) [format #%02x%02x%02x $light(0) \
 		$light(1) $light(2)]
     }
-    if ![info exists new(selectBackground)] {
+    if {![info exists new(selectBackground)]} {
 	set new(selectBackground) $darkerBg
     }
-    if ![info exists new(troughColor)] {
+    if {![info exists new(troughColor)]} {
 	set new(troughColor) $darkerBg
     }
-    if ![info exists new(selectColor)] {
+    if {![info exists new(selectColor)]} {
 	set new(selectColor) #b03060
     }
 
@@ -95,15 +108,18 @@ proc tk_setPalette {args} {
     # defaults are currently for this platform.
     toplevel .___tk_set_palette
     wm withdraw .___tk_set_palette
-    foreach q {button canvas checkbutton entry frame label listbox menubutton menu message \
-		 radiobutton scale scrollbar text} {
+    foreach q {
+	button canvas checkbutton entry frame label labelframe
+	listbox menubutton menu message radiobutton scale scrollbar
+	spinbox text
+    } {
 	$q .___tk_set_palette.$q
     }
 
     # Walk the widget hierarchy, recoloring all existing windows.
     # The option database must be set according to what we do here, 
     # but it breaks things if we set things in the database while 
-    # we are changing colors...so, tkRecolorTree now returns the
+    # we are changing colors...so, ::tk::RecolorTree now returns the
     # option database changes that need to be made, and they
     # need to be evalled here to take effect.
     # We have to walk the whole widget tree instead of just 
@@ -112,9 +128,9 @@ proc tk_setPalette {args} {
     # of widgets that we don't currently know about, so we'll
     # walk the whole hierarchy just in case.
 
-    eval [tkRecolorTree . new]
+    eval [tk::RecolorTree . new]
 
-    catch {destroy .___tk_set_palette}
+    destroy .___tk_set_palette
 
     # Change the option database so that future windows will get the
     # same colors.
@@ -123,13 +139,13 @@ proc tk_setPalette {args} {
 	option add *$option $new($option) widgetDefault
     }
 
-    # Save the options in the global variable tkPalette, for use the
+    # Save the options in the variable ::tk::Palette, for use the
     # next time we change the options.
 
-    array set tkPalette [array get new]
+    array set ::tk::Palette [array get new]
 }
 
-# tkRecolorTree --
+# ::tk::RecolorTree --
 # This procedure changes the colors in a window and all of its
 # descendants, according to information provided by the colors
 # argument. This looks at the defaults provided by the option 
@@ -144,23 +160,34 @@ proc tk_setPalette {args} {
 #			is named after a widget configuration option, and
 #			each value is the value for that option.
 
-proc tkRecolorTree {w colors} {
-    global tkPalette
+proc ::tk::RecolorTree {w colors} {
     upvar $colors c
     set result {}
+    set prototype .___tk_set_palette.[string tolower [winfo class $w]]
+    if {![winfo exists $prototype]} {
+	unset prototype
+    }
     foreach dbOption [array names c] {
 	set option -[string tolower $dbOption]
-	if {![catch {$w config $option} value]} {
+	set class [string replace $dbOption 0 0 [string toupper \
+		[string index $dbOption 0]]]
+	if {![catch {$w configure $option} value]} {
 	    # if the option database has a preference for this
 	    # dbOption, then use it, otherwise use the defaults
 	    # for the widget.
-	    set defaultcolor [option get $w $dbOption widgetDefault]
-	    if {[string match {} $defaultcolor]} {
-		set defaultcolor [winfo rgb . [lindex $value 3]]
-	    } else {
+	    set defaultcolor [option get $w $dbOption $class]
+	    if {[string match {} $defaultcolor] || \
+		    ([info exists prototype] && \
+		    [$prototype cget $option] ne "$defaultcolor")} {
+		set defaultcolor [lindex $value 3]
+	    }
+	    if {![string match {} $defaultcolor]} {
 		set defaultcolor [winfo rgb . $defaultcolor]
 	    }
-	    set chosencolor [winfo rgb . [lindex $value 4]]
+	    set chosencolor [lindex $value 4]
+	    if {![string match {} $chosencolor]} {
+		set chosencolor [winfo rgb . $chosencolor]
+	    }
 	    if {[string match $defaultcolor $chosencolor]} {
 		# Change the option database so that future windows will get
 		# the same colors.
@@ -171,12 +198,12 @@ proc tkRecolorTree {w colors} {
 	}
     }
     foreach child [winfo children $w] {
-	append result ";\n[tkRecolorTree $child c]"
+	append result ";\n[::tk::RecolorTree $child c]"
     }
     return $result
 }
 
-# tkDarken --
+# ::tk::Darken --
 # Given a color name, computes a new color value that darkens (or
 # brightens) the given color by a given percent.
 #
@@ -186,33 +213,32 @@ proc tkRecolorTree {w colors} {
 #		percent: 50 means darken by 50%, 110 means brighten
 #		by 10%.
 
-proc tkDarken {color percent} {
-    set l [winfo rgb . $color]
-    set red [expr [lindex $l 0]/256]
-    set green [expr [lindex $l 1]/256]
-    set blue [expr [lindex $l 2]/256]
-    set red [expr ($red*$percent)/100]
+proc ::tk::Darken {color percent} {
+    foreach {red green blue} [winfo rgb . $color] {
+	set red [expr {($red/256)*$percent/100}]
+	set green [expr {($green/256)*$percent/100}]
+	set blue [expr {($blue/256)*$percent/100}]
+	break
+    }
     if {$red > 255} {
 	set red 255
     }
-    set green [expr ($green*$percent)/100]
     if {$green > 255} {
 	set green 255
     }
-    set blue [expr ($blue*$percent)/100]
     if {$blue > 255} {
 	set blue 255
     }
-    format #%02x%02x%02x $red $green $blue
+    return [format "#%02x%02x%02x" $red $green $blue]
 }
 
-# tk_bisque --
+# ::tk_bisque --
 # Reset the Tk color palette to the old "bisque" colors.
 #
 # Arguments:
 # None.
 
-proc tk_bisque {} {
+proc ::tk_bisque {} {
     tk_setPalette activeBackground #e6ceb1 activeForeground black \
 	    background #ffe4c4 disabledForeground #b0b0b0 foreground black \
 	    highlightBackground #ffe4c4 highlightColor black \
