@@ -11,7 +11,7 @@
  *	application event target.
  *
  * Copyright 2001, Apple Computer, Inc.
- * Copyright (c) 2005-2007 Daniel A. Steffen <das@users.sourceforge.net>
+ * Copyright (c) 2005-2008 Daniel A. Steffen <das@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -59,8 +59,6 @@
  *	acting in its behalf permission to use and distribute the
  *	software in accordance with the terms specified in this
  *	license.
- *
- * RCS: @(#) $Id: tkMacOSXCarbonEvents.c,v 1.3.2.17 2007/06/29 03:22:01 das Exp $
  */
 
 #include "tkMacOSXPrivate.h"
@@ -80,21 +78,28 @@
 static OSStatus CarbonEventHandlerProc(EventHandlerCallRef callRef,
 	EventRef event, void *userData);
 static OSStatus InstallStandardApplicationEventHandler(void);
-static void ExitRaelEventHandlerProc(EventHandlerCallRef callRef,
-	EventRef event, void *userData) __attribute__ ((__noreturn__));
 static void CarbonTimerProc(EventLoopTimerRef timer, void *userData);
 
 /*
  * Static data used by several functions in this file:
  */
 
-static jmp_buf exitRaelJmpBuf;
 static EventLoopTimerRef carbonTimer = NULL;
 static int carbonTimerEnabled = 0;
 static EventHandlerUPP carbonEventHandlerUPP = NULL;
 static Tcl_Interp *carbonEventInterp = NULL;
 static int inTrackingLoop = 0;
 
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1050
+/*
+ * For InstallStandardApplicationEventHandler():
+ */
+
+static jmp_buf exitRaelJmpBuf;
+static void ExitRaelEventHandlerProc(EventHandlerCallRef callRef,
+	EventRef event, void *userData) __attribute__ ((__noreturn__));
+#endif
+
 
 /*
  *----------------------------------------------------------------------
@@ -130,8 +135,9 @@ CarbonEventHandlerProc(
     bzero(&eventStatus, sizeof(eventStatus));
 
 #ifdef TK_MAC_DEBUG_CARBON_EVENTS
-    if (macEvent.eKind != kEventMouseMoved &&
-	    macEvent.eKind != kEventMouseDragged) {
+    if (!(macEvent.eClass == kEventClassMouse && (
+	    macEvent.eKind == kEventMouseMoved ||
+	    macEvent.eKind == kEventMouseDragged))) {
 	TkMacOSXDbgMsg("Started handling %s",
 		TkMacOSXCarbonEventToAscii(event));
 	TkMacOSXInitNamedDebugSymbol(HIToolbox, void, _DebugPrintEvent,
@@ -216,46 +222,21 @@ TkMacOSXInitCarbonEvents(
 	    applicationEventTypes, (void *) carbonEventInterp, NULL);
 
 #ifdef TK_MAC_DEBUG_CARBON_EVENTS
-    TkMacOSXInitNamedDebugSymbol(HIToolbox, void, _TraceEventByName,
-	    CFStringRef);
-    if (_TraceEventByName) {
-	/* Carbon-internal event debugging (c.f. Technote 2124) */
-	_TraceEventByName(CFSTR("kEventRawKeyDown"));
-	_TraceEventByName(CFSTR("kEventRawKeyRepeat"));
-	_TraceEventByName(CFSTR("kEventRawKeyUp"));
-	_TraceEventByName(CFSTR("kEventRawKeyModifiersChanged"));
-	_TraceEventByName(CFSTR("kEventRawKeyRepeat"));
-	_TraceEventByName(CFSTR("kEventMenuBeginTracking"));
-	_TraceEventByName(CFSTR("kEventMenuEndTracking"));
-	_TraceEventByName(CFSTR("kEventMenuOpening"));
-	_TraceEventByName(CFSTR("kEventMenuTargetItem"));
-	_TraceEventByName(CFSTR("kEventCommandProcess"));
-	_TraceEventByName(CFSTR("kEventCommandUpdateStatus"));
-	_TraceEventByName(CFSTR("kEventAppActivated"));
-	_TraceEventByName(CFSTR("kEventAppDeactivated"));
-	_TraceEventByName(CFSTR("kEventAppQuit"));
-	_TraceEventByName(CFSTR("kEventAppHidden"));
-	_TraceEventByName(CFSTR("kEventAppShown"));
-	_TraceEventByName(CFSTR("kEventAppAvailableWindowBoundsChanged"));
-	_TraceEventByName(CFSTR("kEventAppearanceScrollBarVariantChanged"));
-	_TraceEventByName(CFSTR("kEventMouseDown"));
-	_TraceEventByName(CFSTR("kEventMouseUp"));
-#if 0
-	_TraceEventByName(CFSTR("kEventMouseMoved"));
-	_TraceEventByName(CFSTR("kEventMouseDragged"));
-#endif
-	_TraceEventByName(CFSTR("kEventMouseWheelMoved"));
-	_TraceEventByName(CFSTR("kEventMouseScroll"));
-	_TraceEventByName(CFSTR("kEventWindowActivated"));
-	_TraceEventByName(CFSTR("kEventWindowDeactivated"));
-	_TraceEventByName(CFSTR("kEventWindowUpdate"));
-	_TraceEventByName(CFSTR("kEventWindowExpanded"));
-	_TraceEventByName(CFSTR("kEventWindowBoundsChanged"));
-	_TraceEventByName(CFSTR("kEventWindowDragStarted"));
-	_TraceEventByName(CFSTR("kEventWindowDragCompleted"));
-	_TraceEventByName(CFSTR("kEventWindowConstrain"));
-	_TraceEventByName(CFSTR("kEventWindowGetRegion"));
-	_TraceEventByName(CFSTR("kEventWindowDrawContent"));
+    TkMacOSXInitNamedSymbol(HIToolbox, void, DebugTraceEvent, OSType, UInt32,
+	    Boolean);
+    if (DebugTraceEvent) {
+	unsigned int i;
+	const EventTypeSpec *e;
+
+	for (i = 0, e = dispatcherEventTypes;
+		i < GetEventTypeCount(dispatcherEventTypes); i++, e++) {
+	    DebugTraceEvent(e->eventClass, e->eventKind, 1);
+	}
+	for (i = 0, e = applicationEventTypes;
+		i < GetEventTypeCount(applicationEventTypes); i++, e++) {
+	    DebugTraceEvent(e->eventClass, e->eventKind, 1);
+	}
+	DebugTraceEvent = NULL; /* Only enable tracing once. */
     }
 #endif /* TK_MAC_DEBUG_CARBON_EVENTS */
 }
@@ -289,7 +270,7 @@ TkMacOSXInstallWindowCarbonEventHandler(
 	{kEventClassWindow,	 kEventWindowActivated},
 	{kEventClassWindow,	 kEventWindowDeactivated},
 	{kEventClassWindow,	 kEventWindowUpdate},
-	{kEventClassWindow,	 kEventWindowExpanded},
+	{kEventClassWindow,	 kEventWindowExpanding},
 	{kEventClassWindow,	 kEventWindowBoundsChanged},
 	{kEventClassWindow,	 kEventWindowDragStarted},
 	{kEventClassWindow,	 kEventWindowDragCompleted},
@@ -302,6 +283,24 @@ TkMacOSXInstallWindowCarbonEventHandler(
 	    carbonEventHandlerUPP, GetEventTypeCount(windowEventTypes),
 	    windowEventTypes, (void *) (interp ? interp : carbonEventInterp),
 	    NULL);
+
+#ifdef TK_MAC_DEBUG_CARBON_EVENTS
+    TkMacOSXInitNamedSymbol(HIToolbox, void, DebugTraceEvent, OSType, UInt32,
+	    Boolean);
+    if (DebugTraceEvent) {
+	unsigned int i;
+	const EventTypeSpec *e;
+
+	for (i = 0, e = windowEventTypes;
+		i < GetEventTypeCount(windowEventTypes); i++, e++) {
+	    if (!(e->eventClass == kEventClassMouse && (
+		    e->eventKind == kEventMouseMoved ||
+		    e->eventKind == kEventMouseDragged))) {
+		DebugTraceEvent(e->eventClass, e->eventKind, 1);
+	    }
+	}
+    }
+#endif /* TK_MAC_DEBUG_CARBON_EVENTS */
 }
 
 /*
@@ -324,65 +323,92 @@ TkMacOSXInstallWindowCarbonEventHandler(
 static OSStatus
 InstallStandardApplicationEventHandler(void)
 {
-   /*
-    * This is a hack to workaround missing Carbon API to install the standard
-    * application event handler (InstallStandardEventHandler() does not work
-    * on the application target). The only way to install the standard app
-    * handler is to call RunApplicationEventLoop(), but since we are running
-    * our own event loop, we'll immediately need to break out of RAEL again:
-    * we do this via longjmp out of the ExitRaelEventHandlerProc event handler
-    * called first off from RAEL by posting a high priority dummy event.
-    * This workaround is derived from a similar approach in Technical Q&A 1061.
-    */
-    enum {
-	kExitRaelEvent = 'ExiT'
-    };
-    const EventTypeSpec exitRaelEventType = {
-	kExitRaelEvent, kExitRaelEvent
-    };
-    EventHandlerUPP exitRaelEventHandler;
-    EventHandlerRef exitRaelEventHandlerRef = NULL;
-    EventRef exitRaelEvent = NULL;
     OSStatus err = memFullErr;
 
-    exitRaelEventHandler = NewEventHandlerUPP(
-	    (EventHandlerProcPtr) ExitRaelEventHandlerProc);
-    if (exitRaelEventHandler) {
-	err = ChkErr(InstallEventHandler, GetEventDispatcherTarget(),
-		exitRaelEventHandler, 1, &exitRaelEventType, NULL,
-		&exitRaelEventHandlerRef);
-    }
-    if (err == noErr) {
-	err = ChkErr(CreateEvent, NULL, kExitRaelEvent, kExitRaelEvent,
-		GetCurrentEventTime(), kEventAttributeNone, &exitRaelEvent);
-    }
-    if (err == noErr) {
-	err = ChkErr(PostEventToQueue, GetMainEventQueue(), exitRaelEvent,
-		kEventPriorityHigh);
-    }
-    if (err == noErr) {
-	if (!setjmp(exitRaelJmpBuf)) {
-	    RunApplicationEventLoop();
-
-	    /*
-	     * This point should never be reached!
-	     */
-
-	    Tcl_Panic("RunApplicationEventLoop exited !");
+    TK_IF_HI_TOOLBOX(5,
+       /*
+	* The approach below does not work correctly in Leopard, it leads to
+	* crashes in [NSView unlockFocus] whenever HIToolbox uses Cocoa (Help
+	* menu, Nav Services, Color Picker). While it is now possible to
+	* install the standard app handler with InstallStandardEventHandler(),
+	* to fully replicate RAEL the standard menubar event handler also needs
+	* to be installed. Unfortunately there appears to be no public API to
+	* obtain the menubar event target. As a workaround, for now we resort
+	* to calling the HIToolbox-internal GetMenuBarEventTarget() directly
+	* (symbol acquired via TkMacOSXInitNamedSymbol() from HIToolbox
+	* version 343, may not exist in later versions).
+	*/
+	err = ChkErr(InstallStandardEventHandler, GetApplicationEventTarget());
+	TkMacOSXInitNamedSymbol(HIToolbox, EventTargetRef,
+		GetMenuBarEventTarget, void);
+	if (GetMenuBarEventTarget) {
+	    ChkErr(InstallStandardEventHandler, GetMenuBarEventTarget());
+	} else {
+	    TkMacOSXDbgMsg("Unable to install standard menubar event handler");
 	}
-    }
-    if (exitRaelEvent) {
-	ReleaseEvent(exitRaelEvent);
-    }
-    if (exitRaelEventHandlerRef) {
-	RemoveEventHandler(exitRaelEventHandlerRef);
-    }
-    if (exitRaelEventHandler) {
-	DisposeEventHandlerUPP(exitRaelEventHandler);
-    }
+    ) TK_ELSE_HI_TOOLBOX (5,
+       /*
+	* This is a hack to workaround missing Carbon API to install the
+	* standard application event handler (InstallStandardEventHandler()
+	* does not work on the application target). The only way to install the
+	* standard app handler is to call RunApplicationEventLoop(), but since
+	* we are running our own event loop, we'll immediately need to break
+	* out of RAEL again: we do this via longjmp out of the
+	* ExitRaelEventHandlerProc event handler called first off from RAEL by
+	* posting a high priority dummy event. This workaround is derived from
+	* a similar approach in Technical Q&A 1061.
+	*/
+	enum {
+	    kExitRaelEvent = 'ExiT'
+	};
+	const EventTypeSpec exitRaelEventType = {
+	    kExitRaelEvent, kExitRaelEvent
+	};
+	EventHandlerUPP exitRaelEventHandler;
+	EventHandlerRef exitRaelEventHandlerRef = NULL;
+	EventRef exitRaelEvent = NULL;
+
+	exitRaelEventHandler = NewEventHandlerUPP(
+		(EventHandlerProcPtr) ExitRaelEventHandlerProc);
+	if (exitRaelEventHandler) {
+	    err = ChkErr(InstallEventHandler, GetEventDispatcherTarget(),
+		    exitRaelEventHandler, 1, &exitRaelEventType, NULL,
+		    &exitRaelEventHandlerRef);
+	}
+	if (err == noErr) {
+	    err = ChkErr(CreateEvent, NULL, kExitRaelEvent, kExitRaelEvent,
+		    GetCurrentEventTime(), kEventAttributeNone,
+		    &exitRaelEvent);
+	}
+	if (err == noErr) {
+	    err = ChkErr(PostEventToQueue, GetMainEventQueue(), exitRaelEvent,
+		    kEventPriorityHigh);
+	}
+	if (err == noErr) {
+	    if (!setjmp(exitRaelJmpBuf)) {
+		RunApplicationEventLoop();
+
+		/*
+		 * This point should never be reached!
+		 */
+
+		Tcl_Panic("RunApplicationEventLoop exited !");
+	    }
+	}
+	if (exitRaelEvent) {
+	    ReleaseEvent(exitRaelEvent);
+	}
+	if (exitRaelEventHandlerRef) {
+	    RemoveEventHandler(exitRaelEventHandlerRef);
+	}
+	if (exitRaelEventHandler) {
+	    DisposeEventHandlerUPP(exitRaelEventHandler);
+	}
+    ) TK_ENDIF
     return err;
 }
 
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1050
 /*
  *----------------------------------------------------------------------
  *
@@ -409,6 +435,7 @@ ExitRaelEventHandlerProc(
 {
     longjmp(exitRaelJmpBuf, 1);
 }
+#endif
 
 /*
  *----------------------------------------------------------------------
