@@ -9,8 +9,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkMacOSXDialog.c,v 1.4.2.21 2007/11/09 06:26:55 das Exp $
  */
 
 #include "tkMacOSXPrivate.h"
@@ -317,12 +315,15 @@ Tk_GetOpenFileObjCmd(
 		if (choiceLen == 0) { initialDir = NULL; }
 		break;
 	    case OPEN_INITFILE:
-		initialFile = Tcl_GetString(objv[i + 1]);
+	    initialFile = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
 		/* empty strings should be like no selection given */
 		if (choiceLen == 0) { initialFile = NULL; }
 		break;
 	    case OPEN_MESSAGE:
 		choice = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
+		if (message) {
+		    CFRelease(message);
+		}
 		message = CFStringCreateWithBytes(NULL, (unsigned char*)
 			choice, choiceLen, kCFStringEncodingUTF8, false);
 		break;
@@ -342,6 +343,9 @@ Tk_GetOpenFileObjCmd(
 		break;
 	    case OPEN_TITLE:
 		choice = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
+		if (title) {
+		    CFRelease(title);
+		}
 		title = CFStringCreateWithBytes(NULL, (unsigned char*)
 			choice, choiceLen, kCFStringEncodingUTF8, false);
 		break;
@@ -376,7 +380,8 @@ Tk_GetOpenFileObjCmd(
 	while (filterPtr && i-- > 0) {
 	    filterPtr = filterPtr->next;
 	}
-	Tcl_SetVar(interp, Tcl_GetString(typeVariablePtr), filterPtr->name, 0);
+	Tcl_SetVar(interp, Tcl_GetString(typeVariablePtr), filterPtr ?
+		filterPtr->name : "", 0);
     }
 #endif
 
@@ -486,6 +491,9 @@ Tk_GetSaveFileObjCmd(
 		break;
 	    case SAVE_MESSAGE:
 		choice = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
+		if (message) {
+		    CFRelease(message);
+		}
 		message = CFStringCreateWithBytes(NULL, (unsigned char*)
 			choice, choiceLen, kCFStringEncodingUTF8, false);
 		break;
@@ -499,6 +507,9 @@ Tk_GetSaveFileObjCmd(
 		break;
 	    case SAVE_TITLE:
 		choice = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
+		if (title) {
+		    CFRelease(title);
+		}
 		title = CFStringCreateWithBytes(NULL, (unsigned char*)
 			choice, choiceLen, kCFStringEncodingUTF8, false);
 		break;
@@ -592,6 +603,9 @@ Tk_ChooseDirectoryObjCmd(clientData, interp, objc, objv)
 		break;
 	    case CHOOSE_MESSAGE:
 		choice = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
+		if (message) {
+		    CFRelease(message);
+		}
 		message = CFStringCreateWithBytes(NULL, (unsigned char*)
 			choice, choiceLen, kCFStringEncodingUTF8, false);
 		break;
@@ -605,6 +619,9 @@ Tk_ChooseDirectoryObjCmd(clientData, interp, objc, objv)
 		break;
 	    case CHOOSE_TITLE:
 		choice = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
+		if (title) {
+		    CFRelease(title);
+		}
 		title = CFStringCreateWithBytes(NULL, (unsigned char*) choice,
 			choiceLen, kCFStringEncodingUTF8, false);
 		break;
@@ -1479,6 +1496,9 @@ Tk_MessageBoxObjCmd(
 #if 0
 	    case ALERT_DETAIL:
 		str = Tcl_GetString(objv[i + 1]);
+		if (finemessageTextCF) {
+		    CFRelease(finemessageTextCF);
+		}
 		finemessageTextCF = CFStringCreateWithCString(NULL, str,
 			kCFStringEncodingUTF8);
 		break;
@@ -1508,6 +1528,9 @@ Tk_MessageBoxObjCmd(
 
 	    case ALERT_MESSAGE:
 		str = Tcl_GetString(objv[i + 1]);
+		if (messageTextCF) {
+		    CFRelease(messageTextCF);
+		}
 		messageTextCF = CFStringCreateWithCString(NULL, str,
 			kCFStringEncodingUTF8);
 		break;
