@@ -1,17 +1,16 @@
 /*
  * tkMacOSXPort.h --
  *
- *	This file is included by all of the Tk C files.  It contains
+ *	This file is included by all of the Tk C files. It contains
  *	information that may be configuration-dependent, such as
  *	#includes for system include files and a few other things.
  *
  * Copyright (c) 1994-1996 Sun Microsystems, Inc.
  * Copyright 2001, Apple Computer, Inc.
+ * Copyright (c) 2005-2007 Daniel A. Steffen <das@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkMacOSXPort.h,v 1.3.2.3 2005/11/27 02:36:46 das Exp $
  */
 
 #ifndef _TKMACPORT
@@ -19,34 +18,26 @@
 
 /*
  * Macro to use instead of "void" for arguments that must have
- * type "void *" in ANSI C;  maps them to type "char *" in
- * non-ANSI systems.  This macro may be used in some of the include
+ * type "void *" in ANSI C; maps them to type "char *" in
+ * non-ANSI systems. This macro may be used in some of the include
  * files below, which is why it is defined here.
  */
 
 #ifndef VOID
 #   ifdef __STDC__
-#       define VOID void
+#	define VOID void
 #   else
-#       define VOID char
+#	define VOID char
 #   endif
 #endif
 
 #include <stdio.h>
 #include <ctype.h>
 #include <fcntl.h>
-#ifdef HAVE_LIMITS_H
-#   include <limits.h>
-#else
-#   include "../compat/limits.h"
-#endif
+#include <limits.h>
 #include <math.h>
 #include <pwd.h>
-#ifdef NO_STDLIB_H
-#   include "../compat/stdlib.h"
-#else
-#   include <stdlib.h>
-#endif
+#include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/file.h>
@@ -62,16 +53,12 @@
 #   include <time.h>
 #else
 #   if HAVE_SYS_TIME_H
-#       include <sys/time.h>
+#	include <sys/time.h>
 #   else
-#       include <time.h>
+#	include <time.h>
 #   endif
 #endif
-#ifdef HAVE_UNISTD_H
-#   include <unistd.h>
-#else
-#   include "../compat/unistd.h"
-#endif
+#include <unistd.h>
 #include <X11/Xlib.h>
 #include <X11/cursorfont.h>
 #include <X11/keysym.h>
@@ -117,13 +104,6 @@
 #endif
 #define MASK_SIZE howmany(FD_SETSIZE, NFDBITS)
 
-/*
- * Not all systems declare the errno variable in errno.h. so this
- * file does it explicitly.
- */
-
-extern int errno;
-
 /*
  * Define "NBBY" (number of bits per byte) if it's not already defined.
  */
@@ -133,18 +113,11 @@ extern int errno;
 #endif
 
 /*
- * Declarations for various library procedures that may not be declared
- * in any other header file.
+ * The following define causes Tk to use its internal keysym hash table
  */
 
-#ifndef strcasecmp
-extern int		strcasecmp _ANSI_ARGS_((CONST char *s1,
-			    CONST char *s2));
-#endif
-#ifndef strncasecmp			    
-extern int		strncasecmp _ANSI_ARGS_((CONST char *s1,
-			    CONST char *s2, size_t n));
-#endif
+#define REDO_KEYSYM_LOOKUP
+
 /*
  * Defines for X functions that are used by Tk but are treated as
  * no-op functions on the Macintosh.
@@ -188,7 +161,7 @@ extern int		strncasecmp _ANSI_ARGS_((CONST char *s1,
 
 #define TkpPrintWindowId(buf,w) \
 	sprintf((buf), "0x%x", (unsigned int) (w))
-	    
+
 /*
  * TkpScanWindowId is just an alias for Tcl_GetInt on Unix.
  */
@@ -197,11 +170,23 @@ extern int		strncasecmp _ANSI_ARGS_((CONST char *s1,
 	Tcl_GetInt((i),(s),(int *) (wp))
 
 /*
- * Magic pixel values for dynamic (or active) colors.
+ * Turn off Tk double-buffering as Aqua windows are already double-buffered.
  */
 
+#define TK_NO_DOUBLE_BUFFERING 1
+
+/*
+ * Magic pixel code values for system colors.
+ *
+ * NOTE: values must be kept in sync with indices into the
+ *	 systemColorMap array in tkMacOSXColor.c !
+ */
+
+#define TRANSPARENT_PIXEL		30
 #define HIGHLIGHT_PIXEL			31
+#define HIGHLIGHT_SECONDARY_PIXEL	32
 #define HIGHLIGHT_TEXT_PIXEL		33
+#define HIGHLIGHT_ALTERNATE_PIXEL	34
 #define CONTROL_TEXT_PIXEL		35
 #define CONTROL_BODY_PIXEL		37
 #define CONTROL_FRAME_PIXEL		39
