@@ -1,24 +1,30 @@
-/* 
+/*
  * tkMacOSXScale.c --
  *
- *	This file implements the Macintosh specific portion of the 
+ *	This file implements the Macintosh specific portion of the
  *	scale widget.
  *
  * Copyright (c) 1996 by Sun Microsystems, Inc.
  * Copyright (c) 1998-2000 by Scriptics Corporation.
+ * Copyright (c) 2006-2007 Daniel A. Steffen <das@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkMacOSXScale.c,v 1.2.2.5 2006/04/28 06:03:00 das Exp $
  */
 
-#include "tkMacOSXInt.h"
+#include "tkMacOSXPrivate.h"
 #include "tkScale.h"
 
+/*
+#ifdef TK_MAC_DEBUG
+#define TK_MAC_DEBUG_SCALE
+#endif
+*/
+
 /*
  * Defines used in this file.
  */
+
 #define slider		1110
 #define inSlider	1
 #define inInc		2
@@ -43,10 +49,10 @@ static ControlActionUPP scaleActionProc = NULL; /* Pointer to func. */
  * Forward declarations for procedures defined later in this file:
  */
 
-static void		MacScaleEventProc _ANSI_ARGS_((ClientData clientData,
-			    XEvent *eventPtr));
-static pascal void	ScaleActionProc _ANSI_ARGS_((ControlRef theControl,
-			    ControlPartCode partCode));
+static void MacScaleEventProc(ClientData clientData, XEvent *eventPtr);
+static pascal void ScaleActionProc(ControlRef theControl,
+	ControlPartCode partCode);
+
 
 /*
  *----------------------------------------------------------------------
@@ -65,20 +71,19 @@ static pascal void	ScaleActionProc _ANSI_ARGS_((ControlRef theControl,
  */
 
 TkScale *
-TkpCreateScale(tkwin)
-    Tk_Window tkwin;
+TkpCreateScale(
+    Tk_Window tkwin)
 {
-    MacScale *macScalePtr;
-    
-    macScalePtr = (MacScale *) ckalloc(sizeof(MacScale));
+    MacScale *macScalePtr = (MacScale *) ckalloc(sizeof(MacScale));
+
     macScalePtr->scaleHandle = NULL;
     if (scaleActionProc == NULL) {
 	scaleActionProc = NewControlActionUPP(ScaleActionProc);
     }
-    
+
     Tk_CreateEventHandler(tkwin, ButtonPressMask,
 	    MacScaleEventProc, (ClientData) macScalePtr);
-	    
+
     return (TkScale *) macScalePtr;
 }
 
@@ -99,16 +104,17 @@ TkpCreateScale(tkwin)
  */
 
 void
-TkpDestroyScale(scalePtr)
-    TkScale *scalePtr;
+TkpDestroyScale(
+    TkScale *scalePtr)
 {
     MacScale *macScalePtr = (MacScale *) scalePtr;
-    
+
     /*
      * Free Macintosh control.
      */
+
     if (macScalePtr->scaleHandle != NULL) {
-        DisposeControl(macScalePtr->scaleHandle);
+	DisposeControl(macScalePtr->scaleHandle);
     }
 }
 
@@ -130,8 +136,8 @@ TkpDestroyScale(scalePtr)
  */
 
 void
-TkpDisplayScale(clientData)
-    ClientData clientData;	/* Widget record for scale. */
+TkpDisplayScale(
+    ClientData clientData)	/* Widget record for scale. */
 {
     TkScale *scalePtr = (TkScale *) clientData;
     Tk_Window tkwin = scalePtr->tkwin;
@@ -141,18 +147,14 @@ TkpDisplayScale(clientData)
     MacScale *macScalePtr = (MacScale *) clientData;
     Rect r;
     WindowRef windowRef;
-    CGrafPtr destPort;        
-    CGrafPtr saveWorld;
-    GDHandle saveDevice;
+    CGrafPtr destPort, savePort;
+    Boolean portChanged;
     MacDrawable *macDraw;
-    SInt32       initialValue;
-    SInt32       minValue;
-    SInt32       maxValue;
-    UInt16       numTicks;
-    
+    SInt32 initialValue, minValue, maxValue;
+    UInt16 numTicks;
 
-#ifdef TK_MAC_DEBUG
-    fprintf(stderr,"TkpDisplayScale\n");
+#ifdef TK_MAC_DEBUG_SCALE
+    TkMacOSXDbgMsg("TkpDisplayScale");
 #endif
     scalePtr->flags &= ~REDRAW_PENDING;
     if ((scalePtr->tkwin == NULL) || !Tk_IsMapped(scalePtr->tkwin)) {
@@ -167,8 +169,7 @@ TkpDisplayScale(clientData)
     if ((scalePtr->flags & INVOKE_COMMAND) && (scalePtr->command != NULL)) {
 	Tcl_Preserve((ClientData) interp);
 	sprintf(string, scalePtr->format, scalePtr->value);
-	result = Tcl_VarEval(interp, scalePtr->command, " ", string,
-		(char *) NULL);
+	result = Tcl_VarEval(interp, scalePtr->command, " ", string, NULL);
 	if (result != TCL_OK) {
 	    Tcl_AddErrorInfo(interp, "\n    (command executed by scale)");
 	    Tcl_BackgroundError(interp);
@@ -184,14 +185,13 @@ TkpDisplayScale(clientData)
 
     /*
      * Now handle the part of redisplay that is the same for
-     * horizontal and vertical scales:  border and traversal
+     * horizontal and vertical scales: border and traversal
      * highlight.
      */
 
     if (scalePtr->highlightWidth != 0) {
-	GC gc;
-    
-	gc = Tk_GCForColor(scalePtr->highlightColorPtr, Tk_WindowId(tkwin));
+	GC gc = Tk_GCForColor(scalePtr->highlightColorPtr, Tk_WindowId(tkwin));
+
 	Tk_DrawFocusHighlight(tkwin, gc, scalePtr->highlightWidth,
 		Tk_WindowId(tkwin));
     }
@@ -204,11 +204,11 @@ TkpDisplayScale(clientData)
     /*
      * Set up port for drawing Macintosh control.
      */
+
     macDraw = (MacDrawable *) Tk_WindowId(tkwin);
     destPort = TkMacOSXGetDrawablePort(Tk_WindowId(tkwin));
-    windowRef = GetWindowFromPort(destPort);
-    GetGWorld(&saveWorld, &saveDevice);
-    SetGWorld(destPort, NULL);
+    windowRef = TkMacOSXDrawableWindow(Tk_WindowId(tkwin));
+    portChanged = QDSwapPort(destPort, &savePort);
     TkMacOSXSetUpClippingRgn(Tk_WindowId(tkwin));
 
     /*
@@ -218,78 +218,74 @@ TkpDisplayScale(clientData)
 #define MAC_OSX_SCROLL_WIDTH 10
 
     if (scalePtr->orient == ORIENT_HORIZONTAL) {
-        int offset;
-        offset = (Tk_Height(tkwin) - MAC_OSX_SCROLL_WIDTH)/2;
-        if (offset < 0) {
-            offset = 0;
-        }
-        
-        r.left = macDraw->xOff + scalePtr->inset;
-        r.top = macDraw->yOff + offset;
-        r.right = macDraw->xOff+Tk_Width(tkwin) - scalePtr->inset;
-        r.bottom = macDraw->yOff + offset + MAC_OSX_SCROLL_WIDTH/2;
+	int offset = (Tk_Height(tkwin) - MAC_OSX_SCROLL_WIDTH)/2;
+
+	if (offset < 0) {
+	    offset = 0;
+	}
+
+	r.left = macDraw->xOff + scalePtr->inset;
+	r.top = macDraw->yOff + offset;
+	r.right = macDraw->xOff+Tk_Width(tkwin) - scalePtr->inset;
+	r.bottom = macDraw->yOff + offset + MAC_OSX_SCROLL_WIDTH/2;
     } else {
-        int offset;
-    
-        offset = (Tk_Width(tkwin) - MAC_OSX_SCROLL_WIDTH)/2;
-        if (offset < 0) {
-            offset = 0;
-        }
-        
-        r.left = macDraw->xOff + offset;
-        r.top = macDraw->yOff + scalePtr->inset;
-        r.right = macDraw->xOff + offset + MAC_OSX_SCROLL_WIDTH/2;
-        r.bottom = macDraw->yOff+Tk_Height(tkwin) - scalePtr->inset;
+	int offset = (Tk_Width(tkwin) - MAC_OSX_SCROLL_WIDTH)/2;
+
+	if (offset < 0) {
+	    offset = 0;
+	}
+
+	r.left = macDraw->xOff + offset;
+	r.top = macDraw->yOff + scalePtr->inset;
+	r.right = macDraw->xOff + offset + MAC_OSX_SCROLL_WIDTH/2;
+	r.bottom = macDraw->yOff+Tk_Height(tkwin) - scalePtr->inset;
     }
 
     if (macScalePtr->scaleHandle == NULL) {
-        
-#ifdef TK_MAC_DEBUG
-        fprintf(stderr,"Initialising scale\n");
+#ifdef TK_MAC_DEBUG_SCALE
+	TkMacOSXDbgMsg("Initialising scale");
 #endif
+	initialValue = scalePtr->value;
+	if (scalePtr->orient == ORIENT_HORIZONTAL) {
+	    minValue = scalePtr->fromValue;
+	    maxValue = scalePtr->toValue;
+	} else {
+	    minValue = scalePtr->fromValue;
+	    maxValue = scalePtr->toValue;
+	}
 
-        initialValue = scalePtr->value;
-        if (scalePtr->orient == ORIENT_HORIZONTAL) {
-            minValue = scalePtr->fromValue;
-            maxValue = scalePtr->toValue;
-        } else {
-            minValue = scalePtr->fromValue;
-            maxValue = scalePtr->toValue;
-        }
-        
-        if (scalePtr->tickInterval == 0) {
-            numTicks = 0;
-        } else {
-            numTicks = (maxValue - minValue)/scalePtr->tickInterval;
-        }
-                
-        CreateSliderControl(windowRef, &r, initialValue, minValue, maxValue,
-                kControlSliderPointsDownOrRight, numTicks, 
-                1, scaleActionProc, 
-                &(macScalePtr->scaleHandle));
-        SetControlReference(macScalePtr->scaleHandle, (UInt32) scalePtr);
+	if (scalePtr->tickInterval == 0) {
+	    numTicks = 0;
+	} else {
+	    numTicks = (maxValue - minValue)/scalePtr->tickInterval;
+	}
 
-	/*
-	 * If we are foremost than make us active.
-	 */
-	if (windowRef == FrontWindow()) {
+	CreateSliderControl(windowRef, &r, initialValue, minValue, maxValue,
+		kControlSliderPointsDownOrRight, numTicks, 1, scaleActionProc,
+		&(macScalePtr->scaleHandle));
+	SetControlReference(macScalePtr->scaleHandle, (UInt32) scalePtr);
+
+	if (IsWindowActive(windowRef)) {
 	    macScalePtr->flags |= ACTIVE;
 	}
     } else {
-        SetControlBounds(macScalePtr->scaleHandle, &r);
-        SetControl32BitValue(macScalePtr->scaleHandle, scalePtr->value);
-        SetControl32BitMinimum(macScalePtr->scaleHandle, scalePtr->fromValue);
-        SetControl32BitMaximum(macScalePtr->scaleHandle, scalePtr->toValue);
+	SetControlBounds(macScalePtr->scaleHandle, &r);
+	SetControl32BitValue(macScalePtr->scaleHandle, scalePtr->value);
+	SetControl32BitMinimum(macScalePtr->scaleHandle, scalePtr->fromValue);
+	SetControl32BitMaximum(macScalePtr->scaleHandle, scalePtr->toValue);
     }
 
     /*
      * Finally draw the control.
      */
+
     SetControlVisibility(macScalePtr->scaleHandle,true,true);
     HiliteControl(macScalePtr->scaleHandle,0);
     Draw1Control(macScalePtr->scaleHandle);
 
-    SetGWorld(saveWorld, saveDevice);
+    if (portChanged) {
+	QDSwapPort(savePort, NULL);
+    }
 done:
     scalePtr->flags &= ~REDRAW_ALL;
 }
@@ -314,57 +310,57 @@ done:
  */
 
 int
-TkpScaleElement(scalePtr, x, y)
-    TkScale *scalePtr;		/* Widget record for scale. */
-    int x, y;			/* Coordinates within scalePtr's window. */
+TkpScaleElement(
+    TkScale *scalePtr,		/* Widget record for scale. */
+    int x, int y)		/* Coordinates within scalePtr's window. */
 {
     MacScale *macScalePtr = (MacScale *) scalePtr;
     ControlPartCode part;
     Point where;
     Rect bounds;
-    CGrafPtr saveWorld;
-    GDHandle saveDevice;
-    GWorldPtr destPort;        
-#ifdef TK_MAC_DEBUG
-    fprintf(stderr,"TkpScaleElement\n");
-#endif
+    CGrafPtr destPort, savePort;
+    Boolean portChanged;
 
+#ifdef TK_MAC_DEBUG_SCALE
+    TkMacOSXDbgMsg("TkpScaleElement");
+#endif
     destPort = TkMacOSXGetDrawablePort(Tk_WindowId(scalePtr->tkwin));
-    GetGWorld(&saveWorld, &saveDevice);
-    SetGWorld(destPort, NULL);
+    portChanged = QDSwapPort(destPort, &savePort);
 
     /*
      * All of the calculations in this procedure mirror those in
-     * DisplayScrollbar.  Be sure to keep the two consistent.
+     * DisplayScrollbar. Be sure to keep the two consistent.
      */
 
-    TkMacOSXWinBounds((TkWindow *) scalePtr->tkwin, &bounds);		
+    TkMacOSXWinBounds((TkWindow *) scalePtr->tkwin, &bounds);
     where.h = x + bounds.left;
     where.v = y + bounds.top;
     part = TestControl(macScalePtr->scaleHandle, where);
-    
-    SetGWorld(saveWorld, saveDevice);
 
-#ifdef TK_MAC_DEBUG
+    if (portChanged) {
+	QDSwapPort(savePort, NULL);
+    }
+
+#ifdef TK_MAC_DEBUG_SCALE
     fprintf (stderr,"ScalePart %d, pos ( %d %d )\n", part, where.h, where.v );
 #endif
-    
+
     switch (part) {
-    	case inSlider:
+	case inSlider:
 	    return SLIDER;
-    	case inInc:
+	case inInc:
 	    if (scalePtr->orient == ORIENT_VERTICAL) {
 		return TROUGH1;
 	    } else {
 		return TROUGH2;
 	    }
-    	case inDecr:
+	case inDecr:
 	    if (scalePtr->orient == ORIENT_VERTICAL) {
 		return TROUGH2;
 	    } else {
 		return TROUGH1;
 	    }
-    	default:
+	default:
 	    return OTHER;
     }
 }
@@ -374,7 +370,7 @@ TkpScaleElement(scalePtr, x, y)
  *
  * MacScaleEventProc --
  *
- *	This procedure is invoked by the Tk dispatcher for 
+ *	This procedure is invoked by the Tk dispatcher for
  *	ButtonPress events on scales.
  *
  * Results:
@@ -382,63 +378,70 @@ TkpScaleElement(scalePtr, x, y)
  *
  * Side effects:
  *	When the window gets deleted, internal structures get
- *	cleaned up.  When it gets exposed, it is redisplayed.
+ *	cleaned up. When it gets exposed, it is redisplayed.
  *
  *--------------------------------------------------------------
  */
 
 static void
-MacScaleEventProc(clientData, eventPtr)
-    ClientData clientData;	/* Information about window. */
-    XEvent *eventPtr;		/* Information about event. */
+MacScaleEventProc(
+    ClientData clientData,	/* Information about window. */
+    XEvent *eventPtr)		/* Information about event. */
 {
     MacScale *macScalePtr = (MacScale *) clientData;
     Point where;
     Rect bounds;
     int part;
-    CGrafPtr saveWorld;
-    GDHandle saveDevice;
-    GWorldPtr destPort;
+    CGrafPtr destPort, savePort;
+    Boolean portChanged;
 
-#ifdef TK_MAC_DEBUG
+#ifdef TK_MAC_DEBUG_SCALE
     fprintf(stderr,"MacScaleEventProc\n" );
 #endif
+
     /*
      * To call Macintosh control routines we must have the port
-     * set to the window containing the control.  We will then test
+     * set to the window containing the control. We will then test
      * which part of the control was hit and act accordingly.
      */
+
     destPort = TkMacOSXGetDrawablePort(Tk_WindowId(macScalePtr->info.tkwin));
-    GetGWorld(&saveWorld, &saveDevice);
-    SetGWorld(destPort, NULL);
+    portChanged = QDSwapPort(destPort, &savePort);
     TkMacOSXSetUpClippingRgn(Tk_WindowId(macScalePtr->info.tkwin));
 
-    TkMacOSXWinBounds((TkWindow *) macScalePtr->info.tkwin, &bounds);		
+    TkMacOSXWinBounds((TkWindow *) macScalePtr->info.tkwin, &bounds);
     where.h = eventPtr->xbutton.x + bounds.left;
     where.v = eventPtr->xbutton.y + bounds.top;
-#ifdef TK_MAC_DEBUG
-    fprintf(stderr,"calling TestControl\n");
+#ifdef TK_MAC_DEBUG_SCALE
+    TkMacOSXDbgMsg("calling TestControl");
 #endif
     part = TestControl(macScalePtr->scaleHandle, where);
     if (part == 0) {
 	return;
     }
-    
-    part = TrackControl(macScalePtr->scaleHandle, where, scaleActionProc);
-    
+
+    TkMacOSXTrackingLoop(1);
+    part = HandleControlClick(macScalePtr->scaleHandle, where,
+	    TkMacOSXModifierState(), scaleActionProc);
+    TkMacOSXTrackingLoop(0);
+
     /*
      * Update the value for the widget.
      */
+
     macScalePtr->info.value = GetControlValue(macScalePtr->scaleHandle);
     /* TkScaleSetValue(&macScalePtr->info, macScalePtr->info.value, 1, 0); */
 
     /*
-     * The TrackControl call will "eat" the ButtonUp event.  We now
+     * The HandleControlClick call will "eat" the ButtonUp event. We now
      * generate a ButtonUp event so Tk will unset implicit grabs etc.
      */
+
     TkGenerateButtonEventForXPointer(Tk_WindowId(macScalePtr->info.tkwin));
 
-    SetGWorld(saveWorld, saveDevice);
+    if (portChanged) {
+	QDSwapPort(savePort, NULL);
+    }
 }
 
 /*
@@ -447,8 +450,8 @@ MacScaleEventProc(clientData, eventPtr)
  * ScaleActionProc --
  *
  *	Callback procedure used by the Macintosh toolbox call
- *	TrackControl.  This call will update the display while
- *	the scrollbar is being manipulated by the user.
+ *	HandleControlClick. This call will update the display
+ *	while the scrollbar is being manipulated by the user.
  *
  * Results:
  *	None.
@@ -467,13 +470,13 @@ ScaleActionProc(
     int value;
     TkScale *scalePtr = (TkScale *) GetControlReference(theControl);
 
-#ifdef TK_MAC_DEBUG
-    fprintf(stderr,"ScaleActionProc\n");
+#ifdef TK_MAC_DEBUG_SCALE
+    TkMacOSXDbgMsg("ScaleActionProc");
 #endif
-    value =  GetControlValue(theControl);
+    value = GetControlValue(theControl);
     TkScaleSetValue(scalePtr, value, 1, 1);
     Tcl_Preserve((ClientData) scalePtr);
-    Tcl_DoOneEvent(TCL_IDLE_EVENTS);
+    TkMacOSXRunTclEventLoop();
     Tcl_Release((ClientData) scalePtr);
 }
 
