@@ -12,8 +12,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkMacOSXXStubs.c,v 1.2.2.20 2007/11/09 06:26:57 das Exp $
  */
 
 #include "tkMacOSXPrivate.h"
@@ -466,12 +464,13 @@ XSelectInput(
     Debugger();
 }
 
-void
+int
 XBell(
     Display* display,
     int percent)
 {
     SysBeep(percent);
+    return Success;
 }
 
 #if 0
@@ -551,7 +550,7 @@ XDrawPoints(
 }
 */
 
-void
+int
 XWarpPointer(
     Display* display,
     Window src_w,
@@ -563,6 +562,7 @@ XWarpPointer(
     int dest_x,
     int dest_y)
 {
+    return Success;
 }
 
 void
@@ -1284,6 +1284,6 @@ Tk_GetUserInactiveTime(Display *dpy)
 void
 Tk_ResetUserInactiveTime(Display *dpy)
 {
-    UpdateSystemActivity(OverallAct);
+    UpdateSystemActivity(UsrActivity);
 }
 #endif
