@@ -3,12 +3,10 @@
  *
  *	This file contains X specific cursor manipulation routines.
  *
- * Copyright (c) 1995 Sun Microsystems, Inc.
+ * Copyright (c) 1995-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkUnixCursor.c,v 1.3 1998/09/14 18:23:55 stanton Exp $
  */
 
 #include "tkPort.h"
@@ -154,7 +152,7 @@ TkGetCursorByName(interp, tkwin, string)
     TkUnixCursor *cursorPtr = NULL;
     Cursor cursor = None;
     int argc;
-    char **argv = NULL;
+    CONST char **argv = NULL;
     Pixmap source = None;
     Pixmap mask = None;
     Display *display = Tk_Display(tkwin);
@@ -196,7 +194,7 @@ TkGetCursorByName(interp, tkwin, string)
 	    fg.red = fg.green = fg.blue = 0;
 	    bg.red = bg.green = bg.blue = 65535;
 	} else {
-	    if (XParseColor(display, Tk_Colormap(tkwin), argv[1],
+	    if (TkParseColor(display, Tk_Colormap(tkwin), argv[1],
 		    &fg) == 0) {
 		Tcl_AppendResult(interp, "invalid color name \"", argv[1],
 			"\"", (char *) NULL);
@@ -206,7 +204,7 @@ TkGetCursorByName(interp, tkwin, string)
 		bg.red = bg.green = bg.blue = 0;
 		maskIndex = namePtr->shape;
 	    } else {
-		if (XParseColor(display, Tk_Colormap(tkwin), argv[2],
+		if (TkParseColor(display, Tk_Colormap(tkwin), argv[2],
 			&bg) == 0) {
 		    Tcl_AppendResult(interp, "invalid color name \"", argv[2],
 			    "\"", (char *) NULL);
@@ -218,7 +216,7 @@ TkGetCursorByName(interp, tkwin, string)
 	if (dispPtr->cursorFont == None) {
 	    dispPtr->cursorFont = XLoadFont(display, CURSORFONT);
 	    if (dispPtr->cursorFont == None) {
-		interp->result = "couldn't load cursor font";
+		Tcl_SetResult(interp, "couldn't load cursor font", TCL_STATIC);
 		goto cleanup;
 	    }
 	}
@@ -264,7 +262,7 @@ TkGetCursorByName(interp, tkwin, string)
 	    goto cleanup;
 	}
 	if (argc == 2) {
-	    if (XParseColor(display, Tk_Colormap(tkwin), argv[1],
+	    if (TkParseColor(display, Tk_Colormap(tkwin), argv[1],
 		    &fg) == 0) {
 		Tcl_AppendResult(interp, "invalid color name \"",
 			argv[1], "\"", (char *) NULL);
@@ -282,17 +280,18 @@ TkGetCursorByName(interp, tkwin, string)
 		goto cleanup;
 	    }
 	    if ((maskWidth != width) && (maskHeight != height)) {
-		interp->result =
-			"source and mask bitmaps have different sizes";
+		Tcl_SetResult(interp,
+			"source and mask bitmaps have different sizes",
+			TCL_STATIC);
 		goto cleanup;
 	    }
-	    if (XParseColor(display, Tk_Colormap(tkwin), argv[2],
+	    if (TkParseColor(display, Tk_Colormap(tkwin), argv[2],
 		    &fg) == 0) {
 		Tcl_AppendResult(interp, "invalid color name \"", argv[2],
 			"\"", (char *) NULL);
 		goto cleanup;
 	    }
-	    if (XParseColor(display, Tk_Colormap(tkwin), argv[3],
+	    if (TkParseColor(display, Tk_Colormap(tkwin), argv[3],
 		    &bg) == 0) {
 		Tcl_AppendResult(interp, "invalid color name \"", argv[3],
 			"\"", (char *) NULL);
@@ -323,6 +322,9 @@ TkGetCursorByName(interp, tkwin, string)
 
 
     badString:
+    if (argv) {
+	ckfree((char *) argv);
+    }
     Tcl_AppendResult(interp, "bad cursor spec \"", string, "\"",
 	    (char *) NULL);
     return NULL;
@@ -348,8 +350,8 @@ TkCursor *
 TkCreateCursorFromData(tkwin, source, mask, width, height, xHot, yHot,
 	fgColor, bgColor)
     Tk_Window tkwin;		/* Window in which cursor will be used. */
-    char *source;		/* Bitmap data for cursor shape. */
-    char *mask;			/* Bitmap data for cursor mask. */
+    CONST char *source;		/* Bitmap data for cursor shape. */
+    CONST char *mask;		/* Bitmap data for cursor mask. */
     int width, height;		/* Dimensions of cursor. */
     int xHot, yHot;		/* Location of hot-spot in cursor. */
     XColor fgColor;		/* Foreground color for cursor. */
@@ -382,7 +384,7 @@ TkCreateCursorFromData(tkwin, source, mask, width, height, xHot, yHot,
 /*
  *----------------------------------------------------------------------
  *
- * TkFreeCursor --
+ * TkpFreeCursor --
  *
  *	This procedure is called to release a cursor allocated by
  *	TkGetCursorByName.
@@ -397,11 +399,10 @@ TkCreateCursorFromData(tkwin, source, mask, width, height, xHot, yHot,
  */
 
 void
-TkFreeCursor(cursorPtr)
+TkpFreeCursor(cursorPtr)
     TkCursor *cursorPtr;
 {
     TkUnixCursor *unixCursorPtr = (TkUnixCursor *) cursorPtr;
     XFreeCursor(unixCursorPtr->display, (Cursor) unixCursorPtr->info.cursor);
     Tk_FreeXId(unixCursorPtr->display, (XID) unixCursorPtr->info.cursor);
-    ckfree((char *) unixCursorPtr);
 }
