@@ -8,8 +8,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkUnixEvent.c,v 1.11.2.5 2005/12/05 22:40:08 hobbs Exp $
  */
 
 #include "tkInt.h"
@@ -343,19 +341,22 @@ static void
 TransferXEventsToTcl(display)
     Display *display;
 {
-    int numFound;
     XEvent event;
 
-    numFound = QLength(display);
-
     /*
-     * Transfer events from the X event queue to the Tk event queue.
+     * Transfer events from the X event queue to the Tk event queue
+     * after XIM event filtering.  KeyPress and KeyRelease events
+     * are filtered in Tk_HandleEvent instead of here, so that Tk's
+     * focus management code can redirect them.
      */
-
-    while (numFound > 0) {
+    while (QLength(display) > 0) {
 	XNextEvent(display, &event);
+	if (event.type != KeyPress && event.type != KeyRelease) {
+	    if (XFilterEvent(&event, None)) {
+		continue;
+	    }
+	}
 	Tk_QueueWindowEvent(&event, TCL_QUEUE_TAIL);
-	numFound--;
     }
 }
 
@@ -463,13 +464,12 @@ DisplayFileProc(clientData, flags)
  *
  * TkUnixDoOneXEvent --
  *
- *	This routine waits for an X event to be processed or for
- *	a timeout to occur.  The timeout is specified as an absolute
- *	time.  This routine is called when Tk needs to wait for a
- *	particular X event without letting arbitrary events be
- *	processed.  The caller will typically call Tk_RestrictEvents
- *	to set up an event filter before calling this routine.  This
- *	routine will service at most one event per invocation.
+ *	This routine waits for an X event to be processed or for a timeout to
+ *	occur. The timeout is specified as an absolute time. This routine is
+ *	called when Tk needs to wait for a particular X event without letting
+ *	arbitrary events be processed. The caller will typically call
+ *	Tk_RestrictEvents to set up an event filter before calling this
+ *	routine. This routine will service at most one event per invocation.
  *
  * Results:
  *	Returns 0 if the timeout has expired, otherwise returns 1.
@@ -482,14 +482,15 @@ DisplayFileProc(clientData, flags)
 
 int
 TkUnixDoOneXEvent(timePtr)
-    Tcl_Time *timePtr;		/* Specifies the absolute time when the
-				 * call should time out. */
+    Tcl_Time *timePtr;		/* Specifies the absolute time when the call
+				 * should time out. */
 {
     TkDisplay *dispPtr;
     static fd_mask readMask[MASK_SIZE];
     struct timeval blockTime, *timeoutPtr;
     Tcl_Time now;
-    int fd, index, bit, numFound, numFdBits = 0;
+    int fd, index, numFound, numFdBits = 0;
+    fd_mask bit;
 
     /*
      * Look for queued events first. 
@@ -500,9 +501,9 @@ TkUnixDoOneXEvent(timePtr)
     }
 
     /*
-     * Compute the next block time and check to see if we have timed out.
-     * Note that HP-UX defines tv_sec to be unsigned so we have to be
-     * careful in our arithmetic.
+     * Compute the next block time and check to see if we have timed out. Note
+     * that HP-UX defines tv_sec to be unsigned so we have to be careful in
+     * our arithmetic.
      */
 
     if (timePtr) {
@@ -525,21 +526,26 @@ TkUnixDoOneXEvent(timePtr)
     }
 
     /*
-     * Set up the select mask for all of the displays.  If a display has
-     * data pending, then we want to poll instead of blocking.
+     * Set up the select mask for all of the displays. If a display has data
+     * pending, then we want to poll instead of blocking.
      */
 
     memset((VOID *) readMask, 0, MASK_SIZE*sizeof(fd_mask));
     for (dispPtr = TkGetDisplayList(); dispPtr != NULL;
-	 dispPtr = dispPtr->nextPtr) {
+	    dispPtr = dispPtr->nextPtr) {
 	XFlush(dispPtr->display);
 	if (QLength(dispPtr->display) > 0) {
 	    blockTime.tv_sec = 0;
 	    blockTime.tv_usec = 0;
 	}
 	fd = ConnectionNumber(dispPtr->display);
+
+	/*
+	 * Assume there are always at least 'fd' bits in the 'readMask' array.
+	 */
+
 	index = fd/(NBBY*sizeof(fd_mask));
-	bit = 1 << (fd%(NBBY*sizeof(fd_mask)));
+	bit = ((fd_mask)1) << (fd%(NBBY*sizeof(fd_mask)));
 	readMask[index] |= bit;
 	if (numFdBits <= fd) {
 	    numFdBits = fd+1;
@@ -550,8 +556,8 @@ TkUnixDoOneXEvent(timePtr)
 	    timeoutPtr);
     if (numFound <= 0) {
 	/*
-	 * Some systems don't clear the masks after an error, so
-	 * we have to do it here.
+	 * Some systems don't clear the masks after an error, so we have to do
+	 * it here.
 	 */
 
 	memset((VOID *) readMask, 0, MASK_SIZE*sizeof(fd_mask));
@@ -562,10 +568,10 @@ TkUnixDoOneXEvent(timePtr)
      */
 
     for (dispPtr = TkGetDisplayList(); dispPtr != NULL;
-	 dispPtr = dispPtr->nextPtr) {
+	    dispPtr = dispPtr->nextPtr) {
 	fd = ConnectionNumber(dispPtr->display);
 	index = fd/(NBBY*sizeof(fd_mask));
-	bit = 1 << (fd%(NBBY*sizeof(fd_mask)));
+	bit = ((fd_mask)1) << (fd%(NBBY*sizeof(fd_mask)));
 	if ((readMask[index] & bit) || (QLength(dispPtr->display) > 0)) {
 	    DisplayFileProc((ClientData)dispPtr, TCL_READABLE);
 	}
@@ -587,8 +593,8 @@ TkUnixDoOneXEvent(timePtr)
     }
 
     /*
-     * We had an event but we did not generate a Tcl event from it. Behave
-     * as though we dealt with it. (JYL&SS)
+     * We had an event but we did not generate a Tcl event from it. Behave as
+     * though we dealt with it. (JYL&SS)
      */
 
     return 1;
@@ -650,31 +656,10 @@ OpenIM(dispPtr)
 {
     unsigned short i;
     XIMStyles *stylePtr;
-    char *modifiers;
 
-    modifiers = XSetLocaleModifiers("");
-    if (modifiers == NULL) {
+    if (XSetLocaleModifiers("") == NULL) {
 	goto error;
     }
-#if 1
-    /* 
-     * This is a temporary hack that can be taken back out again
-     * once Tk has learned how to deal with SCIM
-     */
-    while ((modifiers = strchr(modifiers, '@')) != NULL) {
-	if (strncmp(modifiers, "@im=", 4) == 0) {
-	    /* The first "@im=" entry wins */
-	    const char *scim = "@im=SCIM";
-	    if (strncmp(modifiers, scim, strlen(scim)) == 0) {
-		/* If it is SCIM, we override it */
-		if (XSetLocaleModifiers("@im=local") == NULL) {
-		    goto error;
-		}
-	    }
-	    break;
-	}
-    }
-#endif
 
     dispPtr->inputMethod = XOpenIM(dispPtr->display, NULL, NULL, NULL);
     if (dispPtr->inputMethod == NULL) {
