@@ -8,20 +8,19 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkUnixFont.c,v 1.8.2.2 2001/04/05 22:09:13 hobbs Exp $
  */
  
 #include "tkUnixInt.h"
 #include "tkFont.h"
 #include <netinet/in.h>		/* for htons() prototype */
+#include <arpa/inet.h>		/* inet_ntoa() */
 
 /*
  * The preferred font encodings.
  */
 
 static CONST char *encodingList[] = {
-    "ucs-2be", "iso8859-1", "jis0208", "jis0212", NULL
+    "iso8859-1", "jis0208", "jis0212", NULL
 };
 
 /*
@@ -175,7 +174,7 @@ static Tcl_ThreadDataKey dataKey;
  */
  
 static EncodingAlias encodingAliases[] = {
-    {"gb2312",		"gb2312*"},
+    {"gb2312-raw",	"gb2312*"},
     {"big5",		"big5*"},
     {"cns11643-1",	"cns11643*-1"},
     {"cns11643-1",	"cns11643*.1-0"},
@@ -189,7 +188,14 @@ static EncodingAlias encodingAliases[] = {
     {"tis620",		"tis620*"},
     {"ksc5601",		"ksc5601*"},
     {"dingbats",	"*dingbats"},
+#ifdef WORDS_BIGENDIAN
+    {"unicode",		"iso10646-1"},
+#else
+    /*
+     * ucs-2be is needed if native order isn't BE.
+     */
     {"ucs-2be",		"iso10646-1"},
+#endif
     {NULL,		NULL}
 };
 
@@ -197,13 +203,16 @@ static EncodingAlias encodingAliases[] = {
  * Procedures used only in this file.
  */
 
+static void		FontPkgCleanup _ANSI_ARGS_((ClientData clientData));
 static FontFamily *	AllocFontFamily _ANSI_ARGS_((Display *display,
 			    XFontStruct *fontStructPtr, int base));
 static SubFont *	CanUseFallback _ANSI_ARGS_((UnixFont *fontPtr,
-			    char *fallbackName, int ch));
+			    CONST char *fallbackName, int ch,
+			    SubFont **fixSubFontPtrPtr));
 static SubFont *	CanUseFallbackWithAliases _ANSI_ARGS_((
 			    UnixFont *fontPtr, char *fallbackName,
-			    int ch, Tcl_DString *nameTriedPtr));
+			    int ch, Tcl_DString *nameTriedPtr,
+			    SubFont **fixSubFontPtrPtr));
 static int		ControlUtfProc _ANSI_ARGS_((ClientData clientData,
 			    CONST char *src, int srcLen, int flags,
 			    Tcl_EncodingState *statePtr, char *dst,
@@ -213,7 +222,7 @@ static XFontStruct *	CreateClosestFont _ANSI_ARGS_((Tk_Window tkwin,
 			    CONST TkFontAttributes *faPtr,
 			    CONST TkXLFDAttributes *xaPtr));
 static SubFont *	FindSubFontForChar _ANSI_ARGS_((UnixFont *fontPtr,
-			    int ch));
+			    int ch, SubFont **fixSubFontPtrPtr));
 static void		FontMapInsert _ANSI_ARGS_((SubFont *subFontPtr,
 			    int ch));
 static void		FontMapLoadPage _ANSI_ARGS_((SubFont *subFontPtr,
@@ -246,6 +255,7 @@ static void		ReleaseSubFont _ANSI_ARGS_((Display *display,
 			    SubFont *subFontPtr));
 static int		SeenName _ANSI_ARGS_((CONST char *name,
 			    Tcl_DString *dsPtr));
+#ifndef WORDS_BIGENDIAN
 static int		Ucs2beToUtfProc _ANSI_ARGS_((ClientData clientData,
 			    CONST char *src, int srcLen, int flags,
 			    Tcl_EncodingState *statePtr, char *dst, int dstLen,
@@ -256,7 +266,46 @@ static int		UtfToUcs2beProc _ANSI_ARGS_((ClientData clientData,
 			    Tcl_EncodingState *statePtr, char *dst, int dstLen,
 			    int *srcReadPtr, int *dstWrotePtr,
 			    int *dstCharsPtr));
+#endif
 
+
+/*
+ *-------------------------------------------------------------------------
+ *
+ * FontPkgCleanup --
+ *
+ *	This procedure is called when an application is created.  It
+ *	initializes all the structures that are used by the
+ *	platform-dependent code on a per application basis.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Releases thread-specific resources used by font pkg.
+ *
+ *-------------------------------------------------------------------------
+ */
+
+static void
+FontPkgCleanup(ClientData clientData)
+{
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+    
+    if (tsdPtr->controlFamily.encoding != NULL) {
+	FontFamily *familyPtr = &tsdPtr->controlFamily;
+	int i;
+
+	Tcl_FreeEncoding(familyPtr->encoding);
+	for (i = 0; i < FONTMAP_PAGES; i++) {
+	    if (familyPtr->fontMap[i] != NULL) {
+		ckfree(familyPtr->fontMap[i]);
+	    }
+	}
+	tsdPtr->controlFamily.encoding = NULL;
+    }
+}
 
 /*
  *-------------------------------------------------------------------------
@@ -285,7 +334,7 @@ TkpFontPkgInit(mainPtr)
     Tcl_EncodingType type;
     SubFont dummy;
     int i;
-    
+
     if (tsdPtr->controlFamily.encoding == NULL) {
 	type.encodingName	= "X11ControlChars";
 	type.toUtfProc		= ControlUtfProc;
@@ -293,7 +342,7 @@ TkpFontPkgInit(mainPtr)
 	type.freeProc		= NULL;
 	type.clientData		= NULL;
 	type.nullSize		= 0;
-	
+
 	tsdPtr->controlFamily.refCount = 2;
 	tsdPtr->controlFamily.encoding = Tcl_CreateEncoding(&type);
 	tsdPtr->controlFamily.isTwoByteFont = 0;
@@ -305,9 +354,10 @@ TkpFontPkgInit(mainPtr)
 	    FontMapInsert(&dummy, i + 0x80);
 	}
 
+#ifndef WORDS_BIGENDIAN
 	/*
-	 * UCS-2BE is unicode in big-endian format.
-	 * It is used in iso10646 fonts.
+	 * UCS-2BE is unicode (UCS-2) in big-endian format.  Define this
+	 * if native order isn't BE.  It is used in iso10646 fonts.
 	 */
 
 	type.encodingName	= "ucs-2be";
@@ -317,6 +367,8 @@ TkpFontPkgInit(mainPtr)
 	type.clientData		= NULL;
 	type.nullSize		= 2;
 	Tcl_CreateEncoding(&type);
+#endif
+	Tcl_CreateThreadExitHandler(FontPkgCleanup, NULL);
     }
 }
 
@@ -365,7 +417,7 @@ ControlUtfProc(clientData, src, srcLen, flags, statePtr, dst, dstLen,
 				 * correspond to the bytes stored in the
 				 * output buffer. */
 {
-    CONST char *srcEnd;
+    CONST char *srcStart, *srcEnd;
     char *dstStart, *dstEnd;
     Tcl_UniChar ch;
     int result;
@@ -375,8 +427,9 @@ ControlUtfProc(clientData, src, srcLen, flags, statePtr, dst, dstLen,
 	'a', 'b', 't', 'n', 'v', 'f', 'r'
     };
 
-    result = TCL_OK;    
+    result = TCL_OK;
 
+    srcStart = src;
     srcEnd = src + srcLen;
 
     dstStart = dst;
@@ -406,18 +459,20 @@ ControlUtfProc(clientData, src, srcLen, flags, statePtr, dst, dstLen,
 	    dst += 6;
 	}
     }
-    *srcReadPtr = src - srcEnd;
+    *srcReadPtr = src - srcStart;
     *dstWrotePtr = dst - dstStart;
     *dstCharsPtr = dst - dstStart;
     return result;
 }
 
+#ifndef WORDS_BIGENDIAN
 /*
  *-------------------------------------------------------------------------
  *
  * Ucs2beToUtfProc --
  *
  *	Convert from UCS-2BE (big-endian 16-bit Unicode) to UTF-8.
+ *	This is only defined on LE machines.
  *
  * Results:
  *	Returns TCL_OK if conversion was successful.
@@ -456,40 +511,38 @@ Ucs2beToUtfProc(clientData, src, srcLen, flags, statePtr, dst, dstLen,
 				 * correspond to the bytes stored in the
 				 * output buffer. */
 {
-    CONST Tcl_UniChar *wSrc, *wSrcStart, *wSrcEnd;
+    CONST char *srcStart, *srcEnd;
     char *dstEnd, *dstStart;
     int result, numChars;
-    
+
     result = TCL_OK;
-    if ((srcLen % sizeof(Tcl_UniChar)) != 0) {
+
+    /* check alignment with ucs-2 (2 == sizeof(UCS-2)) */
+    if ((srcLen % 2) != 0) {
 	result = TCL_CONVERT_MULTIBYTE;
-	srcLen /= sizeof(Tcl_UniChar);
-	srcLen *= sizeof(Tcl_UniChar);
+	srcLen--;
     }
 
-    wSrc = (Tcl_UniChar *) src;
-
-    wSrcStart = (Tcl_UniChar *) src;
-    wSrcEnd = (Tcl_UniChar *) (src + srcLen);
+    srcStart = src;
+    srcEnd = src + srcLen;
 
     dstStart = dst;
     dstEnd = dst + dstLen - TCL_UTF_MAX;
 
-    for (numChars = 0; wSrc < wSrcEnd; numChars++) {
+    for (numChars = 0; src < srcEnd; numChars++) {
 	if (dst > dstEnd) {
 	    result = TCL_CONVERT_NOSPACE;
 	    break;
 	}
-	/* 
-	 * On a little-endian machine (Intel) the UCS-2BE is in the
-	 * wrong byte-order in comparison to "unicode", which is
-	 * in native host order.
+	/*
+	 * Need to swap byte-order on little-endian machines (x86) for
+	 * UCS-2BE.  We know this is an LE->BE swap.
 	 */
-	dst += Tcl_UniCharToUtf(htons(*wSrc), dst);
-	wSrc++;
+	dst += Tcl_UniCharToUtf(htons(*((short *)src)), dst);
+	src += 2 /* sizeof(UCS-2) */;
     }
 
-    *srcReadPtr = (char *) wSrc - (char *) wSrcStart;
+    *srcReadPtr = src - srcStart;
     *dstWrotePtr = dst - dstStart;
     *dstCharsPtr = numChars;
     return result;
@@ -500,7 +553,7 @@ Ucs2beToUtfProc(clientData, src, srcLen, flags, statePtr, dst, dstLen,
  *
  * UtfToUcs2beProc --
  *
- *	Convert from UTF-8 to UCS-2BE.
+ *	Convert from UTF-8 to UCS-2BE (fixed 2-byte encoding).
  *
  * Results:
  *	Returns TCL_OK if conversion was successful.
@@ -539,10 +592,10 @@ UtfToUcs2beProc(clientData, src, srcLen, flags, statePtr, dst, dstLen,
 				 * correspond to the bytes stored in the
 				 * output buffer. */
 {
-    CONST char *srcStart, *srcEnd, *srcClose;
-    Tcl_UniChar *wDst, *wDstStart, *wDstEnd;
+    CONST char *srcStart, *srcEnd, *srcClose, *dstStart, *dstEnd;
     int result, numChars;
-    
+    Tcl_UniChar ch;
+
     srcStart = src;
     srcEnd = src + srcLen;
     srcClose = srcEnd;
@@ -550,9 +603,8 @@ UtfToUcs2beProc(clientData, src, srcLen, flags, statePtr, dst, dstLen,
 	srcClose -= TCL_UTF_MAX;
     }
 
-    wDst = (Tcl_UniChar *) dst;
-    wDstStart = (Tcl_UniChar *) dst;
-    wDstEnd = (Tcl_UniChar *) (dst + dstLen - sizeof(Tcl_UniChar));
+    dstStart = dst;
+    dstEnd   = dst + dstLen - 2 /* sizeof(UCS-2) */;
 
     result = TCL_OK;
     for (numChars = 0; src < srcEnd; numChars++) {
@@ -565,22 +617,26 @@ UtfToUcs2beProc(clientData, src, srcLen, flags, statePtr, dst, dstLen,
 	    result = TCL_CONVERT_MULTIBYTE;
 	    break;
 	}
-	if (wDst > wDstEnd) {
+	if (dst > dstEnd) {
 	    result = TCL_CONVERT_NOSPACE;
 	    break;
         }
-	src += Tcl_UtfToUniChar(src, wDst);
+	src += Tcl_UtfToUniChar(src, &ch);
 	/*
-	 * Byte swap for little-endian machines.
+	 * Ensure big-endianness (store big bits first).
+	 * XXX: This hard-codes the assumed size of Tcl_UniChar as 2.
+	 * Make sure to work in char* for Tcl_UtfToUniChar alignment.
+	 * [Bug 1122671]
 	 */
-	*wDst = htons(*wDst);
-	wDst++;
+	*dst++ = (ch >> 8);
+	*dst++ = (ch & 0xFF);
     }
     *srcReadPtr = src - srcStart;
-    *dstWrotePtr = (char *) wDst - (char *) wDstStart;
+    *dstWrotePtr = dst - dstStart;
     *dstCharsPtr = numChars;
     return result;
 }
+#endif /* WORDS_BIGENDIAN */
 
 /*
  *---------------------------------------------------------------------------
@@ -806,8 +862,26 @@ TkpGetFontFamilies(interp, tkwin)
     Tcl_InitHashTable(&familyTable, TCL_STRING_KEYS);
     nameList = ListFonts(Tk_Display(tkwin), "*", &numNames);
     for (i = 0; i < numNames; i++) {
+	char *familyEnd;
+
+	family = strchr(nameList[i] + 1, '-');
+	if (family == NULL) {
+	    /*
+	     * Apparently, sometimes ListFonts() can return a font name with
+	     * zero or one '-' character in it. This is probably indicative of
+	     * a server misconfiguration, but crashing because of it is a very
+	     * bad idea anyway. [Bug 1475865]
+	     */
+
+	    continue;
+	}
+	family++;			/* Advance to char after '-'. */
+	familyEnd = strchr(family, '-');
+	if (familyEnd == NULL) {
+	    continue;			/* See comment above. */
+	}
+	*familyEnd = '\0';
 	family = strchr(nameList[i] + 1, '-') + 1;
-	strchr(family, '-')[0] = '\0';
 	Tcl_CreateHashEntry(&familyTable, family, &new);
     }
     XFreeFontNames(nameList);
@@ -945,7 +1019,7 @@ Tk_MeasureChars(tkfont, source, numBytes, maxLength, flags, lengthPtr)
 	end = source + numBytes;
 	for (p = source; p < end; ) {
 	    next = p + Tcl_UtfToUniChar(p, &ch);
-	    thisSubFontPtr = FindSubFontForChar(fontPtr, ch);
+	    thisSubFontPtr = FindSubFontForChar(fontPtr, ch, &lastSubFontPtr);
 	    if (thisSubFontPtr != lastSubFontPtr) {
 		familyPtr = lastSubFontPtr->familyPtr;
 		Tcl_UtfToExternalDString(familyPtr->encoding, source,
@@ -1004,7 +1078,7 @@ Tk_MeasureChars(tkfont, source, numBytes, maxLength, flags, lengthPtr)
 	    if ((ch < BASE_CHARS) && (fontPtr->widths[ch] != 0)) {
 		newX += fontPtr->widths[ch];
 	    } else {
-		lastSubFontPtr = FindSubFontForChar(fontPtr, ch);
+		lastSubFontPtr = FindSubFontForChar(fontPtr, ch, NULL);
 		familyPtr = lastSubFontPtr->familyPtr;
 		Tcl_UtfToExternal(NULL, familyPtr->encoding, p, next - p,
 			0, NULL, buf, sizeof(buf), NULL, &dstWrote, NULL);
@@ -1114,76 +1188,87 @@ Tk_DrawChars(display, drawable, gc, tkfont, source, numBytes, x, y)
     SubFont *thisSubFontPtr, *lastSubFontPtr;
     Tcl_DString runString;
     CONST char *p, *end, *next;
-    int xStart, needWidth;
+    int xStart, needWidth, window_width, do_width;
     Tcl_UniChar ch;
     FontFamily *familyPtr;
+#ifdef TK_DRAW_CHAR_XWINDOW_CHECK
+    int rx, ry;
+    unsigned int width, height, border_width, depth;
+    Drawable root;
+#endif
 
     fontPtr = (UnixFont *) tkfont;
     lastSubFontPtr = &fontPtr->subFontArray[0];
 
     xStart = x;
 
+#ifdef TK_DRAW_CHAR_XWINDOW_CHECK
+    /*
+     * Get the window width so we can abort drawing outside of the window
+     */
+    if (XGetGeometry(display, drawable, &root, &rx, &ry, &width, &height,
+	    &border_width, &depth) == False) {
+	window_width = INT_MAX;
+    } else {
+	window_width = width;
+    }
+#else
+    /*
+     * This is used by default until we find a solution that doesn't
+     * round-trip to the X server (need to get Tk cached window width).
+     */
+    window_width = 32768;
+#endif
+
     end = source + numBytes;
-    for (p = source; p < end; ) {
-	next = p + Tcl_UtfToUniChar(p, &ch);
-	thisSubFontPtr = FindSubFontForChar(fontPtr, ch);
-	if (thisSubFontPtr != lastSubFontPtr) {
+    needWidth = fontPtr->font.fa.underline + fontPtr->font.fa.overstrike;
+    for (p = source; p <= end; ) {
+	if (p < end) {
+	    next = p + Tcl_UtfToUniChar(p, &ch);
+	    thisSubFontPtr = FindSubFontForChar(fontPtr, ch, &lastSubFontPtr);
+	} else {
+	    next = p + 1;
+	    thisSubFontPtr = lastSubFontPtr;
+	}
+	if ((thisSubFontPtr != lastSubFontPtr)
+		|| (p == end) || (p-source > 200)) {
 	    if (p > source) {
+	        do_width = (needWidth || (p != end)) ? 1 : 0;
 		familyPtr = lastSubFontPtr->familyPtr;
+
 		Tcl_UtfToExternalDString(familyPtr->encoding, source,
 			p - source, &runString);
 		if (familyPtr->isTwoByteFont) {
 		    XDrawString16(display, drawable, gc, x, y, 
 			    (XChar2b *) Tcl_DStringValue(&runString),
 			    Tcl_DStringLength(&runString) / 2);
-			    
-		    x += XTextWidth16(lastSubFontPtr->fontStructPtr,
-			    (XChar2b *) Tcl_DStringValue(&runString),
-			    Tcl_DStringLength(&runString) / 2);
+		    if (do_width) {
+			x += XTextWidth16(lastSubFontPtr->fontStructPtr,
+				(XChar2b *) Tcl_DStringValue(&runString),
+				Tcl_DStringLength(&runString) / 2);
+		    }
 		} else {
 		    XDrawString(display, drawable, gc, x, y,
 			    Tcl_DStringValue(&runString),
 			    Tcl_DStringLength(&runString));
-		    x += XTextWidth(lastSubFontPtr->fontStructPtr,
-			    Tcl_DStringValue(&runString),
-			    Tcl_DStringLength(&runString));
+		    if (do_width) {
+			x += XTextWidth(lastSubFontPtr->fontStructPtr,
+				Tcl_DStringValue(&runString),
+				Tcl_DStringLength(&runString));
+		    }
 		}
 		Tcl_DStringFree(&runString);
 	    }
 	    lastSubFontPtr = thisSubFontPtr;
 	    source = p;
 	    XSetFont(display, gc, lastSubFontPtr->fontStructPtr->fid);
+	    if (x > window_width) {
+	        break;
+	    }
 	}
 	p = next;
     }
 
-    needWidth = fontPtr->font.fa.underline + fontPtr->font.fa.overstrike;
-    if (p > source) {
-	familyPtr = lastSubFontPtr->familyPtr;
-	Tcl_UtfToExternalDString(familyPtr->encoding, source, p - source,
-		&runString);
-	if (familyPtr->isTwoByteFont) {
-	    XDrawString16(display, drawable, gc, x, y, 
-		    (XChar2b *) Tcl_DStringValue(&runString),
-		    Tcl_DStringLength(&runString) >> 1);
-	    if (needWidth) {
-		x += XTextWidth16(lastSubFontPtr->fontStructPtr,
-			(XChar2b *) Tcl_DStringValue(&runString),
-			Tcl_DStringLength(&runString) >> 1);
-	    }
-	} else {
-	    XDrawString(display, drawable, gc, x, y, 
-		    Tcl_DStringValue(&runString),
-		    Tcl_DStringLength(&runString));
-	    if (needWidth) {
-		x += XTextWidth(lastSubFontPtr->fontStructPtr,
-			Tcl_DStringValue(&runString),
-			Tcl_DStringLength(&runString));
-	    }
-	}
-	Tcl_DStringFree(&runString);
-    }
-
     if (lastSubFontPtr != &fontPtr->subFontArray[0]) {
 	XSetFont(display, gc, fontPtr->subFontArray[0].fontStructPtr->fid);
     }
@@ -1199,6 +1284,9 @@ Tk_DrawChars(display, drawable, gc, tkfont, source, numBytes, x, y)
 		(unsigned) (x - xStart), (unsigned) fontPtr->barHeight);
     }
 }
+
+
+
 
 /*
  *-------------------------------------------------------------------------
@@ -1308,14 +1396,14 @@ CreateClosestFont(tkwin, faPtr, xaPtr)
 	FontAttributes got;
 	int scalable;
 	unsigned int score;
-	
+
 	if (TkFontParseXLFD(nameList[nameIdx], &got.fa, &got.xa) != TCL_OK) {
 	    continue;
 	}
 	IdentifySymbolEncodings(&got);
 	scalable = (got.fa.size == 0);
 	score = RankAttributes(&want, &got);
-	if (score <= bestScore[scalable]) {
+	if (score < bestScore[scalable]) {
 	    bestIdx[scalable] = nameIdx;
 	    bestScore[scalable] = score;
 	}
@@ -1427,7 +1515,7 @@ InitFont(tkwin, fontStructPtr, fontPtr)
     InitSubFont(display, fontStructPtr, 1, &fontPtr->subFontArray[0]);
 
     fontPtr->controlSubFont	= fontPtr->subFontArray[0];
-    subFontPtr			= FindSubFontForChar(fontPtr, '0');
+    subFontPtr			= FindSubFontForChar(fontPtr, '0', NULL);
     controlPtr			= &fontPtr->controlSubFont;
     controlPtr->fontStructPtr	= subFontPtr->fontStructPtr;
     controlPtr->familyPtr	= &tsdPtr->controlFamily;
@@ -1744,19 +1832,24 @@ FreeFontFamily(familyPtr)
  * Side effects:
  *	The contents of fontPtr are modified to cache the results
  *	of the lookup and remember any SubFonts that were dynamically 
- *	loaded.
+ *	loaded.  The table of SubFonts might be extended, and if a non-NULL
+ *	reference to a subfont pointer is available, it is updated if it
+ *	previously pointed into the old subfont table.
  *
  *-------------------------------------------------------------------------
  */
 
 static SubFont *
-FindSubFontForChar(fontPtr, ch)
+FindSubFontForChar(fontPtr, ch, fixSubFontPtrPtr)
     UnixFont *fontPtr;		/* The font object with which the character
 				 * will be displayed. */
     int ch;			/* The Unicode character to be displayed. */
+    SubFont **fixSubFontPtrPtr;	/* Subfont reference to fix up if we
+				 * reallocate our subfont table. */
 {
     int i, j, k, numNames;
-    char *faceName, *fallback;
+    Tk_Uid faceName; 
+    char *fallback;
     char **aliases, **nameList, **anyFallbacks;
     char ***fontFallbacks;
     SubFont *subFontPtr;
@@ -1792,7 +1885,7 @@ FindSubFontForChar(fontPtr, ch)
      
     faceName = fontPtr->font.fa.family;
     if (SeenName(faceName, &ds) == 0) {
-	subFontPtr = CanUseFallback(fontPtr, faceName, ch);
+	subFontPtr = CanUseFallback(fontPtr, faceName, ch, fixSubFontPtrPtr);
 	if (subFontPtr != NULL) {
 	    goto end;
 	}
@@ -1832,7 +1925,8 @@ FindSubFontForChar(fontPtr, ch)
 	 */
 
 	for (j = 0; (fallback = fontFallbacks[i][j]) != NULL; j++) {
-	    subFontPtr = CanUseFallbackWithAliases(fontPtr, fallback, ch, &ds);
+	    subFontPtr = CanUseFallbackWithAliases(fontPtr, fallback, ch, &ds,
+		    fixSubFontPtrPtr);
 	    if (subFontPtr != NULL) {
 		goto end;
 	    }
@@ -1845,7 +1939,8 @@ FindSubFontForChar(fontPtr, ch)
 
     anyFallbacks = TkFontGetGlobalClass();
     for (i = 0; (fallback = anyFallbacks[i]) != NULL; i++) {
-	subFontPtr = CanUseFallbackWithAliases(fontPtr, fallback, ch, &ds);
+	subFontPtr = CanUseFallbackWithAliases(fontPtr, fallback, ch, &ds,
+		fixSubFontPtrPtr);
 	if (subFontPtr != NULL) {
 	    goto end;
 	}
@@ -1861,7 +1956,8 @@ FindSubFontForChar(fontPtr, ch)
 	fallback = strchr(nameList[i] + 1, '-') + 1;
 	strchr(fallback, '-')[0] = '\0';
 	if (SeenName(fallback, &ds) == 0) {
-	    subFontPtr = CanUseFallback(fontPtr, fallback, ch);
+	    subFontPtr = CanUseFallback(fontPtr, fallback, ch,
+		    fixSubFontPtrPtr);
 	    if (subFontPtr != NULL) {
 		XFreeFontNames(nameList);
 		goto end;
@@ -1988,7 +2084,7 @@ FontMapLoadPage(subFontPtr, row)
     int row;			/* Index of the page to be loaded into 
 				 * the cache. */
 {
-    char src[TCL_UTF_MAX], buf[16];
+    char buf[16], src[TCL_UTF_MAX];
     int minHi, maxHi, minLo, maxLo, scale, checkLo;
     int i, end, bitOffset, isTwoByteFont, n;
     Tcl_Encoding encoding;
@@ -2025,7 +2121,7 @@ FontMapLoadPage(subFontPtr, row)
     end = (row + 1) << FONTMAP_SHIFT;
     for (i = row << FONTMAP_SHIFT; i < end; i++) {
 	int hi, lo;
-	
+
 	if (Tcl_UtfToExternal(NULL, encoding, src, Tcl_UniCharToUtf(i, src),
         	TCL_ENCODING_STOPONERROR, NULL, buf, sizeof(buf), NULL, 
 		NULL, NULL) != TCL_OK) {
@@ -2065,13 +2161,16 @@ FontMapLoadPage(subFontPtr, row)
  * Side effects:
  *	If the name and/or one of its aliases was rejected, the
  *	rejected string is recorded in nameTriedPtr so that it won't
- *	be tried again.
+ *	be tried again.  The table of SubFonts might be extended, and if
+ *	a non-NULL reference to a subfont pointer is available, it is
+ *	updated if it previously pointed into the old subfont table.
  *
  *---------------------------------------------------------------------------
  */
 
 static SubFont *
-CanUseFallbackWithAliases(fontPtr, faceName, ch, nameTriedPtr)
+CanUseFallbackWithAliases(fontPtr, faceName, ch, nameTriedPtr,
+	fixSubFontPtrPtr)
     UnixFont *fontPtr;		/* The font object that will own the new
 				 * screen font. */
     char *faceName;		/* Desired face name for new screen font. */
@@ -2081,13 +2180,15 @@ CanUseFallbackWithAliases(fontPtr, faceName, ch, nameTriedPtr)
 				 * been tried.  It is possible for the same
 				 * face name to be queried multiple times when
 				 * trying to find a suitable screen font. */
+    SubFont **fixSubFontPtrPtr;	/* Subfont reference to fix up if we
+				 * reallocate our subfont table. */
 {
     SubFont *subFontPtr;
     char **aliases;
     int i;
     
     if (SeenName(faceName, nameTriedPtr) == 0) {
-	subFontPtr = CanUseFallback(fontPtr, faceName, ch);
+	subFontPtr = CanUseFallback(fontPtr, faceName, ch, fixSubFontPtrPtr);
 	if (subFontPtr != NULL) {
 	    return subFontPtr;
 	}
@@ -2096,7 +2197,8 @@ CanUseFallbackWithAliases(fontPtr, faceName, ch, nameTriedPtr)
     if (aliases != NULL) {
 	for (i = 0; aliases[i] != NULL; i++) {
 	    if (SeenName(aliases[i], nameTriedPtr) == 0) {
-		subFontPtr = CanUseFallback(fontPtr, aliases[i], ch);
+		subFontPtr = CanUseFallback(fontPtr, aliases[i], ch,
+			fixSubFontPtrPtr);
 		if (subFontPtr != NULL) {
 		    return subFontPtr;
 		}
@@ -2166,18 +2268,23 @@ SeenName(name, dsPtr)
  *
  * Side effects:				       
  *	The font object's subFontArray is updated to contain a reference
- *	to the newly allocated SubFont.
+ *	to the newly allocated SubFont.  The table of SubFonts might be
+ *	extended, and if a non-NULL reference to a subfont pointer is
+ *	available, it is updated if it previously pointed into the old
+ *	subfont table.
  *
  *-------------------------------------------------------------------------
  */
 
 static SubFont *
-CanUseFallback(fontPtr, faceName, ch)
+CanUseFallback(fontPtr, faceName, ch, fixSubFontPtrPtr)
     UnixFont *fontPtr;		/* The font object that will own the new
 				 * screen font. */
-    char *faceName;		/* Desired face name for new screen font. */
+    CONST char *faceName;	/* Desired face name for new screen font. */
     int ch;			/* The Unicode character that the new
 				 * screen font must be able to display. */
+    SubFont **fixSubFontPtrPtr;	/* Subfont reference to fix up if we
+				 * reallocate our subfont table. */
 {
     int i, nameIdx, numNames, srcLen;
     Tk_Uid hateFoundry;
@@ -2309,7 +2416,7 @@ CanUseFallback(fontPtr, faceName, ch)
 
 	scalable = (got.fa.size == 0);
 	score = RankAttributes(&want, &got);
-	if (score <= bestScore[scalable]) {
+	if (score < bestScore[scalable]) {
 	    bestIdx[scalable] = nameIdx;
 	    bestScore[scalable] = score;
 	}
@@ -2372,6 +2479,14 @@ CanUseFallback(fontPtr, faceName, ch)
 	newPtr = (SubFont *) ckalloc(sizeof(SubFont) * (fontPtr->numSubFonts + 1));
 	memcpy((char *) newPtr, fontPtr->subFontArray,
 		fontPtr->numSubFonts * sizeof(SubFont));
+	if (fixSubFontPtrPtr != NULL) {
+	    register SubFont *fixSubFontPtr = *fixSubFontPtrPtr;
+
+	    if (fixSubFontPtr != &fontPtr->controlSubFont) {
+		*fixSubFontPtrPtr =
+			newPtr + (fixSubFontPtr - fontPtr->subFontArray);
+	    }
+	}
 	if (fontPtr->subFontArray != fontPtr->staticSubFonts) {
 	    ckfree((char *) fontPtr->subFontArray);
 	}
@@ -2631,6 +2746,13 @@ GetFontAttributes(display, fontStructPtr, faPtr)
     } else {
 	TkInitFontAttributes(&faPtr->fa);
 	TkInitXLFDAttributes(&faPtr->xa);
+    }
+    /*
+     * Do last ditch check for family.  It seems that some X servers can
+     * fail on the X font calls above, slipping through earlier checks.
+     * X-Win32 5.4 is one of these.
+     */
+    if (faPtr->fa.family == NULL) {
 	faPtr->fa.family = Tk_GetUid("");
 	faPtr->xa.foundry = Tk_GetUid("");
 	faPtr->xa.charset = Tk_GetUid("");
@@ -2779,5 +2901,3 @@ GetEncodingAlias(name)
     }
     return name;
 }
-
-
