@@ -8,17 +8,53 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkUnixKey.c,v 1.5 2000/04/10 22:43:13 ericm Exp $
  */
 
 #include "tkInt.h"
+#include <X11/XKBlib.h>
 
 /*
  * Prototypes for local procedures defined in this file:
  */
 
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tk_SetCaretPos --
+ *
+ *	This enables correct placement of the XIM caret.  This is called
+ *	by widgets to indicate their cursor placement, and the caret
+ *	location is used by TkpGetString to place the XIM caret.
+ *	This is currently only used for over-the-spot XIM.
+ *
+ * Results:
+ *	None
+ *
+ * Side effects:
+ *	None
+ *
+ *----------------------------------------------------------------------
+ */
 
+void
+Tk_SetCaretPos(tkwin, x, y, height)
+    Tk_Window tkwin;
+    int	      x;
+    int	      y;
+    int	      height;
+{
+    TkCaret *caretPtr = &(((TkWindow *) tkwin)->dispPtr->caret);
+
+    /*
+     * Use height for best placement of the XIM over-the-spot box.
+     */
+
+    caretPtr->winPtr = ((TkWindow *) tkwin);
+    caretPtr->x = x;
+    caretPtr->y = y;
+    caretPtr->height = height;
+}
 
 /*
  *----------------------------------------------------------------------
@@ -49,6 +85,9 @@ TkpGetString(winPtr, eventPtr, dsPtr)
     int len;
     Tcl_DString buf;
     Status status;
+#ifdef TK_USE_INPUT_METHODS
+    TkDisplay *dispPtr = winPtr->dispPtr;
+#endif
 
     /*
      * Overallocate the dstring to the maximum stack amount.
@@ -56,11 +95,16 @@ TkpGetString(winPtr, eventPtr, dsPtr)
 
     Tcl_DStringInit(&buf);
     Tcl_DStringSetLength(&buf, TCL_DSTRING_STATIC_SIZE-1);
-    
+
 #ifdef TK_USE_INPUT_METHODS
-    if (winPtr->dispPtr->useInputMethods
+    if ((dispPtr->flags & TK_DISPLAY_USE_IM)
 	    && (winPtr->inputContext != NULL)
 	    && (eventPtr->type == KeyPress)) {
+#if TK_XIM_SPOT
+	XVaNestedList preedit_attr;
+	XPoint spot;
+#endif
+
 	len = XmbLookupString(winPtr->inputContext, &eventPtr->xkey,
 		Tcl_DStringValue(&buf), Tcl_DStringLength(&buf),
 		(KeySym *) NULL, &status);
@@ -76,6 +120,21 @@ TkpGetString(winPtr, eventPtr, dsPtr)
 	if ((status != XLookupChars) && (status != XLookupBoth)) {
 	    len = 0;
 	}
+
+#if TK_XIM_SPOT
+	/*
+	 * Adjust the XIM caret position.  We might want to check that
+	 * this is the right caret.winPtr as well.
+	 */
+	if (dispPtr->flags & TK_DISPLAY_XIM_SPOT) {
+	    spot.x = dispPtr->caret.x;
+	    spot.y = dispPtr->caret.y + dispPtr->caret.height;
+	    preedit_attr = XVaCreateNestedList(0, XNSpotLocation, &spot, NULL);
+	    XSetICValues(winPtr->inputContext,
+		    XNPreeditAttributes, preedit_attr, NULL);
+	    XFree(preedit_attr);
+	}
+#endif
     } else {
 	len = XLookupString(&eventPtr->xkey, Tcl_DStringValue(&buf),
 		Tcl_DStringLength(&buf), (KeySym *) NULL,
@@ -97,7 +156,7 @@ TkpGetString(winPtr, eventPtr, dsPtr)
 /*
  * When mapping from a keysym to a keycode, need
  * information about the modifier state that should be used
- * so that when they call XKeycodeToKeysym taking into
+ * so that when they call XkbKeycodeToKeysym taking into
  * account the xkey.state, they will get back the original
  * keysym.
  */
@@ -121,14 +180,14 @@ TkpSetKeycodeAndState(tkwin, keySym, eventPtr)
     }
     if (keycode != 0) {
 	for (state = 0; state < 4; state++) {
-	    if (XKeycodeToKeysym(display, keycode, state) == keySym) {
+	    if (XkbKeycodeToKeysym(display, keycode, 0, state) == keySym) {
 		if (state & 1) {
 		    eventPtr->xkey.state |= ShiftMask;
 		}
 		if (state & 2) {
 		    TkDisplay *dispPtr;
 
-		    dispPtr = ((TkWindow *) tkwin)->dispPtr; 
+		    dispPtr = ((TkWindow *) tkwin)->dispPtr;
 		    eventPtr->xkey.state |= dispPtr->modeModMask;
 		}
 		break;
@@ -190,7 +249,8 @@ TkpGetKeySym(dispPtr, eventPtr)
 	    && (eventPtr->xkey.state & LockMask))) {
 	index += 1;
     }
-    sym = XKeycodeToKeysym(dispPtr->display, eventPtr->xkey.keycode, index);
+    sym = XkbKeycodeToKeysym(dispPtr->display, eventPtr->xkey.keycode, 0,
+	    index);
 
     /*
      * Special handling:  if the key was shifted because of Lock, but
@@ -205,8 +265,8 @@ TkpGetKeySym(dispPtr, eventPtr)
 		|| ((sym >= XK_Agrave) && (sym <= XK_Odiaeresis))
 		|| ((sym >= XK_Ooblique) && (sym <= XK_Thorn)))) {
 	    index &= ~1;
-	    sym = XKeycodeToKeysym(dispPtr->display, eventPtr->xkey.keycode,
-		    index);
+	    sym = XkbKeycodeToKeysym(dispPtr->display, eventPtr->xkey.keycode,
+		    0, index);
 	}
     }
 
@@ -216,8 +276,8 @@ TkpGetKeySym(dispPtr, eventPtr)
      */
 
     if ((index & 1) && (sym == NoSymbol)) {
-	sym = XKeycodeToKeysym(dispPtr->display, eventPtr->xkey.keycode,
-		index & ~1);
+	sym = XkbKeycodeToKeysym(dispPtr->display, eventPtr->xkey.keycode,
+		0, index & ~1);
     }
     return sym;
 }
@@ -267,7 +327,7 @@ TkpInitKeymapInfo(dispPtr)
 	if (*codePtr == 0) {
 	    continue;
 	}
-	keysym = XKeycodeToKeysym(dispPtr->display, *codePtr, 0);
+	keysym = XkbKeycodeToKeysym(dispPtr->display, *codePtr, 0, 0);
 	if (keysym == XK_Shift_Lock) {
 	    dispPtr->lockUsage = LU_SHIFT;
 	    break;
@@ -293,7 +353,7 @@ TkpInitKeymapInfo(dispPtr)
 	if (*codePtr == 0) {
 	    continue;
 	}
-	keysym = XKeycodeToKeysym(dispPtr->display, *codePtr, 0);
+	keysym = XkbKeycodeToKeysym(dispPtr->display, *codePtr, 0, 0);
 	if (keysym == XK_Mode_switch) {
 	    dispPtr->modeModMask |= ShiftMask << (i/modMapPtr->max_keypermod);
 	}
