@@ -7,12 +7,9 @@
  *
  * Copyright (c) 1991-1993 The Regents of the University of California.
  * Copyright (c) 1994-1996 Sun Microsystems, Inc.
- * Copyright (c) 1998 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkUnixPort.h,v 1.3 1998/09/30 19:01:22 rjohnson Exp $
  */
 
 #ifndef _UNIXPORT
@@ -38,7 +35,7 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <fcntl.h>
-#ifdef HAVE_LIMITS_H
+#ifndef NO_LIMITS_H
 #   include <limits.h>
 #else
 #   include "../compat/limits.h"
@@ -70,7 +67,7 @@
 #       include <time.h>
 #   endif
 #endif
-#ifdef HAVE_UNISTD_H
+#ifndef NO_UNISTD_H
 #   include <unistd.h>
 #else
 #   include "../compat/unistd.h"
@@ -120,13 +117,6 @@
 #endif
 #define MASK_SIZE howmany(FD_SETSIZE, NFDBITS)
 
-/*
- * Not all systems declare the errno variable in errno.h. so this
- * file does it explicitly.
- */
-
-extern int errno;
-
 /*
  * Define "NBBY" (number of bits per byte) if it's not already defined.
  */
@@ -135,28 +125,50 @@ extern int errno;
 #   define NBBY 8
 #endif
 
-/*
- * These macros are just wrappers for the equivalent X Region calls.
- */
+#ifdef __CYGWIN__
+#   define UINT unsigned int
+#   define HWND void *
+#   define HDC void *
+#   define HINSTANCE void *
+#   define COLORREF void *
+#   define HMENU void *
+#   define TkWinDCState void
+#   define HPALETTE void *
+#   define WNDPROC void *
+#   define WPARAM void *
+#   define LPARAM void *
+#   define LRESULT void *
 
-#define TkClipBox(rgn, rect) XClipBox((Region) rgn, rect)
-#define TkCreateRegion() (TkRegion) XCreateRegion()
-#define TkDestroyRegion(rgn) XDestroyRegion((Region) rgn)
-#define TkIntersectRegion(a, b, r) XIntersectRegion((Region) a, \
-	(Region) b, (Region) r)
-#define TkRectInRegion(r, x, y, w, h) XRectInRegion((Region) r, x, y, w, h)
-#define TkSetRegion(d, gc, rgn) XSetRegion(d, gc, (Region) rgn)
-#define TkUnionRectWithRegion(rect, src, ret) XUnionRectWithRegion(rect, \
-	(Region) src, (Region) ret)
+    extern int TkPutImage(unsigned long *, int, Display *, Drawable,
+	    GC, XImage *, int, int, int, int, unsigned int, unsigned int);
 
-/*
- * The TkPutImage macro strips off the color table information, which isn't
- * needed for X.
- */
+#else /* !__CYGWIN__ */
+    /*
+     * The TkPutImage macro strips off the color table information, which isn't
+     * needed for X.
+     */
 
-#define TkPutImage(colors, ncolors, display, pixels, gc, image, destx, desty, srcx, srcy, width, height) \
-	XPutImage(display, pixels, gc, image, destx, desty, srcx, \
-	srcy, width, height);
+#   define TkPutImage(colors, ncolors, display, pixels, gc, image, srcx, srcy, destx, desty, width, height) \
+		XPutImage(display, pixels, gc, image, srcx, srcy, destx, \
+		desty, width, height);
+
+    /*
+     * These macros are just wrappers for the equivalent X Region calls.
+     */
+
+#   define TkClipBox(rgn, rect) XClipBox((Region) (rgn), (rect))
+#   define TkCreateRegion() (TkRegion) XCreateRegion()
+#   define TkDestroyRegion(rgn) XDestroyRegion((Region) (rgn))
+#   define TkIntersectRegion(a, b, r) XIntersectRegion((Region) (a), \
+	(Region) (b), (Region) (r))
+#   define TkRectInRegion(r, x, y, w, h) XRectInRegion((Region) (r), (x), (y), (w), (h))
+#   define TkSetRegion(d, gc, rgn) XSetRegion((d), (gc), (Region) (rgn))
+#   define TkSubtractRegion(a, b, r) XSubtractRegion((Region) (a), \
+	(Region) (b), (Region) (r))
+#   define TkUnionRectWithRegion(rect, src, ret) XUnionRectWithRegion((rect), \
+	(Region) (src), (Region) (ret))
+
+#endif /* !__CYGWIN__ */
 
 /*
  * Supply macros for seek offsets, if they're not already provided by
@@ -180,15 +192,17 @@ extern int errno;
  * in any other header file.
  */
 
-extern void		panic _ANSI_ARGS_(TCL_VARARGS(char *, string));
 
 /*
  * These functions do nothing under Unix, so we just eliminate calls to them.
  */
 
+#define TkpButtonSetDefaults(specPtr) {}
 #define TkpDestroyButton(butPtr) {}
 #define TkSelUpdateClipboard(a,b) {}
+#ifndef __CYGWIN__
 #define TkSetPixmapColormap(p,c) {}
+#endif
 
 /*
  * These calls implement native bitmaps which are not supported under
@@ -201,30 +215,21 @@ extern void		panic _ANSI_ARGS_(TCL_VARARGS(char *, string));
 
 /*
  * This macro stores a representation of the window handle in a string.
+ * This should perhaps use the real size of an XID.
  */
 
+#ifndef __CYGWIN__
 #define TkpPrintWindowId(buf,w) \
-	sprintf((buf), "0x%x", (unsigned int) (w))
-	    
-/*
- * TkpScanWindowId is just an alias for Tcl_GetInt on Unix.
- */
-
-#define TkpScanWindowId(i,s,wp) \
-	Tcl_GetInt((i),(s),(wp))
-	    
-/*
- * This macro indicates that entry and text widgets should display
- * the selection highlight regardless of which window has the focus.
- */
-
-#define ALWAYS_SHOW_SELECTION
+	sprintf((buf), "%#08lx", (unsigned long) (w))
+#endif
 
 /*
  * The following declaration is used to get access to a private Tcl interface
  * that is needed for portability reasons.
  */
 
-EXTERN void		TclpGetTime _ANSI_ARGS_((Tcl_Time *time));
+#ifndef _TCLINT
+#include <tclInt.h>
+#endif
 
 #endif /* _UNIXPORT */
