@@ -1,4 +1,4 @@
-/* 
+/*
  * tkWinColor.c --
  *
  *	Functions to map color names to system color values.
@@ -8,8 +8,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkWinColor.c,v 1.6 2000/07/06 03:17:44 mo Exp $
  */
 
 #include "tkWinInt.h"
@@ -40,36 +38,36 @@ typedef struct {
 
 
 static SystemColorEntry sysColors[] = {
-    "3dDarkShadow",		COLOR_3DDKSHADOW,
-    "3dLight",			COLOR_3DLIGHT,
-    "ActiveBorder",		COLOR_ACTIVEBORDER,
-    "ActiveCaption",		COLOR_ACTIVECAPTION,
-    "AppWorkspace",		COLOR_APPWORKSPACE,
-    "Background",		COLOR_BACKGROUND,
-    "ButtonFace",		COLOR_BTNFACE,
-    "ButtonHighlight",		COLOR_BTNHIGHLIGHT,
-    "ButtonShadow",		COLOR_BTNSHADOW,
-    "ButtonText",		COLOR_BTNTEXT,
-    "CaptionText",		COLOR_CAPTIONTEXT,
-    "DisabledText",		COLOR_GRAYTEXT,
-    "GrayText",			COLOR_GRAYTEXT,
-    "Highlight",		COLOR_HIGHLIGHT,
-    "HighlightText",		COLOR_HIGHLIGHTTEXT,
-    "InactiveBorder",		COLOR_INACTIVEBORDER,
-    "InactiveCaption",		COLOR_INACTIVECAPTION,
-    "InactiveCaptionText",	COLOR_INACTIVECAPTIONTEXT,
-    "InfoBackground",		COLOR_INFOBK,
-    "InfoText",			COLOR_INFOTEXT,
-    "Menu",			COLOR_MENU,
-    "MenuText",			COLOR_MENUTEXT,
-    "Scrollbar",		COLOR_SCROLLBAR,
-    "Window",			COLOR_WINDOW,
-    "WindowFrame",		COLOR_WINDOWFRAME,
-    "WindowText",		COLOR_WINDOWTEXT,
-    NULL,			0
+    {"3dDarkShadow",		COLOR_3DDKSHADOW},
+    {"3dLight",			COLOR_3DLIGHT},
+    {"ActiveBorder",		COLOR_ACTIVEBORDER},
+    {"ActiveCaption",		COLOR_ACTIVECAPTION},
+    {"AppWorkspace",		COLOR_APPWORKSPACE},
+    {"Background",		COLOR_BACKGROUND},
+    {"ButtonFace",		COLOR_BTNFACE},
+    {"ButtonHighlight",		COLOR_BTNHIGHLIGHT},
+    {"ButtonShadow",		COLOR_BTNSHADOW},
+    {"ButtonText",		COLOR_BTNTEXT},
+    {"CaptionText",		COLOR_CAPTIONTEXT},
+    {"DisabledText",		COLOR_GRAYTEXT},
+    {"GrayText",			COLOR_GRAYTEXT},
+    {"Highlight",		COLOR_HIGHLIGHT},
+    {"HighlightText",		COLOR_HIGHLIGHTTEXT},
+    {"InactiveBorder",		COLOR_INACTIVEBORDER},
+    {"InactiveCaption",		COLOR_INACTIVECAPTION},
+    {"InactiveCaptionText",	COLOR_INACTIVECAPTIONTEXT},
+    {"InfoBackground",		COLOR_INFOBK},
+    {"InfoText",			COLOR_INFOTEXT},
+    {"Menu",			COLOR_MENU},
+    {"MenuText",			COLOR_MENUTEXT},
+    {"Scrollbar",		COLOR_SCROLLBAR},
+    {"Window",			COLOR_WINDOW},
+    {"WindowFrame",		COLOR_WINDOWFRAME},
+    {"WindowText",		COLOR_WINDOWTEXT},
+    {NULL,			0}
 };
 
-typedef struct ThreadSpecificData { 
+typedef struct ThreadSpecificData {
     int ncolors;
 } ThreadSpecificData;
 static Tcl_ThreadDataKey dataKey;
@@ -106,7 +104,7 @@ FindSystemColor(name, colorPtr, indexPtr)
     int *indexPtr;		/* Out parameter to store color index. */
 {
     int l, u, r, i;
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
             Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     /*
@@ -203,7 +201,7 @@ TkpGetColor(tkwin, name)
 
     if (((strncasecmp(name, "system", 6) == 0)
 	    && FindSystemColor(name+6, &color, &index))
-	    || XParseColor(Tk_Display(tkwin), Tk_Colormap(tkwin), name,
+	    || TkParseColor(Tk_Display(tkwin), Tk_Colormap(tkwin), name,
 		    &color)) {
 	winColPtr = (WinColor *) ckalloc(sizeof(WinColor));
 	winColPtr->info.color = color;
@@ -211,7 +209,7 @@ TkpGetColor(tkwin, name)
 
 	XAllocColor(Tk_Display(tkwin), Tk_Colormap(tkwin),
 		&winColPtr->info.color);
- 	return (TkColor *) winColPtr; 
+ 	return (TkColor *) winColPtr;
     }
     return (TkColor *) NULL;
 }
@@ -310,7 +308,7 @@ TkWinIndexOfColor(colorPtr)
     register WinColor *winColPtr = (WinColor *) colorPtr;
     if (winColPtr->info.magic == COLOR_MAGIC) {
 	return winColPtr->index;
-    }    
+    }
     return -1;
 }
 
@@ -356,7 +354,7 @@ XAllocColor(display, colormap, color)
 	/*
 	 * Find the nearest existing palette entry.
 	 */
-	
+
 	newPixel = RGB(entry.peRed, entry.peGreen, entry.peBlue);
 	index = GetNearestPaletteIndex(cmap->palette, newPixel);
 	GetPaletteEntries(cmap->palette, index, 1, &closeEntry);
@@ -372,7 +370,7 @@ XAllocColor(display, colormap, color)
 	 * those values and just put the new color in as if the colors
 	 * had not matched.
 	 */
-	
+
 	if ((index >= cmap->size) || (newPixel != closePixel)) {
 	    if (cmap->size == sizePalette) {
 		color->red   = closeEntry.peRed * 257;
@@ -399,11 +397,11 @@ XAllocColor(display, colormap, color)
 	}
 	Tcl_SetHashValue(entryPtr, (ClientData)refCount);
     } else {
-	
+
 	/*
 	 * Determine what color will actually be used on non-colormap systems.
 	 */
-	
+
 	color->pixel = GetNearestColor(dc,
 		RGB(entry.peRed, entry.peGreen, entry.peBlue));
 	color->red    = GetRValue(color->pixel) * 257;
@@ -432,7 +430,7 @@ XAllocColor(display, colormap, color)
  *----------------------------------------------------------------------
  */
 
-void
+int
 XFreeColors(display, colormap, pixels, npixels, planes)
     Display* display;
     Colormap colormap;
@@ -451,7 +449,7 @@ XFreeColors(display, colormap, pixels, npixels, planes)
     /*
      * We don't have to do anything for non-palette devices.
      */
-    
+
     if (GetDeviceCaps(dc, RASTERCAPS) & RC_PALETTE) {
 
 	/*
@@ -487,6 +485,7 @@ XFreeColors(display, colormap, pixels, npixels, planes)
 	}
     }
     ReleaseDC(NULL, dc);
+    return Success;
 }
 
 /*
@@ -569,7 +568,7 @@ XCreateColormap(display, w, visual, alloc)
  *----------------------------------------------------------------------
  */
 
-void
+int
 XFreeColormap(display, colormap)
     Display* display;
     Colormap colormap;
@@ -580,6 +579,7 @@ XFreeColormap(display, colormap)
     }
     Tcl_DeleteHashTable(&cmap->refCounts);
     ckfree((char *) cmap);
+    return Success;
 }
 
 /*
