@@ -7,8 +7,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * SCCS: @(#) tkWinCursor.c 1.9 96/10/31 10:41:10
  */
 
 #include "tkWinInt.h"
@@ -24,6 +22,14 @@ typedef struct {
     int system;			/* 1 if cursor is a system cursor, else 0. */
 } TkWinCursor;
 
+/*
+ * The HAND cursor is only present when WINVER >= 0x0500. If this is
+ * not available at runtime, it will default to the unix-style cursor.
+ */
+#ifndef IDC_HAND
+#define IDC_HAND MAKEINTRESOURCE(32649)
+#endif
+
 /*
  * The table below is used to map from the name of a predefined cursor
  * to its resource identifier.
@@ -38,7 +44,7 @@ static struct CursorName {
     {"ibeam",			IDC_IBEAM},
     {"icon",			IDC_ICON},
     {"no",			IDC_NO},
-    {"size",			IDC_SIZE},
+    {"size",			IDC_SIZEALL},
     {"size_ne_sw",		IDC_SIZENESW},
     {"size_ns",			IDC_SIZENS},
     {"size_nw_se",		IDC_SIZENWSE},
@@ -46,12 +52,13 @@ static struct CursorName {
     {"uparrow",			IDC_UPARROW},
     {"wait",			IDC_WAIT},
     {"crosshair",		IDC_CROSS},
-    {"fleur",			IDC_SIZE},
+    {"fleur",			IDC_SIZEALL},
     {"sb_v_double_arrow",	IDC_SIZENS},
     {"sb_h_double_arrow",	IDC_SIZEWE},
     {"center_ptr",		IDC_UPARROW},
     {"watch",			IDC_WAIT},
     {"xterm",			IDC_IBEAM},
+    {"hand2",			IDC_HAND},
     {NULL,			0}
 };
 
@@ -87,34 +94,76 @@ TkGetCursorByName(interp, tkwin, string)
 {
     struct CursorName *namePtr;
     TkWinCursor *cursorPtr;
+    int argc;
+    CONST char **argv = NULL;
 
     /*
-     * Check for the cursor in the system cursor set.
+     * All cursor names are valid lists of one element (for
+     * Unix-compatability), even unadorned system cursor names.
      */
 
-    for (namePtr = cursorNames; namePtr->name != NULL; namePtr++) {
-	if (strcmp(namePtr->name, string) == 0) {
-	    break;
-	}
+    if (Tcl_SplitList(interp, string, &argc, &argv) != TCL_OK) {
+	return NULL;
+    }
+    if (argc == 0) {
+	goto badCursorSpec;
     }
 
     cursorPtr = (TkWinCursor *) ckalloc(sizeof(TkWinCursor));
     cursorPtr->info.cursor = (Tk_Cursor) cursorPtr;
     cursorPtr->winCursor = NULL;
-    if (namePtr->name != NULL) {
-	cursorPtr->winCursor = LoadCursor(NULL, namePtr->id);
-	cursorPtr->system = 1;
+    cursorPtr->system = 0;
+
+    if (argv[0][0] == '@') {
+	/*
+	 * Check for system cursor of type @<filename>, where only
+	 * the name is allowed.  This accepts any of:
+	 *	-cursor @/winnt/cursors/globe.ani
+	 *	-cursor @C:/Winnt/cursors/E_arrow.cur
+	 *	-cursor {@C:/Program\ Files/Cursors/bart.ani}
+	 *      -cursor {{@C:/Program Files/Cursors/bart.ani}}
+	 *	-cursor [list @[file join "C:/Program Files" Cursors bart.ani]]
+	 */
+
+	if (Tcl_IsSafe(interp)) {
+	    Tcl_AppendResult(interp, "can't get cursor from a file in",
+		    " a safe interpreter", (char *) NULL);
+	    ckfree((char *) argv);
+	    ckfree((char *) cursorPtr);
+	    return NULL;
+	}
+	cursorPtr->winCursor = LoadCursorFromFile(&(argv[0][1]));
+    } else {
+	/*
+	 * Check for the cursor in the system cursor set.
+	 */
+	for (namePtr = cursorNames; namePtr->name != NULL; namePtr++) {
+	    if (strcmp(namePtr->name, argv[0]) == 0) {
+		cursorPtr->winCursor = LoadCursor(NULL, namePtr->id);
+		break;
+	    }
+	}
+
+	if (cursorPtr->winCursor == NULL) {
+	    /*
+	     * Hmm, it is not in the system cursor set.  Check to see
+	     * if it is one of our application resources.
+	     */
+	    cursorPtr->winCursor = LoadCursor(Tk_GetHINSTANCE(), argv[0]);
+	} else {
+	    cursorPtr->system = 1;
+	}
     }
+
     if (cursorPtr->winCursor == NULL) {
-	cursorPtr->winCursor = LoadCursor(TkWinGetTkModule(), string);
-	cursorPtr->system = 0;
-    }
-    if (cursorPtr->winCursor == NULL) {
-	ckfree((char *)cursorPtr);
+	ckfree((char *) cursorPtr);
+    badCursorSpec:
+	ckfree((char *) argv);
 	Tcl_AppendResult(interp, "bad cursor spec \"", string, "\"",
 		(char *) NULL);
 	return NULL;
     } else {
+	ckfree((char *) argv);
 	return (TkCursor *) cursorPtr;
     }
 }
@@ -139,8 +188,8 @@ TkCursor *
 TkCreateCursorFromData(tkwin, source, mask, width, height, xHot, yHot,
 	fgColor, bgColor)
     Tk_Window tkwin;		/* Window in which cursor will be used. */
-    char *source;		/* Bitmap data for cursor shape. */
-    char *mask;			/* Bitmap data for cursor mask. */
+    CONST char *source;		/* Bitmap data for cursor shape. */
+    CONST char *mask;		/* Bitmap data for cursor mask. */
     int width, height;		/* Dimensions of cursor. */
     int xHot, yHot;		/* Location of hot-spot in cursor. */
     XColor fgColor;		/* Foreground color for cursor. */
@@ -152,7 +201,7 @@ TkCreateCursorFromData(tkwin, source, mask, width, height, xHot, yHot,
 /*
  *----------------------------------------------------------------------
  *
- * TkFreeCursor --
+ * TkpFreeCursor --
  *
  *	This procedure is called to release a cursor allocated by
  *	TkGetCursorByName.
@@ -167,11 +216,9 @@ TkCreateCursorFromData(tkwin, source, mask, width, height, xHot, yHot,
  */
 
 void
-TkFreeCursor(cursorPtr)
+TkpFreeCursor(cursorPtr)
     TkCursor *cursorPtr;
 {
-    TkWinCursor *winCursorPtr = (TkWinCursor *) cursorPtr;
-    ckfree((char *) winCursorPtr);
 }
 
 /*
