@@ -1,4 +1,4 @@
-/* 
+/*
  * tkWinDraw.c --
  *
  *	This file contains the Xlib emulation functions pertaining to
@@ -9,8 +9,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkWinDraw.c,v 1.12.2.3 2007/12/05 19:17:32 hobbs Exp $
  */
 
 #include "tkWinInt.h"
@@ -26,7 +24,7 @@
  * Translation table between X gc functions and Win32 raster op modes.
  */
 
-int tkpWinRopModes[] = {
+CONST int tkpWinRopModes[] = {
     R2_BLACK,			/* GXclear */
     R2_MASKPEN,			/* GXand */
     R2_MASKPENNOT,		/* GXandReverse */
@@ -57,7 +55,7 @@ int tkpWinRopModes[] = {
 #define SRCORREVERSE	(DWORD)0x00DD0228 /* dest = source OR (NOT dest) */
 #define SRCNAND		(DWORD)0x007700E6 /* dest = NOT (source AND dest) */
 
-int tkpWinBltModes[] = {
+CONST int tkpWinBltModes[] = {
     BLACKNESS,			/* GXclear */
     SRCAND,			/* GXand */
     SRCERASE,			/* GXandReverse */
@@ -95,9 +93,10 @@ int tkpWinBltModes[] = {
 /*
  * Macros used later in the file.
  */
-
-#define MIN(a,b)	((a>b) ? b : a)
-#define MAX(a,b)	((a<b) ? b : a)
+#ifndef MIN
+#   define MIN(a,b)	((a>b) ? b : a)
+#   define MAX(a,b)	((a<b) ? b : a)
+#endif
 
 /*
  * The followng typedef is used to pass Windows GDI drawing functions.
@@ -118,7 +117,7 @@ static Tcl_ThreadDataKey dataKey;
 
 static POINT *		ConvertPoints _ANSI_ARGS_((XPoint *points, int npoints,
 			    int mode, RECT *bbox));
-static void		DrawOrFillArc _ANSI_ARGS_((Display *display,
+static int		DrawOrFillArc _ANSI_ARGS_((Display *display,
 			    Drawable d, GC gc, int x, int y,
 			    unsigned int width, unsigned int height,
 			    int start, int extent, int fill));
@@ -157,7 +156,7 @@ TkWinGetDrawableDC(display, d, state)
 
     if (twdPtr->type == TWD_WINDOW) {
 	TkWindow *winPtr = twdPtr->window.winPtr;
-    
+
  	dc = GetDC(twdPtr->window.handle);
 	if (winPtr == NULL) {
 	    cmap = DefaultColormap(display, DefaultScreen(display));
@@ -221,7 +220,7 @@ TkWinReleaseDrawableDC(d, dc, state)
  *	Returns the converted array of POINTs.
  *
  * Side effects:
- *	Allocates a block of memory in thread local storage that 
+ *	Allocates a block of memory in thread local storage that
  *      should not be freed.
  *
  *----------------------------------------------------------------------
@@ -234,7 +233,7 @@ ConvertPoints(points, npoints, mode, bbox)
     int mode;			/* CoordModeOrigin or CoordModePrevious. */
     RECT *bbox;			/* Bounding box of points. */
 {
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
             Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     int i;
 
@@ -257,7 +256,7 @@ ConvertPoints(points, npoints, mode, bbox)
 
     bbox->left = bbox->right = points[0].x;
     bbox->top = bbox->bottom = points[0].y;
-    
+
     if (mode == CoordModeOrigin) {
 	for (i = 0; i < npoints; i++) {
 	    tsdPtr->winPoints[i].x = points[i].x;
@@ -300,7 +299,7 @@ ConvertPoints(points, npoints, mode, bbox)
  *----------------------------------------------------------------------
  */
 
-void
+int
 XCopyArea(display, src, dest, gc, src_x, src_y, width, height, dest_x, dest_y)
     Display* display;
     Drawable src;
@@ -336,6 +335,7 @@ XCopyArea(display, src, dest, gc, src_x, src_y, width, height, dest_x, dest_y)
 	TkWinReleaseDrawableDC(dest, destDC, &destState);
     }
     TkWinReleaseDrawableDC(src, srcDC, &srcState);
+    return Success;
 }
 
 /*
@@ -357,7 +357,7 @@ XCopyArea(display, src, dest, gc, src_x, src_y, width, height, dest_x, dest_y)
  *----------------------------------------------------------------------
  */
 
-void
+int
 XCopyPlane(display, src, dest, gc, src_x, src_y, width, height, dest_x,
 	dest_y, plane)
     Display* display;
@@ -487,6 +487,7 @@ XCopyPlane(display, src, dest, gc, src_x, src_y, width, height, dest_x,
 	TkWinReleaseDrawableDC(dest, destDC, &destState);
     }
     TkWinReleaseDrawableDC(src, srcDC, &srcState);
+    return Success;
 }
 
 /*
@@ -506,7 +507,7 @@ XCopyPlane(display, src, dest, gc, src_x, src_y, width, height, dest_x,
  *----------------------------------------------------------------------
  */
 
-void
+int
 TkPutImage(colors, ncolors, display, d, gc, image, src_x, src_y, dest_x,
 	dest_y, width, height)
     unsigned long *colors;		/* Array of pixel values used by this
@@ -516,7 +517,7 @@ TkPutImage(colors, ncolors, display, d, gc, image, src_x, src_y, dest_x,
     Drawable d;				/* Destination drawable. */
     GC gc;
     XImage* image;			/* Source image. */
-    int src_x, src_y;			/* Offset of subimage. */      
+    int src_x, src_y;			/* Offset of subimage. */
     int dest_x, dest_y;			/* Position of subimage origin in
 					 * drawable.  */
     unsigned int width, height;		/* Dimensions of subimage. */
@@ -550,22 +551,22 @@ TkPutImage(colors, ncolors, display, d, gc, image, src_x, src_y, dest_x,
 	}
 	SetTextColor(dc, gc->foreground);
 	SetBkColor(dc, gc->background);
-    } else {    
+    } else {
 	int i, usePalette;
 
 	/*
 	 * Do not use a palette for TrueColor images.
 	 */
-	
+
 	usePalette = (image->bits_per_pixel < 16);
-	
+
 	if (usePalette) {
 	    infoPtr = (BITMAPINFO*) ckalloc(sizeof(BITMAPINFOHEADER)
 		    + sizeof(RGBQUAD)*ncolors);
 	} else {
 	    infoPtr = (BITMAPINFO*) ckalloc(sizeof(BITMAPINFOHEADER));
 	}
-	
+
 	infoPtr->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
 	infoPtr->bmiHeader.biWidth = image->width;
 	infoPtr->bmiHeader.biHeight = -image->height; /* Top-down order */
@@ -596,13 +597,14 @@ TkPutImage(colors, ncolors, display, d, gc, image, src_x, src_y, dest_x,
 	panic("Fail to allocate bitmap\n");
 	DeleteDC(dcMem);
     	TkWinReleaseDrawableDC(d, dc, &state);
-	return;
+	return BadValue;
     }
     bitmap = SelectObject(dcMem, bitmap);
     BitBlt(dc, dest_x, dest_y, width, height, dcMem, src_x, src_y, SRCCOPY);
     DeleteObject(SelectObject(dcMem, bitmap));
     DeleteDC(dcMem);
     TkWinReleaseDrawableDC(d, dc, &state);
+    return Success;
 }
 
 /*
@@ -621,7 +623,7 @@ TkPutImage(colors, ncolors, display, d, gc, image, src_x, src_y, dest_x,
  *----------------------------------------------------------------------
  */
 
-void
+int
 XFillRectangles(display, d, gc, rectangles, nrectangles)
     Display* display;
     Drawable d;
@@ -636,7 +638,7 @@ XFillRectangles(display, d, gc, rectangles, nrectangles)
     HBRUSH brush, oldBrush;
 
     if (d == None) {
-	return;
+	return BadDrawable;
     }
 
     dc = TkWinGetDrawableDC(display, d, &state);
@@ -659,7 +661,7 @@ XFillRectangles(display, d, gc, rectangles, nrectangles)
 	/*
 	 * Select stipple pattern into destination dc.
 	 */
-	
+
 	stipple = CreatePatternBrush(twdPtr->bitmap.handle);
 	SetBrushOrgEx(dc, gc->ts_x_origin, gc->ts_y_origin, NULL);
 	oldBrush = SelectObject(dc, stipple);
@@ -691,7 +693,7 @@ XFillRectangles(display, d, gc, rectangles, nrectangles)
 	    SelectObject(dcMem, oldBitmap);
 	    DeleteObject(bitmap);
 	}
-	
+
 	DeleteDC(dcMem);
 	SelectObject(dc, oldBrush);
 	DeleteObject(stipple);
@@ -709,7 +711,7 @@ XFillRectangles(display, d, gc, rectangles, nrectangles)
 	    HPEN newPen = CreatePen(PS_NULL, 0, gc->foreground);
 	    HPEN oldPen = SelectObject(dc, newPen);
 	    oldBrush = SelectObject(dc, brush);
-	    
+
 	    for (i = 0; i < nrectangles; i++) {
 		Rectangle(dc, rectangles[i].x, rectangles[i].y,
 		    rectangles[i].x + rectangles[i].width + 1,
@@ -723,6 +725,7 @@ XFillRectangles(display, d, gc, rectangles, nrectangles)
     }
     DeleteObject(brush);
     TkWinReleaseDrawableDC(d, dc, &state);
+    return Success;
 }
 
 /*
@@ -752,11 +755,11 @@ RenderObject(dc, gc, points, npoints, mode, pen, func)
     HPEN pen;
     WinDrawFunc func;
 {
-    RECT rect;
+    RECT rect = {0, 0, 0, 0};
     HPEN oldPen;
     HBRUSH oldBrush;
     POINT *winPoints = ConvertPoints(points, npoints, mode, &rect);
-    
+
     if ((gc->fill_style == FillStippled
 	    || gc->fill_style == FillOpaqueStippled)
 	    && gc->stipple != None) {
@@ -767,7 +770,7 @@ RenderObject(dc, gc, points, npoints, mode, pen, func)
 	HBITMAP oldBitmap;
 	int i;
 	HBRUSH oldMemBrush;
-	
+
 	if (twdPtr->type != TWD_BITMAP) {
 	    panic("unexpected drawable type in stipple");
 	}
@@ -787,7 +790,7 @@ RenderObject(dc, gc, points, npoints, mode, pen, func)
 	/*
 	 * Select stipple pattern into destination dc.
 	 */
-	
+
 	SetBrushOrgEx(dc, gc->ts_x_origin, gc->ts_y_origin, NULL);
 	oldBrush = SelectObject(dc, CreatePatternBrush(twdPtr->bitmap.handle));
 
@@ -795,7 +798,7 @@ RenderObject(dc, gc, points, npoints, mode, pen, func)
 	 * Create temporary drawing surface containing a copy of the
 	 * destination equal in size to the bounding box of the object.
 	 */
-	
+
 	dcMem = CreateCompatibleDC(dc);
 	oldBitmap = SelectObject(dcMem, CreateCompatibleBitmap(dc, width,
 		height));
@@ -804,7 +807,7 @@ RenderObject(dc, gc, points, npoints, mode, pen, func)
 
 	/*
 	 * Translate the object for rendering in the temporary drawing
-	 * surface. 
+	 * surface.
 	 */
 
 	for (i = 0; i < npoints; i++) {
@@ -872,7 +875,7 @@ RenderObject(dc, gc, points, npoints, mode, pen, func)
  *----------------------------------------------------------------------
  */
 
-void
+int
 XDrawLines(display, d, gc, points, npoints, mode)
     Display* display;
     Drawable d;
@@ -884,9 +887,9 @@ XDrawLines(display, d, gc, points, npoints, mode)
     HPEN pen;
     TkWinDCState state;
     HDC dc;
-    
+
     if (d == None) {
-	return;
+	return BadDrawable;
     }
 
     dc = TkWinGetDrawableDC(display, d, &state);
@@ -895,8 +898,9 @@ XDrawLines(display, d, gc, points, npoints, mode)
     SetBkMode(dc, TRANSPARENT);
     RenderObject(dc, gc, points, npoints, mode, pen, Polyline);
     DeleteObject(pen);
-    
+
     TkWinReleaseDrawableDC(d, dc, &state);
+    return Success;
 }
 
 /*
@@ -915,7 +919,7 @@ XDrawLines(display, d, gc, points, npoints, mode)
  *----------------------------------------------------------------------
  */
 
-void
+int
 XFillPolygon(display, d, gc, points, npoints, shape, mode)
     Display* display;
     Drawable d;
@@ -930,7 +934,7 @@ XFillPolygon(display, d, gc, points, npoints, shape, mode)
     HDC dc;
 
     if (d == None) {
-	return;
+	return BadDrawable;
     }
 
     dc = TkWinGetDrawableDC(display, d, &state);
@@ -939,6 +943,7 @@ XFillPolygon(display, d, gc, points, npoints, shape, mode)
     RenderObject(dc, gc, points, npoints, mode, pen, Polygon);
 
     TkWinReleaseDrawableDC(d, dc, &state);
+    return Success;
 }
 
 /*
@@ -957,7 +962,7 @@ XFillPolygon(display, d, gc, points, npoints, shape, mode)
  *----------------------------------------------------------------------
  */
 
-void
+int
 XDrawRectangle(display, d, gc, x, y, width, height)
     Display* display;
     Drawable d;
@@ -973,7 +978,7 @@ XDrawRectangle(display, d, gc, x, y, width, height)
     HDC dc;
 
     if (d == None) {
-	return;
+	return BadDrawable;
     }
 
     dc = TkWinGetDrawableDC(display, d, &state);
@@ -989,6 +994,7 @@ XDrawRectangle(display, d, gc, x, y, width, height)
     DeleteObject(SelectObject(dc, oldPen));
     SelectObject(dc, oldBrush);
     TkWinReleaseDrawableDC(d, dc, &state);
+    return Success;
 }
 
 /*
@@ -1007,7 +1013,7 @@ XDrawRectangle(display, d, gc, x, y, width, height)
  *----------------------------------------------------------------------
  */
 
-void
+int
 XDrawArc(display, d, gc, x, y, width, height, start, extent)
     Display* display;
     Drawable d;
@@ -1021,7 +1027,7 @@ XDrawArc(display, d, gc, x, y, width, height, start, extent)
 {
     display->request++;
 
-    DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, 0);
+    return DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, 0);
 }
 
 /*
@@ -1040,7 +1046,7 @@ XDrawArc(display, d, gc, x, y, width, height, start, extent)
  *----------------------------------------------------------------------
  */
 
-void
+int
 XFillArc(display, d, gc, x, y, width, height, start, extent)
     Display* display;
     Drawable d;
@@ -1054,7 +1060,7 @@ XFillArc(display, d, gc, x, y, width, height, start, extent)
 {
     display->request++;
 
-    DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, 1);
+    return DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, 1);
 }
 
 /*
@@ -1074,7 +1080,7 @@ XFillArc(display, d, gc, x, y, width, height, start, extent)
  *----------------------------------------------------------------------
  */
 
-static void
+static int
 DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, fill)
     Display *display;
     Drawable d;
@@ -1094,7 +1100,7 @@ DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, fill)
     double radian_start, radian_end, xr, yr;
 
     if (d == None) {
-	return;
+	return BadDrawable;
     }
 
     dc = TkWinGetDrawableDC(display, d, &state);
@@ -1164,6 +1170,7 @@ DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, fill)
     }
     DeleteObject(SelectObject(dc, oldPen));
     TkWinReleaseDrawableDC(d, dc, &state);
+    return Success;
 }
 
 /*
@@ -1228,24 +1235,24 @@ SetUpGraphicsPort(gc)
 	switch (gc->cap_style) {
 	    case CapNotLast:
 	    case CapButt:
-		style |= PS_ENDCAP_FLAT; 
+		style |= PS_ENDCAP_FLAT;
 		break;
 	    case CapRound:
-		style |= PS_ENDCAP_ROUND; 
+		style |= PS_ENDCAP_ROUND;
 		break;
 	    default:
-		style |= PS_ENDCAP_SQUARE; 
+		style |= PS_ENDCAP_SQUARE;
 		break;
 	}
 	switch (gc->join_style) {
-	    case JoinMiter: 
-		style |= PS_JOIN_MITER; 
+	    case JoinMiter:
+		style |= PS_JOIN_MITER;
 		break;
 	    case JoinRound:
-		style |= PS_JOIN_ROUND; 
+		style |= PS_JOIN_ROUND;
 		break;
 	    default:
-		style |= PS_JOIN_BEVEL; 
+		style |= PS_JOIN_BEVEL;
 		break;
 	}
 	return ExtCreatePen(style, gc->line_width, &lb, 0, NULL);
@@ -1349,7 +1356,7 @@ TkWinFillRect(dc, x, y, width, height, pixel)
  *----------------------------------------------------------------------
  */
 
-void 
+void
 TkpDrawHighlightBorder(tkwin, fgGC, bgGC, highlightWidth, drawable)
     Tk_Window tkwin;
     GC fgGC;
