@@ -10,8 +10,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkWinInt.h,v 1.14.2.5 2004/09/23 01:49:08 hobbs Exp $
  */
 
 #ifndef _TKWININT
@@ -123,7 +121,6 @@ typedef struct {
  */
 
 #define TK_WIN_TOPLEVEL_CLASS_NAME "TkTopLevel"
-#define TK_WIN_TOPLEVEL_NOCDC_CLASS_NAME "TkTopLevelNoCDC"
 #define TK_WIN_CHILD_CLASS_NAME "TkChild"
 
 /*
@@ -131,8 +128,8 @@ typedef struct {
  * Win32 raster and BitBlt op modes.
  */
 
-extern int tkpWinRopModes[];
-extern int tkpWinBltModes[];
+extern CONST int tkpWinRopModes[];
+extern CONST int tkpWinBltModes[];
 
 /*
  * The following defines are used with TkWinGetBorderPixels to get the
@@ -157,9 +154,6 @@ extern int tkpWinBltModes[];
 #define TCL_STORAGE_CLASS DLLEXPORT
 #endif
 
-EXTERN LRESULT CALLBACK	TkWinChildProc _ANSI_ARGS_((HWND hwnd, UINT message,
-			    WPARAM wParam, LPARAM lParam));
-
 /*
  * Special proc needed as tsd accessor function between
  * tkWinX.c:GenerateXEvent and tkWinClipboard.c:UpdateClipboard
