@@ -8,8 +8,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkWinMenu.c,v 1.21.2.9 2007/06/09 23:52:40 hobbs Exp $
  */
 
 #define OEMRESOURCE
@@ -52,6 +50,10 @@
 #define MENU_SYSTEM_MENU	    MENU_PLATFORM_FLAG1
 #define MENU_RECONFIGURE_PENDING    MENU_PLATFORM_FLAG2
 
+#ifndef WM_UNINITMENUPOPUP
+#define WM_UNINITMENUPOPUP              0x0125
+#endif
+
 static int indicatorDimensions[2];
 				/* The dimensions of the indicator space
 				 * in a menu entry. Calculated at init
@@ -195,38 +197,35 @@ GetNewID(mePtr, menuIDPtr)
     TkMenuEntry *mePtr;		/* The menu we are working with */
     WORD *menuIDPtr;		/* The resulting id */
 {
-    int found = 0;
-    int newEntry;
-    Tcl_HashEntry *commandEntryPtr;
-    WORD returnID;
     ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
-            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+    WORD curID = tsdPtr->lastCommandID;
 
-    WORD curID = tsdPtr->lastCommandID + 1;
+    while (1) {
+	Tcl_HashEntry *commandEntryPtr;
+	int new;
 
-    /*
-     * The following code relies on WORD wrapping when the highest value is
-     * incremented.
-     */
-    
-    while (curID != tsdPtr->lastCommandID) {
-    	commandEntryPtr = Tcl_CreateHashEntry(&tsdPtr->commandTable,
-		(char *) curID, &newEntry);
-    	if (newEntry == 1) {
-    	    found = 1;
-    	    returnID = curID;
-    	    break;
-    	}
-    	curID++;
-    }
+	/*
+	 * Try the next ID number, taking care to wrap rather than stray
+	 * into the system menu IDs.  [Bug 3235256]
+	 */
+	if (++curID >= 0xF000) {
+	    curID = 1;
+	}
 
-    if (found) {
-    	Tcl_SetHashValue(commandEntryPtr, (char *) mePtr);
-    	*menuIDPtr = returnID;
-    	tsdPtr->lastCommandID = returnID;
-    	return TCL_OK;
-    } else {
-    	return TCL_ERROR;
+	/* Return error when we've checked all IDs without success. */
+	if (curID == tsdPtr->lastCommandID) {
+	    return TCL_ERROR;
+	}
+
+	commandEntryPtr = Tcl_CreateHashEntry(&tsdPtr->commandTable,
+		(char *) curID, &new);
+	if (new) {
+	    Tcl_SetHashValue(commandEntryPtr, (char *) mePtr);
+	    *menuIDPtr = curID;
+	    tsdPtr->lastCommandID = curID;
+	    return TCL_OK;
+	}
     }
 }
 
@@ -543,7 +542,7 @@ ReconfigureWindowsMenu(
     UINT flags;
     UINT itemID;
     int i, count, systemMenu = 0, base;
-    int width, height;
+    int width = 0, height = 0;
     Tcl_DString translatedText;
   
     if (NULL == winMenuHdl) {
@@ -940,6 +939,19 @@ TkWinHandleMenuEvent(phwnd, pMessage, pwParam, plParam, plResult)
             Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     switch (*pMessage) {
+ 	case WM_UNINITMENUPOPUP:
+	    hashEntryPtr = Tcl_FindHashEntry(&tsdPtr->winMenuTable,
+		    (char *) *pwParam);
+	    if (hashEntryPtr != NULL) {
+   		menuPtr = (TkMenu *) Tcl_GetHashValue(hashEntryPtr);
+		if ((menuPtr->menuRefPtr != NULL)
+			&& (menuPtr->menuRefPtr->parentEntryPtr != NULL)) {
+		    TkPostSubmenu(menuPtr->interp,
+			    menuPtr->menuRefPtr->parentEntryPtr->menuPtr, NULL);
+		}
+	    }
+	    break;
+
 	case WM_INITMENU:
 	    TkMenuInit();
 	    hashEntryPtr = Tcl_FindHashEntry(&tsdPtr->winMenuTable, 
@@ -1657,7 +1669,7 @@ DrawMenuEntryAccelerator(menuPtr, mePtr, d, gc, tkfont, fmPtr,
 {
     int baseline;
     int leftEdge = x + mePtr->indicatorSpace + mePtr->labelWidth;
-    char *accel;
+    char *accel = NULL;
     
     if (mePtr->accelPtr != NULL) {
 	accel = Tcl_GetStringFromObj(mePtr->accelPtr, NULL);
@@ -1726,8 +1738,7 @@ DrawMenuEntryArrow(menuPtr, mePtr, d, gc,
     COLORREF oldBgColor;
     RECT rect;
 
-    if (!drawArrow || (mePtr->type != CASCADE_ENTRY) ||
-            (mePtr->state != ENTRY_DISABLED))
+    if (!drawArrow || (mePtr->type != CASCADE_ENTRY))
         return;
 
     oldFgColor = gc->foreground;
@@ -1743,7 +1754,8 @@ DrawMenuEntryArrow(menuPtr, mePtr, d, gc,
         gc->background = activeBgColor->pixel;
     }
 
-    gc->foreground = GetSysColor(COLOR_GRAYTEXT);
+    gc->foreground = GetSysColor((mePtr->state == ENTRY_DISABLED) ?
+	    COLOR_GRAYTEXT : COLOR_MENUTEXT);
 
     rect.top = y + GetSystemMetrics(SM_CYBORDER);
     rect.bottom = y + height - GetSystemMetrics(SM_CYBORDER);
@@ -2468,7 +2480,7 @@ GetMenuLabelGeometry(mePtr, tkfont, fmPtr, widthPtr, heightPtr)
 					 * portion */
 {
     TkMenu *menuPtr = mePtr->menuPtr;
-    int haveImage = 0, haveText = 0;
+    int haveImage = 0;
  
     if (mePtr->image != NULL) {
     	Tk_SizeOfImage(mePtr->image, widthPtr, heightPtr);
@@ -2955,7 +2967,14 @@ SetDefaults(
     TEXTMETRIC tm;
     int pointSize;
     HFONT menuFont;
-    NONCLIENTMETRICS ncMetrics;
+    /* See: [Bug #3239768] tk8.4.19 (and later) WIN32 menu font support */
+    struct {
+        NONCLIENTMETRICS metrics;
+#if (WINVER < 0x0600)
+        int padding;
+#endif
+    } nc;
+    OSVERSIONINFO os;
 
     /*
      * Set all of the default options. The loop will terminate when we run 
@@ -2973,10 +2992,17 @@ SetDefaults(
     }
     Tcl_DStringInit(&menuFontDString);
 
-    ncMetrics.cbSize = sizeof(ncMetrics);
-    SystemParametersInfo(SPI_GETNONCLIENTMETRICS, sizeof(ncMetrics),
-	    &ncMetrics, 0);
-    menuFont = CreateFontIndirect(&ncMetrics.lfMenuFont);
+    nc.metrics.cbSize = sizeof(nc);
+
+    os.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+    GetVersionEx(&os);
+    if (os.dwMajorVersion < 6) {
+	nc.metrics.cbSize -= sizeof(int);
+    }
+
+    SystemParametersInfo(SPI_GETNONCLIENTMETRICS, nc.metrics.cbSize,
+	    &nc.metrics, 0);
+    menuFont = CreateFontIndirect(&nc.metrics.lfMenuFont);
     SelectObject(scratchDC, menuFont);
     GetTextMetrics(scratchDC, &tm);
     GetTextFace(scratchDC, LF_FACESIZE, faceName);
