@@ -8,8 +8,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkWinScrlbr.c,v 1.5.2.1 2001/04/04 07:57:18 hobbs Exp $
  */
 
 #include "tkWinInt.h"
@@ -87,9 +85,10 @@ static void		UpdateScrollbarMetrics _ANSI_ARGS_((void));
  * The class procedure table for the scrollbar widget.
  */
 
-TkClassProcs tkpScrollbarProcs = {
+Tk_ClassProcs tkpScrollbarProcs = {
+    sizeof(Tk_ClassProcs),	/* size */
+    NULL,			/* worldChangedProc */
     CreateProc,			/* createProc */
-    NULL,			/* geometryProc */
     ModalLoopProc,		/* modalProc */
 };
 
@@ -184,6 +183,12 @@ UpdateScrollbar(scrollPtr)
 		    * (MAX_SCROLL - (scrollInfo.nPage - 1)));
     } else {
 	scrollInfo.nPos = 0;
+	/*
+	 * Disable the scrollbar when there is nothing to scroll.
+	 * This is standard Windows style (see eg Notepad).
+	 * Also prevents possible crash on XP+ systems [Bug #624116].
+	 */
+	scrollInfo.fMask |= SIF_DISABLENOSCROLL;
     }
     SetScrollInfo(scrollPtr->hwnd, SB_CTL, &scrollInfo, TRUE);
 }
@@ -241,7 +246,7 @@ CreateProc(tkwin, parentWin, instanceData)
 
     for (winPtr = ((TkWindow*)tkwin)->nextPtr; winPtr != NULL;
 	 winPtr = winPtr->nextPtr) {
-	if ((winPtr->window != None) && !(winPtr->flags & TK_TOP_LEVEL)) {
+	if ((winPtr->window != None) && !(winPtr->flags & TK_TOP_HIERARCHY)) {
 	    TkWinSetWindowPos(scrollPtr->hwnd, Tk_GetHWND(winPtr->window),
 		    Below);
 	    break;
@@ -577,7 +582,7 @@ ScrollbarProc(hwnd, message, wParam, lParam)
 	    }
 
 	    interp = scrollPtr->info.interp;
-	    code = Tcl_GlobalEval(interp, cmdString.string);
+	    code = Tcl_EvalEx(interp, cmdString.string, -1, TCL_EVAL_GLOBAL);
 	    if (code != TCL_OK && code != TCL_CONTINUE && code != TCL_BREAK) {
 		Tcl_AddErrorInfo(interp, "\n    (scrollbar command)");
 		Tcl_BackgroundError(interp);
