@@ -1,7 +1,7 @@
-/* 
+/*
  * tkWinX.c --
  *
- *	This file contains Windows emulation procedures for X routines. 
+ *	This file contains Windows emulation procedures for X routines.
  *
  * Copyright (c) 1995-1996 Sun Microsystems, Inc.
  * Copyright (c) 1994 Software Research Associates, Inc.
@@ -9,8 +9,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkWinX.c,v 1.25.2.6 2005/02/28 22:10:26 hobbs Exp $
  */
 
 #include "tkWinInt.h"
@@ -22,7 +20,7 @@
  */
 
 #ifndef _WIN32_IE
-#define _WIN32_IE 0x0300
+#define _WIN32_IE 0x0501 /* IE 5 */
 #endif
 
 #include <commctrl.h>
@@ -39,6 +37,15 @@
 
 #include <imm.h>
 
+/*
+ * WM_UNICHAR is a message for Unicode input on all windows systems.
+ * Perhaps this definition should be moved in another file.
+ */
+#ifndef WM_UNICHAR
+#define WM_UNICHAR     0x0109
+#define UNICODE_NOCHAR 0xFFFF
+#endif
+
 static TkWinProcs asciiProcs = {
     0,
 
@@ -146,8 +153,8 @@ TkGetServerInfo(interp, tkwin)
 
     os.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
     GetVersionEx(&os);
-    sprintf(buffer, "Windows %d.%d %d %s", os.dwMajorVersion,
-	    os.dwMinorVersion, os.dwBuildNumber,
+    sprintf(buffer, "Windows %d.%d %d %s", (int)os.dwMajorVersion,
+	    (int)os.dwMinorVersion, (int)os.dwBuildNumber,
 #ifdef _WIN64
 	    "Win64"
 #else
@@ -226,6 +233,9 @@ void
 TkWinXInit(hInstance)
     HINSTANCE hInstance;
 {
+    CHARSETINFO lpCs;
+    DWORD lpCP;
+
     if (childClassInitialized != 0) {
 	return;
     }
@@ -246,21 +256,7 @@ TkWinXInit(hInstance)
 	tkWinProcs = &asciiProcs;
     }
 
-    /*
-     * When threads are enabled, we cannot use CLASSDC because
-     * threads will then write into the same device context.
-     * 
-     * This is a hack; we should add a subsystem that manages
-     * device context on a per-thread basis.  See also tkWinWm.c,
-     * which also initializes a WNDCLASS structure.
-     */
-
-#ifdef TCL_THREADS
     childClass.style = CS_HREDRAW | CS_VREDRAW;
-#else
-    childClass.style = CS_HREDRAW | CS_VREDRAW | CS_CLASSDC;
-#endif
-
     childClass.cbClsExtra = 0;
     childClass.cbWndExtra = 0;
     childClass.hInstance = hInstance;
@@ -280,6 +276,17 @@ TkWinXInit(hInstance)
 	panic("Unable to register TkChild class");
     }
 
+    /*
+     * Initialize input language info
+     */
+
+    if (GetLocaleInfo(LANGIDFROMLCID(GetKeyboardLayout(0)),
+	       LOCALE_IDEFAULTANSICODEPAGE | LOCALE_RETURN_NUMBER,
+	       (LPTSTR) &lpCP, sizeof(lpCP)/sizeof(TCHAR))
+	    && TranslateCharsetInfo((DWORD *)lpCP, &lpCs, TCI_SRCCODEPAGE)) {
+	UpdateInputLanguage(lpCs.ciCharset);
+    }
+
     /*
      * Make sure we cleanup on finalize.
      */
@@ -310,7 +317,7 @@ TkWinXCleanup(clientData)
     /*
      * Clean up our own class.
      */
-    
+
     if (childClassInitialized) {
         childClassInitialized = 0;
         UnregisterClass(TK_WIN_CHILD_CLASS_NAME, hInstance);
@@ -324,7 +331,7 @@ TkWinXCleanup(clientData)
     /*
      * And let the window manager clean up its own class(es).
      */
-    
+
     TkWinWmCleanup(hInstance);
 }
 
@@ -333,12 +340,12 @@ TkWinXCleanup(clientData)
  *
  * TkWinGetPlatformId --
  *
- *	Determines whether running under NT, 95, or Win32s, to allow 
+ *	Determines whether running under NT, 95, or Win32s, to allow
  *	runtime conditional code.  Win32s is no longer supported.
  *
  * Results:
  *	The return value is one of:
- *	    VER_PLATFORM_WIN32s		Win32s on Windows 3.1. 
+ *	    VER_PLATFORM_WIN32s		Win32s on Windows 3.1.
  *	    VER_PLATFORM_WIN32_WINDOWS	Win32 on Windows 95.
  *	    VER_PLATFORM_WIN32_NT	Win32 on Windows NT
  *
@@ -357,10 +364,10 @@ TkWinGetPlatformId()
 	os.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
 	GetVersionEx(&os);
 	tkPlatformId = os.dwPlatformId;
-        
-        /* Set tkWinTheme to be TK_THEME_WIN_XP or TK_THEME_WIN_CLASSIC.
-         * The TK_THEME_WIN_CLASSIC could be set even when running
-         * under XP if the windows classic theme was selected. */
+
+	/* Set tkWinTheme to be TK_THEME_WIN_XP or TK_THEME_WIN_CLASSIC.
+	 * The TK_THEME_WIN_CLASSIC could be set even when running
+	 * under XP if the windows classic theme was selected. */
 	if ((os.dwPlatformId == VER_PLATFORM_WIN32_NT) &&
 	        (os.dwMajorVersion == 5 && os.dwMinorVersion == 1)) {
 	    HKEY   hKey;
@@ -373,7 +380,7 @@ TkWinGetPlatformId()
                     KEY_READ, &hKey) != ERROR_SUCCESS) {
                 tkWinTheme = TK_THEME_WIN_XP;
 	    } else {
-	        RegQueryValueEx(hKey, szCurrent, NULL, NULL, pBuffer, &dwSize);
+	        RegQueryValueEx(hKey, szCurrent, NULL, NULL, (BYTE *) pBuffer, &dwSize);
 	        RegCloseKey(hKey);
 	        if (strcmp(pBuffer, "Windows Standard") == 0) {
 	            tkWinTheme = TK_THEME_WIN_CLASSIC;
@@ -561,11 +568,11 @@ TkpOpenDisplay(display_name)
     Screen *screen;
     TkWinDrawable *twdPtr;
     Display *display;
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     if (tsdPtr->winDisplay != NULL) {
-	if (strcmp(tsdPtr->winDisplay->display->display_name, display_name) 
+	if (strcmp(tsdPtr->winDisplay->display->display_name, display_name)
                 == 0) {
 	    return tsdPtr->winDisplay;
 	} else {
@@ -646,7 +653,7 @@ TkpCloseDisplay(dispPtr)
 {
     Display *display = dispPtr->display;
     HWND hwnd;
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     if (dispPtr != tsdPtr->winDisplay) {
@@ -657,7 +664,7 @@ TkpCloseDisplay(dispPtr)
     /*
      * Force the clipboard to be rendered if we are the clipboard owner.
      */
-    
+
     if (dispPtr->clipWindow) {
 	hwnd = Tk_GetHWND(Tk_WindowId(dispPtr->clipWindow));
 	if (GetClipboardOwner() == hwnd) {
@@ -752,12 +759,13 @@ TkClipCleanup(dispPtr)
  *----------------------------------------------------------------------
  */
 
-void
+int
 XBell(display, percent)
     Display* display;
     int percent;
 {
     MessageBeep(MB_OK);
+    return Success;
 }
 
 /*
@@ -825,6 +833,22 @@ TkWinChildProc(hwnd, message, wParam, lParam)
 	    result =  TkWinEmbeddedEventProc(hwnd, message, wParam, lParam);
 	    break;
 
+	case WM_UNICHAR:
+	    if (wParam == UNICODE_NOCHAR) {
+		/* If wParam is UNICODE_NOCHAR and the application processes
+		 * this message, then return TRUE. */
+		result = 1;
+	    } else {
+		/* If the event was translated, we must return 0 */
+		if (Tk_TranslateWinEvent(hwnd, message, wParam, lParam,
+			    &result)) {
+		    result = 0;
+		} else {
+		    result = 1;
+		}
+	    }
+	    break;
+
 	default:
 	    if (!Tk_TranslateWinEvent(hwnd, message, wParam, lParam,
 		    &result)) {
@@ -914,6 +938,7 @@ Tk_TranslateWinEvent(hwnd, message, wParam, lParam, resultPtr)
 	case WM_SETFOCUS:
 	case WM_KILLFOCUS:
 	case WM_DESTROYCLIPBOARD:
+	case WM_UNICHAR:
 	case WM_CHAR:
 	case WM_SYSKEYDOWN:
 	case WM_SYSKEYUP:
@@ -957,7 +982,7 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 {
     XEvent event;
     TkWindow *winPtr = (TkWindow *)Tk_HWNDToWindow(hwnd);
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     if (!winPtr || winPtr->window == None) {
@@ -1061,6 +1086,7 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 	     */
 
 	case WM_CHAR:
+	case WM_UNICHAR:
 	case WM_SYSKEYDOWN:
 	case WM_SYSKEYUP:
 	case WM_KEYDOWN:
@@ -1151,31 +1177,31 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 		     * Synthesize both a KeyPress and a KeyRelease.
 		     * Strings generated by Input Method Editor are handled
 		     * in the following manner:
-		     * 1. A series of WM_KEYDOWN & WM_KEYUP messages that 
+		     * 1. A series of WM_KEYDOWN & WM_KEYUP messages that
 		     *    cause GetTranslatedKey() to be called and return
-		     *    immediately because the WM_KEYDOWNs have no 
-		     *	  associated WM_CHAR messages -- the IME window is 
-		     *	  accumulating the characters and translating them 
+		     *    immediately because the WM_KEYDOWNs have no
+		     *	  associated WM_CHAR messages -- the IME window is
+		     *	  accumulating the characters and translating them
 		     *    itself.  In the "bind" command, you get an event
-		     *	  with a mystery keysym and %A == "" for each 
+		     *	  with a mystery keysym and %A == "" for each
 		     *	  WM_KEYDOWN that actually was meant for the IME.
 		     * 2. A WM_KEYDOWN corresponding to the "confirm typing"
-		     *    character.  This causes GetTranslatedKey() to be 
+		     *    character.  This causes GetTranslatedKey() to be
 		     *	  called.
-		     * 3. A WM_IME_NOTIFY message saying that the IME is 
-		     *	  done.  A side effect of this message is that 
+		     * 3. A WM_IME_NOTIFY message saying that the IME is
+		     *	  done.  A side effect of this message is that
 		     *    GetTranslatedKey() thinks this means that there
 		     *	  are no WM_CHAR messages and returns immediately.
 		     *    In the "bind" command, you get an another event
 		     *	  with a mystery keysym and %A == "".
-		     * 4. A sequence of WM_CHAR messages that correspond to 
-		     *	  the characters in the IME window.  A bunch of 
-		     *    simulated KeyPress/KeyRelease events will be 
-		     *    generated, one for each character.  Adjacent 
+		     * 4. A sequence of WM_CHAR messages that correspond to
+		     *	  the characters in the IME window.  A bunch of
+		     *    simulated KeyPress/KeyRelease events will be
+		     *    generated, one for each character.  Adjacent
 		     *    WM_CHAR messages may actually specify the high
 		     *	  and low bytes of a multi-byte character -- in that
 		     *    case the two WM_CHAR messages will be combined into
-		     *	  one event.  It is the event-consumer's 
+		     *	  one event.  It is the event-consumer's
 		     *	  responsibility to convert the string returned from
 		     *	  XLookupString from system encoding to UTF-8.
 		     * 5. And finally we get the WM_KEYUP for the "confirm
@@ -1201,6 +1227,21 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 		    Tk_QueueWindowEvent(&event, TCL_QUEUE_TAIL);
 		    event.type = KeyRelease;
 		    break;
+
+		case WM_UNICHAR: {
+		    char buffer[TCL_UTF_MAX+1];
+		    int i;
+		    event.type = KeyPress;
+		    event.xany.send_event = -3;
+		    event.xkey.keycode = wParam;
+		    event.xkey.nbytes = Tcl_UniCharToUtf(wParam, buffer);
+		    for (i=0; i<event.xkey.nbytes && i<TCL_UTF_MAX; ++i) {
+			event.xkey.trans_chars[i] = buffer[i];
+		    }
+		    Tk_QueueWindowEvent(&event, TCL_QUEUE_TAIL);
+		    event.type = KeyRelease;
+		    break;
+		}
 	    }
 	    break;
 	}
@@ -1216,7 +1257,7 @@ GenerateXEvent(hwnd, message, wParam, lParam)
  *
  * GetState --
  *
- *	This function constructs a state mask for the mouse buttons 
+ *	This function constructs a state mask for the mouse buttons
  *	and modifier keys as they were before the event occured.
  *
  * Results:
@@ -1310,7 +1351,7 @@ GetTranslatedKey(xkey)
     XKeyEvent *xkey;
 {
     MSG msg;
-    
+
     xkey->nbytes = 0;
 
     while ((xkey->nbytes < XMaxTransChars)
@@ -1355,13 +1396,12 @@ GetTranslatedKey(xkey)
  *
  * UpdateInputLanguage --
  *
- *	Gets called when a WM_INPUTLANGCHANGE message is received
- *      by the TK child window procedure. This message is sent
- *      by the Input Method Editor system when the user chooses
- *      a different input method. All subsequent WM_CHAR
- *      messages will contain characters in the new encoding. We record
- *      the new encoding so that TkpGetString() knows how to
- *      correctly translate the WM_CHAR into unicode.
+ *	Gets called when a WM_INPUTLANGCHANGE message is received by the Tk
+ *	child window procedure. This message is sent by the Input Method
+ *	Editor system when the user chooses a different input method. All
+ *	subsequent WM_CHAR messages will contain characters in the new
+ *	encoding. We record the new encoding so that TkpGetString() knows how
+ *	to correctly translate the WM_CHAR into unicode.
  *
  * Results:
  *	Records the new encoding in keyInputEncoding.
@@ -1475,7 +1515,7 @@ TkWinGetUnicodeEncoding()
  *
  *      This function correctly processes the composition data and
  *      sends the UNICODE values of the composed characters to
- *      TK's event queue. 
+ *      TK's event queue.
  *
  * Results:
  *	If this function has processed the composition data, returns 1.
@@ -1724,7 +1764,7 @@ TkpGetMS()
 void
 TkWinUpdatingClipboard(int mode)
 {
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     tsdPtr->updatingClipboard = mode;
