@@ -8,8 +8,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: winMain.c,v 1.7.6.1 1999/11/03 00:43:09 hobbs Exp $
  */
 
 #include <tk.h>
@@ -32,19 +30,36 @@
  */
 
 static void		setargv _ANSI_ARGS_((int *argcPtr, char ***argvPtr));
-static void		WishPanic _ANSI_ARGS_(TCL_VARARGS(char *,format));
+static Tcl_PanicProc	WishPanic;
 
 #ifdef TK_TEST
 extern int		Tktest_Init(Tcl_Interp *interp);
 #endif /* TK_TEST */
 
-#ifdef TCL_TEST
-extern int		TclObjTest_Init _ANSI_ARGS_((Tcl_Interp *interp));
-extern int		Tcltest_Init _ANSI_ARGS_((Tcl_Interp *interp));
-#endif /* TCL_TEST */
-
 static BOOL consoleRequired = TRUE;
 
+/*
+ * The following #if block allows you to change the AppInit
+ * function by using a #define of TCL_LOCAL_APPINIT instead
+ * of rewriting this entire file.  The #if checks for that
+ * #define and uses Tcl_AppInit if it doesn't exist.
+ */
+    
+#ifndef TK_LOCAL_APPINIT
+#define TK_LOCAL_APPINIT Tcl_AppInit    
+#endif
+extern int TK_LOCAL_APPINIT _ANSI_ARGS_((Tcl_Interp *interp));
+    
+/*
+ * The following #if block allows you to change how Tcl finds the startup
+ * script, prime the library or encoding paths, fiddle with the argv,
+ * etc., without needing to rewrite Tk_Main()
+ */
+
+#ifdef TK_LOCAL_MAIN_HOOK
+extern int TK_LOCAL_MAIN_HOOK _ANSI_ARGS_((int *argc, char ***argv));
+#endif
+
 
 /*
  *----------------------------------------------------------------------
@@ -72,9 +87,19 @@ WinMain(hInstance, hPrevInstance, lpszCmdLine, nCmdShow)
 {
     char **argv;
     int argc;
+    char buffer[MAX_PATH+1];
+    char *p;
 
     Tcl_SetPanicProc(WishPanic);
 
+    /*
+     * Create the console channels and install them as the standard
+     * channels.  All I/O will be discarded until Tk_CreateConsoleWindow is
+     * called to attach the console to a text widget.
+     */
+
+    consoleRequired = TRUE;
+
     /*
      * Set up the default locale to be standard "C" locale so parsing
      * is performed correctly.
@@ -84,24 +109,23 @@ WinMain(hInstance, hPrevInstance, lpszCmdLine, nCmdShow)
     setargv(&argc, &argv);
 
     /*
-     * Increase the application queue size from default value of 8.
-     * At the default value, cross application SendMessage of WM_KILLFOCUS
-     * will fail because the handler will not be able to do a PostMessage!
-     * This is only needed for Windows 3.x, since NT dynamically expands
-     * the queue.
+     * Replace argv[0] with full pathname of executable, and forward
+     * slashes substituted for backslashes.
      */
 
-    SetMessageQueue(64);
+    GetModuleFileName(NULL, buffer, sizeof(buffer));
+    argv[0] = buffer;
+    for (p = buffer; *p != '\0'; p++) {
+	if (*p == '\\') {
+	    *p = '/';
+	}
+    }
 
-    /*
-     * Create the console channels and install them as the standard
-     * channels.  All I/O will be discarded until Tk_CreateConsoleWindow is
-     * called to attach the console to a text widget.
-     */
+#ifdef TK_LOCAL_MAIN_HOOK
+    TK_LOCAL_MAIN_HOOK(&argc, &argv);
+#endif
 
-    consoleRequired = TRUE;
-
-    Tk_Main(argc, argv, Tcl_AppInit);
+    Tk_Main(argc, argv, TK_LOCAL_APPINIT);
     return 1;
 }
 
@@ -147,17 +171,22 @@ Tcl_AppInit(interp)
 	    goto error;
 	}
     }
+#if defined(STATIC_BUILD) && defined(TCL_USE_STATIC_PACKAGES)
+    {
+	extern Tcl_PackageInitProc Registry_Init;
+	extern Tcl_PackageInitProc Dde_Init;
 
-#ifdef TCL_TEST
-    if (Tcltest_Init(interp) == TCL_ERROR) {
-	return TCL_ERROR;
-    }
-    Tcl_StaticPackage(interp, "Tcltest", Tcltest_Init,
-            (Tcl_PackageInitProc *) NULL);
-    if (TclObjTest_Init(interp) == TCL_ERROR) {
-	return TCL_ERROR;
-    }
-#endif /* TCL_TEST */
+	if (Registry_Init(interp) == TCL_ERROR) {
+	    return TCL_ERROR;
+	}
+	Tcl_StaticPackage(interp, "registry", Registry_Init, NULL);
+
+	if (Dde_Init(interp) == TCL_ERROR) {
+	    return TCL_ERROR;
+	}
+	Tcl_StaticPackage(interp, "dde", Dde_Init, NULL);
+   }
+#endif
 
 #ifdef TK_TEST
     if (Tktest_Init(interp) == TCL_ERROR) {
@@ -196,13 +225,13 @@ error:
  */
 
 void
-WishPanic TCL_VARARGS_DEF(char *,arg1)
+WishPanic TCL_VARARGS_DEF(CONST char *,arg1)
 {
     va_list argList;
     char buf[1024];
-    char *format;
+    CONST char *format;
     
-    format = TCL_VARARGS_START(char *,arg1,argList);
+    format = TCL_VARARGS_START(CONST char *,arg1,argList);
     vsprintf(buf, format, argList);
 
     MessageBeep(MB_ICONEXCLAMATION);
@@ -213,7 +242,6 @@ WishPanic TCL_VARARGS_DEF(char *,arg1)
 #endif
     ExitProcess(1);
 }
-
 /*
  *-------------------------------------------------------------------------
  *
@@ -331,6 +359,7 @@ setargv(argcPtr, argvPtr)
     *argvPtr = argv;
 }
 
+#if !defined(__GNUC__) || defined(TK_TEST)
 /*
  *----------------------------------------------------------------------
  *
@@ -358,15 +387,6 @@ int main(int argc, char **argv)
      */
 
     setlocale(LC_ALL, "C");
-    /*
-     * Increase the application queue size from default value of 8.
-     * At the default value, cross application SendMessage of WM_KILLFOCUS
-     * will fail because the handler will not be able to do a PostMessage!
-     * This is only needed for Windows 3.x, since NT dynamically expands
-     * the queue.
-     */
-
-    SetMessageQueue(64);
 
     /*
      * Create the console channels and install them as the standard
@@ -379,4 +399,4 @@ int main(int argc, char **argv)
     Tk_Main(argc, argv, Tcl_AppInit);
     return 0;
 }
-
+#endif /* !__GNUC__ || TK_TEST */
