@@ -1,4 +1,4 @@
-/* 
+/*
  * xdraw.c --
  *
  *	This file contains generic procedures related to X drawing
@@ -8,8 +8,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: xdraw.c,v 1.2 1998/09/14 18:24:03 stanton Exp $
  */
 
 #include "tk.h"
@@ -19,7 +17,7 @@
  *
  * XDrawLine --
  *
- *	Draw a single line between two points in a given drawable. 
+ *	Draw a single line between two points in a given drawable.
  *
  * Results:
  *	None.
@@ -30,7 +28,7 @@
  *----------------------------------------------------------------------
  */
 
-void
+int
 XDrawLine(display, d, gc, x1, y1, x2, y2)
     Display* display;
     Drawable d;
@@ -43,7 +41,7 @@ XDrawLine(display, d, gc, x1, y1, x2, y2)
     points[0].y = y1;
     points[1].x = x2;
     points[1].y = y2;
-    XDrawLines(display, d, gc, points, 2, CoordModeOrigin);
+    return XDrawLines(display, d, gc, points, 2, CoordModeOrigin);
 }
 
 /*
@@ -63,7 +61,7 @@ XDrawLine(display, d, gc, x1, y1, x2, y2)
  *----------------------------------------------------------------------
  */
 
-void
+int
 XFillRectangle(display, d, gc, x, y, width, height)
     Display* display;
     Drawable d;
@@ -78,5 +76,5 @@ XFillRectangle(display, d, gc, x, y, width, height)
     rectangle.y = y;
     rectangle.width = width;
     rectangle.height = height;
-    XFillRectangles(display, d, gc, &rectangle, 1);
+    return XFillRectangles(display, d, gc, &rectangle, 1);
 }
