@@ -1,25 +1,87 @@
-/* 
+/*
  * xgc.c --
  *
  *	This file contains generic routines for manipulating X graphics
- *	contexts. 
+ *	contexts.
  *
  * Copyright (c) 1995-1996 Sun Microsystems, Inc.
+ * Copyright (c) 2002-2007 Daniel A. Steffen <das@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: xgc.c,v 1.2 1998/09/14 18:24:03 stanton Exp $
  */
 
 #include <tkInt.h>
 
-#ifdef MAC_TCL
-#	include <Xlib.h>
-#else
+#if !defined(MAC_OSX_TK)
 #	include <X11/Xlib.h>
 #endif
+#ifdef MAC_OSX_TK
+#	include <tkMacOSXInt.h>
+#	include <X11/Xlib.h>
+#	include <X11/X.h>
+#	define Cursor XCursor
+#	define Region XRegion
+#endif
 
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * AllocClipMask --
+ *
+ *	Static helper proc to allocate new or clear existing TkpClipMask.
+ *
+ * Results:
+ *	Returns ptr to the new/cleared TkpClipMask.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static TkpClipMask *AllocClipMask(GC gc) {
+    TkpClipMask *clip_mask = (TkpClipMask*) gc->clip_mask;
+
+    if (clip_mask == None) {
+	clip_mask = (TkpClipMask*) ckalloc(sizeof(TkpClipMask));
+	gc->clip_mask = (Pixmap) clip_mask;
+#ifdef MAC_OSX_TK
+    } else if (clip_mask->type == TKP_CLIP_REGION) {
+	TkpReleaseRegion(clip_mask->value.region);
+#endif
+    }
+    return clip_mask;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * FreeClipMask --
+ *
+ *	Static helper proc to free TkpClipMask.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void FreeClipMask(GC gc) {
+    if (gc->clip_mask != None) {
+#ifdef MAC_OSX_TK
+	if (((TkpClipMask*) gc->clip_mask)->type == TKP_CLIP_REGION) {
+	    TkpReleaseRegion(((TkpClipMask*) gc->clip_mask)->value.region);
+	}
+#endif
+	ckfree((char*) gc->clip_mask);
+	gc->clip_mask = None;
+    }
+}
 
 /*
  *----------------------------------------------------------------------
@@ -29,7 +91,7 @@
  *	Allocate a new GC, and initialize the specified fields.
  *
  * Results:
- *	Returns a newly allocated GC. 
+ *	Returns a newly allocated GC.
  *
  * Side effects:
  *	None.
@@ -46,16 +108,26 @@ XCreateGC(display, d, mask, values)
 {
     GC gp;
 
-    gp = (XGCValues *)ckalloc(sizeof(XGCValues));
+/*
+ * In order to have room for a dash list, MAX_DASH_LIST_SIZE extra chars are
+ * defined, which is invisible from the outside. The list is assumed to end
+ * with a 0-char, so this must be set explicitely during initialization.
+ */
+
+#define MAX_DASH_LIST_SIZE 10
+
+    gp = (XGCValues *)ckalloc(sizeof(XGCValues) + MAX_DASH_LIST_SIZE);
     if (!gp) {
 	return None;
     }
 
     gp->function = 	(mask & GCFunction) 	?values->function	:GXcopy;
     gp->plane_mask = 	(mask & GCPlaneMask) 	?values->plane_mask 	:~0;
-    gp->foreground = 	(mask & GCForeground) 	?values->foreground 	:0;
-    gp->background = 	(mask & GCBackground) 	?values->background 	:0xffffff;
-    gp->line_width = 	(mask & GCLineWidth)	?values->line_width	:0;	
+    gp->foreground = 	(mask & GCForeground) 	?values->foreground 	:
+	    BlackPixelOfScreen(DefaultScreenOfDisplay(display));
+    gp->background = 	(mask & GCBackground) 	?values->background 	:
+	    WhitePixelOfScreen(DefaultScreenOfDisplay(display));
+    gp->line_width = 	(mask & GCLineWidth)	?values->line_width	:1;
     gp->line_style = 	(mask & GCLineStyle)	?values->line_style	:LineSolid;
     gp->cap_style =  	(mask & GCCapStyle)	?values->cap_style	:0;
     gp->join_style = 	(mask & GCJoinStyle)	?values->join_style	:0;
@@ -73,13 +145,14 @@ XCreateGC(display, d, mask, values)
     gp->clip_y_origin = (mask & GCClipYOrigin)	?values->clip_y_origin	:0;
     gp->dash_offset = 	(mask & GCDashOffset)	?values->dash_offset	:0;
     gp->dashes = 	(mask & GCDashList)	?values->dashes		:4;
+    (&(gp->dashes))[1] = 	0;
 
+    gp->clip_mask = None;
     if (mask & GCClipMask) {
-	gp->clip_mask = (Pixmap)ckalloc(sizeof(TkpClipMask));
-	((TkpClipMask*)gp->clip_mask)->type = TKP_CLIP_PIXMAP;
-	((TkpClipMask*)gp->clip_mask)->value.pixmap = values->clip_mask;
-    } else {
-	gp->clip_mask = None;
+	TkpClipMask *clip_mask = AllocClipMask(gp);
+
+	clip_mask->type = TKP_CLIP_PIXMAP;
+	clip_mask->value.pixmap = values->clip_mask;
     }
 
     return gp;
@@ -102,7 +175,7 @@ XCreateGC(display, d, mask, values)
  *----------------------------------------------------------------------
  */
 
-void
+int
 XChangeGC(d, gc, mask, values)
     Display * d;
     GC gc;
@@ -113,7 +186,7 @@ XChangeGC(d, gc, mask, values)
     if (mask & GCPlaneMask) { gc->plane_mask = values->plane_mask; }
     if (mask & GCForeground) { gc->foreground = values->foreground; }
     if (mask & GCBackground) { gc->background = values->background; }
-    if (mask & GCLineWidth) { gc->line_width = values->line_width; }	
+    if (mask & GCLineWidth) { gc->line_width = values->line_width; }
     if (mask & GCLineStyle) { gc->line_style = values->line_style; }
     if (mask & GCCapStyle) { gc->cap_style = values->cap_style; }
     if (mask & GCJoinStyle) { gc->join_style = values->join_style; }
@@ -131,7 +204,8 @@ XChangeGC(d, gc, mask, values)
     if (mask & GCClipYOrigin) { gc->clip_y_origin = values->clip_y_origin; }
     if (mask & GCClipMask) { XSetClipMask(d, gc, values->clip_mask); }
     if (mask & GCDashOffset) { gc->dash_offset = values->dash_offset; }
-    if (mask & GCDashList) { gc->dashes = values->dashes; }
+    if (mask & GCDashList) { gc->dashes = values->dashes; (&(gc->dashes))[1] = 0;}
+    return Success;
 }
 
 /*
@@ -150,17 +224,15 @@ XChangeGC(d, gc, mask, values)
  *----------------------------------------------------------------------
  */
 
-void
-XFreeGC(d, gc)
+int XFreeGC(d, gc)
     Display * d;
     GC gc;
 {
     if (gc != None) {
-	if (gc->clip_mask != None) {
-	    ckfree((char*) gc->clip_mask);
-	}
+	FreeClipMask(gc);
 	ckfree((char *) gc);
     }
+    return Success;
 }
 
 /*
@@ -180,52 +252,82 @@ XFreeGC(d, gc)
  *----------------------------------------------------------------------
  */
 
-void 
+int
 XSetForeground(display, gc, foreground)
     Display *display;
     GC gc;
     unsigned long foreground;
 {
     gc->foreground = foreground;
+    return Success;
 }
 
-void 
+int
 XSetBackground(display, gc, background)
     Display *display;
     GC gc;
     unsigned long background;
 {
     gc->background = background;
+    return Success;
 }
 
-void
+int
+XSetDashes(display, gc, dash_offset, dash_list, n)
+    Display* display;
+    GC gc;
+    int dash_offset;
+    _Xconst char* dash_list;
+    int n;
+{
+    char *p = &(gc->dashes);
+
+#ifdef TkWinDeleteBrush
+    TkWinDeleteBrush(gc->fgBrush);
+    TkWinDeletePen(gc->fgPen);
+    TkWinDeleteBrush(gc->bgBrush);
+    TkWinDeletePen(gc->fgExtPen);
+#endif
+    gc->dash_offset = dash_offset;
+    if (n > MAX_DASH_LIST_SIZE) n = MAX_DASH_LIST_SIZE;
+    while (n-- > 0) {
+	*p++ = *dash_list++;
+    }
+    *p = 0;
+    return Success;
+}
+
+int
 XSetFunction(display, gc, function)
     Display *display;
     GC gc;
     int function;
 {
     gc->function = function;
+    return Success;
 }
 
-void
+int
 XSetFillRule(display, gc, fill_rule)
     Display *display;
     GC gc;
     int fill_rule;
 {
     gc->fill_rule = fill_rule;
+    return Success;
 }
 
-void
+int
 XSetFillStyle(display, gc, fill_style)
     Display *display;
     GC gc;
     int fill_style;
 {
     gc->fill_style = fill_style;
+    return Success;
 }
 
-void
+int
 XSetTSOrigin(display, gc, x, y)
     Display *display;
     GC gc;
@@ -233,36 +335,40 @@ XSetTSOrigin(display, gc, x, y)
 {
     gc->ts_x_origin = x;
     gc->ts_y_origin = y;
+    return Success;
 }
 
-void
+int
 XSetFont(display, gc, font)
     Display *display;
     GC gc;
     Font font;
 {
     gc->font = font;
+    return Success;
 }
 
-void
+int
 XSetArcMode(display, gc, arc_mode)
     Display *display;
     GC gc;
     int arc_mode;
 {
     gc->arc_mode = arc_mode;
+    return Success;
 }
 
-void
+int
 XSetStipple(display, gc, stipple)
     Display *display;
     GC gc;
     Pixmap stipple;
 {
     gc->stipple = stipple;
+    return Success;
 }
 
-void
+int
 XSetLineAttributes(display, gc, line_width, line_style, cap_style,
 	join_style)
     Display *display;
@@ -276,9 +382,10 @@ XSetLineAttributes(display, gc, line_width, line_style, cap_style,
     gc->line_style = line_style;
     gc->cap_style = cap_style;
     gc->join_style = join_style;
+    return Success;
 }
 
-void
+int
 XSetClipOrigin(display, gc, clip_x_origin, clip_y_origin)
     Display* display;
     GC gc;
@@ -287,6 +394,7 @@ XSetClipOrigin(display, gc, clip_x_origin, clip_y_origin)
 {
     gc->clip_x_origin = clip_x_origin;
     gc->clip_y_origin = clip_y_origin;
+    return Success;
 }
 
 /*
@@ -296,10 +404,10 @@ XSetClipOrigin(display, gc, clip_x_origin, clip_y_origin)
  *
  *	Sets the clipping region/pixmap for a GC.
  *
- *	Note that unlike the Xlib equivalent, it is not safe to delete
- *	the region after setting it into the GC.  The only use of
- *	TkSetRegion is currently in ImgPhotoDisplay, which uses the GC
- *	immediately.
+ *	Note that unlike the Xlib equivalent, it is not safe to delete the
+ *	region after setting it into the GC (except on Mac OS X). The only
+ *	uses of TkSetRegion are currently in DisplayFrame and in
+ *	ImgPhotoDisplay, which use the GC immediately.
  *
  * Results:
  *	None.
@@ -317,37 +425,192 @@ TkSetRegion(display, gc, r)
     TkRegion r;
 {
     if (r == None) {
-	if (gc->clip_mask) {
-	    ckfree((char*) gc->clip_mask);
-	    gc->clip_mask = None;
-	}
-	return;
-    }
+	FreeClipMask(gc);
+    } else {
+	TkpClipMask *clip_mask = AllocClipMask(gc);
 
-    if (gc->clip_mask == None) {
-	gc->clip_mask = (Pixmap)ckalloc(sizeof(TkpClipMask));
+	clip_mask->type = TKP_CLIP_REGION;
+	clip_mask->value.region = r;
+#ifdef MAC_OSX_TK
+	TkpRetainRegion(r);
+#endif
     }
-    ((TkpClipMask*)gc->clip_mask)->type = TKP_CLIP_REGION;
-    ((TkpClipMask*)gc->clip_mask)->value.region = r;
 }
 
-void
+int
 XSetClipMask(display, gc, pixmap)
     Display* display;
     GC gc;
     Pixmap pixmap;
 {
     if (pixmap == None) {
-	if (gc->clip_mask) {
-	    ckfree((char*) gc->clip_mask);
-	    gc->clip_mask = None;
-	}
-	return;
-    }
+	FreeClipMask(gc);
+    } else {
+	TkpClipMask *clip_mask = AllocClipMask(gc);
 
-    if (gc->clip_mask == None) {
-	gc->clip_mask = (Pixmap)ckalloc(sizeof(TkpClipMask));
+	clip_mask->type = TKP_CLIP_PIXMAP;
+	clip_mask->value.pixmap = pixmap;
     }
-    ((TkpClipMask*)gc->clip_mask)->type = TKP_CLIP_PIXMAP;
-    ((TkpClipMask*)gc->clip_mask)->value.pixmap = pixmap;
+    return Success;
 }
+
+/*
+ * Some additional dummy functions (hopefully implemented soon).
+ */
+
+#if 0
+Cursor
+XCreateFontCursor(display, shape)
+    Display* display;
+    unsigned int shape;
+{
+    return (Cursor) 0;
+}
+
+void
+XDrawImageString(display, d, gc, x, y, string, length)
+    Display* display;
+    Drawable d;
+    GC gc;
+    int x;
+    int y;
+    _Xconst char* string;
+    int length;
+{
+}
+#endif
+
+void
+XDrawPoint(display, d, gc, x, y)
+    Display* display;
+    Drawable d;
+    GC gc;
+    int x;
+    int y;
+{
+    XDrawLine(display, d, gc, x, y, x, y);
+}
+
+void
+XDrawPoints(display, d, gc, points, npoints, mode)
+    Display* display;
+    Drawable d;
+    GC gc;
+    XPoint* points;
+    int npoints;
+    int mode;
+{
+    int i;
+
+    for (i=0; i<npoints; i++) {
+	XDrawPoint(display, d, gc, points[i].x, points[i].y);
+    }
+}
+
+#if !defined(MAC_OSX_TK)
+void
+XDrawSegments(display, d, gc, segments, nsegments)
+    Display* display;
+    Drawable d;
+    GC gc;
+    XSegment* segments;
+    int nsegments;
+{
+}
+#endif
+
+#if 0
+char *
+XFetchBuffer(display, nbytes_return, buffer)
+    Display* display;
+    int* nbytes_return;
+    int buffer;
+{
+    return (char *) 0;
+}
+
+Status XFetchName(display, w, window_name_return)
+    Display* display;
+    Window w;
+    char** window_name_return;
+{
+    return (Status) 0;
+}
+
+Atom *XListProperties(display, w, num_prop_return)
+    Display* display;
+    Window w;
+    int* num_prop_return;
+{
+    return (Atom *) 0;
+}
+
+void
+XMapRaised(display, w)
+    Display* display;
+    Window w;
+{
+}
+
+void
+XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height)
+    Display* display;
+    Drawable d;
+    GC gc;
+    XImage* image;
+    int src_x;
+    int src_y;
+    int dest_x;
+    int dest_y;
+    unsigned int width;
+    unsigned int height;
+{
+}
+
+void
+XQueryTextExtents(display, font_ID, string, nchars, direction_return,
+	font_ascent_return, font_descent_return, overall_return)
+    Display* display;
+    XID font_ID;
+    _Xconst char* string;
+    int nchars;
+    int* direction_return;
+    int* font_ascent_return;
+    int* font_descent_return;
+    XCharStruct* overall_return;
+{
+}
+
+void
+XReparentWindow(display, w, parent, x, y)
+    Display* display;
+    Window w;
+    Window parent;
+    int x;
+    int y;
+{
+}
+
+void
+XRotateBuffers(display, rotate)
+    Display* display;
+    int rotate;
+{
+}
+
+void
+XStoreBuffer(display, bytes, nbytes, buffer)
+    Display* display;
+    _Xconst char* bytes;
+    int nbytes;
+    int buffer;
+{
+}
+
+void
+XUndefineCursor(display, w)
+    Display* display;
+    Window w;
+{
+}
+#endif
